@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -22,6 +22,7 @@
 #include <cmath>
 
 #include "SDL_lib.hxx"
+#include "Logger.hxx"
 #include "FrameBuffer.hxx"
 #include "Settings.hxx"
 #include "System.hxx"
@@ -40,25 +41,18 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 SoundSDL2::SoundSDL2(OSystem& osystem, AudioSettings& audioSettings)
   : Sound(osystem),
-    myIsInitializedFlag(false),
-    myVolume(100),
-    myVolumeFactor(0xffff),
-    myDevice(0),
-    myEmulationTiming(nullptr),
-    myCurrentFragment(nullptr),
-    myUnderrun(false),
     myAudioSettings(audioSettings)
 {
   ASSERT_MAIN_THREAD;
 
-  myOSystem.logMessage("SoundSDL2::SoundSDL2 started ...", 2);
+  Logger::debug("SoundSDL2::SoundSDL2 started ...");
 
   if (SDL_InitSubSystem(SDL_INIT_AUDIO) < 0) {
     ostringstream buf;
 
     buf << "WARNING: Failed to initialize SDL audio system! " << endl
         << "         " << SDL_GetError() << endl;
-    myOSystem.logMessage(buf.str(), 0);
+    Logger::error(buf.str());
     return;
   }
 
@@ -66,9 +60,9 @@ SoundSDL2::SoundSDL2(OSystem& osystem, AudioSettings& audioSettings)
   if(!openDevice())
     return;
 
-  mute(true);
+  SoundSDL2::mute(true);
 
-  myOSystem.logMessage("SoundSDL2::SoundSDL2 initialized", 2);
+  Logger::debug("SoundSDL2::SoundSDL2 initialized");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -106,7 +100,7 @@ bool SoundSDL2::openDevice()
 
     buf << "WARNING: Couldn't open SDL audio device! " << endl
         << "         " << SDL_GetError() << endl;
-    myOSystem.logMessage(buf.str(), 0);
+    Logger::error(buf.str());
 
     return myIsInitializedFlag = false;
   }
@@ -119,8 +113,8 @@ void SoundSDL2::setEnabled(bool state)
   myAudioSettings.setEnabled(state);
   if (myAudioQueue) myAudioQueue->ignoreOverflows(!state);
 
-  myOSystem.logMessage(state ? "SoundSDL2::setEnabled(true)" :
-                               "SoundSDL2::setEnabled(false)", 2);
+  Logger::debug(state ? "SoundSDL2::setEnabled(true)" :
+                "SoundSDL2::setEnabled(false)");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -137,13 +131,13 @@ void SoundSDL2::open(shared_ptr<AudioQueue> audioQueue,
 
   myEmulationTiming = emulationTiming;
 
-  myOSystem.logMessage("SoundSDL2::open started ...", 2);
+  Logger::debug("SoundSDL2::open started ...");
   mute(true);
 
   audioQueue->ignoreOverflows(!myAudioSettings.enabled());
   if(!myAudioSettings.enabled())
   {
-    myOSystem.logMessage("Sound disabled\n", 1);
+    Logger::info("Sound disabled\n");
     return;
   }
 
@@ -159,12 +153,12 @@ void SoundSDL2::open(shared_ptr<AudioQueue> audioQueue,
   // Show some info
   myAboutString = about();
   if(myAboutString != pre_about)
-    myOSystem.logMessage(myAboutString, 1);
+    Logger::info(myAboutString);
 
   // And start the SDL sound subsystem ...
   mute(false);
 
-  myOSystem.logMessage("SoundSDL2::open finished", 2);
+  Logger::debug("SoundSDL2::open finished");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -177,8 +171,6 @@ void SoundSDL2::close()
   if (myAudioQueue) myAudioQueue->closeSink(myCurrentFragment);
   myAudioQueue.reset();
   myCurrentFragment = nullptr;
-
-  myOSystem.logMessage("SoundSDL2::close", 2);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -191,6 +183,22 @@ bool SoundSDL2::mute(bool state)
   return oldstate;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool SoundSDL2::toggleMute()
+{
+  bool enabled = myAudioSettings.enabled();
+
+  setEnabled(!enabled);
+  myOSystem.console().initializeAudio();
+
+  string message = "Sound ";
+  message += !enabled ? "unmuted" : "muted";
+
+  myOSystem.frameBuffer().showMessage(message);
+
+  return enabled;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void SoundSDL2::setVolume(uInt32 percent)
 {
@@ -200,7 +208,7 @@ void SoundSDL2::setVolume(uInt32 percent)
     myVolume = percent;
 
     SDL_LockAudioDevice(myDevice);
-    myVolumeFactor = static_cast<float>(percent) / 100.f;
+    myVolumeFactor = static_cast<float>(percent) / 100.F;
     SDL_UnlockAudioDevice(myDevice);
   }
 }
@@ -223,6 +231,15 @@ void SoundSDL2::adjustVolume(Int8 direction)
 
   setVolume(percent);
 
+  // Enable audio if it is currently disabled
+  bool enabled = myAudioSettings.enabled();
+
+  if(percent > 0 && !enabled)
+  {
+    setEnabled(!enabled);
+    myOSystem.console().initializeAudio();
+  }
+
   // Now show an onscreen message
   strval << percent;
   message = "Volume set to ";
@@ -253,8 +270,8 @@ string SoundSDL2::about() const
     case AudioSettings::Preset::highQualityLowLag:
       buf << "High quality, low lag" << endl;
       break;
-    case AudioSettings::Preset::veryHighQualityVeryLowLag:
-      buf << "Very high quality, very low lag" << endl;
+    case AudioSettings::Preset::ultraQualityMinimalLag:
+      buf << "Ultra quality, minimal lag" << endl;
       break;
   }
   buf << "    Fragment size: " << uInt32(myHardwareSpec.samples) << " bytes" << endl
@@ -284,7 +301,8 @@ void SoundSDL2::processFragment(float* stream, uInt32 length)
 {
   myResampler->fillFragment(stream, length);
 
-  for (uInt32 i = 0; i < length; i++) stream[i] = stream[i] * myVolumeFactor;
+  for (uInt32 i = 0; i < length; ++i)
+    stream[i] *= myVolumeFactor;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -305,8 +323,6 @@ void SoundSDL2::initResampler()
     return nextFragment;
   };
 
-  StaggeredLogger::Logger logger = [this](string msg) { myOSystem.logMessage(msg, 1); };
-
   Resampler::Format formatFrom =
     Resampler::Format(myEmulationTiming->audioSampleRate(), myAudioQueue->fragmentSize(), myAudioQueue->isStereo());
   Resampler::Format formatTo =
@@ -314,15 +330,15 @@ void SoundSDL2::initResampler()
 
   switch (myAudioSettings.resamplingQuality()) {
     case AudioSettings::ResamplingQuality::nearestNeightbour:
-      myResampler = make_unique<SimpleResampler>(formatFrom, formatTo, nextFragmentCallback, logger);
+      myResampler = make_unique<SimpleResampler>(formatFrom, formatTo, nextFragmentCallback);
       break;
 
     case AudioSettings::ResamplingQuality::lanczos_2:
-      myResampler = make_unique<LanczosResampler>(formatFrom, formatTo, nextFragmentCallback, 2, logger);
+      myResampler = make_unique<LanczosResampler>(formatFrom, formatTo, nextFragmentCallback, 2);
       break;
 
     case AudioSettings::ResamplingQuality::lanczos_3:
-      myResampler = make_unique<LanczosResampler>(formatFrom, formatTo, nextFragmentCallback, 3, logger);
+      myResampler = make_unique<LanczosResampler>(formatFrom, formatTo, nextFragmentCallback, 3);
       break;
 
     default:
