@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -17,6 +17,7 @@
 
 #include <thread>
 #include "AtariNTSC.hxx"
+#include "PhosphorHandler.hxx"
 
 // blitter related
 #ifndef restrict
@@ -31,29 +32,41 @@
 #endif
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void AtariNTSC::initialize(const Setup& setup, const uInt8* palette)
+void AtariNTSC::initialize(const Setup& setup)
 {
   init(myImpl, setup);
-  initializePalette(palette);
+  generateKernels();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void AtariNTSC::initializePalette(const uInt8* palette)
+void AtariNTSC::setPalette(const PaletteArray& palette)
 {
-  // Palette stores R/G/B data for 'palette_size' entries
-  for ( uInt32 entry = 0; entry < palette_size; ++entry )
+  uInt8* ptr = myRGBPalette.data();
+  for(size_t i = 0; i < palette.size(); ++i)
   {
-    float r = myImpl.to_float [*palette++];
-    float g = myImpl.to_float [*palette++];
-    float b = myImpl.to_float [*palette++];
+    *ptr++ = (palette[i] >> 16) & 0xff;
+    *ptr++ = (palette[i] >> 8) & 0xff;
+    *ptr++ = palette[i] & 0xff;
+  }
+  generateKernels();
+}
 
-    float y, i, q = RGB_TO_YIQ( r, g, b, y, i );
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void AtariNTSC::generateKernels()
+{
+  const uInt8* ptr = myRGBPalette.data();
+  for(size_t entry = 0; entry < myRGBPalette.size() / 3; ++entry)
+  {
+    float r = myImpl.to_float[*ptr++],
+          g = myImpl.to_float[*ptr++],
+          b = myImpl.to_float[*ptr++];
+    float y, i, q;  RGB_TO_YIQ( r, g, b, y, i, q );
 
     // Generate kernel
-    int ir, ig, ib = YIQ_TO_RGB( y, i, q, myImpl.to_rgb, int, ir, ig );
+    int ir, ig, ib;  YIQ_TO_RGB( y, i, q, myImpl.to_rgb.data(), ir, ig, ib );
     uInt32 rgb = PACK_RGB( ir, ig, ib );
 
-    uInt32* kernel = myColorTable[entry];
+    uInt32* kernel = myColorTable[entry].data();
     genKernel(myImpl, y, i, q, kernel);
 
     for ( uInt32 c = 0; c < rgb_kernel_size / 2; ++c )
@@ -77,12 +90,12 @@ void AtariNTSC::enableThreading(bool enable)
   }
   else
   {
-    systemThreads = std::min(4u, systemThreads);
+    systemThreads = std::max<uInt32>(1, std::min<uInt32>(4, systemThreads - 1));
 
     myWorkerThreads = systemThreads - 1;
     myTotalThreads  = systemThreads;
 
-    myThreads = make_unique<std::thread[]>(myWorkerThreads);
+    myThreads = make_unique<std::thread[]>(myWorkerThreads);  // NOLINT
   }
 }
 
@@ -132,50 +145,53 @@ void AtariNTSC::renderThread(const uInt8* atari_in, const uInt32 in_width,
     uInt32* restrict line_out = static_cast<uInt32*>(rgb_out);
     ++line_in;
 
+    // shift right by 2 pixel
+    line_out[0] = line_out[1] = 0;
+    line_out += 2;
+
     for(uInt32 n = chunk_count; n; --n)
     {
       // order of input and output pixels must not be altered
-      ATARI_NTSC_COLOR_IN(0, line_in[0]);
-      ATARI_NTSC_RGB_OUT_8888(0, line_out[0]);
-      ATARI_NTSC_RGB_OUT_8888(1, line_out[1]);
-      ATARI_NTSC_RGB_OUT_8888(2, line_out[2]);
-      ATARI_NTSC_RGB_OUT_8888(3, line_out[3]);
+      ATARI_NTSC_COLOR_IN(0, line_in[0])
+      ATARI_NTSC_RGB_OUT_8888(0, line_out[0])
+      ATARI_NTSC_RGB_OUT_8888(1, line_out[1])
+      ATARI_NTSC_RGB_OUT_8888(2, line_out[2])
+      ATARI_NTSC_RGB_OUT_8888(3, line_out[3])
 
-      ATARI_NTSC_COLOR_IN(1, line_in[1]);
-      ATARI_NTSC_RGB_OUT_8888(4, line_out[4]);
-      ATARI_NTSC_RGB_OUT_8888(5, line_out[5]);
-      ATARI_NTSC_RGB_OUT_8888(6, line_out[6]);
+      ATARI_NTSC_COLOR_IN(1, line_in[1])
+      ATARI_NTSC_RGB_OUT_8888(4, line_out[4])
+      ATARI_NTSC_RGB_OUT_8888(5, line_out[5])
+      ATARI_NTSC_RGB_OUT_8888(6, line_out[6])
 
       line_in += 2;
       line_out += 7;
     }
 
     // finish final pixels
-    ATARI_NTSC_COLOR_IN(0, line_in[0]);
-    ATARI_NTSC_RGB_OUT_8888(0, line_out[0]);
-    ATARI_NTSC_RGB_OUT_8888(1, line_out[1]);
-    ATARI_NTSC_RGB_OUT_8888(2, line_out[2]);
-    ATARI_NTSC_RGB_OUT_8888(3, line_out[3]);
+    ATARI_NTSC_COLOR_IN(0, line_in[0])
+    ATARI_NTSC_RGB_OUT_8888(0, line_out[0])
+    ATARI_NTSC_RGB_OUT_8888(1, line_out[1])
+    ATARI_NTSC_RGB_OUT_8888(2, line_out[2])
+    ATARI_NTSC_RGB_OUT_8888(3, line_out[3])
 
-    ATARI_NTSC_COLOR_IN(1, NTSC_black);
-    ATARI_NTSC_RGB_OUT_8888(4, line_out[4]);
-    ATARI_NTSC_RGB_OUT_8888(5, line_out[5]);
-    ATARI_NTSC_RGB_OUT_8888(6, line_out[6]);
+    ATARI_NTSC_COLOR_IN(1, NTSC_black)
+    ATARI_NTSC_RGB_OUT_8888(4, line_out[4])
+    ATARI_NTSC_RGB_OUT_8888(5, line_out[5])
+    ATARI_NTSC_RGB_OUT_8888(6, line_out[6])
 
-    line_in += 2;
     line_out += 7;
 
-    ATARI_NTSC_COLOR_IN(0, NTSC_black);
-    ATARI_NTSC_RGB_OUT_8888(0, line_out[0]);
-    ATARI_NTSC_RGB_OUT_8888(1, line_out[1]);
-    ATARI_NTSC_RGB_OUT_8888(2, line_out[2]);
-    ATARI_NTSC_RGB_OUT_8888(3, line_out[3]);
+    ATARI_NTSC_COLOR_IN(0, NTSC_black)
+    ATARI_NTSC_RGB_OUT_8888(0, line_out[0])
+    ATARI_NTSC_RGB_OUT_8888(1, line_out[1])
+    ATARI_NTSC_RGB_OUT_8888(2, line_out[2])
+    ATARI_NTSC_RGB_OUT_8888(3, line_out[3])
 
-    ATARI_NTSC_COLOR_IN(1, NTSC_black);
-    ATARI_NTSC_RGB_OUT_8888(4, line_out[4]);
+    ATARI_NTSC_COLOR_IN(1, NTSC_black)
+    ATARI_NTSC_RGB_OUT_8888(4, line_out[4])
 #if 0
-    ATARI_NTSC_RGB_OUT_8888(5, line_out[5]);
-    ATARI_NTSC_RGB_OUT_8888(6, line_out[6]);
+    ATARI_NTSC_RGB_OUT_8888(5, line_out[5])
+    ATARI_NTSC_RGB_OUT_8888(6, line_out[6])
 #endif
 
     atari_in += in_width;
@@ -205,89 +221,93 @@ void AtariNTSC::renderWithPhosphorThread(const uInt8* atari_in, const uInt32 in_
     uInt32* restrict line_out = static_cast<uInt32*>(rgb_out);
     ++line_in;
 
+    // shift right by 2 pixel
+    line_out[0] = line_out[1] = 0;
+    line_out += 2;
+
     for(uInt32 n = chunk_count; n; --n)
     {
       // order of input and output pixels must not be altered
-      ATARI_NTSC_COLOR_IN(0, line_in[0]);
-      ATARI_NTSC_RGB_OUT_8888(0, line_out[0]);
-      ATARI_NTSC_RGB_OUT_8888(1, line_out[1]);
-      ATARI_NTSC_RGB_OUT_8888(2, line_out[2]);
-      ATARI_NTSC_RGB_OUT_8888(3, line_out[3]);
+      ATARI_NTSC_COLOR_IN(0, line_in[0])
+      ATARI_NTSC_RGB_OUT_8888(0, line_out[0])
+      ATARI_NTSC_RGB_OUT_8888(1, line_out[1])
+      ATARI_NTSC_RGB_OUT_8888(2, line_out[2])
+      ATARI_NTSC_RGB_OUT_8888(3, line_out[3])
 
-      ATARI_NTSC_COLOR_IN(1, line_in[1]);
-      ATARI_NTSC_RGB_OUT_8888(4, line_out[4]);
-      ATARI_NTSC_RGB_OUT_8888(5, line_out[5]);
-      ATARI_NTSC_RGB_OUT_8888(6, line_out[6]);
+      ATARI_NTSC_COLOR_IN(1, line_in[1])
+      ATARI_NTSC_RGB_OUT_8888(4, line_out[4])
+      ATARI_NTSC_RGB_OUT_8888(5, line_out[5])
+      ATARI_NTSC_RGB_OUT_8888(6, line_out[6])
 
       line_in += 2;
       line_out += 7;
     }
 
     // finish final pixels
-    ATARI_NTSC_COLOR_IN(0, line_in[0]);
-    ATARI_NTSC_RGB_OUT_8888(0, line_out[0]);
-    ATARI_NTSC_RGB_OUT_8888(1, line_out[1]);
-    ATARI_NTSC_RGB_OUT_8888(2, line_out[2]);
-    ATARI_NTSC_RGB_OUT_8888(3, line_out[3]);
+    ATARI_NTSC_COLOR_IN(0, line_in[0])
+    ATARI_NTSC_RGB_OUT_8888(0, line_out[0])
+    ATARI_NTSC_RGB_OUT_8888(1, line_out[1])
+    ATARI_NTSC_RGB_OUT_8888(2, line_out[2])
+    ATARI_NTSC_RGB_OUT_8888(3, line_out[3])
 
-    ATARI_NTSC_COLOR_IN(1, NTSC_black);
-    ATARI_NTSC_RGB_OUT_8888(4, line_out[4]);
-    ATARI_NTSC_RGB_OUT_8888(5, line_out[5]);
-    ATARI_NTSC_RGB_OUT_8888(6, line_out[6]);
+    ATARI_NTSC_COLOR_IN(1, NTSC_black)
+    ATARI_NTSC_RGB_OUT_8888(4, line_out[4])
+    ATARI_NTSC_RGB_OUT_8888(5, line_out[5])
+    ATARI_NTSC_RGB_OUT_8888(6, line_out[6])
 
-    line_in += 2;
     line_out += 7;
 
-    ATARI_NTSC_COLOR_IN(0, NTSC_black);
-    ATARI_NTSC_RGB_OUT_8888(0, line_out[0]);
-    ATARI_NTSC_RGB_OUT_8888(1, line_out[1]);
-    ATARI_NTSC_RGB_OUT_8888(2, line_out[2]);
-    ATARI_NTSC_RGB_OUT_8888(3, line_out[3]);
+    ATARI_NTSC_COLOR_IN(0, NTSC_black)
+    ATARI_NTSC_RGB_OUT_8888(0, line_out[0])
+    ATARI_NTSC_RGB_OUT_8888(1, line_out[1])
+    ATARI_NTSC_RGB_OUT_8888(2, line_out[2])
+    ATARI_NTSC_RGB_OUT_8888(3, line_out[3])
 
-    ATARI_NTSC_COLOR_IN(1, NTSC_black);
-    ATARI_NTSC_RGB_OUT_8888(4, line_out[4]);
+    ATARI_NTSC_COLOR_IN(1, NTSC_black)
+    ATARI_NTSC_RGB_OUT_8888(4, line_out[4])
 #if 0
-    ATARI_NTSC_RGB_OUT_8888(5, line_out[5]);
-    ATARI_NTSC_RGB_OUT_8888(6, line_out[6]);
+    ATARI_NTSC_RGB_OUT_8888(5, line_out[5])
+    ATARI_NTSC_RGB_OUT_8888(6, line_out[6])
 #endif
 
     // Do phosphor mode (blend the resulting frames)
-    // Note: The code assumes that AtariNTSC::outWidth(kTIAW) == outPitch == 565
+    // Note: The unrolled code assumed that AtariNTSC::outWidth(kTIAW) == outPitch == 565
+    // Now this got changed to 568 so the final 5 calculations got removed.
     for (uInt32 x = AtariNTSC::outWidth(in_width) / 8; x; --x)
     {
       // Store back into displayed frame buffer (for next frame)
-      rgb_in[bufofs] = getRGBPhosphor(out[bufofs], rgb_in[bufofs]);
+      rgb_in[bufofs] = PhosphorHandler::getPixel(out[bufofs], rgb_in[bufofs]);
       ++bufofs;
-      rgb_in[bufofs] = getRGBPhosphor(out[bufofs], rgb_in[bufofs]);
+      rgb_in[bufofs] = PhosphorHandler::getPixel(out[bufofs], rgb_in[bufofs]);
       ++bufofs;
-      rgb_in[bufofs] = getRGBPhosphor(out[bufofs], rgb_in[bufofs]);
+      rgb_in[bufofs] = PhosphorHandler::getPixel(out[bufofs], rgb_in[bufofs]);
       ++bufofs;
-      rgb_in[bufofs] = getRGBPhosphor(out[bufofs], rgb_in[bufofs]);
+      rgb_in[bufofs] = PhosphorHandler::getPixel(out[bufofs], rgb_in[bufofs]);
       ++bufofs;
-      rgb_in[bufofs] = getRGBPhosphor(out[bufofs], rgb_in[bufofs]);
+      rgb_in[bufofs] = PhosphorHandler::getPixel(out[bufofs], rgb_in[bufofs]);
       ++bufofs;
-      rgb_in[bufofs] = getRGBPhosphor(out[bufofs], rgb_in[bufofs]);
+      rgb_in[bufofs] = PhosphorHandler::getPixel(out[bufofs], rgb_in[bufofs]);
       ++bufofs;
-      rgb_in[bufofs] = getRGBPhosphor(out[bufofs], rgb_in[bufofs]);
+      rgb_in[bufofs] = PhosphorHandler::getPixel(out[bufofs], rgb_in[bufofs]);
       ++bufofs;
-      rgb_in[bufofs] = getRGBPhosphor(out[bufofs], rgb_in[bufofs]);
+      rgb_in[bufofs] = PhosphorHandler::getPixel(out[bufofs], rgb_in[bufofs]);
       ++bufofs;
     }
-    // finish final pixels
-    rgb_in[bufofs] = getRGBPhosphor(out[bufofs], rgb_in[bufofs]);
+    // finish final 565 % 8 = 5 pixels
+    /*rgb_in[bufofs] = PhosphorHandler::getPixel(out[bufofs], rgb_in[bufofs]);
     ++bufofs;
-    rgb_in[bufofs] = getRGBPhosphor(out[bufofs], rgb_in[bufofs]);
+    rgb_in[bufofs] = PhosphorHandler::getPixel(out[bufofs], rgb_in[bufofs]);
     ++bufofs;
-    rgb_in[bufofs] = getRGBPhosphor(out[bufofs], rgb_in[bufofs]);
+    rgb_in[bufofs] = PhosphorHandler::getPixel(out[bufofs], rgb_in[bufofs]);
     ++bufofs;
-    rgb_in[bufofs] = getRGBPhosphor(out[bufofs], rgb_in[bufofs]);
-    ++bufofs;
-    rgb_in[bufofs] = getRGBPhosphor(out[bufofs], rgb_in[bufofs]);
+    rgb_in[bufofs] = PhosphorHandler::getPixel(out[bufofs], rgb_in[bufofs]);
     ++bufofs;
+    rgb_in[bufofs] = PhosphorHandler::getPixel(out[bufofs], rgb_in[bufofs]);
+    ++bufofs;*/
 #if 0
-    rgb_in[bufofs] = getRGBPhosphor(out[bufofs], rgb_in[bufofs]);
+    rgb_in[bufofs] = PhosphorHandler::getPixel(out[bufofs], rgb_in[bufofs]);
     ++bufofs;
-    rgb_in[bufofs] = getRGBPhosphor(out[bufofs], rgb_in[bufofs]);
+    rgb_in[bufofs] = PhosphorHandler::getPixel(out[bufofs], rgb_in[bufofs]);
     ++bufofs;
 #endif
 
@@ -296,37 +316,18 @@ void AtariNTSC::renderWithPhosphorThread(const uInt8* atari_in, const uInt32 in_
   }
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-inline uInt32 AtariNTSC::getRGBPhosphor(const uInt32 c, const uInt32 p) const
-{
-#define TO_RGB(color, red, green, blue)   \
-    const uInt8 red   = uInt8(color >> 16); \
-    const uInt8 green = uInt8(color >> 8);\
-    const uInt8 blue  = uInt8(color);
-
-  TO_RGB(c, rc, gc, bc);
-  TO_RGB(p, rp, gp, bp);
-
-  // Mix current calculated frame with previous displayed frame
-  const uInt8 rn = myPhosphorPalette[rc][rp];
-  const uInt8 gn = myPhosphorPalette[gc][gp];
-  const uInt8 bn = myPhosphorPalette[bc][bp];
-
-  return (rn << 16) | (gn << 8) | bn;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void AtariNTSC::init(init_t& impl, const Setup& setup)
 {
-  impl.brightness = float(setup.brightness) * (0.5f * rgb_unit) + rgb_offset;
-  impl.contrast   = float(setup.contrast)   * (0.5f * rgb_unit) + rgb_unit;
+  impl.brightness = setup.brightness * (0.5F * rgb_unit) + rgb_offset;
+  impl.contrast   = setup.contrast   * (0.5F * rgb_unit) + rgb_unit;
 
-  impl.artifacts = float(setup.artifacts);
+  impl.artifacts = setup.artifacts;
   if ( impl.artifacts > 0 )
     impl.artifacts *= artifacts_max - artifacts_mid;
   impl.artifacts = impl.artifacts * artifacts_mid + artifacts_mid;
 
-  impl.fringing = float(setup.fringing);
+  impl.fringing = setup.fringing;
   if ( impl.fringing > 0 )
     impl.fringing *= fringing_max - fringing_mid;
   impl.fringing = impl.fringing * fringing_mid + fringing_mid;
@@ -336,30 +337,30 @@ void AtariNTSC::init(init_t& impl, const Setup& setup)
   /* generate gamma table */
   if (true)  /* was (gamma_size > 1) */
   {
-    float const to_float = 1.0f / (gamma_size - 1/*(gamma_size > 1)*/);
-    float const gamma = 1.1333f - float(setup.gamma) * 0.5f;
+    float const to_float = 1.F / (gamma_size - 1/*(gamma_size > 1)*/);
+    float const gamma = 1.1333F - setup.gamma * 0.5F;
     /* match common PC's 2.2 gamma to TV's 2.65 gamma */
     int i;
     for ( i = 0; i < gamma_size; i++ )
       impl.to_float [i] =
-          float(pow( i * to_float, gamma )) * impl.contrast + impl.brightness;
+          powf( i * to_float, gamma ) * impl.contrast + impl.brightness;
   }
 
   /* setup decoder matricies */
   {
-    float hue = float(setup.hue) * PI + PI / 180 * ext_decoder_hue;
-    float sat = float(setup.saturation) + 1;
-    hue += PI / 180 * (std_decoder_hue - ext_decoder_hue);
+    float hue = setup.hue * BSPF::PI_f + BSPF::PI_f / 180 * ext_decoder_hue;
+    float sat = setup.saturation + 1;
+    hue += BSPF::PI_f / 180 * (std_decoder_hue - ext_decoder_hue);
 
-    float s = float(sin( hue )) * sat;
-    float c = float(cos( hue )) * sat;
-    float* out = impl.to_rgb;
+    float s = sinf( hue ) * sat;
+    float c = cosf( hue ) * sat;
+    float* out = impl.to_rgb.data();
     int n;
 
     n = burst_count;
     do
     {
-      float const* in = default_decoder;
+      float const* in = default_decoder.data();
       int n2 = 3;
       do
       {
@@ -371,7 +372,7 @@ void AtariNTSC::init(init_t& impl, const Setup& setup)
       while ( --n2 );
     #if 0  // burst_count is always 0
       if ( burst_count > 1 )
-        ROTATE_IQ( s, c, 0.866025f, -0.5f ); /* +120 degrees */
+        ROTATE_IQ( s, c, 0.866025F, -0.5F ); /* +120 degrees */
     #endif
     }
     while ( --n );
@@ -381,50 +382,49 @@ void AtariNTSC::init(init_t& impl, const Setup& setup)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void AtariNTSC::initFilters(init_t& impl, const Setup& setup)
 {
-  float kernels [kernel_size * 2];
+  std::array<float, kernel_size * 2> kernels;
 
   /* generate luma (y) filter using sinc kernel */
   {
     /* sinc with rolloff (dsf) */
-    float const rolloff = 1 + float(setup.sharpness) * 0.032f;
-    float const maxh = 32;
-    float const pow_a_n = float(pow( rolloff, maxh ));
+    float const rolloff = 1 + setup.sharpness * 0.032F;
+    constexpr float maxh = 32;
+    float const pow_a_n = powf( rolloff, maxh );
     float sum;
-    int i;
     /* quadratic mapping to reduce negative (blurring) range */
-    float to_angle = float(setup.resolution) + 1;
-    to_angle = PI / maxh * float(LUMA_CUTOFF) * (to_angle * to_angle + 1);
+    float to_angle = setup.resolution + 1;
+    to_angle = BSPF::PI_f / maxh * luma_cutoff * (to_angle * to_angle + 1.F);
 
     kernels [kernel_size * 3 / 2] = maxh; /* default center value */
-    for ( i = 0; i < kernel_half * 2 + 1; i++ )
+    for ( int i = 0; i < kernel_half * 2 + 1; i++ )
     {
       int x = i - kernel_half;
       float angle = x * to_angle;
       /* instability occurs at center point with rolloff very close to 1.0 */
-      if ( x || pow_a_n > 1.056 || pow_a_n < 0.981 )
+      if ( x || pow_a_n > 1.056F || pow_a_n < 0.981F )
       {
-        float rolloff_cos_a = rolloff * float(cos( angle ));
+        float rolloff_cos_a = rolloff * cosf( angle );
         float num = 1 - rolloff_cos_a -
-            pow_a_n * float(cos( maxh * angle )) +
-            pow_a_n * rolloff * float(cos( (maxh - 1) * angle ));
+            pow_a_n * cosf( maxh * angle ) +
+            pow_a_n * rolloff * cosf( (maxh - 1) * angle );
         float den = 1 - rolloff_cos_a - rolloff_cos_a + rolloff * rolloff;
         float dsf = num / den;
-        kernels [kernel_size * 3 / 2 - kernel_half + i] = dsf - 0.5f;
+        kernels [kernel_size * 3 / 2 - kernel_half + i] = dsf - 0.5F;
       }
     }
 
     /* apply blackman window and find sum */
     sum = 0;
-    for ( i = 0; i < kernel_half * 2 + 1; i++ )
+    for ( int i = 0; i < kernel_half * 2 + 1; i++ )
     {
-      float x = PI * 2 / (kernel_half * 2) * i;
-      float blackman = 0.42f - 0.5f * float(cos( x )) + 0.08f * float(cos( x * 2 ));
+      float x = BSPF::PI_f * 2 / (kernel_half * 2) * i;
+      float blackman = 0.42F - 0.5F * cosf( x ) + 0.08F * cosf( x * 2 );
       sum += (kernels [kernel_size * 3 / 2 - kernel_half + i] *= blackman);
     }
 
     /* normalize kernel */
-    sum = 1.0f / sum;
-    for ( i = 0; i < kernel_half * 2 + 1; i++ )
+    sum = 1.0F / sum;
+    for ( int i = 0; i < kernel_half * 2 + 1; i++ )
     {
       int x = kernel_size * 3 / 2 - kernel_half + i;
       kernels [x] *= sum;
@@ -433,9 +433,8 @@ void AtariNTSC::initFilters(init_t& impl, const Setup& setup)
 
   /* generate chroma (iq) filter using gaussian kernel */
   {
-    float const cutoff_factor = -0.03125f;
-    float cutoff = float(setup.bleed);
-    int i;
+    constexpr float cutoff_factor = -0.03125F;
+    float cutoff = setup.bleed;
 
     if ( cutoff < 0 )
     {
@@ -443,22 +442,22 @@ void AtariNTSC::initFilters(init_t& impl, const Setup& setup)
       cutoff *= cutoff;
       cutoff *= cutoff;
       cutoff *= cutoff;
-      cutoff *= -30.0f / 0.65f;
+      cutoff *= -30.0F / 0.65F;
     }
-    cutoff = cutoff_factor - 0.65f * cutoff_factor * cutoff;
+    cutoff = cutoff_factor - 0.65F * cutoff_factor * cutoff;
 
-    for ( i = -kernel_half; i <= kernel_half; i++ )
-      kernels [kernel_size / 2 + i] = float(exp( i * i * cutoff ));
+    for ( int i = -kernel_half; i <= kernel_half; i++ )
+      kernels [kernel_size / 2 + i] = expf( i * i * cutoff );
 
     /* normalize even and odd phases separately */
-    for ( i = 0; i < 2; i++ )
+    for ( int i = 0; i < 2; i++ )
     {
       float sum = 0;
       int x;
       for ( x = i; x < kernel_size; x += 2 )
         sum += kernels [x];
 
-      sum = 1.0f / sum;
+      sum = 1.0F / sum;
       for ( x = i; x < kernel_size; x += 2 )
       {
         kernels [x] *= sum;
@@ -467,15 +466,14 @@ void AtariNTSC::initFilters(init_t& impl, const Setup& setup)
   }
 
   /* generate linear rescale kernels */
-  float weight = 1.0f;
-  float* out = impl.kernel;
+  float weight = 1.0F;
+  float* out = impl.kernel.data();
   int n = rescale_out;
   do
   {
     float remain = 0;
-    int i;
-    weight -= 1.0f / rescale_in;
-    for ( i = 0; i < kernel_size * 2; i++ )
+    weight -= 1.0F / rescale_in;
+    for ( int i = 0; i < kernel_size * 2; i++ )
     {
       float cur = kernels [i];
       float m = cur * weight;
@@ -491,7 +489,7 @@ void AtariNTSC::initFilters(init_t& impl, const Setup& setup)
 void AtariNTSC::genKernel(init_t& impl, float y, float i, float q, uInt32* out)
 {
   /* generate for each scanline burst phase */
-  float const* to_rgb = impl.to_rgb;
+  float const* to_rgb = impl.to_rgb.data();
   int burst_remain = burst_count;
   y -= rgb_offset;
   do
@@ -500,7 +498,7 @@ void AtariNTSC::genKernel(init_t& impl, float y, float i, float q, uInt32* out)
     Convolve these with kernels which: filter respective components, apply
     sharpening, and rescale horizontally. Convert resulting yiq to rgb and pack
     into integer. Based on algorithm by NewRisingSun. */
-    pixel_info_t const* pixel = atari_ntsc_pixels;
+    pixel_info_t const* pixel = atari_ntsc_pixels.data();
     int alignment_remain = alignment_count;
     do
     {
@@ -534,7 +532,7 @@ void AtariNTSC::genKernel(init_t& impl, float y, float i, float q, uInt32* out)
         else
           k -= kernel_size * 2 * (rescale_out - 1) + 2;
         {
-          int r, g, b = YIQ_TO_RGB( fy, fi, fq, to_rgb, int, r, g );
+          int r, g, b;  YIQ_TO_RGB( fy, fi, fq, to_rgb, r, g, b );
           *out++ = PACK_RGB( r, g, b ) - rgb_bias;
         }
       }
@@ -546,24 +544,26 @@ void AtariNTSC::genKernel(init_t& impl, float y, float i, float q, uInt32* out)
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 const AtariNTSC::Setup AtariNTSC::TV_Composite = {
-  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.15, 0.0, 0.0, 0.0
+  0.0F, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F, 0.15F, 0.0F, 0.0F, 0.0F
 };
 const AtariNTSC::Setup AtariNTSC::TV_SVideo = {
-  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.45, -1.0, -1.0, 0.0
+  0.0F, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F, 0.45F, -1.0F, -1.0F, 0.0F
 };
 const AtariNTSC::Setup AtariNTSC::TV_RGB = {
-  0.0, 0.0, 0.0, 0.0, 0.2, 0.0, 0.70, -1.0, -1.0, -1.0
+  0.0F, 0.0F, 0.0F, 0.0F, 0.2F, 0.0F, 0.70F, -1.0F, -1.0F, -1.0F
 };
 const AtariNTSC::Setup AtariNTSC::TV_Bad = {
-  0.1, -0.3, 0.3, 0.25, 0.2, 0.0, 0.1, 0.5, 0.5, 0.5
+  0.1F, -0.3F, 0.3F, 0.25F, 0.2F, 0.0F, 0.1F, 0.5F, 0.5F, 0.5F
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const AtariNTSC::pixel_info_t AtariNTSC::atari_ntsc_pixels[alignment_count] = {
-  { PIXEL_OFFSET( -4, -9 ), { 1, 1, 1, 1            } },
-  { PIXEL_OFFSET(  0, -5 ), {            1, 1, 1, 1 } },
-};
+const std::array<AtariNTSC::pixel_info_t, AtariNTSC::alignment_count>
+AtariNTSC::atari_ntsc_pixels = { {
+  { PIXEL_OFFSET1(-4, -9), PIXEL_OFFSET2(-4), { 1, 1, 1, 1            } },
+  { PIXEL_OFFSET1( 0, -5), PIXEL_OFFSET2( 0), {            1, 1, 1, 1 } }
+} };
 
-const float AtariNTSC::default_decoder[6] = {
-  0.9563f, 0.6210f, -0.2721f, -0.6474f, -1.1070f, 1.7046f
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+const std::array<float, 6> AtariNTSC::default_decoder = {
+  0.9563F, 0.6210F, -0.2721F, -0.6474F, -1.1070F, 1.7046F
 };
