@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -20,16 +20,8 @@
 
 #include "NTSCFilter.hxx"
 
-constexpr double scaleFrom100(double x) { return (x/50.0) - 1.0;     }
-constexpr uInt32 scaleTo100(double x)   { return uInt32(50*(x+1.0)); }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-NTSCFilter::NTSCFilter()
-  : mySetup(AtariNTSC::TV_Composite),
-    myPreset(PRESET_OFF),
-    myCurrentAdjustable(0)
-{
-}
+constexpr float scaleFrom100(float x) { return (x/50.F) - 1.F;     }
+constexpr uInt32 scaleTo100(float x)  { return uInt32(50*(x+1.F)); }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string NTSCFilter::setPreset(Preset preset)
@@ -38,30 +30,30 @@ string NTSCFilter::setPreset(Preset preset)
   string msg = "disabled";
   switch(myPreset)
   {
-    case PRESET_COMPOSITE:
+    case Preset::COMPOSITE:
       mySetup = AtariNTSC::TV_Composite;
       msg = "COMPOSITE";
       break;
-    case PRESET_SVIDEO:
+    case Preset::SVIDEO:
       mySetup = AtariNTSC::TV_SVideo;
       msg = "S-VIDEO";
       break;
-    case PRESET_RGB:
+    case Preset::RGB:
       mySetup = AtariNTSC::TV_RGB;
       msg = "RGB";
       break;
-    case PRESET_BAD:
+    case Preset::BAD:
       mySetup = AtariNTSC::TV_Bad;
       msg = "BAD ADJUST";
       break;
-    case PRESET_CUSTOM:
+    case Preset::CUSTOM:
       mySetup = myCustomSetup;
       msg = "CUSTOM";
       break;
     default:
       return msg;
   }
-  myNTSC.initialize(mySetup, myTIAPalette);
+  myNTSC.initialize(mySetup);
   return msg;
 }
 
@@ -70,11 +62,11 @@ string NTSCFilter::getPreset() const
 {
   switch(myPreset)
   {
-    case PRESET_COMPOSITE:  return "COMPOSITE";
-    case PRESET_SVIDEO:     return "S-VIDEO";
-    case PRESET_RGB:        return "RGB";
-    case PRESET_BAD:        return "BAD ADJUST";
-    case PRESET_CUSTOM:     return "CUSTOM";
+    case Preset::COMPOSITE: return "COMPOSITE";
+    case Preset::SVIDEO:    return "S-VIDEO";
+    case Preset::RGB:       return "RGB";
+    case Preset::BAD:       return "BAD ADJUST";
+    case Preset::CUSTOM:    return "CUSTOM";
     default:                return "Disabled";
   }
 }
@@ -82,7 +74,7 @@ string NTSCFilter::getPreset() const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string NTSCFilter::setNextAdjustable()
 {
-  if(myPreset != PRESET_CUSTOM)
+  if(myPreset != Preset::CUSTOM)
     return "'Custom' TV mode not selected";
 
   myCurrentAdjustable = (myCurrentAdjustable + 1) % 10;
@@ -96,7 +88,7 @@ string NTSCFilter::setNextAdjustable()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string NTSCFilter::setPreviousAdjustable()
 {
-  if(myPreset != PRESET_CUSTOM)
+  if(myPreset != Preset::CUSTOM)
     return "'Custom' TV mode not selected";
 
   if(myCurrentAdjustable == 0) myCurrentAdjustable = 9;
@@ -111,7 +103,7 @@ string NTSCFilter::setPreviousAdjustable()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string NTSCFilter::increaseAdjustable()
 {
-  if(myPreset != PRESET_CUSTOM)
+  if(myPreset != Preset::CUSTOM)
     return "'Custom' TV mode not selected";
 
   uInt32 newval = scaleTo100(*ourCustomAdjustables[myCurrentAdjustable].value);
@@ -129,7 +121,7 @@ string NTSCFilter::increaseAdjustable()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string NTSCFilter::decreaseAdjustable()
 {
-  if(myPreset != PRESET_CUSTOM)
+  if(myPreset != Preset::CUSTOM)
     return "'Custom' TV mode not selected";
 
   uInt32 newval = scaleTo100(*ourCustomAdjustables[myCurrentAdjustable].value);
@@ -149,16 +141,16 @@ string NTSCFilter::decreaseAdjustable()
 void NTSCFilter::loadConfig(const Settings& settings)
 {
   // Load adjustables for custom mode
-  myCustomSetup.hue = BSPF::clamp(settings.getFloat("tv.hue"), -1.0f, 1.0f);
-  myCustomSetup.saturation = BSPF::clamp(settings.getFloat("tv.saturation"), -1.0f, 1.0f);
-  myCustomSetup.contrast = BSPF::clamp(settings.getFloat("tv.contrast"), -1.0f, 1.0f);
-  myCustomSetup.brightness = BSPF::clamp(settings.getFloat("tv.brightness"), -1.0f, 1.0f);
-  myCustomSetup.sharpness = BSPF::clamp(settings.getFloat("tv.sharpness"), -1.0f, 1.0f);
-  myCustomSetup.gamma = BSPF::clamp(settings.getFloat("tv.gamma"), -1.0f, 1.0f);
-  myCustomSetup.resolution = BSPF::clamp(settings.getFloat("tv.resolution"), -1.0f, 1.0f);
-  myCustomSetup.artifacts = BSPF::clamp(settings.getFloat("tv.artifacts"), -1.0f, 1.0f);
-  myCustomSetup.fringing = BSPF::clamp(settings.getFloat("tv.fringing"), -1.0f, 1.0f);
-  myCustomSetup.bleed = BSPF::clamp(settings.getFloat("tv.bleed"), -1.0f, 1.0f);
+  myCustomSetup.hue = BSPF::clamp(settings.getFloat("tv.hue"), -1.0F, 1.0F);
+  myCustomSetup.saturation = BSPF::clamp(settings.getFloat("tv.saturation"), -1.0F, 1.0F);
+  myCustomSetup.contrast = BSPF::clamp(settings.getFloat("tv.contrast"), -1.0F, 1.0F);
+  myCustomSetup.brightness = BSPF::clamp(settings.getFloat("tv.brightness"), -1.0F, 1.0F);
+  myCustomSetup.sharpness = BSPF::clamp(settings.getFloat("tv.sharpness"), -1.0F, 1.0F);
+  myCustomSetup.gamma = BSPF::clamp(settings.getFloat("tv.gamma"), -1.0F, 1.0F);
+  myCustomSetup.resolution = BSPF::clamp(settings.getFloat("tv.resolution"), -1.0F, 1.0F);
+  myCustomSetup.artifacts = BSPF::clamp(settings.getFloat("tv.artifacts"), -1.0F, 1.0F);
+  myCustomSetup.fringing = BSPF::clamp(settings.getFloat("tv.fringing"), -1.0F, 1.0F);
+  myCustomSetup.bleed = BSPF::clamp(settings.getFloat("tv.bleed"), -1.0F, 1.0F);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -182,15 +174,15 @@ void NTSCFilter::getAdjustables(Adjustable& adjustable, Preset preset) const
 {
   switch(preset)
   {
-    case PRESET_COMPOSITE:
+    case Preset::COMPOSITE:
       convertToAdjustable(adjustable, AtariNTSC::TV_Composite);  break;
-    case PRESET_SVIDEO:
+    case Preset::SVIDEO:
       convertToAdjustable(adjustable, AtariNTSC::TV_SVideo);  break;
-    case PRESET_RGB:
+    case Preset::RGB:
       convertToAdjustable(adjustable, AtariNTSC::TV_RGB);  break;
-    case PRESET_BAD:
+    case Preset::BAD:
       convertToAdjustable(adjustable, AtariNTSC::TV_Bad);  break;
-    case PRESET_CUSTOM:
+    case Preset::CUSTOM:
       convertToAdjustable(adjustable, myCustomSetup);  break;
     default:
       break;
@@ -232,7 +224,7 @@ void NTSCFilter::convertToAdjustable(Adjustable& adjustable,
 AtariNTSC::Setup NTSCFilter::myCustomSetup = AtariNTSC::TV_Composite;
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const NTSCFilter::AdjustableTag NTSCFilter::ourCustomAdjustables[10] = {
+const std::array<NTSCFilter::AdjustableTag, 10> NTSCFilter::ourCustomAdjustables = { {
   { "contrast", &myCustomSetup.contrast },
   { "brightness", &myCustomSetup.brightness },
   { "hue", &myCustomSetup.hue },
@@ -243,4 +235,4 @@ const NTSCFilter::AdjustableTag NTSCFilter::ourCustomAdjustables[10] = {
   { "artifacts", &myCustomSetup.artifacts },
   { "fringing", &myCustomSetup.fringing },
   { "bleeding", &myCustomSetup.bleed }
-};
+} };
