@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -45,18 +45,15 @@ using std::right;
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartDebug::CartDebug(Debugger& dbg, Console& console, const OSystem& osystem)
   : DebuggerSystem(dbg, console),
-    myOSystem(osystem),
-    myDebugWidget(nullptr),
-    myAddrToLineIsROM(true),
-    myLabelLength(8)   // longest pre-defined label
+    myOSystem(osystem)
 {
   // Add case sensitive compare for user labels
   // TODO - should user labels be case insensitive too?
-  auto usrCmp = [](const string& a, const string& b) { return a < b; };
+  const auto usrCmp = [](const string& a, const string& b) { return a < b; };
   myUserAddresses = LabelToAddr(usrCmp);
 
   // Add case insensitive compare for system labels
-  auto sysCmp = [](const string& a, const string& b) {
+  const auto sysCmp = [](const string& a, const string& b) {
       return BSPF::compareIgnoreCase(a, b) < 0;
   };
   mySystemAddresses = LabelToAddr(sysCmp);
@@ -74,11 +71,11 @@ CartDebug::CartDebug(Debugger& dbg, Console& console, const OSystem& osystem)
   // Banksizes greater than 4096 indicate multi-bank ROMs, but we handle only
   // 4K pieces at a time
   // Banksizes less than 4K use the actual value
-  uInt32 banksize = 0;
+  size_t banksize = 0;
   myConsole.cartridge().getImage(banksize);
 
   BankInfo info;
-  info.size = std::min(banksize, 4096u);
+  info.size = std::min<size_t>(banksize, 4_KB);
   for(uInt32 i = 0; i < myConsole.cartridge().bankCount(); ++i)
     myBankInfo.push_back(info);
 
@@ -120,7 +117,7 @@ CartDebug::CartDebug(Debugger& dbg, Console& console, const OSystem& osystem)
 
   // Add settings for Distella
   DiStella::settings.gfxFormat =
-    myOSystem.settings().getInt("dis.gfxformat") == 16 ? Base::F_16 : Base::F_2;
+    myOSystem.settings().getInt("dis.gfxformat") == 16 ? Base::Fmt::_16 : Base::Fmt::_2;
   DiStella::settings.resolveCode =
     myOSystem.settings().getBool("dis.resolve");
   DiStella::settings.showAddresses =
@@ -179,16 +176,16 @@ string CartDebug::toString()
 
   switch(Base::format())
   {
-    case Base::F_16:
-    case Base::F_10:
+    case Base::Fmt::_16:
+    case Base::Fmt::_10:
       bytesPerLine = 0x10;
       break;
 
-    case Base::F_2:
+    case Base::Fmt::_2:
       bytesPerLine = 0x04;
       break;
 
-    case Base::F_DEFAULT:
+    case Base::Fmt::_DEFAULT:
     default:
       return DebuggerParser::red("invalid base, this is a BUG");
   }
@@ -204,7 +201,7 @@ string CartDebug::toString()
     // bytes have been previously output
     if(state.rport[i] - curraddr > bytesPerLine || bytesSoFar >= 256)
     {
-      char port[37];
+      char port[37];  // NOLINT (convert to stringstream)
       std::snprintf(port, 36, "%04x: (rport = %04x, wport = %04x)\n",
               state.rport[i], state.rport[i], state.wport[i]);
       port[2] = port[3] = 'x';
@@ -214,7 +211,7 @@ string CartDebug::toString()
     curraddr = state.rport[i];
     buf << Base::HEX2 << (curraddr & 0x00ff) << ": ";
 
-    for(uInt8 j = 0; j < bytesPerLine; ++j)
+    for(uInt32 j = 0; j < bytesPerLine; ++j)
     {
       buf << myDebugger.invIfChanged(state.ram[i+j], oldstate.ram[i+j]) << " ";
 
@@ -244,7 +241,7 @@ bool CartDebug::disassemble(bool force)
   if(changed)
   {
     // Are we disassembling from ROM or ZP RAM?
-    BankInfo& info = (PC & 0x1000) ? myBankInfo[getBank()] :
+    BankInfo& info = (PC & 0x1000) ? myBankInfo[getBank(PC)] :
         myBankInfo[myBankInfo.size()-1];
 
     // If the offset has changed, all old addresses must be 'converted'
@@ -384,7 +381,8 @@ bool CartDebug::addDirective(CartDebug::DisasmType type,
     return false;
 
   if(bank < 0)  // Do we want the current bank or ZP RAM?
-    bank = (myDebugger.cpuDebug().pc() & 0x1000) ? getBank() : int(myBankInfo.size())-1;
+    bank = (myDebugger.cpuDebug().pc() & 0x1000) ?
+      getBank(myDebugger.cpuDebug().pc()) : int(myBankInfo.size())-1;
 
   bank = std::min(bank, bankCount());
   BankInfo& info = myBankInfo[bank];
@@ -506,9 +504,15 @@ bool CartDebug::addDirective(CartDebug::DisasmType type,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartDebug::getBank()
+int CartDebug::getBank(uInt16 address)
 {
-  return myConsole.cartridge().getBank();
+  return myConsole.cartridge().getBank(address);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartDebug::getPCBank()
+{
+  return myConsole.cartridge().getBank(myDebugger.cpuDebug().pc());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -523,8 +527,8 @@ bool CartDebug::addLabel(const string& label, uInt16 address)
   // Only user-defined labels can be added or redefined
   switch(addressType(address))
   {
-    case ADDR_TIA:
-    case ADDR_IO:
+    case AddrType::TIA:
+    case AddrType::IO:
       return false;
     default:
       removeLabel(label);
@@ -562,7 +566,7 @@ bool CartDebug::getLabel(ostream& buf, uInt16 addr, bool isRead, int places) con
 {
   switch(addressType(addr))
   {
-    case ADDR_TIA:
+    case AddrType::TIA:
     {
       if(isRead)
       {
@@ -591,7 +595,7 @@ bool CartDebug::getLabel(ostream& buf, uInt16 addr, bool isRead, int places) con
       return true;
     }
 
-    case ADDR_IO:
+    case AddrType::IO:
     {
       uInt16 a = addr & 0xFF, offset = addr & 0xFD00;
       if(a <= 0x97)
@@ -611,7 +615,7 @@ bool CartDebug::getLabel(ostream& buf, uInt16 addr, bool isRead, int places) con
       return true;
     }
 
-    case ADDR_ZPRAM:
+    case AddrType::ZPRAM:
     {
       // RAM can use user-defined labels; otherwise we default to
       // standard mnemonics
@@ -634,7 +638,7 @@ bool CartDebug::getLabel(ostream& buf, uInt16 addr, bool isRead, int places) con
       return true;
     }
 
-    case ADDR_ROM:
+    case AddrType::ROM:
     {
       // These addresses can never be in the system labels list
       const auto& iter = myUserLabels.find(addr);
@@ -658,6 +662,8 @@ bool CartDebug::getLabel(ostream& buf, uInt16 addr, bool isRead, int places) con
     case 8:
       buf << "$" << Base::HEX8 << addr;
       return true;
+    default:
+      break;
   }
 
   return false;
@@ -696,7 +702,7 @@ string CartDebug::loadListFile()
     if(lst.isFile() && lst.isReadable())
       myListFile = lst.getPath();
     else
-      return DebuggerParser::red("list file not found in:\n  " + lst.getShortPath());
+      return DebuggerParser::red("list file \'" + lst.getShortPath() + "\' not found");
   }
 
   FilesystemNode node(myListFile);
@@ -741,7 +747,7 @@ string CartDebug::loadListFile()
   }
   myDebugger.rom().invalidate();
 
-  return "loaded " + node.getShortPath() + " OK";
+  return "list file '" + node.getShortPath() + "' loaded OK";
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -756,7 +762,7 @@ string CartDebug::loadSymbolFile()
     if(sym.isFile() && sym.isReadable())
       mySymbolFile = sym.getPath();
     else
-      return DebuggerParser::red("symbol file not found in:\n  " + sym.getShortPath());
+      return DebuggerParser::red("symbol file \'" + sym.getShortPath() + "\' not found");
   }
 
   FilesystemNode node(mySymbolFile);
@@ -787,12 +793,12 @@ string CartDebug::loadSymbolFile()
       if (iter == myUserLabels.end() || !BSPF::equalsIgnoreCase(label, iter->second))
       {
         // Check for period, and strip leading number
-        string::size_type pos = label.find_first_of(".", 0);
+        string::size_type pos = label.find_first_of('.', 0);
         if(pos != string::npos)
           addLabel(label.substr(pos), value);
         else
         {
-          pos = label.find_last_of("$");
+          pos = label.find_last_of('$');
           if (pos == string::npos || pos != label.length() - 1)
             addLabel(label, value);
         }
@@ -801,32 +807,22 @@ string CartDebug::loadSymbolFile()
   }
   myDebugger.rom().invalidate();
 
-  return "loaded " + node.getShortPath() + " OK";
+  return "symbol file '" + node.getShortPath() + "' loaded OK";
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string CartDebug::loadConfigFile()
 {
-  // There are two possible locations for loading config files
-  //   (in order of decreasing relevance):
-  // 1) ROM dir based on properties entry name
-  // 2) CFG dir based on properties entry name
+  // The default naming/location for config files is the ROM dir based on the
+  // actual ROM filename
 
   if(myCfgFile == "")
   {
-    const string& propsname =
-      myConsole.properties().get(Cartridge_Name) + ".cfg";
-
-    FilesystemNode case1(myOSystem.romFile().getParent().getPath() + propsname);
-    FilesystemNode case2(myOSystem.cfgDir() + propsname);
-
-    if(case1.isFile() && case1.isReadable())
-      myCfgFile = case1.getPath();
-    else if(case2.isFile() && case2.isReadable())
-      myCfgFile = case2.getPath();
+    FilesystemNode cfg(myOSystem.romFile().getPathWithExt("") + ".cfg");
+    if(cfg.isFile() && cfg.isReadable())
+      myCfgFile = cfg.getPath();
     else
-      return DebuggerParser::red("config file not found in:\n  " +
-          case1.getShortPath() + "\n  " + case2.getShortPath());
+      return DebuggerParser::red("config file \'" + cfg.getShortPath() + "\' not found");
   }
 
   FilesystemNode node(myCfgFile);
@@ -910,7 +906,7 @@ string CartDebug::loadConfigFile()
   stringstream retVal;
   if(myConsole.cartridge().bankCount() > 1)
     retVal << DebuggerParser::red("config file for multi-bank ROM not fully supported\n");
-  retVal << "loaded " << node.getShortPath() << " OK";
+  retVal << "config file '" << node.getShortPath() << "' loaded OK";
   return retVal.str();
 
 }
@@ -918,29 +914,25 @@ string CartDebug::loadConfigFile()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string CartDebug::saveConfigFile()
 {
-  // While there are two possible locations for loading config files,
-  // the main 'config' directory is used whenever possible when saving,
-  // unless the rom-specific file already exists
+  // The default naming/location for config files is the ROM dir based on the
+  // actual ROM filename
 
-  FilesystemNode node;
-
-  FilesystemNode case0(myCfgFile);
-  if(myCfgFile != "" && case0.isFile() && case0.isWritable())
-    node = case0;
-  else
+  FilesystemNode cfg;
+  if(myCfgFile == "")
   {
-    const string& propsname =
-      myConsole.properties().get(Cartridge_Name) + ".cfg";
-
-    node = FilesystemNode(myOSystem.cfgDir() + propsname);
+    cfg = FilesystemNode(myOSystem.romFile().getPathWithExt("") + ".cfg");
+    if(cfg.isFile() && cfg.isWritable())
+      myCfgFile = cfg.getPath();
+    else
+      return DebuggerParser::red("config file \'" + cfg.getShortPath() + "\' not writable");
   }
 
-  const string& name = myConsole.properties().get(Cartridge_Name);
-  const string& md5 = myConsole.properties().get(Cartridge_MD5);
+  const string& name = myConsole.properties().get(PropType::Cart_Name);
+  const string& md5 = myConsole.properties().get(PropType::Cart_MD5);
 
-  ofstream out(node.getPath());
+  ofstream out(cfg.getPath());
   if(!out.is_open())
-    return "Unable to save directives to " + node.getShortPath();
+    return "Unable to save directives to " + cfg.getShortPath();
 
   // Store all bank information
   out << "//Stella.pro: \"" << name << "\"" << endl
@@ -955,7 +947,7 @@ string CartDebug::saveConfigFile()
   stringstream retVal;
   if(myConsole.cartridge().bankCount() > 1)
     retVal << DebuggerParser::red("config file for multi-bank ROM not fully supported\n");
-  retVal << "saved " << node.getShortPath() << " OK";
+  retVal << "config file '" << cfg.getShortPath() << "' saved OK";
   return retVal.str();
 }
 
@@ -965,7 +957,7 @@ string CartDebug::saveDisassembly()
   if(myDisasmFile == "")
   {
     const string& propsname =
-      myConsole.properties().get(Cartridge_Name) + ".asm";
+      myConsole.properties().get(PropType::Cart_Name) + ".asm";
 
     myDisasmFile = FilesystemNode(myOSystem.defaultSaveDir() + propsname).getPath();
   }
@@ -996,7 +988,7 @@ string CartDebug::saveDisassembly()
   settings.fFlag = DiStella::settings.fFlag;
   settings.rFlag = DiStella::settings.rFlag;
   settings.bytesWidth = 8+1;  // same as Stella debugger
-  settings.bFlag = DiStella::settings.bFlag;; // process break routine (TODO)
+  settings.bFlag = DiStella::settings.bFlag; // process break routine (TODO)
 
   Disassembly disasm;
   disasm.list.reserve(2048);
@@ -1044,7 +1036,7 @@ string CartDebug::saveDisassembly()
         }
         case CartDebug::GFX:
         {
-          buf << ".byte   " << (settings.gfxFormat == Base::F_2 ? "%" : "$")
+          buf << ".byte   " << (settings.gfxFormat == Base::Fmt::_2 ? "%" : "$")
               << tag.bytes << " ; |";
           for(int c = 12; c < 20; ++c)
             buf << ((tag.disasm[c] == '\x1e') ? "#" : " ");
@@ -1053,7 +1045,7 @@ string CartDebug::saveDisassembly()
         }
         case CartDebug::PGFX:
         {
-          buf << ".byte   " << (settings.gfxFormat == Base::F_2 ? "%" : "$")
+          buf << ".byte   " << (settings.gfxFormat == Base::Fmt::_2 ? "%" : "$")
               << tag.bytes << " ; |";
           for(int c = 12; c < 20; ++c)
             buf << ((tag.disasm[c] == '\x1f') ? "*" : " ");
@@ -1080,8 +1072,8 @@ string CartDebug::saveDisassembly()
   out << "; Disassembly of " << myOSystem.romFile().getShortPath() << "\n"
       << "; Disassembled " << std::put_time(&timeinfo, "%c\n")
       << "; Using Stella " << STELLA_VERSION << "\n;\n"
-      << "; ROM properties name : " << myConsole.properties().get(Cartridge_Name) << "\n"
-      << "; ROM properties MD5  : " << myConsole.properties().get(Cartridge_MD5) << "\n"
+      << "; ROM properties name : " << myConsole.properties().get(PropType::Cart_Name) << "\n"
+      << "; ROM properties MD5  : " << myConsole.properties().get(PropType::Cart_MD5) << "\n"
       << "; Bankswitch type     : " << myConsole.cartridge().about() << "\n;\n"
       << "; Legend: * = CODE not yet run (tentative code)\n"
       << ";         D = DATA directive (referenced in some way)\n"
@@ -1212,7 +1204,7 @@ string CartDebug::saveDisassembly()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string CartDebug::saveRom()
 {
-  const string& rom = myConsole.properties().get(Cartridge_Name) + ".a26";
+  const string& rom = myConsole.properties().get(PropType::Cart_Name) + ".a26";
 
   FilesystemNode node(myOSystem.defaultSaveDir() + rom);
   ofstream out(node.getPath(), std::ios::binary);
@@ -1317,21 +1309,21 @@ CartDebug::AddrType CartDebug::addressType(uInt16 addr) const
   if(addr % 0x2000 < 0x1000)
   {
     if((addr & 0x00ff) < 0x80)
-      return ADDR_TIA;
+      return AddrType::TIA;
     else
     {
       switch(addr & 0x0f00)
       {
         case 0x000:  case 0x100:  case 0x400:  case 0x500:
         case 0x800:  case 0x900:  case 0xc00:  case 0xd00:
-          return ADDR_ZPRAM;
+          return AddrType::ZPRAM;
         case 0x200:  case 0x300:  case 0x600:  case 0x700:
         case 0xa00:  case 0xb00:  case 0xe00:  case 0xf00:
-          return ADDR_IO;
+          return AddrType::IO;
       }
     }
   }
-  return ADDR_ROM;
+  return AddrType::ROM;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1379,11 +1371,11 @@ void CartDebug::addressTypeAsString(ostream& buf, uInt16 addr) const
         debugger  = myDebugger.getAccessFlags(addr) & 0xFC,
         label     = myDisLabels[addr & 0xFFF];
 
-  buf << endl << "directive: " << Base::toString(directive, Base::F_2_8) << " ";
+  buf << endl << "directive: " << Base::toString(directive, Base::Fmt::_2_8) << " ";
   disasmTypeAsString(buf, directive);
-  buf << endl << "emulation: " << Base::toString(debugger, Base::F_2_8) << " ";
+  buf << endl << "emulation: " << Base::toString(debugger, Base::Fmt::_2_8) << " ";
   disasmTypeAsString(buf, debugger);
-  buf << endl << "tentative: " << Base::toString(label, Base::F_2_8) << " ";
+  buf << endl << "tentative: " << Base::toString(label, Base::Fmt::_2_8) << " ";
   disasmTypeAsString(buf, label);
   buf << endl;
 }
@@ -1451,13 +1443,13 @@ void CartDebug::disasmTypeAsString(ostream& buf, uInt8 flags) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* const CartDebug::ourTIAMnemonicR[16] = {
+std::array<const char*, 16> CartDebug::ourTIAMnemonicR = {
   "CXM0P", "CXM1P", "CXP0FB", "CXP1FB", "CXM0FB", "CXM1FB", "CXBLPF", "CXPPMM",
   "INPT0", "INPT1", "INPT2", "INPT3", "INPT4", "INPT5", "$1e", "$1f"
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* const CartDebug::ourTIAMnemonicW[64] = {
+std::array<const char*, 64> CartDebug::ourTIAMnemonicW = {
   "VSYNC", "VBLANK", "WSYNC", "RSYNC", "NUSIZ0", "NUSIZ1", "COLUP0", "COLUP1",
   "COLUPF", "COLUBK", "CTRLPF", "REFP0", "REFP1", "PF0", "PF1", "PF2",
   "RESP0", "RESP1", "RESM0", "RESM1", "RESBL", "AUDC0", "AUDC1", "AUDF0",
@@ -1469,7 +1461,7 @@ const char* const CartDebug::ourTIAMnemonicW[64] = {
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* const CartDebug::ourIOMnemonic[24] = {
+std::array<const char*, 24> CartDebug::ourIOMnemonic = {
   "SWCHA", "SWACNT", "SWCHB", "SWBCNT", "INTIM", "TIMINT",
   "$286", "$287", "$288", "$289", "$28a", "$28b", "$28c",
   "$28d", "$28e", "$28f", "$290", "$291", "$292", "$293",
@@ -1477,7 +1469,7 @@ const char* const CartDebug::ourIOMnemonic[24] = {
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* const CartDebug::ourZPMnemonic[128] = {
+std::array<const char*, 128> CartDebug::ourZPMnemonic = {
   "ram_80", "ram_81", "ram_82", "ram_83", "ram_84", "ram_85", "ram_86", "ram_87",
   "ram_88", "ram_89", "ram_8A", "ram_8B", "ram_8C", "ram_8D", "ram_8E", "ram_8F",
   "ram_90", "ram_91", "ram_92", "ram_93", "ram_94", "ram_95", "ram_96", "ram_97",
