@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -49,23 +49,21 @@
 
 #include "RomWidget.hxx"
 #include "Expression.hxx"
-#include "PackedBitArray.hxx"
 #include "YaccParser.hxx"
 
 #include "TIA.hxx"
 #include "Debugger.hxx"
 #include "DispatchResult.hxx"
 
+using Common::Base;
+
 Debugger* Debugger::myStaticDebugger = nullptr;
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Debugger::Debugger(OSystem& osystem, Console& console)
   : DialogContainer(osystem),
     myConsole(console),
-    mySystem(console.system()),
-    myDialog(nullptr),
-    myWidth(DebuggerDialog::kSmallFontMinW),
-    myHeight(DebuggerDialog::kSmallFontMinH)
+    mySystem(console.system())
 {
   // Init parser
   myParser = make_unique<DebuggerParser>(*this, osystem.settings());
@@ -86,13 +84,14 @@ Debugger::Debugger(OSystem& osystem, Console& console)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Debugger::~Debugger()
 {
+  delete myDialog;  myDialog = nullptr;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Debugger::initialize()
 {
-  const GUI::Size& s = myOSystem.settings().getSize("dbg.res");
-  const GUI::Size& d = myOSystem.frameBuffer().desktopSize();
+  const Common::Size& s = myOSystem.settings().getSize("dbg.res");
+  const Common::Size& d = myOSystem.frameBuffer().desktopSize();
   myWidth = s.w;  myHeight = s.h;
 
   // The debugger dialog is resizable, within certain bounds
@@ -102,11 +101,10 @@ void Debugger::initialize()
   myWidth  = std::min(myWidth,  uInt32(d.w));
   myHeight = std::min(myHeight, uInt32(d.h));
 
-  myOSystem.settings().setValue("dbg.res", GUI::Size(myWidth, myHeight));
+  myOSystem.settings().setValue("dbg.res", Common::Size(myWidth, myHeight));
 
-  delete myBaseDialog;  myBaseDialog = myDialog = nullptr;
+  delete myDialog;  myDialog = nullptr;
   myDialog = new DebuggerDialog(myOSystem, *this, 0, 0, myWidth, myHeight);
-  myBaseDialog = myDialog;
 
   myCartDebug->setDebugWidget(&(myDialog->cartDebug()));
 
@@ -154,7 +152,7 @@ bool Debugger::startWithFatalError(const string& message)
 void Debugger::quit(bool exitrom)
 {
   if(exitrom)
-    myOSystem.eventHandler().handleEvent(Event::LauncherMode, 1);
+    myOSystem.eventHandler().handleEvent(Event::ExitMode);
   else
   {
     myOSystem.eventHandler().leaveDebugMode();
@@ -177,13 +175,12 @@ string Debugger::autoExec(StringList* history)
   buf << myParser->exec(romname, history) << endl;
 
   // Init builtins
-  for(uInt32 i = 0; i < NUM_BUILTIN_FUNCS; ++i)
+  for(const auto& func: ourBuiltinFunctions)
   {
     // TODO - check this for memory leaks
-    int res = YaccParser::parse(ourBuiltinFunctions[i].defn);
+    int res = YaccParser::parse(func.defn);
     if(res == 0)
-      addFunction(ourBuiltinFunctions[i].name, ourBuiltinFunctions[i].defn,
-                  YaccParser::getResult(), true);
+      addFunction(func.name, func.defn, YaccParser::getResult(), true);
     else
       cerr << "ERROR in builtin function!" << endl;
   }
@@ -191,7 +188,7 @@ string Debugger::autoExec(StringList* history)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-PackedBitArray& Debugger::breakPoints() const
+BreakpointMap& Debugger::breakPoints() const
 {
   return mySystem.m6502().breakPoints();
 }
@@ -221,7 +218,7 @@ const string Debugger::invIfChanged(int reg, int oldReg)
 
   bool changed = reg != oldReg;
   if(changed) ret += "\177";
-  ret += Common::Base::toString(reg, Common::Base::F_16_2);
+  ret += Common::Base::toString(reg, Common::Base::Fmt::_16_2);
   if(changed) ret += "\177";
 
   return ret;
@@ -262,6 +259,14 @@ void Debugger::saveState(int state)
   myOSystem.state().saveState(state);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Debugger::saveAllStates()
+{
+  // Saving states is implicitly a read-only operation, so we keep the
+  // system locked, so no changes can occur
+  myOSystem.state().rewindManager().saveAllStates();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Debugger::loadState(int state)
 {
@@ -274,6 +279,18 @@ void Debugger::loadState(int state)
   lockSystem();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Debugger::loadAllStates()
+{
+  // We're loading new states, so we start with a clean slate
+  mySystem.clearDirtyPages();
+
+  // State loading could initiate a bankswitch, so we allow it temporarily
+  unlockSystem();
+  myOSystem.state().rewindManager().loadAllStates();
+  lockSystem();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int Debugger::step()
 {
@@ -309,8 +326,17 @@ int Debugger::trace()
     uInt64 startCycle = mySystem.cycles();
     int targetPC = myCpuDebug->pc() + 3; // return address
 
+    // set temporary breakpoint at target PC (if not existing already)
+    Int8 bank = myCartDebug->getBank(targetPC);
+    if(!checkBreakPoint(targetPC, bank))
+    {
+      // add temporary breakpoint and remove later
+      setBreakPoint(targetPC, bank, BreakpointMap::ONE_SHOT);
+    }
+
     unlockSystem();
-    myOSystem.console().tia().updateScanlineByTrace(targetPC).flushLineCache();
+    mySystem.m6502().execute(11900000); // max. ~10 seconds
+    myOSystem.console().tia().flushLineCache();
     lockSystem();
 
     addState("trace");
@@ -321,24 +347,44 @@ int Debugger::trace()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::toggleBreakPoint(uInt16 bp)
+bool Debugger::setBreakPoint(uInt16 addr, uInt8 bank, uInt32 flags)
 {
-  breakPoints().initialize();
-  breakPoints().toggle(bp);
+  bool exists = checkBreakPoint(addr, bank);
+
+  if(exists)
+    return false;
+
+  breakPoints().add(addr, bank, flags);
+  return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::setBreakPoint(uInt16 bp, bool set)
+bool Debugger::clearBreakPoint(uInt16 addr, uInt8 bank)
 {
-  breakPoints().initialize();
-  if(set) breakPoints().set(bp);
-  else    breakPoints().clear(bp);
+  bool exists = checkBreakPoint(addr, bank);
+
+  if(!exists)
+    return false;
+
+  breakPoints().erase(addr, bank);
+  return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Debugger::breakPoint(uInt16 bp)
+bool Debugger::checkBreakPoint(uInt16 addr, uInt8 bank)
 {
-  return breakPoints().isSet(bp);
+  return breakPoints().check(addr, bank);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Debugger::toggleBreakPoint(uInt16 addr, uInt8 bank)
+{
+  if(checkBreakPoint(addr, bank))
+    clearBreakPoint(addr, bank);
+  else
+    setBreakPoint(addr, bank);
+
+  return breakPoints().check(addr, bank);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -569,7 +615,7 @@ uInt16 Debugger::unwindStates(const uInt16 numStates, string& message)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Debugger::clearAllBreakPoints()
 {
-  breakPoints().clearAll();
+  breakPoints().clear();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -612,7 +658,7 @@ void Debugger::saveOldState(bool clearDirtyPages)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::addState(string rewindMsg)
+void Debugger::addState(const string& rewindMsg)
 {
   // Add another rewind level to the Time Machine buffer
   RewindManager& r = myOSystem.state().rewindManager();
@@ -650,7 +696,6 @@ void Debugger::setQuitState()
   // execute one instruction on quit. If we're
   // sitting at a breakpoint/trap, this will get us past it.
   // Somehow this feels like a hack to me, but I don't know why
-  //	if(breakPoints().isSet(myCpuDebug->pc()))
   mySystem.m6502().execute(1);
 }
 
@@ -667,8 +712,8 @@ bool Debugger::addFunction(const string& name, const string& definition,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Debugger::isBuiltinFunction(const string& name)
 {
-  for(uInt32 i = 0; i < NUM_BUILTIN_FUNCS; ++i)
-    if(name == ourBuiltinFunctions[i].name)
+  for(const auto& func: ourBuiltinFunctions)
+    if(name == func.name)
       return true;
   return false;
 }
@@ -709,7 +754,7 @@ const string& Debugger::getFunctionDef(const string& name) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const FunctionDefMap Debugger::getFunctionDefMap() const
+const Debugger::FunctionDefMap Debugger::getFunctionDefMap() const
 {
   return myFunctionDefs;
 }
@@ -721,39 +766,39 @@ string Debugger::builtinHelp() const
   uInt32 len, c_maxlen = 0, i_maxlen = 0;
 
   // Get column widths for aligned output (functions)
-  for(uInt32 i = 0; i < NUM_BUILTIN_FUNCS; ++i)
+  for(const auto& func: ourBuiltinFunctions)
   {
-    len = uInt32(ourBuiltinFunctions[i].name.size());
+    len = uInt32(func.name.size());
     if(len > c_maxlen)  c_maxlen = len;
-    len = uInt32(ourBuiltinFunctions[i].defn.size());
+    len = uInt32(func.defn.size());
     if(len > i_maxlen)  i_maxlen = len;
   }
 
   buf << std::setfill(' ') << endl << "Built-in functions:" << endl;
-  for(uInt32 i = 0; i < NUM_BUILTIN_FUNCS; ++i)
+  for(const auto& func: ourBuiltinFunctions)
   {
-    buf << std::setw(c_maxlen) << std::left << ourBuiltinFunctions[i].name
+    buf << std::setw(c_maxlen) << std::left << func.name
         << std::setw(2) << std::right << "{"
-        << std::setw(i_maxlen) << std::left << ourBuiltinFunctions[i].defn
+        << std::setw(i_maxlen) << std::left << func.defn
         << std::setw(4) << "}"
-        << ourBuiltinFunctions[i].help
+        << func.help
         << endl;
   }
 
   // Get column widths for aligned output (pseudo-registers)
   c_maxlen = 0;
-  for(uInt32 i = 0; i < NUM_PSEUDO_REGS; ++i)
+  for(const auto& reg: ourPseudoRegisters)
   {
-    len = uInt32(ourPseudoRegisters[i].name.size());
+    len = uInt32(reg.name.size());
     if(len > c_maxlen)  c_maxlen = len;
   }
 
   buf << endl << "Pseudo-registers:" << endl;
-  for(uInt32 i = 0; i < NUM_PSEUDO_REGS; ++i)
+  for(const auto& reg: ourPseudoRegisters)
   {
-    buf << std::setw(c_maxlen) << std::left << ourPseudoRegisters[i].name
+    buf << std::setw(c_maxlen) << std::left << reg.name
         << std::setw(2) << " "
-        << std::setw(i_maxlen) << std::left << ourPseudoRegisters[i].help
+        << std::setw(i_maxlen) << std::left << reg.help
         << endl;
   }
 
@@ -773,9 +818,9 @@ void Debugger::getCompletions(const char* in, StringList& list) const
         list.push_back(l);
     }
 
-    for(uInt32 i = 0; i < NUM_PSEUDO_REGS; ++i)
-      if(BSPF::matches(ourPseudoRegisters[i].name, in))
-        list.push_back(ourPseudoRegisters[i].name);
+    for(const auto& reg: ourPseudoRegisters)
+      if(BSPF::matches(reg.name, in))
+        list.push_back(reg.name);
   }
 }
 
@@ -796,11 +841,11 @@ void Debugger::unlockSystem()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Debugger::canExit() const
 {
-  return !myDialogStack.empty() && myDialogStack.top() == baseDialog();
+  return baseDialogIsActive();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Debugger::BuiltinFunction Debugger::ourBuiltinFunctions[NUM_BUILTIN_FUNCS] = {
+std::array<Debugger::BuiltinFunction, 18> Debugger::ourBuiltinFunctions = { {
   // left joystick:
   { "_joy0left",    "!(*SWCHA & $40)", "Left joystick moved left" },
   { "_joy0right",   "!(*SWCHA & $80)", "Left joystick moved right" },
@@ -824,11 +869,12 @@ Debugger::BuiltinFunction Debugger::ourBuiltinFunctions[NUM_BUILTIN_FUNCS] = {
   { "_diff0a",    "*SWCHB & $40",     "Left diff. set to A (hard)" },
   { "_diff1b",    "!(*SWCHB & $80)",  "Right diff. set to B (easy)" },
   { "_diff1a",    "*SWCHB & $80",     "Right diff. set to A (hard)" }
-};
+} };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // Names are defined here, but processed in YaccParser
-Debugger::PseudoRegister Debugger::ourPseudoRegisters[NUM_PSEUDO_REGS] = {
+std::array<Debugger::PseudoRegister, 11> Debugger::ourPseudoRegisters = { {
+// Debugger::PseudoRegister Debugger::ourPseudoRegisters[NUM_PSEUDO_REGS] = {
   { "_bank",      "Currently selected bank" },
   { "_cclocks",   "Color clocks on current scanline" },
   { "_cycleshi",  "Higher 32 bits of number of cycles since emulation started" },
@@ -843,4 +889,4 @@ Debugger::PseudoRegister Debugger::ourPseudoRegisters[NUM_PSEUDO_REGS] = {
   // CPU address access functions:
   /*{ "__lastread", "last CPU read address" },
   { "__lastwrite", "last CPU write address" },*/
-};
+} };
