@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -42,16 +42,16 @@ Cartridge0840Widget::Cartridge0840Widget(
          << "$" << (start + 0xFFF) << " (hotspot = $" << spot << ")\n";
   }
 
-  int xpos = 10,
+  int xpos = 2,
       ypos = addBaseInformation(size, "Fred X. Quimby", info.str()) + myLineHeight;
 
   VariantList items;
   VarList::push_back(items, "0 ($800)");
   VarList::push_back(items, "1 ($840)");
   myBank =
-    new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth("0 ($800) "),
-                    myLineHeight, items, "Set bank ",
-                    _font.getStringWidth("Set bank "), kBankChanged);
+    new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth("0 ($800)"),
+                    myLineHeight, items, "Set bank     ",
+                    0, kBankChanged);
   myBank->setTarget(this);
   addFocusWidget(myBank);
 }
@@ -87,7 +87,7 @@ string Cartridge0840Widget::bankState()
 {
   ostringstream& buf = buffer();
 
-  static const char* const spot[] = { "$800", "$840" };
+  static const std::array<string, 2> spot = { "$800", "$840" };
   buf << "Bank = " << std::dec << myCart.getBank()
       << ", hotspot = " << spot[myCart.getBank()];
 
