@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -25,7 +25,7 @@ Cartridge2KWidget::Cartridge2KWidget(
   : CartDebugWidget(boss, lfont, nfont, x, y, w, h)
 {
   // Eventually, we should query this from the debugger/disassembler
-  uInt16 size = cart.mySize;
+  size_t size = cart.mySize;
   uInt16 start = (cart.myImage[size-3] << 8) | cart.myImage[size-4];
   start -= start % size;
 
