@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -27,23 +27,25 @@ Cartridge3EPlusWidget::Cartridge3EPlusWidget(
   : CartDebugWidget(boss, lfont, nfont, x, y, w, h),
     myCart(cart)
 {
-  uInt32 size = cart.mySize;
+  size_t size = cart.mySize;
 
   ostringstream info;
   info << "3EPlus cartridge - (64K ROM + RAM)\n"
        << "  4-64K ROM (1K banks), 32K RAM (512b banks)\n"
        << "Each 1K ROM selected by writing to $3F\n"
           "Each 512b RAM selected by writing to $3E\n"
-          "  Lower 512B of bank x (R)\n"
-          "  Upper 512B of bank x (+$200) (W)\n"
+          "  Lower 512b of bank x (R)\n"
+          "  Upper 512b of bank x (+$200) (W)\n"
        << "Startup bank = 0/-1/-1/0 (ROM)\n";
 
   // Eventually, we should query this from the debugger/disassembler
-  uInt16 start = (cart.myImage[size-3] << 8) | cart.myImage[size-4];
+  //uInt16 start = (cart.myImage[size-3] << 8) | cart.myImage[size-4];
+  // Currently the cart starts at bank 0. If we change that, we have to change this too.
+  uInt16 start = (cart.myImage[0x400-3] << 8) | cart.myImage[0x400 - 4];
   start -= start % 0x1000;
   info << "Bank RORG" << " = $" << Common::Base::HEX4 << start << "\n";
 
-  int xpos = 10,
+  int xpos = 2,
       ypos = addBaseInformation(size, "T. Jentzsch", info.str()) +
                                 myLineHeight;
 
@@ -135,7 +137,7 @@ void Cartridge3EPlusWidget::loadConfig()
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge3EPlusWidget::handleCommand(CommandSender* sender,
-                                        int cmd, int data, int id)
+                                          int cmd, int data, int id)
 {
   uInt16 segment = 0;
   switch(cmd)
@@ -152,6 +154,8 @@ void Cartridge3EPlusWidget::handleCommand(CommandSender* sender,
     case kBank3Changed:
       segment = 3;
       break;
+    default:
+      break;
   }
 
   // Ignore bank if either number or type hasn't been selected
@@ -291,8 +295,8 @@ string Cartridge3EPlusWidget::internalRamDescription()
 {
   ostringstream desc;
   desc << "Accessible 512b at a time via:\n"
-       << "  $F000/$F400/$F800/etc used for Read Access\n"
-       << "  $F200/$F600/$FA00/etc used for Write Access (+$200)";
+       << "  $f000/$f400/$f800/$fc00 for Read Access\n"
+       << "  $f200/$f600/$fa00/$fe00 for Write Access (+$200)";
 
   return desc.str();
 }
@@ -328,6 +332,6 @@ uInt8 Cartridge3EPlusWidget::internalRamGetValue(int addr)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const Cartridge3EPlusWidget::BankID Cartridge3EPlusWidget::bankEnum[4] = {
+const std::array<Cartridge3EPlusWidget::BankID, 4> Cartridge3EPlusWidget::bankEnum = {
   kBank0Changed, kBank1Changed, kBank2Changed, kBank3Changed
 };
