@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -42,7 +42,7 @@ CartridgeCMWidget::CartridgeCMWidget(
     "used to control the cart functionality\n"
     "Startup bank = 3 (ROM), RAM disabled\n";
 
-  int xpos = 10,
+  int xpos = 2,
       ypos = addBaseInformation(size, "CompuMate", info) + myLineHeight;
 
   VariantList items;
@@ -52,8 +52,8 @@ CartridgeCMWidget::CartridgeCMWidget(
   VarList::push_back(items, " 3 ");
   myBank =
     new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth(" 0 "),
-                    myLineHeight, items, "Set bank ",
-                    _font.getStringWidth("Set bank "), kBankChanged);
+                    myLineHeight, items, "Set bank     ",
+                    0, kBankChanged);
   myBank->setTarget(this);
   addFocusWidget(myBank);
 
@@ -73,7 +73,7 @@ CartridgeCMWidget::CartridgeCMWidget(
         myFontHeight, "Current column ", TextAlign::Left);
   xpos += lwidth;
 
-  myColumn = new DataGridWidget(boss, _nfont, xpos, ypos-2, 1, 1, 2, 8, Common::Base::F_16);
+  myColumn = new DataGridWidget(boss, _nfont, xpos, ypos-2, 1, 1, 2, 8, Common::Base::Fmt::_16);
   myColumn->setTarget(this);
   myColumn->setEditable(false);
 
