@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -28,7 +28,7 @@ CartridgeCVPlusWidget::CartridgeCVPlusWidget(
   : CartDebugWidget(boss, lfont, nfont, x, y, w, h),
     myCart(cart)
 {
-  uInt32 size = cart.mySize;
+  size_t size = cart.mySize;
 
   ostringstream info;
   info << "LS_Dracon CV+ cartridge, 1K RAM, 2-256 2K ROM\n"
@@ -37,7 +37,7 @@ CartridgeCVPlusWidget::CartridgeCVPlusWidget(
        << "2048 bytes ROM @ $F800 - $FFFF, by writing to $3D\n"
        << "Startup bank = " << cart.startBank() << "\n";
 
-  int xpos = 10,
+  int xpos = 2,
       ypos = addBaseInformation(size, "LS_Dracon / Stephen Anthony",
                                 info.str()) + myLineHeight;
 
@@ -48,7 +48,7 @@ CartridgeCVPlusWidget::CartridgeCVPlusWidget(
   ostringstream label;
   label << "Set bank ($F800 - $FFFF) ";
   myBank =
-    new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth("xxx ($3D) "),
+    new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth("xxx ($3D)"),
                     myLineHeight, items, label.str(),
                     _font.getStringWidth(label.str()), kBankChanged);
   myBank->setTarget(this);
