@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -27,7 +27,7 @@ CartridgeDASHWidget::CartridgeDASHWidget(
   : CartDebugWidget(boss, lfont, nfont, x, y, w, h),
     myCart(cart)
 {
-  uInt32 size = cart.mySize;
+  size_t size = cart.mySize;
 
   ostringstream info;
   info << "DASH cartridge - (64K ROM + RAM)\n"
@@ -43,7 +43,7 @@ CartridgeDASHWidget::CartridgeDASHWidget(
   start -= start % 0x1000;
   info << "Bank RORG" << " = $" << Common::Base::HEX4 << start << "\n";
 
-  int xpos = 10,
+  int xpos = 2,
       ypos = addBaseInformation(size, "A. Davie & T. Jentzsch", info.str()) +
                                 myLineHeight;
 
@@ -152,6 +152,8 @@ void CartridgeDASHWidget::handleCommand(CommandSender* sender,
     case kBank3Changed:
       segment = 3;
       break;
+    default:
+      break;
   }
 
   // Ignore bank if either number or type hasn't been selected
@@ -364,6 +366,6 @@ uInt8 CartridgeDASHWidget::internalRamGetValue(int addr)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const CartridgeDASHWidget::BankID CartridgeDASHWidget::bankEnum[4] = {
+const std::array<CartridgeDASHWidget::BankID, 4> CartridgeDASHWidget::bankEnum = {
   kBank0Changed, kBank1Changed, kBank2Changed, kBank3Changed
 };
