@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -27,7 +27,7 @@ CartridgeDPCWidget::CartridgeDPCWidget(
   : CartDebugWidget(boss, lfont, nfont, x, y, w, h),
     myCart(cart)
 {
-  uInt16 size = cart.mySize;
+  size_t size = cart.mySize;
 
   ostringstream info;
   info << "DPC cartridge, two 4K banks + 2K display bank\n"
@@ -45,7 +45,7 @@ CartridgeDPCWidget::CartridgeDPCWidget(
          << "$" << (start + 0xFFF) << " (hotspot = $F" << (spot+i) << ")\n";
   }
 
-  int xpos = 10,
+  int xpos = 2,
       ypos = addBaseInformation(size, "Activision (Pitfall II)", info.str()) +
               myLineHeight;
 
@@ -53,9 +53,9 @@ CartridgeDPCWidget::CartridgeDPCWidget(
   VarList::push_back(items, "0 ($FFF8)");
   VarList::push_back(items, "1 ($FFF9)");
   myBank =
-    new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth("0 ($FFFx) "),
-                    myLineHeight, items, "Set bank ",
-                    _font.getStringWidth("Set bank "), kBankChanged);
+    new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth("0 ($FFFx)"),
+                    myLineHeight, items, "Set bank     ",
+                    0, kBankChanged);
   myBank->setTarget(this);
   addFocusWidget(myBank);
   ypos += myLineHeight + 8;
@@ -72,48 +72,48 @@ CartridgeDPCWidget::CartridgeDPCWidget(
         myFontHeight, "Top Registers ", TextAlign::Left);
   xpos += lwidth;
 
-  myTops = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 2, 8, Common::Base::F_16);
+  myTops = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 2, 8, Common::Base::Fmt::_16);
   myTops->setTarget(this);
   myTops->setEditable(false);
 
   // Bottom registers
-  xpos = 18;  ypos += myLineHeight + 4;
+  xpos = 10;  ypos += myLineHeight + 4;
   new StaticTextWidget(boss, _font, xpos, ypos, lwidth,
         myFontHeight, "Bottom Registers ", TextAlign::Left);
   xpos += lwidth;
 
-  myBottoms = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 2, 8, Common::Base::F_16);
+  myBottoms = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 2, 8, Common::Base::Fmt::_16);
   myBottoms->setTarget(this);
   myBottoms->setEditable(false);
 
   // Counter registers
-  xpos = 18;  ypos += myLineHeight + 4;
+  xpos = 10;  ypos += myLineHeight + 4;
   new StaticTextWidget(boss, _font, xpos, ypos, lwidth,
         myFontHeight, "Counter Registers ", TextAlign::Left);
   xpos += lwidth;
 
-  myCounters = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 4, 16, Common::Base::F_16_4);
+  myCounters = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 4, 16, Common::Base::Fmt::_16_4);
   myCounters->setTarget(this);
   myCounters->setEditable(false);
 
   // Flag registers
-  xpos = 18;  ypos += myLineHeight + 4;
+  xpos = 10;  ypos += myLineHeight + 4;
   new StaticTextWidget(boss, _font, xpos, ypos, lwidth,
         myFontHeight, "Flag Registers ", TextAlign::Left);
   xpos += lwidth;
 
-  myFlags = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 2, 8, Common::Base::F_16);
+  myFlags = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 2, 8, Common::Base::Fmt::_16);
   myFlags->setTarget(this);
   myFlags->setEditable(false);
 
   // Music mode
-  xpos = 10;  ypos += myLineHeight + 12;
+  xpos = 2;  ypos += myLineHeight + 12;
   lwidth = _font.getStringWidth("Music mode (DF5/DF6/DF7) ");
   new StaticTextWidget(boss, _font, xpos, ypos, lwidth,
         myFontHeight, "Music mode (DF5/DF6/DF7) ", TextAlign::Left);
   xpos += lwidth;
 
-  myMusicMode = new DataGridWidget(boss, _nfont, xpos, ypos-2, 3, 1, 2, 8, Common::Base::F_16);
+  myMusicMode = new DataGridWidget(boss, _nfont, xpos, ypos-2, 3, 1, 2, 8, Common::Base::Fmt::_16);
   myMusicMode->setTarget(this);
   myMusicMode->setEditable(false);
 
@@ -123,7 +123,7 @@ CartridgeDPCWidget::CartridgeDPCWidget(
         myFontHeight, "Current random number ", TextAlign::Left);
   xpos += lwidth;
 
-  myRandom = new DataGridWidget(boss, _nfont, xpos, ypos-2, 1, 1, 2, 8, Common::Base::F_16);
+  myRandom = new DataGridWidget(boss, _nfont, xpos, ypos-2, 1, 1, 2, 8, Common::Base::Fmt::_16);
   myRandom->setTarget(this);
   myRandom->setEditable(false);
 }
@@ -229,7 +229,7 @@ string CartridgeDPCWidget::bankState()
 {
   ostringstream& buf = buffer();
 
-  static const char* const spot[] = { "$FFF8", "$FFF9" };
+  static constexpr std::array<const char*, 2> spot = { "$FFF8", "$FFF9" };
   buf << "Bank = " << std::dec << myCart.getBank()
       << ", hotspot = " << spot[myCart.getBank()];
 
