@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -33,11 +33,12 @@ CartDebugWidget::CartDebugWidget(GuiObject* boss, const GUI::Font& lfont,
     myFontWidth(lfont.getMaxCharWidth()),
     myFontHeight(lfont.getFontHeight()),
     myLineHeight(lfont.getLineHeight()),
-    myButtonHeight(myLineHeight + 4),
-    myDesc(nullptr)   { }
+    myButtonHeight(myLineHeight + 4)
+{
+}
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartDebugWidget::addBaseInformation(int bytes, const string& manufacturer,
+int CartDebugWidget::addBaseInformation(size_t bytes, const string& manufacturer,
         const string& desc, const uInt16 maxlines)
 {
   const int lwidth = _font.getStringWidth("Manufacturer "),
@@ -45,24 +46,22 @@ int CartDebugWidget::addBaseInformation(int bytes, const string& manufacturer,
   EditTextWidget* w = nullptr;
   ostringstream buf;
 
-  int x = 10, y = 10;
+  int x = 2, y = 8;
 
   // Add ROM size, manufacturer and bankswitch info
-  new StaticTextWidget(_boss, _font, x, y, lwidth,
-        myFontHeight, "ROM Size ", TextAlign::Left);
+  new StaticTextWidget(_boss, _font, x, y + 1, "ROM Size ");
   buf << bytes << " bytes";
   if(bytes >= 1024)
     buf << " / " << (bytes/1024) << "KB";
 
-  w = new EditTextWidget(_boss, _nfont, x+lwidth, y,
-        fwidth, myLineHeight, buf.str());
+  w = new EditTextWidget(_boss, _nfont, x+lwidth, y - 1,
+                         fwidth, myLineHeight, buf.str());
   w->setEditable(false);
   y += myLineHeight + 4;
 
-  new StaticTextWidget(_boss, _font, x, y, lwidth,
-        myFontHeight, "Manufacturer ", TextAlign::Left);
-  w = new EditTextWidget(_boss, _nfont, x+lwidth, y,
-        fwidth, myLineHeight, manufacturer);
+  new StaticTextWidget(_boss, _font, x, y + 1, "Manufacturer ");
+  w = new EditTextWidget(_boss, _nfont, x+lwidth, y - 1,
+                         fwidth, myLineHeight, manufacturer);
   w->setEditable(false);
   y += myLineHeight + 4;
 
@@ -72,11 +71,11 @@ int CartDebugWidget::addBaseInformation(int bytes, const string& manufacturer,
   if(lines < 3) lines = 3;
   if(lines > maxlines) lines = maxlines;
 
-  new StaticTextWidget(_boss, _font, x, y, lwidth,
-        myFontHeight, "Description ", TextAlign::Left);
-  myDesc = new StringListWidget(_boss, _nfont, x+lwidth, y,
-               fwidth, lines * myLineHeight, false);
+  new StaticTextWidget(_boss, _font, x, y + 1, "Description ");
+  myDesc = new StringListWidget(_boss, _nfont, x+lwidth, y - 1,
+                                fwidth, lines * myLineHeight, false);
   myDesc->setEditable(false);
+  myDesc->setEnabled(false);
   myDesc->setList(sl);
 
   y += myDesc->getHeight() + 4;
@@ -93,6 +92,5 @@ void CartDebugWidget::invalidate()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartDebugWidget::loadConfig()
 {
-  if(myDesc)
-    myDesc->setSelected(0);
+  //myDesc->setSelected(0);
 }
