@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -30,7 +30,7 @@ CartridgeF0Widget::CartridgeF0Widget(
 
   ostringstream info;
   info << "64K Megaboy F0 cartridge, 16 4K banks\n"
-       << "Startup bank = " << cart.startBank() << " or undetermined\n"
+       << "Startup bank = #" << cart.startBank() << " or undetermined\n"
        << "Bankswitch triggered by accessing $1FF0\n";
 
   // Eventually, we should query this from the debugger/disassembler
@@ -42,7 +42,7 @@ CartridgeF0Widget::CartridgeF0Widget(
          << " - " << "$" << (start + 0xFFF) << "\n";
   }
 
-  int xpos = 10,
+  int xpos = 2,
       ypos = addBaseInformation(size, "Dynacom Megaboy",
                                 info.str()) + myLineHeight;
 
@@ -64,9 +64,9 @@ CartridgeF0Widget::CartridgeF0Widget(
   VarList::push_back(items, " 14");
   VarList::push_back(items, " 15");
   myBank =
-    new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth(" 15 "),
-                    myLineHeight, items, "Set bank ",
-                    _font.getStringWidth("Set bank "), kBankChanged);
+    new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth(" 15"),
+                    myLineHeight, items, "Set bank    #",
+                    0, kBankChanged);
   myBank->setTarget(this);
   addFocusWidget(myBank);
 }
@@ -102,7 +102,7 @@ string CartridgeF0Widget::bankState()
 {
   ostringstream& buf = buffer();
 
-  buf << "Bank = " << std::dec << myCart.getBank() << ", hotspot = $FFF0";
+  buf << "Bank = #" << std::dec << myCart.getBank() << ", hotspot = $FFF0";
 
   return buf.str();
 }
