@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -28,7 +28,7 @@ CartridgeFA2Widget::CartridgeFA2Widget(
   : CartDebugWidget(boss, lfont, nfont, x, y, w, h),
     myCart(cart)
 {
-  uInt16 size = cart.mySize;
+  size_t size = cart.mySize;
 
   ostringstream info;
   info << "Modified FA RAM+, six or seven 4K banks\n"
@@ -47,7 +47,7 @@ CartridgeFA2Widget::CartridgeFA2Widget(
          << "$" << (start + 0xFFF) << " (hotspot = $F" << (spot+i) << ")\n";
   }
 
-  int xpos = 10,
+  int xpos = 2,
       ypos = addBaseInformation(size, "Chris D. Walton (Star Castle 2600)",
                 info.str(), 15) + myLineHeight;
 
@@ -62,9 +62,9 @@ CartridgeFA2Widget::CartridgeFA2Widget(
     VarList::push_back(items, "6 ($FFFB)");
 
   myBank =
-    new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth("0 ($FFFx) "),
-                    myLineHeight, items, "Set bank ",
-                    _font.getStringWidth("Set bank "), kBankChanged);
+    new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth("0 ($FFFx)"),
+                    myLineHeight, items, "Set bank     ",
+                    0, kBankChanged);
   myBank->setTarget(this);
   addFocusWidget(myBank);
   ypos += myLineHeight + 20;
@@ -140,6 +140,9 @@ void CartridgeFA2Widget::handleCommand(CommandSender* sender,
     case kFlashSave:
       myCart.flash(2);
       break;
+
+    default:
+      break;
   }
 }
 
@@ -148,7 +151,7 @@ string CartridgeFA2Widget::bankState()
 {
   ostringstream& buf = buffer();
 
-  static const char* const spot[] = {
+  static constexpr std::array<const char*, 7> spot = {
     "$FFF5", "$FFF6", "$FFF7", "$FFF8", "$FFF9", "$FFFA", "$FFFB"
   };
   buf << "Bank = " << std::dec << myCart.getBank()
