@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -27,7 +27,7 @@ CartridgeMDMWidget::CartridgeMDMWidget(
   : CartDebugWidget(boss, lfont, nfont, x, y, w, h),
     myCart(cart)
 {
-  uInt32 size = myCart.mySize;
+  size_t size = myCart.mySize;
 
   ostringstream info;
   info << "Menu Driven Megacart, containing up to 128 4K banks\n"
@@ -35,11 +35,11 @@ CartridgeMDMWidget::CartridgeMDMWidget(
        << "\nBanks are selected by reading from $800 - $BFF, where the lower "
           "byte determines the 4K bank to use.";
 
-  int xpos = 10,
+  int xpos = 2,
       ypos = addBaseInformation(size, "Edwin Blink", info.str(), 15) + myLineHeight;
 
   VariantList items;
-  for(uInt32 i = 0x800; i < (0x800u + myCart.bankCount()); ++i)
+  for(uInt32 i = 0x800; i < (0x800U + myCart.bankCount()); ++i)
   {
     info.str("");
     info << std::dec << (i & 0xFF) << " ($" << Common::Base::HEX4 << i << ")";
@@ -47,14 +47,14 @@ CartridgeMDMWidget::CartridgeMDMWidget(
   }
 
   myBank =
-    new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth("xxx ($0FFF) "),
-                    myLineHeight, items, "Set bank ",
-                    _font.getStringWidth("Set bank "), kBankChanged);
+    new PopUpWidget(boss, _font, xpos, ypos, _font.getStringWidth("xxx ($0FFF)"),
+                    myLineHeight, items, "Set bank     ",
+                    0, kBankChanged);
   myBank->setTarget(this);
   addFocusWidget(myBank);
 
   xpos += myBank->getWidth() + 30;
-  myBankDisabled = new CheckboxWidget(boss, _font, xpos, ypos,
+  myBankDisabled = new CheckboxWidget(boss, _font, xpos, ypos + 1,
                                       "Bankswitching is locked/disabled",
                                       kBankDisabled);
   myBankDisabled->setTarget(this);
@@ -65,6 +65,7 @@ CartridgeMDMWidget::CartridgeMDMWidget(
 void CartridgeMDMWidget::loadConfig()
 {
   myBank->setSelectedIndex(myCart.getBank());
+  myBank->setEnabled(!myCart.myBankingDisabled);
   myBankDisabled->setState(myCart.myBankingDisabled);
 
   CartDebugWidget::loadConfig();
@@ -84,6 +85,7 @@ void CartridgeMDMWidget::handleCommand(CommandSender* sender,
   else if(cmd == kBankDisabled)
   {
     myCart.myBankingDisabled = myBankDisabled->getState();
+    myBank->setEnabled(!myCart.myBankingDisabled);
   }
 }
 
