@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -44,18 +44,17 @@ CartRamWidget::CartRamWidget(
 
   EditTextWidget* etw = nullptr;
   ostringstream buf;
-  int xpos = 2, ypos = 5;
+  int xpos = 2, ypos = 8;
 
   // Add RAM size
-  new StaticTextWidget(_boss, _font, xpos, ypos, lwidth,
-                       myFontHeight, "RAM Size ", TextAlign::Left);
+  new StaticTextWidget(_boss, _font, xpos, ypos + 1, "RAM Size ");
 
   uInt32 ramsize = cartDebug.internalRamSize();
   buf << ramsize << " bytes";
   if(ramsize >= 1024)
     buf << " / " << (ramsize/1024) << "KB";
 
-  etw = new EditTextWidget(boss, nfont, xpos+lwidth, ypos,
+  etw = new EditTextWidget(boss, nfont, xpos+lwidth, ypos - 1,
                          fwidth, myLineHeight, buf.str());
   etw->setEditable(false);
   ypos += myLineHeight + 4;
@@ -69,9 +68,8 @@ CartRamWidget::CartRamWidget(
   if(lines < 3) lines = 3;
   if(lines > maxlines) lines = maxlines;
 
-  new StaticTextWidget(_boss, _font, xpos, ypos, lwidth,
-                       myFontHeight, "Description ", TextAlign::Left);
-  myDesc = new StringListWidget(boss, nfont, xpos+lwidth, ypos,
+  new StaticTextWidget(_boss, _font, xpos, ypos + 1, "Description ");
+  myDesc = new StringListWidget(boss, nfont, xpos+lwidth, ypos - 1,
                                 fwidth, lines * myLineHeight, false);
   myDesc->setEditable(false);
   myDesc->setList(sl);
@@ -112,17 +110,12 @@ CartRamWidget::InternalRamWidget::InternalRamWidget(GuiObject* boss,
         int x, int y, int w, int h,
         CartDebugWidget& dbg)
   : RamWidget(boss, lfont, nfont, x, y, w, h,
-      dbg.internalRamSize(), std::min(dbg.internalRamSize() / 16, 16u),
-      std::min(dbg.internalRamSize() / 16, 16u) * 16),
+      dbg.internalRamSize(), std::min(dbg.internalRamSize() / 16, 16U),
+      std::min(dbg.internalRamSize() / 16, 16U) * 16),
     myCart(dbg)
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartRamWidget::InternalRamWidget::~InternalRamWidget()
-{
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 CartRamWidget::InternalRamWidget::getValue(int addr) const
 {
