@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -43,7 +43,7 @@ CartridgeX07Widget::CartridgeX07Widget(
          << " - " << "$" << (start + 0xFFF) << "\n";
   }
 
-  int xpos = 10,
+  int xpos = 2,
       ypos = addBaseInformation(size, "AtariAge / John Payson / Fred Quimby",
                                 info.str()) + myLineHeight;
 
@@ -65,9 +65,9 @@ CartridgeX07Widget::CartridgeX07Widget(
   VarList::push_back(items, " 14");
   VarList::push_back(items, " 15");
   myBank =
-    new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth(" 15 "),
-                    myLineHeight, items, "Set bank ",
-                    _font.getStringWidth("Set bank "), kBankChanged);
+    new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth(" 15"),
+                    myLineHeight, items, "Set bank     ",
+                    0, kBankChanged);
   myBank->setTarget(this);
   addFocusWidget(myBank);
 }
