@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -46,7 +46,7 @@ CpuWidget::CpuWidget(GuiObject* boss, const GUI::Font& lfont, const GUI::Font& n
   new StaticTextWidget(boss, lfont, xpos, ypos+1, lwidth-2, fontHeight,
                        "PC ", TextAlign::Left);
   myPCGrid =
-    new DataGridWidget(boss, nfont, xpos + lwidth, ypos, 1, 1, 4, 16, Common::Base::F_16);
+    new DataGridWidget(boss, nfont, xpos + lwidth, ypos, 1, 1, 4, 16, Common::Base::Fmt::_16);
   myPCGrid->setTarget(this);
   myPCGrid->setID(kPCRegID);
   addFocusWidget(myPCGrid);
@@ -60,7 +60,7 @@ CpuWidget::CpuWidget(GuiObject* boss, const GUI::Font& lfont, const GUI::Font& n
   // Create a 1x4 grid with labels for the other CPU registers
   xpos = x + lwidth;  ypos += myPCGrid->getHeight() + 1;
   myCpuGrid =
-    new DataGridWidget(boss, nfont, xpos, ypos, 1, 4, 2, 8, Common::Base::F_16);
+    new DataGridWidget(boss, nfont, xpos, ypos, 1, 4, 2, 8, Common::Base::Fmt::_16);
   myCpuGrid->setTarget(this);
   myCpuGrid->setID(kCpuRegID);
   addFocusWidget(myCpuGrid);
@@ -68,14 +68,14 @@ CpuWidget::CpuWidget(GuiObject* boss, const GUI::Font& lfont, const GUI::Font& n
   // Create a 1x4 grid with decimal and binary values for the other CPU registers
   xpos = x + lwidth + myPCGrid->getWidth() + 10;
   myCpuGridDecValue =
-    new DataGridWidget(boss, nfont, xpos, ypos, 1, 4, 3, 8, Common::Base::F_10);
+    new DataGridWidget(boss, nfont, xpos, ypos, 1, 4, 3, 8, Common::Base::Fmt::_10);
   myCpuGridDecValue->setTarget(this);
   myCpuGridDecValue->setID(kCpuRegDecID);
   addFocusWidget(myCpuGridDecValue);
 
   xpos += myCpuGridDecValue->getWidth() + 5;
   myCpuGridBinValue =
-    new DataGridWidget(boss, nfont, xpos, ypos, 1, 4, 8, 8, Common::Base::F_2);
+    new DataGridWidget(boss, nfont, xpos, ypos, 1, 4, 8, 8, Common::Base::Fmt::_2);
   myCpuGridBinValue->setTarget(this);
   myCpuGridBinValue->setID(kCpuRegBinID);
   addFocusWidget(myCpuGridBinValue);
@@ -100,7 +100,7 @@ CpuWidget::CpuWidget(GuiObject* boss, const GUI::Font& lfont, const GUI::Font& n
 
   // Add labels for other CPU registers
   xpos = x;
-  string labels[4] = { "SP ", "A ", "X ", "Y " };
+  const std::array<string, 4> labels = { "SP ", "A ", "X ", "Y " };
   for(int row = 0; row < 4; ++row)
   {
     new StaticTextWidget(boss, lfont, xpos, ypos + row*lineHeight + 1,
@@ -118,8 +118,8 @@ CpuWidget::CpuWidget(GuiObject* boss, const GUI::Font& lfont, const GUI::Font& n
 
   // Set the strings to be used in the PSRegister
   // We only do this once because it's the state that changes, not the strings
-  const char* const offstr[] = { "n", "v", "-", "b", "d", "i", "z", "c" };
-  const char* const onstr[]  = { "N", "V", "-", "B", "D", "I", "Z", "C" };
+  const std::array<string, 8> offstr = { "n", "v", "-", "b", "d", "i", "z", "c" };
+  const std::array<string, 8> onstr  = { "N", "V", "-", "B", "D", "I", "Z", "C" };
   StringList off, on;
   for(int i = 0; i < 8; ++i)
   {
@@ -170,6 +170,9 @@ void CpuWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
           addr  = myCpuGridBinValue->getSelectedAddr();
           value = myCpuGridBinValue->getSelectedValue();
           break;
+
+        default:
+          break;
       }
 
       switch(addr)
@@ -211,6 +214,9 @@ void CpuWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
           myCpuGridDecValue->setValueInternal(3, value);
           myCpuGridBinValue->setValueInternal(3, value);
           break;
+
+        default:
+          break;
       }
       break;
 
@@ -247,8 +253,15 @@ void CpuWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
         case kPSRegC:
           dbg.setC(state);
           break;
+
+        default:
+          break;
       }
     }
+    break;
+
+    default:
+      break;
   }
 }
 
