@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -31,27 +31,19 @@
 DataGridWidget::DataGridWidget(GuiObject* boss, const GUI::Font& font,
                                int x, int y, int cols, int rows,
                                int colchars, int bits,
-                               Common::Base::Format base,
+                               Common::Base::Fmt base,
                                bool useScrollbar)
   : EditableWidget(boss, font, x, y,
                    cols*(colchars * font.getMaxCharWidth() + 8) + 1,
                    font.getLineHeight()*rows + 1),
     _rows(rows),
     _cols(cols),
-    _currentRow(0),
-    _currentCol(0),
     _rowHeight(font.getLineHeight()),
     _colWidth(colchars * font.getMaxCharWidth() + 8),
     _bits(bits),
-    _crossGrid(false),
-    _base(base),
-    _selectedItem(0),
-    _currentKeyDown(KBDK_UNKNOWN),
-    _opsWidget(nullptr),
-    _scrollBar(nullptr)
+    _base(base)
 {
-  _flags = WIDGET_ENABLED | WIDGET_RETAIN_FOCUS | WIDGET_WANTS_RAWDATA;
-  _editMode = false;
+  _flags = Widget::FLAG_ENABLED | Widget::FLAG_RETAIN_FOCUS | Widget::FLAG_WANTS_RAWDATA;
 
   // The item is selected, thus _bgcolor is used to draw the caret and
   // _textcolorhi to erase it
@@ -87,23 +79,23 @@ DataGridWidget::DataGridWidget(GuiObject* boss, const GUI::Font& font,
   EditableWidget::TextFilter f;
   switch(base)
   {
-    case Common::Base::F_16:
-    case Common::Base::F_16_1:
-    case Common::Base::F_16_2:
-    case Common::Base::F_16_4:
-    case Common::Base::F_16_8:
+    case Common::Base::Fmt::_16:
+    case Common::Base::Fmt::_16_1:
+    case Common::Base::Fmt::_16_2:
+    case Common::Base::Fmt::_16_4:
+    case Common::Base::Fmt::_16_8:
       setTextFilter([](char c) {
         return (c >= '0' && c <= '9') || (c >= 'a' && c <= 'f');
       });
       break;
-    case Common::Base::F_10:
+    case Common::Base::Fmt::_10:
       setTextFilter([](char c) {
         return (c >= '0' && c <= '9');
       });
       break;
-    case Common::Base::F_2:
-    case Common::Base::F_2_8:
-    case Common::Base::F_2_16:
+    case Common::Base::Fmt::_2:
+    case Common::Base::Fmt::_2_8:
+    case Common::Base::Fmt::_2_16:
       setTextFilter([](char c) { return (c >= '0' && c <= '1'); });
       break;
     default:
@@ -252,14 +244,14 @@ void DataGridWidget::setRange(int lower, int upper)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DataGridWidget::handleMouseEntered()
 {
-  setFlags(WIDGET_HILITED);
+  setFlags(Widget::FLAG_HILITED);
   setDirty();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DataGridWidget::handleMouseLeft()
 {
-  clearFlags(WIDGET_HILITED);
+  clearFlags(Widget::FLAG_HILITED);
   setDirty();
 }
 
@@ -586,6 +578,9 @@ void DataGridWidget::handleCommand(CommandSender* sender, int cmd,
     case kDGShiftRCmd:
       rshiftCell();
       break;
+
+    default:
+      return;
   }
 }
 
@@ -661,22 +656,20 @@ void DataGridWidget::drawWidget(bool hilite)
 
   // Cross out the grid?
   if(_crossGrid)
-    for(row = 1; row < 4; ++row)
-      s.hLine(_x, _y + (row * lineheight/4), _x + linewidth, kColor);
+  {
+    s.line(_x + 1, _y + 1, _x + _w - 2, _y + _h - 1, kColor);
+    s.line(_x + _w - 2, _y + 1, _x + 1, _y + _h - 1, kColor);
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-GUI::Rect DataGridWidget::getEditRect() const
+Common::Rect DataGridWidget::getEditRect() const
 {
-  GUI::Rect r(1, 0, _colWidth, _rowHeight);
   const int rowoffset = _currentRow * _rowHeight;
   const int coloffset = _currentCol * _colWidth + 4;
-  r.top += rowoffset;
-  r.bottom += rowoffset;
-  r.left += coloffset;
-  r.right += coloffset - 5;
 
-  return r;
+  return Common::Rect(1 + coloffset, rowoffset,
+                      _colWidth + coloffset - 5, _rowHeight + rowoffset);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -709,22 +702,22 @@ void DataGridWidget::endEditMode()
   {
     switch(_base)
     {
-      case Common::Base::F_16:
-      case Common::Base::F_16_1:
-      case Common::Base::F_16_2:
-      case Common::Base::F_16_4:
-      case Common::Base::F_16_8:
+      case Common::Base::Fmt::_16:
+      case Common::Base::Fmt::_16_1:
+      case Common::Base::Fmt::_16_2:
+      case Common::Base::Fmt::_16_4:
+      case Common::Base::Fmt::_16_8:
         editString().insert(0, 1, '$');
         break;
-      case Common::Base::F_2:
-      case Common::Base::F_2_8:
-      case Common::Base::F_2_16:
+      case Common::Base::Fmt::_2:
+      case Common::Base::Fmt::_2_8:
+      case Common::Base::Fmt::_2_16:
         editString().insert(0, 1, '\\');
         break;
-      case Common::Base::F_10:
+      case Common::Base::Fmt::_10:
         editString().insert(0, 1, '#');
         break;
-      case Common::Base::F_DEFAULT:
+      case Common::Base::Fmt::_DEFAULT:
       default:  // TODO - properly handle all other cases
         break;
     }
