@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -46,15 +46,13 @@
 #include "StellaMediumFont.hxx"
 #include "OptionsDialog.hxx"
 #include "StateManager.hxx"
+#include "FrameManager.hxx"
 #include "DebuggerDialog.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 DebuggerDialog::DebuggerDialog(OSystem& osystem, DialogContainer& parent,
                                int x, int y, int w, int h)
-  : Dialog(osystem, parent, x, y, w, h),
-    myTab(nullptr),
-    myRomTab(nullptr),
-    myFatalError(nullptr)
+  : Dialog(osystem, parent, x, y, w, h)
 {
   createFont();  // Font is sized according to available space
 
@@ -66,12 +64,20 @@ DebuggerDialog::DebuggerDialog(OSystem& osystem, DialogContainer& parent,
   // Inform the TIA output widget about its associated zoom widget
   myTiaOutput->setZoomWidget(myTiaZoom);
 
-  myOptions = make_unique<OptionsDialog>(osystem, parent, this, w, h, OptionsDialog::debugger);
+  myOptions = make_unique<OptionsDialog>(osystem, parent, this, w, h,
+                                         Menu::AppMode::debugger);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::loadConfig()
 {
+  // set initial focus to myPrompt
+  if (myFirstLoad)
+  {
+    setFocus(myPrompt);
+    myFirstLoad = false;
+  }
+
   myTab->loadConfig();
   myTiaInfo->loadConfig();
   myTiaOutput->loadConfig();
@@ -84,124 +90,19 @@ void DebuggerDialog::loadConfig()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DebuggerDialog::handleKeyDown(StellaKey key, StellaMod mod)
+void DebuggerDialog::handleKeyDown(StellaKey key, StellaMod mod, bool repeated)
 {
   if(key == KBDK_GRAVE && !StellaModTest::isShift(mod))
   {
     // Swallow backtick, so we don't see it when exiting the debugger
     instance().eventHandler().enableTextEvents(false);
   }
-  else if(key == KBDK_F12)
+
+  // special debugger keys first (cannot be remapped)
+  if (StellaModTest::isControl(mod))
   {
-    instance().debugger().parser().run("savesnap");
-    return;
-  }
-  else if(StellaModTest::isAlt(mod) && !StellaModTest::isControl(mod))
-  {
-    switch(key)
+    switch (key)
     {
-      case KBDK_LEFT:  // Alt-left(-shift) rewinds 1(10) states
-        if(StellaModTest::isShift(mod))
-          doRewind10();
-        else
-          doRewind();
-        return;
-      case KBDK_RIGHT:  // Alt-right(-shift) unwinds 1(10) states
-        if(StellaModTest::isShift(mod))
-          doUnwind10();
-        else
-          doUnwind();
-        return;
-      case KBDK_DOWN:  // Alt-down rewinds to start of list
-        doRewindAll();
-        return;
-      case KBDK_UP:  // Alt-up rewinds to end of list
-        doUnwindAll();
-        return;
-
-      case KBDK_Z:
-        if(StellaModTest::isShift(mod))
-          instance().console().toggleP0Collision();
-        else
-          instance().console().toggleP0Bit();
-        return;
-
-      case KBDK_X:
-        if(StellaModTest::isShift(mod))
-          instance().console().toggleP1Collision();
-        else
-          instance().console().toggleP1Bit();
-        return;
-
-      case KBDK_C:
-        if(StellaModTest::isShift(mod))
-          instance().console().toggleM0Collision();
-        else
-          instance().console().toggleM0Bit();
-        return;
-
-      case KBDK_V:
-        if(StellaModTest::isShift(mod))
-          instance().console().toggleM1Collision();
-        else
-          instance().console().toggleM1Bit();
-        return;
-
-      case KBDK_B:
-        if(StellaModTest::isShift(mod))
-          instance().console().toggleBLCollision();
-        else
-          instance().console().toggleBLBit();
-        return;
-
-      case KBDK_N:
-        if(StellaModTest::isShift(mod))
-          instance().console().togglePFCollision();
-        else
-          instance().console().togglePFBit();
-        return;
-
-      case KBDK_COMMA:
-        instance().console().toggleFixedColors();
-        return;
-
-      case KBDK_PERIOD:
-        if(StellaModTest::isShift(mod))
-          instance().console().toggleCollisions();
-        else
-          instance().console().toggleBits();
-        return;
-
-      case KBDK_T:  // Alt-t toggles Time Machine
-        instance().state().toggleTimeMachine();
-        break;
-
-      default:
-        break;
-    }
-  }
-  else if(StellaModTest::isControl(mod))
-  {
-    switch(key)
-    {
-#if 0
-      case KBDK_R:
-        if(StellaModTest::isAlt(mod))
-          doRewindAll();
-        else if(StellaModTest::isShift(mod))
-          doRewind10();
-        else
-          doRewind();
-        return;
-      case KBDK_Y:
-        if(StellaModTest::isAlt(mod))
-          doUnwindAll();
-        else if(StellaModTest::isShift(mod))
-          doUnwind10();
-        else
-          doUnwind();
-        return;
-#endif
       case KBDK_S:
         doStep();
         return;
@@ -218,6 +119,83 @@ void DebuggerDialog::handleKeyDown(StellaKey key, StellaMod mod)
         break;
     }
   }
+
+  // handle emulation keys second (can be remapped)
+  Event::Type event = instance().eventHandler().eventForKey(EventMode::kEmulationMode, key, mod);
+  switch (event)
+  {
+    // events which can be handled 1:1
+    case Event::ToggleP0Collision:
+    case Event::ToggleP0Bit:
+    case Event::ToggleP1Collision:
+    case Event::ToggleP1Bit:
+    case Event::ToggleM0Collision:
+    case Event::ToggleM0Bit:
+    case Event::ToggleM1Collision:
+    case Event::ToggleM1Bit:
+    case Event::ToggleBLCollision:
+    case Event::ToggleBLBit:
+    case Event::TogglePFCollision:
+    case Event::TogglePFBit:
+    case Event::ToggleFixedColors:
+    case Event::ToggleCollisions:
+    case Event::ToggleBits:
+
+    case Event::ToggleTimeMachine:
+
+    case Event::SaveState:
+    case Event::SaveAllStates:
+    case Event::PreviousState :
+    case Event::NextState:
+    case Event::LoadState:
+    case Event::LoadAllStates:
+
+    case Event::ConsoleColor:
+    case Event::ConsoleBlackWhite:
+    case Event::ConsoleColorToggle:
+    case Event::Console7800Pause:
+    case Event::ConsoleLeftDiffA:
+    case Event::ConsoleLeftDiffB:
+    case Event::ConsoleLeftDiffToggle:
+    case Event::ConsoleRightDiffA:
+    case Event::ConsoleRightDiffB:
+    case Event::ConsoleRightDiffToggle:
+      instance().eventHandler().handleEvent(event);
+      return;
+
+    // events which need special handling in debugger
+    case Event::TakeSnapshot:
+      instance().debugger().parser().run("savesnap");
+      return;
+
+    case Event::Rewind1Menu:
+      doRewind();
+      return;
+
+    case Event::Rewind10Menu:
+      doRewind10();
+      return;
+
+    case Event::RewindAllMenu:
+      doRewindAll();
+      return;
+
+    case Event::Unwind1Menu:
+      doUnwind();
+      return;
+
+    case Event::Unwind10Menu:
+      doUnwind10();
+      return;
+
+    case Event::UnwindAllMenu:
+      doUnwindAll();
+      return;
+
+    default:
+      break;
+  }
+
   Dialog::handleKeyDown(key, mod);
 }
 
@@ -392,7 +370,7 @@ void DebuggerDialog::createFont()
           myLFont = make_unique<GUI::Font>(GUI::consoleMediumDesc);
           myNFont = make_unique<GUI::Font>(GUI::consoleMediumDesc);
           break;
-      };
+      }
     }
   else
   {
@@ -421,35 +399,35 @@ void DebuggerDialog::createFont()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::showFatalMessage(const string& msg)
 {
-  myFatalError = make_unique<GUI::MessageBox>(this, *myLFont, msg, _w/2, _h/2,
-                          kDDExitFatalCmd, "Exit ROM", "Continue", "Fatal error");
+  myFatalError = make_unique<GUI::MessageBox>(this, *myLFont, msg, _w-20, _h-20,
+                                              kDDExitFatalCmd, "Exit ROM", "Continue", "Fatal error");
   myFatalError->show();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::addTiaArea()
 {
-  const GUI::Rect& r = getTiaBounds();
+  const Common::Rect& r = getTiaBounds();
   myTiaOutput =
-    new TiaOutputWidget(this, *myNFont, r.left, r.top, r.width(), r.height());
+    new TiaOutputWidget(this, *myNFont, r.x(), r.y(), r.w(), r.h());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::addTabArea()
 {
-  const GUI::Rect& r = getTabBounds();
+  const Common::Rect& r = getTabBounds();
   const int vBorder = 4;
 
   // The tab widget
   // Since there are two tab widgets in this dialog, we specifically
   // assign an ID of 0
-  myTab = new TabWidget(this, *myLFont, r.left, r.top + vBorder,
-                        r.width(), r.height() - vBorder);
+  myTab = new TabWidget(this, *myLFont, r.x(), r.y() + vBorder,
+                        r.w(), r.h() - vBorder);
   myTab->setID(0);
   addTabWidget(myTab);
 
-  const int widWidth  = r.width() - vBorder;
-  const int widHeight = r.height() - myTab->getTabHeight() - vBorder - 4;
+  const int widWidth  = r.w() - vBorder;
+  const int widHeight = r.h() - myTab->getTabHeight() - vBorder - 4;
   int tabID;
 
   // The Prompt/console tab
@@ -487,31 +465,30 @@ void DebuggerDialog::addTabArea()
 void DebuggerDialog::addStatusArea()
 {
   const int lineHeight = myLFont->getLineHeight();
-  const GUI::Rect& r = getStatusBounds();
+  const Common::Rect& r = getStatusBounds();
   int xpos, ypos;
 
-  xpos = r.left;  ypos = r.top;
-  myTiaInfo = new TiaInfoWidget(this, *myLFont, *myNFont, xpos, ypos, r.width());
+  xpos = r.x();  ypos = r.y();
+  myTiaInfo = new TiaInfoWidget(this, *myLFont, *myNFont, xpos, ypos, r.w());
 
-  ypos += myTiaInfo->getHeight() + 10;
-  myTiaZoom = new TiaZoomWidget(this, *myNFont, xpos+10, ypos,
-                                r.width()-10, r.height()-lineHeight-ypos-10);
+  ypos += myTiaInfo->getHeight() + 8;
+  myTiaZoom = new TiaZoomWidget(this, *myNFont, xpos + 10, ypos,
+                                r.w() - 10, r.h() - lineHeight - ypos - 3);
   addToFocusList(myTiaZoom->getFocusList());
 
-  xpos += 10;  ypos += myTiaZoom->getHeight() + 10;
+  xpos += 10;  ypos += myTiaZoom->getHeight() + 6;
   myMessageBox = new EditTextWidget(this, *myLFont,
                                     xpos, ypos, myTiaZoom->getWidth(),
                                     myLFont->getLineHeight(), "");
   myMessageBox->setEditable(false, false);
-  myMessageBox->clearFlags(WIDGET_RETAIN_FOCUS);
+  myMessageBox->clearFlags(Widget::FLAG_RETAIN_FOCUS);
   myMessageBox->setTextColor(kTextColorEm);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::addRomArea()
 {
-  static uInt32 LEFT_ARROW[11] =
-  {
+  static constexpr std::array<uInt32, 11> LEFT_ARROW = {
     0b0000010,
     0b0000110,
     0b0001110,
@@ -524,8 +501,7 @@ void DebuggerDialog::addRomArea()
     0b0000110,
     0b0000010
   };
-  static uInt32 RIGHT_ARROW[11] =
-  {
+  static constexpr std::array<uInt32, 11> RIGHT_ARROW = {
     0b0100000,
     0b0110000,
     0b0111000,
@@ -539,59 +515,60 @@ void DebuggerDialog::addRomArea()
     0b0100000
   };
 
-  const GUI::Rect& r = getRomBounds();
+  const Common::Rect& r = getRomBounds();
   const int VBORDER = 4;
   WidgetArray wid1, wid2;
   ButtonWidget* b;
 
   int bwidth  = myLFont->getStringWidth("Frame +1 "),
       bheight = myLFont->getLineHeight() + 2;
-  int buttonX = r.right - bwidth - 5, buttonY = r.top + 5;
+  int buttonX = r.x() + r.w() - bwidth - 5, buttonY = r.y() + 5;
 
   b = new ButtonWidget(this, *myLFont, buttonX, buttonY,
-                       bwidth, bheight, "Step", kDDStepCmd);
+                       bwidth, bheight, "Step", kDDStepCmd, true);
   wid2.push_back(b);
   buttonY += bheight + 4;
   b = new ButtonWidget(this, *myLFont, buttonX, buttonY,
-                       bwidth, bheight, "Trace", kDDTraceCmd);
+                       bwidth, bheight, "Trace", kDDTraceCmd, true);
   wid2.push_back(b);
   buttonY += bheight + 4;
   b = new ButtonWidget(this, *myLFont, buttonX, buttonY,
-                       bwidth, bheight, "Scan +1", kDDSAdvCmd);
+                       bwidth, bheight, "Scan +1", kDDSAdvCmd, true);
   wid2.push_back(b);
   buttonY += bheight + 4;
   b = new ButtonWidget(this, *myLFont, buttonX, buttonY,
-                       bwidth, bheight, "Frame +1", kDDAdvCmd);
+                       bwidth, bheight, "Frame +1", kDDAdvCmd, true);
   wid2.push_back(b);
   buttonY += bheight + 4;
   b = new ButtonWidget(this, *myLFont, buttonX, buttonY,
                        bwidth, bheight, "Exit", kDDExitCmd);
   wid2.push_back(b);
+  addCancelWidget(b);
 
   bwidth = bheight; // 7 + 12;
   bheight = bheight * 3 + 4 * 2;
   buttonX -= (bwidth + 5);
-  buttonY = r.top + 5;
+  buttonY = r.y() + 5;
 
   myRewindButton =
     new ButtonWidget(this, *myLFont, buttonX, buttonY,
-                     bwidth, bheight, LEFT_ARROW, 7, 11, kDDRewindCmd);
-  myRewindButton->clearFlags(WIDGET_ENABLED);
+                     bwidth, bheight, LEFT_ARROW.data(), 7, 11, kDDRewindCmd, true);
+  myRewindButton->clearFlags(Widget::FLAG_ENABLED);
 
   buttonY += bheight + 4;
   bheight = (myLFont->getLineHeight() + 2) * 2 + 4 * 1;
 
   myUnwindButton =
     new ButtonWidget(this, *myLFont, buttonX, buttonY,
-                     bwidth, bheight, RIGHT_ARROW, 7, 11, kDDUnwindCmd);
-  myUnwindButton->clearFlags(WIDGET_ENABLED);
+                     bwidth, bheight, RIGHT_ARROW.data(), 7, 11, kDDUnwindCmd, true);
+  myUnwindButton->clearFlags(Widget::FLAG_ENABLED);
 
   int xpos = buttonX - 8*myLFont->getMaxCharWidth() - 20, ypos = 30;
 
   bwidth = myLFont->getStringWidth("Options " + ELLIPSIS);
   bheight = myLFont->getLineHeight() + 2;
 
-  b = new ButtonWidget(this, *myLFont, xpos, r.top + 5, bwidth, bheight,
+  b = new ButtonWidget(this, *myLFont, xpos, r.y() + 5, bwidth, bheight,
                        "Options" + ELLIPSIS, kDDOptionsCmd);
   wid1.push_back(b);
   wid1.push_back(myRewindButton);
@@ -599,16 +576,16 @@ void DebuggerDialog::addRomArea()
 
   DataGridOpsWidget* ops = new DataGridOpsWidget(this, *myLFont, xpos, ypos);
 
-  int max_w = xpos - r.left - 10;
-  xpos = r.left + 10;  ypos = 10;
+  int max_w = xpos - r.x() - 10;
+  xpos = r.x() + 10;  ypos = 10;
   myCpu = new CpuWidget(this, *myLFont, *myNFont, xpos, ypos, max_w);
   addToFocusList(myCpu->getFocusList());
 
   addToFocusList(wid1);
   addToFocusList(wid2);
 
-  xpos = r.left + 10;  ypos += myCpu->getHeight() + 10;
-  myRam = new RiotRamWidget(this, *myLFont, *myNFont, xpos, ypos, r.width() - 10);
+  xpos = r.x() + 10;  ypos += myCpu->getHeight() + 10;
+  myRam = new RiotRamWidget(this, *myLFont, *myNFont, xpos, ypos, r.w() - 10);
   addToFocusList(myRam->getFocusList());
 
   // Add the DataGridOpsWidget to any widgets which contain a
@@ -619,9 +596,9 @@ void DebuggerDialog::addRomArea()
   ////////////////////////////////////////////////////////////////////
   // Disassembly area
 
-  xpos = r.left + VBORDER;  ypos += myRam->getHeight() + 5;
-  const int tabWidth  = r.width() - VBORDER - 1;
-  const int tabHeight = r.height() - ypos - 1;
+  xpos = r.x() + VBORDER;  ypos += myRam->getHeight() + 5;
+  const int tabWidth  = r.w() - VBORDER - 1;
+  const int tabHeight = r.h() - ypos - 1;
   int tabID;
 
   // Since there are two tab widgets in this dialog, we specifically
@@ -632,7 +609,7 @@ void DebuggerDialog::addRomArea()
   addTabWidget(myRomTab);
 
   // The main disassembly tab
-  tabID = myRomTab->addTab("   Disassembly   ");
+  tabID = myRomTab->addTab("  Disassembly  ", TabWidget::AUTO_WIDTH);
   myRom = new RomWidget(myRomTab, *myLFont, *myNFont, 2, 2, tabWidth - 1,
                         tabHeight - myRomTab->getTabHeight() - 2);
   myRomTab->setParentWidget(tabID, myRom);
@@ -680,47 +657,43 @@ void DebuggerDialog::addRomArea()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-GUI::Rect DebuggerDialog::getTiaBounds() const
+Common::Rect DebuggerDialog::getTiaBounds() const
 {
-  // The area showing the TIA image (NTSC and PAL supported, up to 260 lines)
-  GUI::Rect r(0, 0, 320, std::max(260, int(_h * 0.35)));
-  return r;
+  // The area showing the TIA image (NTSC and PAL supported, up to 274 lines without scaling)
+  return Common::Rect(0, 0, 320, std::max(int(FrameManager::Metrics::baseHeightPAL), int(_h * 0.35)));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-GUI::Rect DebuggerDialog::getRomBounds() const
+Common::Rect DebuggerDialog::getRomBounds() const
 {
   // The ROM area is the full area to the right of the tabs
-  const GUI::Rect& status = getStatusBounds();
-  GUI::Rect r(status.right + 1, 0, _w, _h);
-
-  return r;
+  const Common::Rect& status = getStatusBounds();
+  return Common::Rect(status.x() + status.w() + 1, 0, _w, _h);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-GUI::Rect DebuggerDialog::getStatusBounds() const
+Common::Rect DebuggerDialog::getStatusBounds() const
 {
   // The status area is the full area to the right of the TIA image
   // extending as far as necessary
   // 30% of any space above 1030 pixels will be allocated to this area
-  const GUI::Rect& tia = getTiaBounds();
+  const Common::Rect& tia = getTiaBounds();
 
-  int x1 = tia.right + 1;
+  int x1 = tia.x() + tia.w() + 1;
   int y1 = 0;
-  int x2 = tia.right + 225 + (_w > 1030 ? int(0.35 * (_w - 1030)) : 0);
-  int y2 = tia.bottom;
-  GUI::Rect r(x1, y1, x2, y2);
+  int x2 = tia.x() + tia.w() + 225 + (_w > 1030 ? int(0.35 * (_w - 1030)) : 0);
+  int y2 = tia.y() + tia.h();
 
-  return r;
+  return Common::Rect(x1, y1, x2, y2);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-GUI::Rect DebuggerDialog::getTabBounds() const
+Common::Rect DebuggerDialog::getTabBounds() const
 {
   // The tab area is the full area below the TIA image
-  const GUI::Rect& tia    = getTiaBounds();
-  const GUI::Rect& status = getStatusBounds();
-  GUI::Rect r(0, tia.bottom + 1, status.right + 1, _h);
+  const Common::Rect& tia    = getTiaBounds();
+  const Common::Rect& status = getStatusBounds();
 
-  return r;
+  return Common::Rect(0, tia.y() + tia.h() + 1,
+                      status.x() + status.w() + 1, _h);
 }
