@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -29,6 +29,7 @@
 
 #define PROMPT  "> "
 
+// TODO: Github issue #361
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PromptWidget::PromptWidget(GuiObject* boss, const GUI::Font& font,
                            int x, int y, int w, int h)
@@ -41,8 +42,8 @@ PromptWidget::PromptWidget(GuiObject* boss, const GUI::Font& font,
     _firstTime(true),
     _exitedEarly(false)
 {
-  _flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS |
-           WIDGET_WANTS_TAB | WIDGET_WANTS_RAWDATA;
+  _flags = Widget::FLAG_ENABLED | Widget::FLAG_CLEARBG | Widget::FLAG_RETAIN_FOCUS |
+           Widget::FLAG_WANTS_TAB | Widget::FLAG_WANTS_RAWDATA;
   _textcolor = kTextColor;
   _bgcolor = kWidColor;
   _bgcolorlo = kDlgColor;
@@ -209,7 +210,7 @@ bool PromptWidget::handleKeyDown(StellaKey key, StellaMod mod)
       int lastDelimPos = -1;
       char delimiter = '\0';
 
-      char inputStr[256];
+      char inputStr[256];  // NOLINT  (will be rewritten soon)
       for (int i = 0; i < len; i++)
       {
         inputStr[i] = buffer(_promptStartPos + i) & 0x7f;
@@ -328,6 +329,7 @@ bool PromptWidget::handleKeyDown(StellaKey key, StellaMod mod)
       break;
 
     case KBDK_DELETE:
+    case KBDK_KP_PERIOD: // actually the num delete
       killChar(+1);
       dirty = true;
       break;
@@ -490,16 +492,14 @@ void PromptWidget::insertIntoPrompt(const char* str)
 void PromptWidget::handleCommand(CommandSender* sender, int cmd,
                                  int data, int id)
 {
-  switch (cmd)
+  if(cmd == GuiObject::kSetPositionCmd)
   {
-    case GuiObject::kSetPositionCmd:
-      int newPos = int(data) + _linesPerPage - 1 + _firstLineInBuffer;
-      if (newPos != _scrollLine)
-      {
-        _scrollLine = newPos;
-        setDirty();
-      }
-      break;
+    int newPos = int(data) + _linesPerPage - 1 + _firstLineInBuffer;
+    if (newPos != _scrollLine)
+    {
+      _scrollLine = newPos;
+      setDirty();
+    }
   }
 }
 
@@ -689,7 +689,11 @@ void PromptWidget::textPaste()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PromptWidget::addToHistory(const char* str)
 {
+#if defined(BSPF_WINDOWS)
+  strncpy_s(_history[_historyIndex], kLineBufferSize, str, kLineBufferSize - 1);
+#else
   strncpy(_history[_historyIndex], str, kLineBufferSize - 1);
+#endif
   _historyIndex = (_historyIndex + 1) % kHistorySize;
   _historyLine = 0;
 
@@ -780,7 +784,6 @@ void PromptWidget::nextLine()
   updateScrollBuffer();
 }
 
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // Call this (at least) when the current line changes or when a new line is added
 void PromptWidget::updateScrollBuffer()
@@ -806,7 +809,8 @@ void PromptWidget::updateScrollBuffer()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int PromptWidget::printf(const char* format, ...)
+// TODO: rewrite this (cert-dcl50-cpp)
+int PromptWidget::printf(const char* format, ...)  // NOLINT
 {
   va_list argptr;
 
@@ -819,7 +823,7 @@ int PromptWidget::printf(const char* format, ...)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int PromptWidget::vprintf(const char* format, va_list argptr)
 {
-  char buf[2048];  // Note: generates warnings about 'nonliteral' format
+  char buf[2048];  // NOLINT  (will be rewritten soon)
   int count = std::vsnprintf(buf, sizeof(buf), format, argptr);
 
   print(buf);
