@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -77,31 +77,33 @@ RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& lfont,
   bits->setList(off, on);
 
   // SWCHA bits in 'poke' mode
-  CREATE_IO_REGS("SWCHA(W)", mySWCHAWriteBits, kSWCHABitsID, true);
+  CREATE_IO_REGS("SWCHA(W)", mySWCHAWriteBits, kSWCHABitsID, true)
   col = xpos + 20;  // remember this for adding widgets to the second column
 
   // SWACNT bits
   xpos = 10;  ypos += lineHeight + 5;
-  CREATE_IO_REGS("SWACNT", mySWACNTBits, kSWACNTBitsID, true);
+  CREATE_IO_REGS("SWACNT", mySWACNTBits, kSWACNTBitsID, true)
 
   // SWCHA bits in 'peek' mode
   xpos = 10;  ypos += lineHeight + 5;
-  CREATE_IO_REGS("SWCHA(R)", mySWCHAReadBits, kSWCHARBitsID, true);
+  CREATE_IO_REGS("SWCHA(R)", mySWCHAReadBits, kSWCHARBitsID, true)
 
   // SWCHB bits in 'poke' mode
   xpos = 10;  ypos += 2 * lineHeight;
-  CREATE_IO_REGS("SWCHB(W)", mySWCHBWriteBits, kSWCHBBitsID, true);
+  CREATE_IO_REGS("SWCHB(W)", mySWCHBWriteBits, kSWCHBBitsID, true)
 
   // SWBCNT bits
   xpos = 10;  ypos += lineHeight + 5;
-  CREATE_IO_REGS("SWBCNT", mySWBCNTBits, kSWBCNTBitsID, true);
+  CREATE_IO_REGS("SWBCNT", mySWBCNTBits, kSWBCNTBitsID, true)
 
   // SWCHB bits in 'peek' mode
   xpos = 10;  ypos += lineHeight + 5;
-  CREATE_IO_REGS("SWCHB(R)", mySWCHBReadBits, 0, false);
+  CREATE_IO_REGS("SWCHB(R)", mySWCHBReadBits, 0, false)
 
   // Timer registers (R/W)
-  const char* const writeNames[] = { "TIM1T", "TIM8T", "TIM64T", "T1024T" };
+  static constexpr std::array<const char*, 4> writeNames = {
+    "TIM1T", "TIM8T", "TIM64T", "T1024T"
+  };
   xpos = 10;  ypos += 2*lineHeight;
   for(int row = 0; row < 4; ++row)
   {
@@ -109,13 +111,15 @@ RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& lfont,
                              lwidth, fontHeight, writeNames[row], TextAlign::Left);
   }
   xpos += t->getWidth() + 5;
-  myTimWrite = new DataGridWidget(boss, nfont, xpos, ypos, 1, 4, 2, 8, Common::Base::F_16);
+  myTimWrite = new DataGridWidget(boss, nfont, xpos, ypos, 1, 4, 2, 8, Common::Base::Fmt::_16);
   myTimWrite->setTarget(this);
   myTimWrite->setID(kTimWriteID);
   addFocusWidget(myTimWrite);
 
   // Timer registers (RO)
-  const char* const readNames[] = { "INTIM", "TIMINT", "Total Clks", "INTIM Clks", "Divider" };
+  static constexpr std::array<const char*, 5> readNames = {
+    "INTIM", "TIMINT", "Total Clks", "INTIM Clks", "Divider"
+  };
   xpos = 10;  ypos += myTimWrite->getHeight() + lineHeight / 2;
   for(int row = 0; row < 5; ++row)
   {
@@ -123,26 +127,27 @@ RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& lfont,
                              10*fontWidth, fontHeight, readNames[row], TextAlign::Left);
   }
   xpos += t->getWidth() + 5;
-  myTimRead = new DataGridWidget(boss, nfont, xpos, ypos, 1, 4, 8, 32, Common::Base::F_16);
+  myTimRead = new DataGridWidget(boss, nfont, xpos, ypos, 1, 4, 8, 32, Common::Base::Fmt::_16);
   myTimRead->setTarget(this);
   myTimRead->setEditable(false);
 
   ypos += myTimRead->getHeight() - 1;
-  myTimDivider = new DataGridWidget(boss, nfont, xpos, ypos, 1, 1, 4, 32, Common::Base::F_10_4);
+  myTimDivider = new DataGridWidget(boss, nfont, xpos, ypos, 1, 1, 4, 32, Common::Base::Fmt::_10_4);
   myTimDivider->setTarget(this);
   myTimDivider->setEditable(false);
 
   // Controller ports
-  const RiotDebug& riot = instance().debugger().riotDebug();
   xpos = col;  ypos = 10;
   myLeftControl = addControlWidget(boss, lfont, xpos, ypos,
-      riot.controller(Controller::Left));
+      instance().console().leftController());
   xpos += myLeftControl->getWidth() + 15;
   myRightControl = addControlWidget(boss, lfont, xpos, ypos,
-      riot.controller(Controller::Right));
+      instance().console().rightController());
 
   // TIA INPTx registers (R), left port
-  const char* const contLeftReadNames[] = { "INPT0", "INPT1", "INPT4" };
+  static constexpr std::array<const char*, 3> contLeftReadNames = {
+    "INPT0", "INPT1", "INPT4"
+  };
   xpos = col;  ypos += myLeftControl->getHeight() + 2 * lineHeight;
   for(int row = 0; row < 3; ++row)
   {
@@ -150,12 +155,14 @@ RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& lfont,
                          5*fontWidth, fontHeight, contLeftReadNames[row], TextAlign::Left);
   }
   xpos += 5*fontWidth + 5;
-  myLeftINPT = new DataGridWidget(boss, nfont, xpos, ypos, 1, 3, 2, 8, Common::Base::F_16);
+  myLeftINPT = new DataGridWidget(boss, nfont, xpos, ypos, 1, 3, 2, 8, Common::Base::Fmt::_16);
   myLeftINPT->setTarget(this);
   myLeftINPT->setEditable(false);
 
   // TIA INPTx registers (R), right port
-  const char* const contRightReadNames[] = { "INPT2", "INPT3", "INPT5" };
+  static constexpr std::array<const char*, 3> contRightReadNames = {
+    "INPT2", "INPT3", "INPT5"
+  };
   xpos = col + myLeftControl->getWidth() + 15;
   for(int row = 0; row < 3; ++row)
   {
@@ -163,7 +170,7 @@ RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& lfont,
                          5*fontWidth, fontHeight, contRightReadNames[row], TextAlign::Left);
   }
   xpos += 5*fontWidth + 5;
-  myRightINPT = new DataGridWidget(boss, nfont, xpos, ypos, 1, 3, 2, 8, Common::Base::F_16);
+  myRightINPT = new DataGridWidget(boss, nfont, xpos, ypos, 1, 3, 2, 8, Common::Base::Fmt::_16);
   myRightINPT->setTarget(this);
   myRightINPT->setEditable(false);
 
@@ -257,22 +264,22 @@ void RiotWidget::loadConfig()
   const RiotState& oldstate = static_cast<const RiotState&>(riot.getOldState());
 
   // Update the SWCHA register booleans (poke mode)
-  IO_REGS_UPDATE(mySWCHAWriteBits, swchaWriteBits);
+  IO_REGS_UPDATE(mySWCHAWriteBits, swchaWriteBits)
 
   // Update the SWACNT register booleans
-  IO_REGS_UPDATE(mySWACNTBits, swacntBits);
+  IO_REGS_UPDATE(mySWACNTBits, swacntBits)
 
   // Update the SWCHA register booleans (peek mode)
-  IO_REGS_UPDATE(mySWCHAReadBits, swchaReadBits);
+  IO_REGS_UPDATE(mySWCHAReadBits, swchaReadBits)
 
   // Update the SWCHB register booleans (poke mode)
-  IO_REGS_UPDATE(mySWCHBWriteBits, swchbWriteBits);
+  IO_REGS_UPDATE(mySWCHBWriteBits, swchbWriteBits)
 
   // Update the SWBCNT register booleans
-  IO_REGS_UPDATE(mySWBCNTBits, swbcntBits);
+  IO_REGS_UPDATE(mySWBCNTBits, swbcntBits)
 
   // Update the SWCHB register booleans (peek mode)
-  IO_REGS_UPDATE(mySWCHBReadBits, swchbReadBits);
+  IO_REGS_UPDATE(mySWCHBReadBits, swchbReadBits)
 
   // Update TIA INPTx registers
   alist.clear();  vlist.clear();  changed.clear();
@@ -354,25 +361,25 @@ void RiotWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
   switch(cmd)
   {
     case DataGridWidget::kItemDataChangedCmd:
-      switch(id)
+      if(id == kTimWriteID)
       {
-        case kTimWriteID:
-          switch(myTimWrite->getSelectedAddr())
-          {
-            case kTim1TID:
-              riot.tim1T(myTimWrite->getSelectedValue());
-              break;
-            case kTim8TID:
-              riot.tim8T(myTimWrite->getSelectedValue());
-              break;
-            case kTim64TID:
-              riot.tim64T(myTimWrite->getSelectedValue());
-              break;
-            case kTim1024TID:
-              riot.tim1024T(myTimWrite->getSelectedValue());
-              break;
-          }
-          break;
+        switch(myTimWrite->getSelectedAddr())
+        {
+          case kTim1TID:
+            riot.tim1T(myTimWrite->getSelectedValue());
+            break;
+          case kTim8TID:
+            riot.tim8T(myTimWrite->getSelectedValue());
+            break;
+          case kTim64TID:
+            riot.tim64T(myTimWrite->getSelectedValue());
+            break;
+          case kTim1024TID:
+            riot.tim1024T(myTimWrite->getSelectedValue());
+            break;
+          default:
+            break;
+        }
       }
       break;
 
@@ -397,18 +404,21 @@ void RiotWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
           break;
         case kSWCHARBitsID:
         {
-          // TODO: Check if there is a nicer way to do this
           value = Debugger::get_bits(mySWCHAReadBits->getState());
-          riot.controller(Controller::Left).set(Controller::One, value & 0b00010000);
-          riot.controller(Controller::Left).set(Controller::Two, value & 0b00100000);
-          riot.controller(Controller::Left).set(Controller::Three, value & 0b01000000);
-          riot.controller(Controller::Left).set(Controller::Four, value & 0b10000000);
-          riot.controller(Controller::Right).set(Controller::One, value & 0b00000001);
-          riot.controller(Controller::Right).set(Controller::Two, value & 0b00000010);
-          riot.controller(Controller::Right).set(Controller::Three, value & 0b00000100);
-          riot.controller(Controller::Right).set(Controller::Four, value & 0b00001000);
+          ControllerLowLevel lport(instance().console().leftController());
+          ControllerLowLevel rport(instance().console().rightController());
+          lport.setPin(Controller::DigitalPin::One,   value & 0b00010000);
+          lport.setPin(Controller::DigitalPin::Two,   value & 0b00100000);
+          lport.setPin(Controller::DigitalPin::Three, value & 0b01000000);
+          lport.setPin(Controller::DigitalPin::Four,  value & 0b10000000);
+          rport.setPin(Controller::DigitalPin::One,   value & 0b00000001);
+          rport.setPin(Controller::DigitalPin::Two,   value & 0b00000010);
+          rport.setPin(Controller::DigitalPin::Three, value & 0b00000100);
+          rport.setPin(Controller::DigitalPin::Four,  value & 0b00001000);
           break;
         }
+        default:
+          break;
       }
       break;
 
@@ -424,6 +434,8 @@ void RiotWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
         case kPauseID:
           handleConsole();
           break;
+        default:
+          break;
       }
       break;
 
@@ -438,6 +450,9 @@ void RiotWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
     case kTVTypeChanged:
       handleConsole();
       break;
+
+    default:
+      break;
   }
 }
 
@@ -447,29 +462,29 @@ ControllerWidget* RiotWidget::addControlWidget(GuiObject* boss, const GUI::Font&
 {
   switch(controller.type())
   {
-    case Controller::AmigaMouse:
+    case Controller::Type::AmigaMouse:
       return new AmigaMouseWidget(boss, font, x, y, controller);
-    case Controller::AtariMouse:
+    case Controller::Type::AtariMouse:
       return new AtariMouseWidget(boss, font, x, y, controller);
-    case Controller::AtariVox:
+    case Controller::Type::AtariVox:
       return new AtariVoxWidget(boss, font, x, y, controller);
-    case Controller::BoosterGrip:
+    case Controller::Type::BoosterGrip:
       return new BoosterWidget(boss, font, x, y, controller);
-    case Controller::Driving:
+    case Controller::Type::Driving:
       return new DrivingWidget(boss, font, x, y, controller);
-    case Controller::Genesis:
+    case Controller::Type::Genesis:
       return new GenesisWidget(boss, font, x, y, controller);
-    case Controller::Joystick:
+    case Controller::Type::Joystick:
       return new JoystickWidget(boss, font, x, y, controller);
-    case Controller::Keyboard:
+    case Controller::Type::Keyboard:
       return new KeyboardWidget(boss, font, x, y, controller);
-//    case Controller::KidVid:      // TODO - implement this
-//    case Controller::MindLink:    // TODO - implement this
-    case Controller::Paddles:
+//    case Controller::Type::KidVid:      // TODO - implement this
+//    case Controller::Type::MindLink:    // TODO - implement this
+    case Controller::Type::Paddles:
       return new PaddleWidget(boss, font, x, y, controller);
-    case Controller::SaveKey:
+    case Controller::Type::SaveKey:
       return new SaveKeyWidget(boss, font, x, y, controller);
-    case Controller::TrakBall:
+    case Controller::Type::TrakBall:
       return new TrakBallWidget(boss, font, x, y, controller);
     default:
       return new NullControlWidget(boss, font, x, y, controller);
