@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -18,7 +18,6 @@
 #include "bspf.hxx"
 #include "Debugger.hxx"
 #include "DiStella.hxx"
-#include "PackedBitArray.hxx"
 #include "Widget.hxx"
 #include "StellaKeys.hxx"
 #include "FBSurface.hxx"
@@ -31,19 +30,9 @@
 RomListWidget::RomListWidget(GuiObject* boss, const GUI::Font& lfont,
                              const GUI::Font& nfont,
                              int x, int y, int w, int h)
-  : EditableWidget(boss, nfont, x, y, 16, 16),
-    _rows(0),
-    _cols(0),
-    _currentPos(0),
-    _selectedItem(-1),
-    _highlightedItem(-1),
-    _editMode(false),
-    _currentKeyDown(KBDK_UNKNOWN),
-    _base(Common::Base::F_DEFAULT),
-    myDisasm(nullptr),
-    myBPState(nullptr)
+  : EditableWidget(boss, nfont, x, y, 16, 16)
 {
-  _flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS;
+  _flags = Widget::FLAG_ENABLED | Widget::FLAG_CLEARBG | Widget::FLAG_RETAIN_FOCUS;
   _bgcolor = kWidColor;
   _bgcolorhi = kWidColor;
   _textcolor = kTextColor;
@@ -86,7 +75,7 @@ RomListWidget::RomListWidget(GuiObject* boss, const GUI::Font& lfont,
         CheckboxWidget::kCheckActionCmd);
     t->setTarget(this);
     t->setID(i);
-    t->setFill(CheckboxWidget::Circle);
+    t->setFill(CheckboxWidget::FillType::Circle);
     t->setTextColor(kTextColorEm);
     ypos += _fontHeight;
 
@@ -98,22 +87,22 @@ RomListWidget::RomListWidget(GuiObject* boss, const GUI::Font& lfont,
   {
     switch(_base)
     {
-      case Common::Base::F_16:
-      case Common::Base::F_16_1:
-      case Common::Base::F_16_2:
-      case Common::Base::F_16_4:
-      case Common::Base::F_16_8:
+      case Common::Base::Fmt::_16:
+      case Common::Base::Fmt::_16_1:
+      case Common::Base::Fmt::_16_2:
+      case Common::Base::Fmt::_16_4:
+      case Common::Base::Fmt::_16_8:
         return (c >= '0' && c <= '9') || (c >= 'a' && c <= 'f') || c == ' ';
 
-      case Common::Base::F_2:
-      case Common::Base::F_2_8:
-      case Common::Base::F_2_16:
+      case Common::Base::Fmt::_2:
+      case Common::Base::Fmt::_2_8:
+      case Common::Base::Fmt::_2_16:
         return c == '0' || c == '1' || c == ' ';
 
-      case Common::Base::F_10:
+      case Common::Base::Fmt::_10:
         return (c >= '0' && c <= '9') || c == ' ';
 
-      case Common::Base::F_DEFAULT:
+      case Common::Base::Fmt::_DEFAULT:
       default:  // TODO - properly handle all other cases
         return false;
     }
@@ -122,20 +111,18 @@ RomListWidget::RomListWidget(GuiObject* boss, const GUI::Font& lfont,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void RomListWidget::setList(const CartDebug::Disassembly& disasm,
-                            const PackedBitArray& state)
+void RomListWidget::setList(const CartDebug::Disassembly& disasm)
 {
   myDisasm = &disasm;
-  myBPState = &state;
 
   // Enable all checkboxes
   for(int i = 0; i < _rows; ++i)
-    myCheckList[i]->setFlags(WIDGET_ENABLED);
+    myCheckList[i]->setFlags(Widget::FLAG_ENABLED);
 
   // Then turn off any extras
   if(int(myDisasm->list.size()) < _rows)
     for(int i = int(myDisasm->list.size()); i < _rows; ++i)
-      myCheckList[i]->clearFlags(WIDGET_ENABLED);
+      myCheckList[i]->clearFlags(Widget::FLAG_ENABLED);
 
   recalc();
 }
@@ -170,8 +157,12 @@ void RomListWidget::setHighlighted(int item)
     _highlightedItem = item;
 
     // Only scroll the list if we're about to pass the page boundary
-    if(_currentPos == 0)
-      _currentPos = _highlightedItem;
+    if (_highlightedItem < _currentPos)
+    {
+      _currentPos -= _rows;
+      if (_currentPos < 0)
+        _currentPos = 0;
+    }
     else if(_highlightedItem == _currentPos + _rows)
       _currentPos += _rows;
 
@@ -248,7 +239,8 @@ void RomListWidget::handleMouseDown(int x, int y, MouseButton b, int clickCount)
     // Set selected and add menu at current x,y mouse location
     _selectedItem = findItem(x, y);
     scrollToSelected();
-    myMenu->show(x + getAbsX(), y + getAbsY(), _selectedItem);
+    myMenu->show(x + getAbsX(), y + getAbsY(),
+                 dialog().surface().dstRect(), _selectedItem);
   }
   else
   {
@@ -290,14 +282,14 @@ void RomListWidget::handleMouseWheel(int x, int y, int direction)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void RomListWidget::handleMouseEntered()
 {
-  setFlags(WIDGET_HILITED);
+  setFlags(Widget::FLAG_HILITED);
   setDirty();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void RomListWidget::handleMouseLeft()
 {
-  clearFlags(WIDGET_HILITED);
+  clearFlags(Widget::FLAG_HILITED);
   setDirty();
 }
 
@@ -433,8 +425,7 @@ void RomListWidget::handleCommand(CommandSender* sender, int cmd, int data, int
     case CheckboxWidget::kCheckActionCmd:
       // We let the parent class handle this
       // Pass it as a kRLBreakpointChangedCmd command, since that's the intent
-      sendCommand(RomListWidget::kBPointChangedCmd, _currentPos+id,
-                  myCheckList[id]->getState());
+      sendCommand(RomListWidget::kBPointChangedCmd, _currentPos+id, 0);
       break;
 
     case GuiObject::kSetPositionCmd:
@@ -469,8 +460,8 @@ void RomListWidget::drawWidget(bool hilite)
   int i, pos, xpos, ypos, len = int(dlist.size());
   ColorId textColor = onTop ? kTextColor : kColor;
 
-  const GUI::Rect& r = getEditRect();
-  const GUI::Rect& l = getLineRect();
+  const Common::Rect& r = getEditRect();
+  const Common::Rect& l = getLineRect();
 
   // Draw a thin frame around the list and to separate columns
   s.frameRect(_x, _y, _w + 1, _h, hilite ? kWidColorHi : kColor);
@@ -479,7 +470,7 @@ void RomListWidget::drawWidget(bool hilite)
   // Draw the list items
   int cycleCountW = _fontWidth * 8,
       noTypeDisasmW = _w - l.x() - _labelWidth,
-      noCodeDisasmW = noTypeDisasmW - r.width(),
+      noCodeDisasmW = noTypeDisasmW - r.w(),
       codeDisasmW = noCodeDisasmW - cycleCountW,
       actualWidth = myDisasm->fieldwidth * _fontWidth;
   if(actualWidth < codeDisasmW)
@@ -491,7 +482,10 @@ void RomListWidget::drawWidget(bool hilite)
     ColorId bytesColor = textColor;
 
     // Draw checkboxes for correct lines (takes scrolling into account)
-    myCheckList[i]->setState(myBPState->isSet(dlist[pos].address));
+    myCheckList[i]->setState(instance().debugger().
+                             checkBreakPoint(dlist[pos].address,
+                             instance().debugger().cartDebug().getBank(dlist[pos].address)));
+
     myCheckList[i]->setDirty();
     myCheckList[i]->draw();
 
@@ -504,11 +498,11 @@ void RomListWidget::drawWidget(bool hilite)
     {
       if(!_editMode)
       {
-        s.fillRect(_x + r.x() - 3, ypos - 1, r.width(), _fontHeight, kTextColorHi);
+        s.fillRect(_x + r.x() - 3, ypos - 1, r.w(), _fontHeight, kTextColorHi);
         bytesColor = kTextColorInv;
       }
       else
-        s.frameRect(_x + r.x() - 3, ypos - 1, r.width(), _fontHeight, kWidColorHi);
+        s.frameRect(_x + r.x() - 3, ypos - 1, r.w(), _fontHeight, kWidColorHi);
     }
 
     // Draw labels
@@ -547,14 +541,14 @@ void RomListWidget::drawWidget(bool hilite)
         if (_selectedItem == pos && _editMode)
         {
           adjustOffset();
-          s.drawString(_font, editString(), _x + r.x(), ypos, r.width(), textColor,
+          s.drawString(_font, editString(), _x + r.x(), ypos, r.w(), textColor,
                        TextAlign::Left, -_editScrollOffset, false);
 
           drawCaret();
         }
         else
         {
-          s.drawString(_font, dlist[pos].bytes, _x + r.x(), ypos, r.width(), bytesColor);
+          s.drawString(_font, dlist[pos].bytes, _x + r.x(), ypos, r.w(), bytesColor);
         }
       }
     }
@@ -568,30 +562,22 @@ void RomListWidget::drawWidget(bool hilite)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-GUI::Rect RomListWidget::getLineRect() const
+Common::Rect RomListWidget::getLineRect() const
 {
-  GUI::Rect r(2, 1, _w, _fontHeight);
-  const int yoffset = (_selectedItem - _currentPos) * _fontHeight,
+  const int yoffset = std::max(0, (_selectedItem - _currentPos) * _fontHeight),
             xoffset = CheckboxWidget::boxSize() + 10;
-  r.top    += yoffset;
-  r.bottom += yoffset;
-  r.left   += xoffset;
-  r.right  -= xoffset - 15;
 
-  return r;
+  return Common::Rect(2 + xoffset, 1 + yoffset,
+                      _w - (xoffset - 15), _fontHeight + yoffset);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-GUI::Rect RomListWidget::getEditRect() const
+Common::Rect RomListWidget::getEditRect() const
 {
-  GUI::Rect r(2, 1, _w, _fontHeight);
-  const int yoffset = (_selectedItem - _currentPos) * _fontHeight;
-  r.top    += yoffset;
-  r.bottom += yoffset;
-  r.left   += _w - _bytesWidth;
-  r.right   = _w;
+  const int yoffset = std::max(0, (_selectedItem - _currentPos) * _fontHeight);
 
-  return r;
+  return Common::Rect(2 + _w - _bytesWidth, 1 + yoffset,
+                      _w, _fontHeight + yoffset);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -629,7 +615,7 @@ void RomListWidget::endEditMode()
   // Send a message that editing finished with a return/enter key press
   // The parent then calls getText() to get the newly entered data
   _editMode = false;
-  sendCommand(RomListWidget::kRomChangedCmd, _selectedItem, _base);
+  sendCommand(RomListWidget::kRomChangedCmd, _selectedItem, static_cast<int>(_base));
 
   // Reset to normal data entry
   EditableWidget::endEditMode();
