@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -33,8 +33,7 @@
 RomWidget::RomWidget(GuiObject* boss, const GUI::Font& lfont, const GUI::Font& nfont,
                      int x, int y, int w, int h)
   : Widget(boss, lfont, x, y, w, h),
-    CommandSender(boss),
-    myListIsDirty(true)
+    CommandSender(boss)
 {
   int xpos, ypos;
   StaticTextWidget* t;
@@ -72,7 +71,7 @@ void RomWidget::loadConfig()
   myListIsDirty |= cart.disassemble(myListIsDirty);
   if(myListIsDirty)
   {
-    myRomList->setList(cart.disassembly(), dbg.breakPoints());
+    myRomList->setList(cart.disassembly());
     myListIsDirty = false;
   }
 
@@ -92,8 +91,7 @@ void RomWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
   {
     case RomListWidget::kBPointChangedCmd:
       // 'data' is the line in the disassemblylist to be accessed
-      // 'id' is the state of the breakpoint at 'data'
-      setBreak(data, id);
+      toggleBreak(data);
       // Refresh the romlist, since the breakpoint may not have
       // actually changed
       myRomList->setDirty();
@@ -103,7 +101,7 @@ void RomWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
     case RomListWidget::kRomChangedCmd:
       // 'data' is the line in the disassemblylist to be accessed
       // 'id' is the base to use for the data to be changed
-      patchROM(data, myRomList->getText(), Common::Base::Format(id));
+      patchROM(data, myRomList->getText(), Common::Base::Fmt(id));
       break;
 
     case RomListWidget::kSetPCCmd:
@@ -142,12 +140,12 @@ void RomWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       // 'data' is the boolean value
       if(data)
       {
-        DiStella::settings.gfxFormat = Common::Base::F_2;
+        DiStella::settings.gfxFormat = Common::Base::Fmt::_2;
         instance().settings().setValue("dis.gfxformat", "2");
       }
       else
       {
-        DiStella::settings.gfxFormat = Common::Base::F_16;
+        DiStella::settings.gfxFormat = Common::Base::Fmt::_16;
         instance().settings().setValue("dis.gfxformat", "16");
       }
       invalidate();
@@ -160,18 +158,22 @@ void RomWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
           DiStella::settings.rFlag);
       invalidate();
       break;
+
+    default:
+      break;
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void RomWidget::setBreak(int disasm_line, bool state)
+void RomWidget::toggleBreak(int disasm_line)
 {
   const CartDebug::DisassemblyList& list =
       instance().debugger().cartDebug().disassembly().list;
   if(disasm_line >= int(list.size()))  return;
 
   if(list[disasm_line].address != 0 && list[disasm_line].bytes != "")
-    instance().debugger().setBreakPoint(list[disasm_line].address, state);
+    instance().debugger().toggleBreakPoint(list[disasm_line].address,
+                                           instance().debugger().cartDebug().getBank(list[disasm_line].address));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -206,7 +208,7 @@ void RomWidget::runtoPC(int disasm_line)
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void RomWidget::patchROM(int disasm_line, const string& bytes,
-                         Common::Base::Format base)
+                         Common::Base::Fmt base)
 {
   const CartDebug::DisassemblyList& list =
       instance().debugger().cartDebug().disassembly().list;
@@ -218,7 +220,7 @@ void RomWidget::patchROM(int disasm_line, const string& bytes,
 
     // Temporarily set to correct base, so we don't have to prefix each byte
     // with the type of data
-    Common::Base::Format oldbase = Common::Base::format();
+    Common::Base::Fmt oldbase = Common::Base::format();
 
     Common::Base::setFormat(base);
     command << "rom #" << list[disasm_line].address << " " << bytes;
