@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -27,11 +27,7 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Cartridge::Cartridge(const Settings& settings, const string& md5)
-  : mySettings(settings),
-    myBankChanged(true),
-    myCodeAccessBase(nullptr),
-    myStartBank(0),
-    myBankLocked(false)
+  : mySettings(settings)
 {
   auto to_uInt32 = [](const string& s, uInt32 pos) {
     return uInt32(std::stoul(s.substr(pos, 8), nullptr, 16));
@@ -58,7 +54,7 @@ void Cartridge::setAbout(const string& about, const string& type,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge::saveROM(ofstream& out) const
 {
-  uInt32 size = 0;
+  size_t size = 0;
 
   const uInt8* image = getImage(size);
   if(image == nullptr || size == 0)
@@ -118,24 +114,24 @@ void Cartridge::pokeRAM(uInt8& dest, uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Cartridge::createCodeAccessBase(uInt32 size)
+void Cartridge::createCodeAccessBase(size_t size)
 {
 #ifdef DEBUGGER_SUPPORT
   myCodeAccessBase = make_unique<uInt8[]>(size);
-  memset(myCodeAccessBase.get(), CartDebug::ROW, size);
+  std::fill_n(myCodeAccessBase.get(), size, CartDebug::ROW);
 #else
   myCodeAccessBase = nullptr;
 #endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Cartridge::initializeRAM(uInt8* arr, uInt32 size, uInt8 val) const
+void Cartridge::initializeRAM(uInt8* arr, size_t size, uInt8 val) const
 {
   if(randomInitialRAM())
-    for(uInt32 i = 0; i < size; ++i)
+    for(size_t i = 0; i < size; ++i)
       arr[i] = mySystem->randGenerator().next();
   else
-    memset(arr, val, size);
+    std::fill_n(arr, size, val);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
