@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -20,24 +20,23 @@
 #include "Cart3E.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Cartridge3E::Cartridge3E(const BytePtr& image, uInt32 size,
+Cartridge3E::Cartridge3E(const ByteBuffer& image, size_t size,
                          const string& md5, const Settings& settings)
   : Cartridge(settings, md5),
-    mySize(size),
-    myCurrentBank(0)
+    mySize(size)
 {
   // Allocate array for the ROM image
   myImage = make_unique<uInt8[]>(mySize);
 
   // Copy the ROM image into my buffer
-  memcpy(myImage.get(), image.get(), mySize);
-  createCodeAccessBase(mySize + 32768);
+  std::copy_n(image.get(), mySize, myImage.get());
+  createCodeAccessBase(mySize + myRAM.size());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge3E::reset()
 {
-  initializeRAM(myRAM, 32768);
+  initializeRAM(myRAM.data(), myRAM.size());
   initializeStartBank(0);
 
   // We'll map the startup bank into the first segment upon reset
@@ -49,14 +48,14 @@ void Cartridge3E::install(System& system)
 {
   mySystem = &system;
 
-  System::PageAccess access(this, System::PA_READWRITE);
+  System::PageAccess access(this, System::PageAccessType::READWRITE);
 
   // The hotspots ($3E and $3F) are in TIA address space, so we claim it here
   for(uInt16 addr = 0x00; addr < 0x40; addr += System::PAGE_SIZE)
     mySystem->setPageAccess(addr, access);
 
   // Setup the second segment to always point to the last ROM slice
-  access.type = System::PA_READ;
+  access.type = System::PageAccessType::READ;
   for(uInt16 addr = 0x1800; addr < 0x2000; addr += System::PAGE_SIZE)
   {
     access.directPeekBase = &myImage[(mySize - 2048) + (addr & 0x07FF)];
@@ -74,25 +73,19 @@ uInt8 Cartridge3E::peek(uInt16 address)
   uInt16 peekAddress = address;
   address &= 0x0FFF;
 
-  if(address < 0x0800)
+  // Due to the way paging is set up, the only way to get here is a TIA read or
+  // attempting to read from the RAM write port
+
+  if(address < 0x0040)  // TIA access
+    return mySystem->tia().peek(address);
+  else if(myCurrentBank >= 256)
   {
-    if(myCurrentBank < 256)
-      return myImage[(address & 0x07FF) + (myCurrentBank << 11)];
-    else
-    {
-      if(address < 0x0400)
-        return myRAM[(address & 0x03FF) + ((myCurrentBank - 256) << 10)];
-      else
-      {
-        // Reading from the write port triggers an unwanted write
-        return peekRAM(myRAM[(address & 0x03FF) + ((myCurrentBank - 256) << 10)], peekAddress);
-      }
-    }
-  }
-  else
-  {
-    return myImage[(address & 0x07FF) + mySize - 2048];
+    // Reading from the write port triggers an unwanted write
+    return peekRAM(myRAM[(address & 0x03FF) + ((myCurrentBank - 256) << 10)], peekAddress);
   }
+
+  // Make compiler happy; should never get here
+  return myImage[(address & 0x07FF) + mySize - 2048];
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -112,10 +105,25 @@ bool Cartridge3E::poke(uInt16 address, uInt8 value)
 
     return mySystem->tia().poke(address, value);
   }
-  else
-    pokeRAM(myRAM[(address & 0x03FF) + ((myCurrentBank - 256) << 10)], pokeAddress, value);
+  else if(myCurrentBank >= 256)
+  {
+    if(address & 0x0400)
+    {
+      pokeRAM(myRAM[(address & 0x03FF) + ((myCurrentBank - 256) << 10)],
+              pokeAddress, value);
+      return true;
+    }
+    else
+    {
+      // Writing to the read port should be ignored, but trigger a break if option enabled
+      uInt8 dummy;
 
-  return true;
+      pokeRAM(dummy, pokeAddress, value);
+      myRamWriteAccess = pokeAddress;
+      return false;
+    }
+  }
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -140,7 +148,7 @@ bool Cartridge3E::bank(uInt16 bank)
     uInt32 offset = myCurrentBank << 11;
 
     // Setup the page access methods for the current bank
-    System::PageAccess access(this, System::PA_READ);
+    System::PageAccess access(this, System::PageAccessType::READ);
 
     // Map ROM image into the system
     for(uInt16 addr = 0x1000; addr < 0x1800; addr += System::PAGE_SIZE)
@@ -159,9 +167,10 @@ bool Cartridge3E::bank(uInt16 bank)
     uInt32 offset = bank << 10;
 
     // Setup the page access methods for the current bank
-    System::PageAccess access(this, System::PA_READ);
+    System::PageAccess access(this, System::PageAccessType::READ);
 
     // Map read-port RAM image into the system
+    // Writes are mapped to poke(), to check for write to the read port
     for(uInt16 addr = 0x1000; addr < 0x1400; addr += System::PAGE_SIZE)
     {
       access.directPeekBase = &myRAM[offset + (addr & 0x03FF)];
@@ -170,11 +179,10 @@ bool Cartridge3E::bank(uInt16 bank)
     }
 
     access.directPeekBase = nullptr;
-    access.type = System::PA_WRITE;
+    access.type = System::PageAccessType::WRITE;
 
     // Map write-port RAM image into the system
-    // Map access to this class, since we need to inspect all accesses to
-    // check if RWP happens
+    // Reads are mapped to peek(), to check for read from write port
     for(uInt16 addr = 0x1400; addr < 0x1800; addr += System::PAGE_SIZE)
     {
       access.codeAccessBase = &myCodeAccessBase[mySize + offset + (addr & 0x03FF)];
@@ -185,9 +193,12 @@ bool Cartridge3E::bank(uInt16 bank)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt16 Cartridge3E::getBank() const
+uInt16 Cartridge3E::getBank(uInt16 address) const
 {
-  return myCurrentBank;
+  if (address & 0x800)
+    return 255; // 256 - 1 // 2K slices, fixed bank
+  else
+    return myCurrentBank;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -220,7 +231,7 @@ bool Cartridge3E::patch(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* Cartridge3E::getImage(uInt32& size) const
+const uInt8* Cartridge3E::getImage(size_t& size) const
 {
   size = mySize;
   return myImage.get();
@@ -232,7 +243,7 @@ bool Cartridge3E::save(Serializer& out) const
   try
   {
     out.putShort(myCurrentBank);
-    out.putByteArray(myRAM, 32768);
+    out.putByteArray(myRAM.data(), myRAM.size());
   }
   catch(...)
   {
@@ -249,7 +260,7 @@ bool Cartridge3E::load(Serializer& in)
   try
   {
     myCurrentBank = in.getShort();
-    in.getByteArray(myRAM, 32768);
+    in.getByteArray(myRAM.data(), myRAM.size());
   }
   catch(...)
   {
