@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -20,7 +20,7 @@
 #include "Cart3EPlus.hxx"
 
 //  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Cartridge3EPlus::Cartridge3EPlus(const BytePtr& image, uInt32 size,
+Cartridge3EPlus::Cartridge3EPlus(const ByteBuffer& image, size_t size,
                                  const string& md5, const Settings& settings)
   : Cartridge(settings, md5),
     mySize(size)
@@ -29,14 +29,14 @@ Cartridge3EPlus::Cartridge3EPlus(const BytePtr& image, uInt32 size,
   myImage = make_unique<uInt8[]>(mySize);
 
   // Copy the ROM image into my buffer
-  memcpy(myImage.get(), image.get(), mySize);
-  createCodeAccessBase(mySize + RAM_TOTAL_SIZE);
+  std::copy_n(image.get(), mySize, myImage.get());
+  createCodeAccessBase(mySize + myRAM.size());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge3EPlus::reset()
 {
-  initializeRAM(myRAM, RAM_TOTAL_SIZE);
+  initializeRAM(myRAM.data(), myRAM.size());
 
   // Remember startup bank (0 per spec, rather than last per 3E scheme).
   // Set this to go to 3rd 1K Bank.
@@ -44,8 +44,8 @@ void Cartridge3EPlus::reset()
 
   // Initialise bank values for all ROM/RAM access
   // This is used to reverse-lookup from address to bank location
-  for(uInt32 b = 0; b < 8; ++b)
-    bankInUse[b] = BANK_UNDEFINED;        // bank is undefined and inaccessible!
+  for(auto& b: bankInUse)
+    b = BANK_UNDEFINED;     // bank is undefined and inaccessible!
 
   initializeBankState();
 
@@ -59,7 +59,7 @@ void Cartridge3EPlus::install(System& system)
 {
   mySystem = &system;
 
-  System::PageAccess access(this, System::PA_READWRITE);
+  System::PageAccess access(this, System::PageAccessType::READWRITE);
 
   // The hotspots are in TIA address space, so we claim it here
   for(uInt16 addr = 0x00; addr < 0x40; addr += System::PAGE_SIZE)
@@ -67,8 +67,8 @@ void Cartridge3EPlus::install(System& system)
 
   // Initialise bank values for all ROM/RAM access
   // This is used to reverse-lookup from address to bank location
-  for(uInt32 b = 0; b < 8; ++b)
-    bankInUse[b] = BANK_UNDEFINED;        // bank is undefined and inaccessible!
+  for(auto& b: bankInUse)
+    b = BANK_UNDEFINED;     // bank is undefined and inaccessible!
 
   initializeBankState();
 
@@ -78,6 +78,18 @@ void Cartridge3EPlus::install(System& system)
   bankROM((3 << BANK_BITS) | 0);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt16 Cartridge3EPlus::getBank(uInt16 address) const
+{
+  return bankInUse[(address & 0xFFF) >> 10]; // 1K slices
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt16 Cartridge3EPlus::bankCount() const
+{
+  return uInt16(mySize >> 10); // 1K slices
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 Cartridge3EPlus::peek(uInt16 address)
 {
@@ -167,17 +179,17 @@ void Cartridge3EPlus::bankRAMSlot(uInt16 bank)
 //cerr << "raw bank=" << std::dec << currentBank << endl
 //     << "startCurrentBank=$" << std::hex << startCurrentBank << endl;
   // Setup the page access methods for the current bank
-  System::PageAccess access(this, System::PA_READ);
+  System::PageAccess access(this, System::PageAccessType::READ);
 
   if(upper)    // We're mapping the write port
   {
     bankInUse[bankNumber * 2 + 1] = Int16(bank);
-    access.type = System::PA_WRITE;
+    access.type = System::PageAccessType::WRITE;
   }
   else         // We're mapping the read port
   {
     bankInUse[bankNumber * 2] = Int16(bank);
-    access.type = System::PA_READ;
+    access.type = System::PageAccessType::READ;
   }
 
   uInt16 start = 0x1000 + (bankNumber << (RAM_BANK_TO_POWER+1)) + (upper ? RAM_WRITE_OFFSET : 0);
@@ -222,7 +234,7 @@ void Cartridge3EPlus::bankROMSlot(uInt16 bank)
   uInt32 startCurrentBank = currentBank << ROM_BANK_TO_POWER;     // Effectively *1K
 
   // Setup the page access methods for the current bank
-  System::PageAccess access(this, System::PA_READ);
+  System::PageAccess access(this, System::PageAccessType::READ);
 
   uInt16 start = 0x1000 + (bankNumber << ROM_BANK_TO_POWER) + (upper ? ROM_BANK_SIZE / 2 : 0);
   uInt16 end = start + ROM_BANK_SIZE / 2 - 1;
@@ -244,7 +256,7 @@ void Cartridge3EPlus::initializeBankState()
     if(bankInUse[b] == BANK_UNDEFINED)
     {
       // All accesses point to peek/poke above
-      System::PageAccess access(this, System::PA_READ);
+      System::PageAccess access(this, System::PageAccessType::READ);
       uInt16 start = 0x1000 + (b << RAM_BANK_TO_POWER);
       uInt16 end = start + RAM_BANK_SIZE - 1;
       for(uInt16 addr = start; addr <= end; addr += System::PAGE_SIZE)
@@ -266,7 +278,7 @@ bool Cartridge3EPlus::patch(uInt16 address, uInt8 value)
   myBankChanged = true;
 
   uInt32 bankNumber = (address >> RAM_BANK_TO_POWER) & 7;   // now 512 byte bank # (ie: 0-7)
-  Int16 whichBankIsThere = bankInUse[bankNumber];           // ROM or RAM bank reference
+  uInt16 whichBankIsThere = bankInUse[bankNumber];           // ROM or RAM bank reference
 
   if (whichBankIsThere == BANK_UNDEFINED) {
 
@@ -295,7 +307,7 @@ bool Cartridge3EPlus::patch(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* Cartridge3EPlus::getImage(uInt32& size) const
+const uInt8* Cartridge3EPlus::getImage(size_t& size) const
 {
   size = mySize;
   return myImage.get();
@@ -306,8 +318,8 @@ bool Cartridge3EPlus::save(Serializer& out) const
 {
   try
   {
-    out.putShortArray(bankInUse, 8);
-    out.putByteArray(myRAM, RAM_TOTAL_SIZE);
+    out.putShortArray(bankInUse.data(), bankInUse.size());
+    out.putByteArray(myRAM.data(), myRAM.size());
   }
   catch (...)
   {
@@ -322,8 +334,8 @@ bool Cartridge3EPlus::load(Serializer& in)
 {
   try
   {
-    in.getShortArray(bankInUse, 8);
-    in.getByteArray(myRAM, RAM_TOTAL_SIZE);
+    in.getShortArray(bankInUse.data(), bankInUse.size());
+    in.getByteArray(myRAM.data(), myRAM.size());
   }
   catch (...)
   {
