@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -19,19 +19,19 @@
 #include "Cart4KSC.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Cartridge4KSC::Cartridge4KSC(const BytePtr& image, uInt32 size,
+Cartridge4KSC::Cartridge4KSC(const ByteBuffer& image, size_t size,
                              const string& md5, const Settings& settings)
   : Cartridge(settings, md5)
 {
   // Copy the ROM image into my buffer
-  memcpy(myImage, image.get(), std::min(4096u, size));
-  createCodeAccessBase(4096);
+  std::copy_n(image.get(), std::min(myImage.size(), size), myImage.begin());
+  createCodeAccessBase(myImage.size());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge4KSC::reset()
 {
-  initializeRAM(myRAM, 128);
+  initializeRAM(myRAM.data(), myRAM.size());
 
   myBankChanged = true;
 }
@@ -41,12 +41,12 @@ void Cartridge4KSC::install(System& system)
 {
   mySystem = &system;
 
-  System::PageAccess access(this, System::PA_READ);
+  System::PageAccess access(this, System::PageAccessType::READ);
 
   // Set the page accessing method for the RAM writing pages
   // Map access to this class, since we need to inspect all accesses to
   // check if RWP happens
-  access.type = System::PA_WRITE;
+  access.type = System::PageAccessType::WRITE;
   for(uInt16 addr = 0x1000; addr < 0x1080; addr += System::PAGE_SIZE)
   {
     access.codeAccessBase = &myCodeAccessBase[addr & 0x007F];
@@ -54,7 +54,7 @@ void Cartridge4KSC::install(System& system)
   }
 
   // Set the page accessing method for the RAM reading pages
-  access.type = System::PA_READ;
+  access.type = System::PageAccessType::READ;
   for(uInt16 addr = 0x1080; addr < 0x1100; addr += System::PAGE_SIZE)
   {
     access.directPeekBase = &myRAM[addr & 0x007F];
@@ -83,8 +83,20 @@ uInt8 Cartridge4KSC::peek(uInt16 address)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge4KSC::poke(uInt16 address, uInt8 value)
 {
-  pokeRAM(myRAM[address & 0x007F], address, value);
-  return true;
+  if (!(address & 0x080))
+  {
+    pokeRAM(myRAM[address & 0x007F], address, value);
+    return true;
+  }
+  else
+  {
+    // Writing to the read port should be ignored, but trigger a break if option enabled
+    uInt8 dummy;
+
+    pokeRAM(dummy, address, value);
+    myRamWriteAccess = address;
+    return false;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -106,10 +118,10 @@ bool Cartridge4KSC::patch(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* Cartridge4KSC::getImage(uInt32& size) const
+const uInt8* Cartridge4KSC::getImage(size_t& size) const
 {
-  size = 4096;
-  return myImage;
+  size = myImage.size();
+  return myImage.data();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -117,7 +129,7 @@ bool Cartridge4KSC::save(Serializer& out) const
 {
   try
   {
-    out.putByteArray(myRAM, 128);
+    out.putByteArray(myRAM.data(), myRAM.size());
   }
   catch(...)
   {
@@ -133,7 +145,7 @@ bool Cartridge4KSC::load(Serializer& in)
 {
   try
   {
-    in.getByteArray(myRAM, 128);
+    in.getByteArray(myRAM.data(), myRAM.size());
   }
   catch(...)
   {
