@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -57,35 +57,34 @@ namespace {
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeCDF::CartridgeCDF(const BytePtr& image, uInt32 size,
+CartridgeCDF::CartridgeCDF(const ByteBuffer& image, size_t size,
                            const string& md5, const Settings& settings)
-  : Cartridge(settings, md5),
-    myAudioCycles(0),
-    myARMCycles(0),
-    myFractionalClocks(0.0)
+  : Cartridge(settings, md5)
 {
   // Copy the ROM image into my buffer
-  memcpy(myImage, image.get(), std::min(32768u, size));
+  std::copy_n(image.get(), std::min(myImage.size(), size), myImage.begin());
 
   // even though the ROM is 32K, only 28K is accessible to the 6507
-  createCodeAccessBase(4096 * 7);
+  createCodeAccessBase(28_KB);
 
   // Pointer to the program ROM (28K @ 0 byte offset)
   // which starts after the 2K CDF Driver and 2K C Code
-  myProgramImage = myImage + 4096;
+  myProgramImage = myImage.data() + 4_KB;
 
   // Pointer to CDF driver in RAM
-  myBusDriverImage = myCDFRAM;
+  myBusDriverImage = myCDFRAM.data();
 
   // Pointer to the display RAM
-  myDisplayImage = myCDFRAM + DSRAM;
+  myDisplayImage = myCDFRAM.data() + DSRAM;
 
   setupVersion();
 
   // Create Thumbulator ARM emulator
   bool devSettings = settings.getBool("dev.settings");
   myThumbEmulator = make_unique<Thumbulator>(
-    reinterpret_cast<uInt16*>(myImage), reinterpret_cast<uInt16*>(myCDFRAM), 32768,
+    reinterpret_cast<uInt16*>(myImage.data()),
+    reinterpret_cast<uInt16*>(myCDFRAM.data()),
+    static_cast<uInt32>(myImage.size()),
     devSettings ? settings.getBool("dev.thumb.trapfatal") : false, thumulatorConfiguration(myCDFSubtype), this);
 
   setInitialState();
@@ -94,7 +93,7 @@ CartridgeCDF::CartridgeCDF(const BytePtr& image, uInt32 size,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeCDF::reset()
 {
-  initializeRAM(myCDFRAM+2048, 8192-2048);
+  initializeRAM(myCDFRAM.data()+2_KB, myCDFRAM.size()-2_KB);
 
   // CDF always starts in bank 6
   initializeStartBank(6);
@@ -112,17 +111,17 @@ void CartridgeCDF::reset()
 void CartridgeCDF::setInitialState()
 {
   // Copy initial CDF driver to Harmony RAM
-  memcpy(myBusDriverImage, myImage, 0x0800);
+  std::copy_n(myImage.begin(), 2_KB, myBusDriverImage);
 
-  for (int i=0; i < 3; ++i)
-    myMusicWaveformSize[i] = 27;
+  myMusicWaveformSize.fill(27);
 
   // Assuming mode starts out with Fast Fetch off and 3-Voice music,
   // need to confirm with Chris
   myMode = 0xFF;
 
   myBankOffset = myLDAimmediateOperandAddress = myJMPoperandAddress = 0;
-  myFastJumpActive = 0;
+  myFastJumpActive = myFastJumpStream = 0;
+
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -137,7 +136,7 @@ void CartridgeCDF::install(System& system)
   mySystem = &system;
 
   // Map all of the accesses to call peek and poke
-  System::PageAccess access(this, System::PA_READ);
+  System::PageAccess access(this, System::PageAccessType::READ);
   for(uInt16 addr = 0x1000; addr < 0x1040; addr += System::PAGE_SIZE)
     mySystem->setPageAccess(addr, access);
 
@@ -185,6 +184,8 @@ inline void CartridgeCDF::callFunction(uInt8 value)
         }
       }
       break;
+    default:
+      break;
   }
 }
 
@@ -409,7 +410,7 @@ bool CartridgeCDF::bank(uInt16 bank)
   myBankOffset = bank << 12;
 
   // Setup the page access methods for the current bank
-  System::PageAccess access(this, System::PA_READ);
+  System::PageAccess access(this, System::PageAccessType::READ);
 
   // Map Program ROM image into the system
   for(uInt16 addr = 0x1040; addr < 0x2000; addr += System::PAGE_SIZE)
@@ -421,7 +422,7 @@ bool CartridgeCDF::bank(uInt16 bank)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt16 CartridgeCDF::getBank() const
+uInt16 CartridgeCDF::getBank(uInt16) const
 {
   return myBankOffset >> 12;
 }
@@ -448,14 +449,13 @@ bool CartridgeCDF::patch(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* CartridgeCDF::getImage(uInt32& size) const
+const uInt8* CartridgeCDF::getImage(size_t& size) const
 {
-  size = 32768;
-  return myImage;
+  size = myImage.size();
+  return myImage.data();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
 uInt32 CartridgeCDF::thumbCallback(uInt8 function, uInt32 value1, uInt32 value2)
 {
   switch (function)
@@ -479,12 +479,14 @@ uInt32 CartridgeCDF::thumbCallback(uInt8 function, uInt32 value1, uInt32 value2)
     case 3:
       myMusicWaveformSize[value1] = value2;
       break;
+
+    default:
+      break;
   }
 
   return 0;
 }
 
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeCDF::save(Serializer& out) const
 {
@@ -504,12 +506,12 @@ bool CartridgeCDF::save(Serializer& out) const
     out.putShort(myJMPoperandAddress);
 
     // Harmony RAM
-    out.putByteArray(myCDFRAM, 8192);
+    out.putByteArray(myCDFRAM.data(), myCDFRAM.size());
 
     // Audio info
-    out.putIntArray(myMusicCounters, 3);
-    out.putIntArray(myMusicFrequencies, 3);
-    out.putByteArray(myMusicWaveformSize, 3);
+    out.putIntArray(myMusicCounters.data(), myMusicCounters.size());
+    out.putIntArray(myMusicFrequencies.data(), myMusicFrequencies.size());
+    out.putByteArray(myMusicWaveformSize.data(), myMusicWaveformSize.size());
 
     // Save cycles and clocks
     out.putLong(myAudioCycles);
@@ -544,12 +546,12 @@ bool CartridgeCDF::load(Serializer& in)
     myJMPoperandAddress = in.getShort();
 
     // Harmony RAM
-    in.getByteArray(myCDFRAM, 8192);
+    in.getByteArray(myCDFRAM.data(), myCDFRAM.size());
 
     // Audio info
-    in.getIntArray(myMusicCounters, 3);
-    in.getIntArray(myMusicFrequencies, 3);
-    in.getByteArray(myMusicWaveformSize, 3);
+    in.getIntArray(myMusicCounters.data(), myMusicCounters.size());
+    in.getIntArray(myMusicFrequencies.data(), myMusicFrequencies.size());
+    in.getByteArray(myMusicWaveformSize.data(), myMusicWaveformSize.size());
 
     // Get cycles and clocks
     myAudioCycles = in.getLong();
@@ -604,13 +606,12 @@ uInt32 CartridgeCDF::getDatastreamIncrement(uInt8 index) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt32 CartridgeCDF::getWaveform(uInt8 index) const
 {
-  uInt32 result;
   uInt16 address = myWaveformBase + index * 4;
 
-  result = myCDFRAM[address + 0]        +  // low byte
-          (myCDFRAM[address + 1] << 8)  +
-          (myCDFRAM[address + 2] << 16) +
-          (myCDFRAM[address + 3] << 24);   // high byte
+  uInt32 result = myCDFRAM[address + 0]        +  // low byte
+                 (myCDFRAM[address + 1] << 8)  +
+                 (myCDFRAM[address + 2] << 16) +
+                 (myCDFRAM[address + 3] << 24);   // high byte
 
   result -= (0x40000000 + DSRAM);
 
@@ -623,13 +624,12 @@ uInt32 CartridgeCDF::getWaveform(uInt8 index) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt32 CartridgeCDF::getSample()
 {
-  uInt32 result;
   uInt16 address = myWaveformBase;
 
-  result = myCDFRAM[address + 0]        +  // low byte
-          (myCDFRAM[address + 1] << 8)  +
-          (myCDFRAM[address + 2] << 16) +
-          (myCDFRAM[address + 3] << 24);   // high byte
+  uInt32 result = myCDFRAM[address + 0]        +  // low byte
+                 (myCDFRAM[address + 1] << 8)  +
+                 (myCDFRAM[address + 2] << 16) +
+                 (myCDFRAM[address + 3] << 24);   // high byte
 
   return result;
 }
