@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -18,48 +18,44 @@
 #include "OSystem.hxx"
 #include "Serializer.hxx"
 #include "System.hxx"
+#include "TimerManager.hxx"
 #include "CartCTY.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeCTY::CartridgeCTY(const BytePtr& image, uInt32 size,
-                           const string& md5, const OSystem& osystem)
-  : Cartridge(osystem.settings(), md5),
-    myOSystem(osystem),
-    myOperationType(0),
-    myTunePosition(0),
-    myLDAimmediate(false),
-    myRandomNumber(0x2B435044),
-    myRamAccessTimeout(0),
-    myAudioCycles(0),
-    myFractionalClocks(0.0),
-    myBankOffset(0)
+CartridgeCTY::CartridgeCTY(const ByteBuffer& image, size_t size,
+                           const string& md5, const Settings& settings)
+  : Cartridge(settings, md5)
 {
   // Copy the ROM image into my buffer
-  memcpy(myImage, image.get(), std::min(32768u, size));
-  createCodeAccessBase(32768);
+  std::copy_n(image.get(), std::min(myImage.size(), size), myImage.begin());
+  createCodeAccessBase(myImage.size());
 
   // Default to no tune data in case user is utilizing an old ROM
-  memset(myTuneData, 0, 28*1024);
+  myTuneData.fill(0);
 
   // Extract tune data if it exists
-  if(size > 32768u)
-    memcpy(myTuneData, image.get() + 32768u, size - 32768u);
+  if(size > myImage.size())
+    std::copy_n(image.get() + myImage.size(), size - myImage.size(), myTuneData.begin());
 
   // Point to the first tune
-  myFrequencyImage = myTuneData;
+  myFrequencyImage = myTuneData.data();
 
-  for(uInt8 i = 0; i < 3; ++i)
-    myMusicCounters[i] = myMusicFrequencies[i] = 0;
+  myMusicCounters.fill(0);
+  myMusicFrequencies.fill(0);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeCTY::reset()
 {
-  initializeRAM(myRAM, 64);
+  initializeRAM(myRAM.data(), myRAM.size());
   initializeStartBank(1);
 
   myRAM[0] = myRAM[1] = myRAM[2] = myRAM[3] = 0xFF;
 
+  myLDAimmediate = false;
+  myRandomNumber = 0x2B435044;
+  myRamAccessTimeout = 0;
+
   myAudioCycles = 0;
   myFractionalClocks = 0.0;
 
@@ -73,7 +69,7 @@ void CartridgeCTY::install(System& system)
   mySystem = &system;
 
   // Map all RAM accesses to call peek and poke
-  System::PageAccess access(this, System::PA_READ);
+  System::PageAccess access(this, System::PageAccessType::READ);
   for(uInt16 addr = 0x1000; addr < 0x1080; addr += System::PAGE_SIZE)
     mySystem->setPageAccess(addr, access);
 
@@ -241,7 +237,7 @@ bool CartridgeCTY::bank(uInt16 bank)
   myBankOffset = bank << 12;
 
   // Setup the page access methods for the current bank
-  System::PageAccess access(this, System::PA_READ);
+  System::PageAccess access(this, System::PageAccessType::READ);
   for(uInt16 addr = 0x1080; addr < 0x2000; addr += System::PAGE_SIZE)
   {
     access.codeAccessBase = &myCodeAccessBase[myBankOffset + (addr & 0x0FFF)];
@@ -251,7 +247,7 @@ bool CartridgeCTY::bank(uInt16 bank)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt16 CartridgeCTY::getBank() const
+uInt16 CartridgeCTY::getBank(uInt16) const
 {
   return myBankOffset >> 12;
 }
@@ -281,10 +277,10 @@ bool CartridgeCTY::patch(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* CartridgeCTY::getImage(uInt32& size) const
+const uInt8* CartridgeCTY::getImage(size_t& size) const
 {
-  size = 32768;
-  return myImage;
+  size = myImage.size();
+  return myImage.data();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -293,7 +289,7 @@ bool CartridgeCTY::save(Serializer& out) const
   try
   {
     out.putShort(getBank());
-    out.putByteArray(myRAM, 64);
+    out.putByteArray(myRAM.data(), myRAM.size());
 
     out.putByte(myOperationType);
     out.putShort(myTunePosition);
@@ -301,9 +297,9 @@ bool CartridgeCTY::save(Serializer& out) const
     out.putInt(myRandomNumber);
     out.putLong(myAudioCycles);
     out.putDouble(myFractionalClocks);
-    out.putIntArray(myMusicCounters, 3);
-    out.putIntArray(myMusicFrequencies, 3);
-    out.putLong(myFrequencyImage - myTuneData);
+    out.putIntArray(myMusicCounters.data(), myMusicCounters.size());
+    out.putIntArray(myMusicFrequencies.data(), myMusicFrequencies.size());
+    out.putLong(myFrequencyImage - myTuneData.data()); // FIXME - storing pointer diff!
   }
   catch(...)
   {
@@ -321,7 +317,7 @@ bool CartridgeCTY::load(Serializer& in)
   {
     // Remember what bank we were in
     bank(in.getShort());
-    in.getByteArray(myRAM, 64);
+    in.getByteArray(myRAM.data(), myRAM.size());
 
     myOperationType = in.getByte();
     myTunePosition = in.getShort();
@@ -329,9 +325,9 @@ bool CartridgeCTY::load(Serializer& in)
     myRandomNumber = in.getInt();
     myAudioCycles = in.getLong();
     myFractionalClocks = in.getDouble();
-    in.getIntArray(myMusicCounters, 3);
-    in.getIntArray(myMusicFrequencies, 3);
-    myFrequencyImage = myTuneData + in.getLong();
+    in.getIntArray(myMusicCounters.data(), myMusicCounters.size());
+    in.getIntArray(myMusicFrequencies.data(), myMusicFrequencies.size());
+    myFrequencyImage = myTuneData.data() + in.getLong();
   }
   catch(...)
   {
@@ -342,9 +338,9 @@ bool CartridgeCTY::load(Serializer& in)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeCTY::setRomName(const string& name)
+void CartridgeCTY::setNVRamFile(const string& nvramdir, const string& romfile)
 {
-  myEEPROMFile = myOSystem.nvramDir() + name + "_eeprom.dat";
+  myEEPROMFile = nvramdir + romfile + "_eeprom.dat";
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -381,7 +377,7 @@ uInt8 CartridgeCTY::ramReadWrite()
         if(index < 7)
         {
           // Add 0.5 s delay for read
-          myRamAccessTimeout = myOSystem.getTicks() + 500000;
+          myRamAccessTimeout = TimerManager::getTicks() + 500000;
           loadTune(index);
         }
         break;
@@ -389,7 +385,7 @@ uInt8 CartridgeCTY::ramReadWrite()
         if(index < 4)
         {
           // Add 0.5 s delay for read
-          myRamAccessTimeout = myOSystem.getTicks() + 500000;
+          myRamAccessTimeout = TimerManager::getTicks() + 500000;
           loadScore(index);
         }
         break;
@@ -397,13 +393,13 @@ uInt8 CartridgeCTY::ramReadWrite()
         if(index < 4)
         {
           // Add 1 s delay for write
-          myRamAccessTimeout = myOSystem.getTicks() + 1000000;
+          myRamAccessTimeout = TimerManager::getTicks() + 1000000;
           saveScore(index);
         }
         break;
       case 4:  // Wipe all score tables
         // Add 1 s delay for write
-        myRamAccessTimeout = myOSystem.getTicks() + 1000000;
+        myRamAccessTimeout = TimerManager::getTicks() + 1000000;
         wipeAllScores();
         break;
     }
@@ -413,7 +409,7 @@ uInt8 CartridgeCTY::ramReadWrite()
   else
   {
     // Have we reached the timeout value yet?
-    if(myOSystem.getTicks() >= myRamAccessTimeout)
+    if(TimerManager::getTicks() >= myRamAccessTimeout)
     {
       myRamAccessTimeout = 0;  // Turn off timer
       myRAM[0] = 0;            // Successful operation
@@ -433,7 +429,7 @@ void CartridgeCTY::loadTune(uInt8 index)
   // Each tune is offset by 4096 bytes
   // Instead of copying non-modifiable data around (as would happen on the
   // Harmony), we simply point to the appropriate tune
-  myFrequencyImage = myTuneData + (index << 12);
+  myFrequencyImage = myTuneData.data() + (index << 12);
 
   // Reset to beginning of tune
   myTunePosition = 0;
@@ -494,20 +490,21 @@ void CartridgeCTY::updateTune()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeCTY::loadScore(uInt8 index)
 {
-  Serializer serializer(myEEPROMFile, true);
+  Serializer serializer(myEEPROMFile, Serializer::Mode::ReadOnly);
   if(serializer)
   {
-    uInt8 scoreRAM[256];
+    std::array<uInt8, 256> scoreRAM;
     try
     {
-      serializer.getByteArray(scoreRAM, 256);
+      serializer.getByteArray(scoreRAM.data(), scoreRAM.size());
     }
     catch(...)
     {
-      memset(scoreRAM, 0, 256);
+      scoreRAM.fill(0);
     }
+
     // Grab 60B slice @ given index (first 4 bytes are ignored)
-    memcpy(myRAM+4, scoreRAM + (index << 6) + 4, 60);
+    std::copy_n(scoreRAM.begin() + (index << 6) + 4, 60, myRAM.begin() + 4);
   }
 }
 
@@ -518,24 +515,24 @@ void CartridgeCTY::saveScore(uInt8 index)
   if(serializer)
   {
     // Load score RAM
-    uInt8 scoreRAM[256];
+    std::array<uInt8, 256> scoreRAM;
     try
     {
-      serializer.getByteArray(scoreRAM, 256);
+      serializer.getByteArray(scoreRAM.data(), scoreRAM.size());
     }
     catch(...)
     {
-      memset(scoreRAM, 0, 256);
+      scoreRAM.fill(0);
     }
 
     // Add 60B RAM to score table @ given index (first 4 bytes are ignored)
-    memcpy(scoreRAM + (index << 6) + 4, myRAM+4, 60);
+    std::copy_n(myRAM.begin() + 4, 60, scoreRAM.begin() + (index << 6) + 4);
 
     // Save score RAM
     serializer.rewind();
     try
     {
-      serializer.putByteArray(scoreRAM, 256);
+      serializer.putByteArray(scoreRAM.data(), scoreRAM.size());
     }
     catch(...)
     {
@@ -552,11 +549,10 @@ void CartridgeCTY::wipeAllScores()
   if(serializer)
   {
     // Erase score RAM
-    uInt8 scoreRAM[256];
-    memset(scoreRAM, 0, 256);
+    std::array<uInt8, 256> scoreRAM = {};
     try
     {
-      serializer.putByteArray(scoreRAM, 256);
+      serializer.putByteArray(scoreRAM.data(), scoreRAM.size());
     }
     catch(...)
     {
@@ -585,7 +581,7 @@ inline void CartridgeCTY::updateMusicModeDataFetchers()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt32 CartridgeCTY::ourFrequencyTable[63] =
+const std::array<uInt32, 63> CartridgeCTY::ourFrequencyTable =
 {
   // this should really be referenced from within the ROM, but its part of
   // the Harmony/Melody CTY Driver, which does not appear to be in the ROM.
