@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -20,24 +20,23 @@
 #include "CartCVPlus.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeCVPlus::CartridgeCVPlus(const BytePtr& image, uInt32 size,
+CartridgeCVPlus::CartridgeCVPlus(const ByteBuffer& image, size_t size,
                                  const string& md5, const Settings& settings)
   : Cartridge(settings, md5),
-    mySize(size),
-    myCurrentBank(0)
+    mySize(size)
 {
   // Allocate array for the ROM image
   myImage = make_unique<uInt8[]>(mySize);
 
   // Copy the ROM image into my buffer
-  memcpy(myImage.get(), image.get(), mySize);
-  createCodeAccessBase(mySize + 1024);
+  std::copy_n(image.get(), mySize, myImage.get());
+  createCodeAccessBase(mySize + myRAM.size());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeCVPlus::reset()
 {
-  initializeRAM(myRAM, 1024);
+  initializeRAM(myRAM.data(), myRAM.size());
   initializeStartBank(0);
 
   // We'll map the startup bank into the first segment upon reset
@@ -49,7 +48,7 @@ void CartridgeCVPlus::install(System& system)
 {
   mySystem = &system;
 
-  System::PageAccess access(this, System::PA_READWRITE);
+  System::PageAccess access(this, System::PageAccessType::READWRITE);
 
   // The hotspot ($3D) is in TIA address space, so we claim it here
   for(uInt16 addr = 0x00; addr < 0x40; addr += System::PAGE_SIZE)
@@ -60,7 +59,7 @@ void CartridgeCVPlus::install(System& system)
   // check if RWP happens
   access.directPeekBase = access.directPokeBase = nullptr;
   access.codeAccessBase = nullptr;
-  access.type = System::PA_WRITE;
+  access.type = System::PageAccessType::WRITE;
   for(uInt16 addr = 0x1400; addr < 0x1800; addr += System::PAGE_SIZE)
   {
     access.codeAccessBase = &myCodeAccessBase[mySize + (addr & 0x03FF)];
@@ -68,7 +67,7 @@ void CartridgeCVPlus::install(System& system)
   }
 
   // Set the page accessing method for the RAM reading pages
-  access.type = System::PA_READ;
+  access.type = System::PageAccessType::READ;
   for(uInt16 addr = 0x1000; addr < 0x1400; addr += System::PAGE_SIZE)
   {
     access.directPeekBase = &myRAM[addr & 0x03FF];
@@ -130,7 +129,7 @@ bool CartridgeCVPlus::bank(uInt16 bank)
   uInt32 offset = myCurrentBank << 11;
 
   // Setup the page access methods for the current bank
-  System::PageAccess access(this, System::PA_READ);
+  System::PageAccess access(this, System::PageAccessType::READ);
 
   // Map ROM image into the system
   for(uInt16 addr = 0x1800; addr < 0x2000; addr += System::PAGE_SIZE)
@@ -144,7 +143,7 @@ bool CartridgeCVPlus::bank(uInt16 bank)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt16 CartridgeCVPlus::getBank() const
+uInt16 CartridgeCVPlus::getBank(uInt16) const
 {
   return myCurrentBank;
 }
@@ -152,7 +151,7 @@ uInt16 CartridgeCVPlus::getBank() const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt16 CartridgeCVPlus::bankCount() const
 {
-  return mySize >> 11;
+  return uInt16(mySize >> 11);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -175,7 +174,7 @@ bool CartridgeCVPlus::patch(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* CartridgeCVPlus::getImage(uInt32& size) const
+const uInt8* CartridgeCVPlus::getImage(size_t& size) const
 {
   size = mySize;
   return myImage.get();
@@ -187,7 +186,7 @@ bool CartridgeCVPlus::save(Serializer& out) const
   try
   {
     out.putShort(myCurrentBank);
-    out.putByteArray(myRAM, 1024);
+    out.putByteArray(myRAM.data(), myRAM.size());
   }
   catch(...)
   {
@@ -204,7 +203,7 @@ bool CartridgeCVPlus::load(Serializer& in)
   try
   {
     myCurrentBank = in.getShort();
-    in.getByteArray(myRAM, 1024);
+    in.getByteArray(myRAM.data(), myRAM.size());
   }
   catch(...)
   {
