@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -16,39 +16,24 @@
 //============================================================================
 
 #include "System.hxx"
+#include "AudioSettings.hxx"
 #include "CartDPC.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeDPC::CartridgeDPC(const BytePtr& image, uInt32 size,
+CartridgeDPC::CartridgeDPC(const ByteBuffer& image, size_t size,
                            const string& md5, const Settings& settings)
   : Cartridge(settings, md5),
-    mySize(size),
-    myAudioCycles(0),
-    myFractionalClocks(0.0),
-    myBankOffset(0)
+    mySize(size)
 {
   // Make a copy of the entire image
-  memcpy(myImage, image.get(), std::min(size, 8192u + 2048u + 256u));
-  createCodeAccessBase(8192);
+  std::copy_n(image.get(), std::min(myImage.size(), size), myImage.begin());
+  createCodeAccessBase(8_KB);
 
   // Pointer to the program ROM (8K @ 0 byte offset)
-  myProgramImage = myImage;
+  myProgramImage = myImage.data();
 
   // Pointer to the display ROM (2K @ 8K offset)
-  myDisplayImage = myProgramImage + 8192;
-
-  // Initialize the DPC data fetcher registers
-  for(int i = 0; i < 8; ++i)
-  {
-    myTops[i] = myBottoms[i] = myFlags[i] = 0;
-    myCounters[i] = 0;
-  }
-
-  // None of the data fetchers are in music mode
-  myMusicMode[0] = myMusicMode[1] = myMusicMode[2] = false;
-
-  // Initialize the DPC's random number generator register (must be non-zero)
-  myRandomNumber = 1;
+  myDisplayImage = myProgramImage + 8_KB;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -60,6 +45,8 @@ void CartridgeDPC::reset()
   // Upon reset we switch to the startup bank
   initializeStartBank(1);
   bank(startBank());
+
+  myDpcPitch = mySettings.getInt(AudioSettings::SETTING_DPC_PITCH);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -68,7 +55,7 @@ void CartridgeDPC::install(System& system)
   mySystem = &system;
 
   // Set the page accessing method for the DPC reading & writing pages
-  System::PageAccess access(this, System::PA_READWRITE);
+  System::PageAccess access(this, System::PageAccessType::READWRITE);
   for(uInt16 addr = 0x1000; addr < 0x1080; addr += System::PAGE_SIZE)
     mySystem->setPageAccess(addr, access);
 
@@ -81,7 +68,7 @@ inline void CartridgeDPC::clockRandomNumberGenerator()
 {
   // Table for computing the input bit of the random number generator's
   // shift register (it's the NOT of the EOR of four bits)
-  static constexpr uInt8 f[16] = {
+  static constexpr std::array<uInt8, 16> f = {
     1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1
   };
 
@@ -102,7 +89,7 @@ inline void CartridgeDPC::updateMusicModeDataFetchers()
   myAudioCycles = mySystem->cycles();
 
   // Calculate the number of DPC OSC clocks since the last update
-  double clocks = ((20000.0 * cycles) / 1193191.66666667) + myFractionalClocks;
+  double clocks = ((myDpcPitch * cycles) / 1193191.66666667) + myFractionalClocks;
   uInt32 wholeClocks = uInt32(clocks);
   myFractionalClocks = clocks - double(wholeClocks);
 
@@ -183,7 +170,7 @@ uInt8 CartridgeDPC::peek(uInt16 address)
         // No, it's a music read
         else
         {
-          static constexpr uInt8 musicAmplitudes[8] = {
+          static constexpr std::array<uInt8, 8> musicAmplitudes = {
               0x00, 0x04, 0x05, 0x09, 0x06, 0x0a, 0x0b, 0x0f
           };
 
@@ -380,7 +367,7 @@ bool CartridgeDPC::bank(uInt16 bank)
   // Remember what bank we're in
   myBankOffset = bank << 12;
 
-  System::PageAccess access(this, System::PA_READ);
+  System::PageAccess access(this, System::PageAccessType::READ);
 
   // Set the page accessing methods for the hot spots
   for(uInt16 addr = (0x1FF8 & ~System::PAGE_MASK); addr < 0x2000;
@@ -391,7 +378,7 @@ bool CartridgeDPC::bank(uInt16 bank)
   }
 
   // Setup the page access methods for the current bank
-  for(uInt16 addr = 0x1080; addr < (0x1FF8U & ~System::PAGE_MASK);
+  for(uInt16 addr = 0x1080; addr < static_cast<uInt16>(0x1FF8U & ~System::PAGE_MASK);
       addr += System::PAGE_SIZE)
   {
     access.directPeekBase = &myProgramImage[myBankOffset + (addr & 0x0FFF)];
@@ -402,7 +389,7 @@ bool CartridgeDPC::bank(uInt16 bank)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt16 CartridgeDPC::getBank() const
+uInt16 CartridgeDPC::getBank(uInt16) const
 {
   return myBankOffset >> 12;
 }
@@ -429,10 +416,10 @@ bool CartridgeDPC::patch(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* CartridgeDPC::getImage(uInt32& size) const
+const uInt8* CartridgeDPC::getImage(size_t& size) const
 {
   size = mySize;
-  return myImage;
+  return myImage.data();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -444,20 +431,20 @@ bool CartridgeDPC::save(Serializer& out) const
     out.putShort(myBankOffset);
 
     // The top registers for the data fetchers
-    out.putByteArray(myTops, 8);
+    out.putByteArray(myTops.data(), myTops.size());
 
     // The bottom registers for the data fetchers
-    out.putByteArray(myBottoms, 8);
+    out.putByteArray(myBottoms.data(), myBottoms.size());
 
     // The counter registers for the data fetchers
-    out.putShortArray(myCounters, 8);
+    out.putShortArray(myCounters.data(), myCounters.size());
 
     // The flag registers for the data fetchers
-    out.putByteArray(myFlags, 8);
+    out.putByteArray(myFlags.data(), myFlags.size());
 
     // The music mode flags for the data fetchers
-    for(int i = 0; i < 3; ++i)
-      out.putBool(myMusicMode[i]);
+    for(const auto& mode: myMusicMode)
+      out.putBool(mode);
 
     // The random number generator register
     out.putByte(myRandomNumber);
@@ -483,20 +470,20 @@ bool CartridgeDPC::load(Serializer& in)
     myBankOffset = in.getShort();
 
     // The top registers for the data fetchers
-    in.getByteArray(myTops, 8);
+    in.getByteArray(myTops.data(), myTops.size());
 
     // The bottom registers for the data fetchers
-    in.getByteArray(myBottoms, 8);
+    in.getByteArray(myBottoms.data(), myBottoms.size());
 
     // The counter registers for the data fetchers
-    in.getShortArray(myCounters, 8);
+    in.getShortArray(myCounters.data(), myCounters.size());
 
     // The flag registers for the data fetchers
-    in.getByteArray(myFlags, 8);
+    in.getByteArray(myFlags.data(), myFlags.size());
 
     // The music mode flags for the data fetchers
-    for(int i = 0; i < 3; ++i)
-      myMusicMode[i] = in.getBool();
+    for(auto& mode: myMusicMode)
+      mode = in.getBool();
 
     // The random number generator register
     myRandomNumber = in.getByte();
