@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -19,20 +19,19 @@
 #include "CartF4SC.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeF4SC::CartridgeF4SC(const BytePtr& image, uInt32 size,
+CartridgeF4SC::CartridgeF4SC(const ByteBuffer& image, size_t size,
                              const string& md5, const Settings& settings)
-  : Cartridge(settings, md5),
-    myBankOffset(0)
+  : Cartridge(settings, md5)
 {
   // Copy the ROM image into my buffer
-  memcpy(myImage, image.get(), std::min(32768u, size));
-  createCodeAccessBase(32768);
+  std::copy_n(image.get(), std::min(myImage.size(), size), myImage.begin());
+  createCodeAccessBase(myImage.size());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeF4SC::reset()
 {
-  initializeRAM(myRAM, 128);
+  initializeRAM(myRAM.data(), myRAM.size());
   initializeStartBank(0);
 
   // Upon reset we switch to the startup bank
@@ -44,12 +43,12 @@ void CartridgeF4SC::install(System& system)
 {
   mySystem = &system;
 
-  System::PageAccess access(this, System::PA_READ);
+  System::PageAccess access(this, System::PageAccessType::READ);
 
   // Set the page accessing method for the RAM writing pages
   // Map access to this class, since we need to inspect all accesses to
   // check if RWP happens
-  access.type = System::PA_WRITE;
+  access.type = System::PageAccessType::WRITE;
   for(uInt16 addr = 0x1000; addr < 0x1080; addr += System::PAGE_SIZE)
   {
     access.codeAccessBase = &myCodeAccessBase[addr & 0x007F];
@@ -57,7 +56,7 @@ void CartridgeF4SC::install(System& system)
   }
 
   // Set the page accessing method for the RAM reading pages
-  access.type = System::PA_READ;
+  access.type = System::PageAccessType::READ;
   for(uInt16 addr = 0x1080; addr < 0x1100; addr += System::PAGE_SIZE)
   {
     access.directPeekBase = &myRAM[addr & 0x007F];
@@ -98,8 +97,20 @@ bool CartridgeF4SC::poke(uInt16 address, uInt8 value)
     return false;
   }
 
-  pokeRAM(myRAM[address & 0x007F], pokeAddress, value);
-  return true;
+  if(!(address & 0x080))
+  {
+    pokeRAM(myRAM[address & 0x007F], pokeAddress, value);
+    return true;
+  }
+  else
+  {
+    // Writing to the read port should be ignored, but trigger a break if option enabled
+    uInt8 dummy;
+
+    pokeRAM(dummy, pokeAddress, value);
+    myRamWriteAccess = pokeAddress;
+    return false;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -110,7 +121,7 @@ bool CartridgeF4SC::bank(uInt16 bank)
   // Remember what bank we're in
   myBankOffset = bank << 12;
 
-  System::PageAccess access(this, System::PA_READ);
+  System::PageAccess access(this, System::PageAccessType::READ);
 
   // Set the page accessing methods for the hot spots
   for(uInt16 addr = (0x1FF4 & ~System::PAGE_MASK); addr < 0x2000;
@@ -121,7 +132,7 @@ bool CartridgeF4SC::bank(uInt16 bank)
   }
 
   // Setup the page access methods for the current bank
-  for(uInt16 addr = 0x1100; addr < (0x1FF4U & ~System::PAGE_MASK);
+  for(uInt16 addr = 0x1100; addr < static_cast<uInt16>(0x1FF4U & ~System::PAGE_MASK);
       addr += System::PAGE_SIZE)
   {
     access.directPeekBase = &myImage[myBankOffset + (addr & 0x0FFF)];
@@ -132,7 +143,7 @@ bool CartridgeF4SC::bank(uInt16 bank)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt16 CartridgeF4SC::getBank() const
+uInt16 CartridgeF4SC::getBank(uInt16) const
 {
   return myBankOffset >> 12;
 }
@@ -162,10 +173,10 @@ bool CartridgeF4SC::patch(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* CartridgeF4SC::getImage(uInt32& size) const
+const uInt8* CartridgeF4SC::getImage(size_t& size) const
 {
-  size = 32768;
-  return myImage;
+  size = myImage.size();
+  return myImage.data();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -174,7 +185,7 @@ bool CartridgeF4SC::save(Serializer& out) const
   try
   {
     out.putShort(myBankOffset);
-    out.putByteArray(myRAM, 128);
+    out.putByteArray(myRAM.data(), myRAM.size());
   }
   catch(...)
   {
@@ -191,7 +202,7 @@ bool CartridgeF4SC::load(Serializer& in)
   try
   {
     myBankOffset = in.getShort();
-    in.getByteArray(myRAM, 128);
+    in.getByteArray(myRAM.data(), myRAM.size());
   }
   catch(...)
   {
