@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -19,20 +19,19 @@
 #include "CartFA.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeFA::CartridgeFA(const BytePtr& image, uInt32 size,
+CartridgeFA::CartridgeFA(const ByteBuffer& image, size_t size,
                          const string& md5, const Settings& settings)
-  : Cartridge(settings, md5),
-    myBankOffset(0)
+  : Cartridge(settings, md5)
 {
   // Copy the ROM image into my buffer
-  memcpy(myImage, image.get(), std::min(12288u, size));
-  createCodeAccessBase(12288);
+  std::copy_n(image.get(), std::min(myImage.size(), size), myImage.begin());
+  createCodeAccessBase(myImage.size());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeFA::reset()
 {
-  initializeRAM(myRAM, 256);
+  initializeRAM(myRAM.data(), myRAM.size());
   initializeStartBank(2);
 
   // Upon reset we switch to the startup bank
@@ -44,12 +43,12 @@ void CartridgeFA::install(System& system)
 {
   mySystem = &system;
 
-  System::PageAccess access(this, System::PA_READ);
+  System::PageAccess access(this, System::PageAccessType::READ);
 
   // Set the page accessing method for the RAM writing pages
   // Map access to this class, since we need to inspect all accesses to
   // check if RWP happens
-  access.type = System::PA_WRITE;
+  access.type = System::PageAccessType::WRITE;
   for(uInt16 addr = 0x1000; addr < 0x1100; addr += System::PAGE_SIZE)
   {
     access.codeAccessBase = &myCodeAccessBase[addr & 0x00FF];
@@ -57,7 +56,7 @@ void CartridgeFA::install(System& system)
   }
 
   // Set the page accessing method for the RAM reading pages
-  access.type = System::PA_READ;
+  access.type = System::PageAccessType::READ;
   for(uInt16 addr = 0x1100; addr < 0x1200; addr += System::PAGE_SIZE)
   {
     access.directPeekBase = &myRAM[addr & 0x00FF];
@@ -128,8 +127,20 @@ bool CartridgeFA::poke(uInt16 address, uInt8 value)
       break;
   }
 
-  pokeRAM(myRAM[address & 0x00FF], address, value);
-  return true;
+  if (!(address & 0x100))
+  {
+    pokeRAM(myRAM[address & 0x00FF], address, value);
+    return true;
+  }
+  else
+  {
+    // Writing to the read port should be ignored, but trigger a break if option enabled
+    uInt8 dummy;
+
+    pokeRAM(dummy, address, value);
+    myRamWriteAccess = address;
+    return false;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -140,7 +151,7 @@ bool CartridgeFA::bank(uInt16 bank)
   // Remember what bank we're in
   myBankOffset = bank << 12;
 
-  System::PageAccess access(this, System::PA_READ);
+  System::PageAccess access(this, System::PageAccessType::READ);
 
   // Set the page accessing methods for the hot spots
   for(uInt16 addr = (0x1FF8 & ~System::PAGE_MASK); addr < 0x2000;
@@ -151,7 +162,7 @@ bool CartridgeFA::bank(uInt16 bank)
   }
 
   // Setup the page access methods for the current bank
-  for(uInt16 addr = 0x1200; addr < (0x1FF8U & ~System::PAGE_MASK);
+  for(uInt16 addr = 0x1200; addr < static_cast<uInt16>(0x1FF8U & ~System::PAGE_MASK);
       addr += System::PAGE_SIZE)
   {
     access.directPeekBase = &myImage[myBankOffset + (addr & 0x0FFF)];
@@ -162,7 +173,7 @@ bool CartridgeFA::bank(uInt16 bank)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt16 CartridgeFA::getBank() const
+uInt16 CartridgeFA::getBank(uInt16) const
 {
   return myBankOffset >> 12;
 }
@@ -192,10 +203,10 @@ bool CartridgeFA::patch(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* CartridgeFA::getImage(uInt32& size) const
+const uInt8* CartridgeFA::getImage(size_t& size) const
 {
-  size = 12288;
-  return myImage;
+  size = myImage.size();
+  return myImage.data();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -204,7 +215,7 @@ bool CartridgeFA::save(Serializer& out) const
   try
   {
     out.putShort(myBankOffset);
-    out.putByteArray(myRAM, 256);
+    out.putByteArray(myRAM.data(), myRAM.size());
   }
   catch(...)
   {
@@ -221,7 +232,7 @@ bool CartridgeFA::load(Serializer& in)
   try
   {
     myBankOffset = in.getShort();
-    in.getByteArray(myRAM, 256);
+    in.getByteArray(myRAM.data(), myRAM.size());
   }
   catch(...)
   {
