@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -18,33 +18,30 @@
 #include "OSystem.hxx"
 #include "Serializer.hxx"
 #include "System.hxx"
+#include "TimerManager.hxx"
 #include "CartFA2.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeFA2::CartridgeFA2(const BytePtr& image, uInt32 size,
-                           const string& md5, const OSystem& osystem)
-  : Cartridge(osystem.settings(), md5),
-    myOSystem(osystem),
-    mySize(28 * 1024),
-    myRamAccessTimeout(0),
-    myBankOffset(0)
+CartridgeFA2::CartridgeFA2(const ByteBuffer& image, size_t size,
+                           const string& md5, const Settings& settings)
+  : Cartridge(settings, md5)
 {
   // 29/32K version of FA2 has valid data @ 1K - 29K
   const uInt8* img_ptr = image.get();
-  if(size >= 29 * 1024)
-    img_ptr += 1024;
+  if(size >= 29_KB)
+    img_ptr += 1_KB;
   else if(size < mySize)
     mySize = size;
 
   // Copy the ROM image into my buffer
-  memcpy(myImage, img_ptr, mySize);
+  std::copy_n(img_ptr, mySize, myImage.begin());
   createCodeAccessBase(mySize);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeFA2::reset()
 {
-  initializeRAM(myRAM, 256);
+  initializeRAM(myRAM.data(), myRAM.size());
   initializeStartBank(0);
 
   // Upon reset we switch to the startup bank
@@ -56,12 +53,12 @@ void CartridgeFA2::install(System& system)
 {
   mySystem = &system;
 
-  System::PageAccess access(this, System::PA_READ);
+  System::PageAccess access(this, System::PageAccessType::READ);
 
   // Set the page accessing method for the RAM writing pages
   // Map access to this class, since we need to inspect all accesses to
   // check if RWP happens
-  access.type = System::PA_WRITE;
+  access.type = System::PageAccessType::WRITE;
   for(uInt16 addr = 0x1000; addr < 0x1100; addr += System::PAGE_SIZE)
   {
     access.codeAccessBase = &myCodeAccessBase[addr & 0x00FF];
@@ -70,7 +67,7 @@ void CartridgeFA2::install(System& system)
 
   // Set the page accessing method for the RAM reading pages
   access.directPokeBase = nullptr;
-  access.type = System::PA_READ;
+  access.type = System::PageAccessType::READ;
   for(uInt16 addr = 0x1100; addr < 0x1200; addr += System::PAGE_SIZE)
   {
     access.directPeekBase = &myRAM[addr & 0x00FF];
@@ -93,7 +90,7 @@ uInt8 CartridgeFA2::peek(uInt16 address)
   {
     case 0x0FF4:
       // Load/save RAM to/from Harmony cart flash
-      if(mySize == 28*1024 && !bankLocked())
+      if(mySize == 28_KB && !bankLocked())
         return ramReadWrite();
       break;
 
@@ -130,7 +127,7 @@ uInt8 CartridgeFA2::peek(uInt16 address)
     case 0x0FFB:
       // Set the current bank to the seventh 4k bank
       // This is only available on 28K ROMs
-      if(mySize == 28*1024)  bank(6);
+      if(mySize == 28_KB)  bank(6);
       break;
 
     default:
@@ -151,7 +148,7 @@ bool CartridgeFA2::poke(uInt16 address, uInt8 value)
   {
     case 0x0FF4:
       // Load/save RAM to/from Harmony cart flash
-      if(mySize == 28*1024 && !bankLocked())
+      if(mySize == 28_KB && !bankLocked())
         ramReadWrite();
       return false;
 
@@ -188,15 +185,27 @@ bool CartridgeFA2::poke(uInt16 address, uInt8 value)
     case 0x0FFB:
       // Set the current bank to the seventh 4k bank
       // This is only available on 28K ROMs
-      if(mySize == 28*1024)  bank(6);
+      if(mySize == 28_KB)  bank(6);
       return false;
 
     default:
       break;
   }
 
-  pokeRAM(myRAM[address & 0x00FF], address, value);
-  return true;
+  if(!(address & 0x100))
+  {
+    pokeRAM(myRAM[address & 0x00FF], address, value);
+    return true;
+  }
+  else
+  {
+    // Writing to the read port should be ignored, but trigger a break if option enabled
+    uInt8 dummy;
+
+    pokeRAM(dummy, address, value);
+    myRamWriteAccess = address;
+    return false;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -207,7 +216,7 @@ bool CartridgeFA2::bank(uInt16 bank)
   // Remember what bank we're in
   myBankOffset = bank << 12;
 
-  System::PageAccess access(this, System::PA_READ);
+  System::PageAccess access(this, System::PageAccessType::READ);
 
   // Set the page accessing methods for the hot spots
   for(uInt16 addr = (0x1FF4 & ~System::PAGE_MASK); addr < 0x2000;
@@ -218,7 +227,7 @@ bool CartridgeFA2::bank(uInt16 bank)
   }
 
   // Setup the page access methods for the current bank
-  for(uInt16 addr = 0x1200; addr < (0x1FF4U & ~System::PAGE_MASK);
+  for(uInt16 addr = 0x1200; addr < static_cast<uInt16>(0x1FF4U & ~System::PAGE_MASK);
       addr += System::PAGE_SIZE)
   {
     access.directPeekBase = &myImage[myBankOffset + (addr & 0x0FFF)];
@@ -229,7 +238,7 @@ bool CartridgeFA2::bank(uInt16 bank)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt16 CartridgeFA2::getBank() const
+uInt16 CartridgeFA2::getBank(uInt16) const
 {
   return myBankOffset >> 12;
 }
@@ -237,7 +246,7 @@ uInt16 CartridgeFA2::getBank() const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt16 CartridgeFA2::bankCount() const
 {
-  return (mySize / 4096);
+  return uInt16(mySize / 4_KB);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -259,10 +268,10 @@ bool CartridgeFA2::patch(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* CartridgeFA2::getImage(uInt32& size) const
+const uInt8* CartridgeFA2::getImage(size_t& size) const
 {
   size = mySize;
-  return myImage;
+  return myImage.data();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -271,7 +280,7 @@ bool CartridgeFA2::save(Serializer& out) const
   try
   {
     out.putShort(myBankOffset);
-    out.putByteArray(myRAM, 256);
+    out.putByteArray(myRAM.data(), myRAM.size());
   }
   catch(...)
   {
@@ -288,7 +297,7 @@ bool CartridgeFA2::load(Serializer& in)
   try
   {
     myBankOffset = in.getShort();
-    in.getByteArray(myRAM, 256);
+    in.getByteArray(myRAM.data(), myRAM.size());
   }
   catch(...)
   {
@@ -303,9 +312,9 @@ bool CartridgeFA2::load(Serializer& in)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeFA2::setRomName(const string& name)
+void CartridgeFA2::setNVRamFile(const string& nvramdir, const string& romfile)
 {
-  myFlashFile = myOSystem.nvramDir() + name + "_flash.dat";
+  myFlashFile = nvramdir + romfile + "_flash.dat";
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -333,7 +342,7 @@ uInt8 CartridgeFA2::ramReadWrite()
   if(myRamAccessTimeout == 0)
   {
     // Remember when the first access was made
-    myRamAccessTimeout = myOSystem.getTicks();
+    myRamAccessTimeout = TimerManager::getTicks();
 
     // We go ahead and do the access now, and only return when a sufficient
     // amount of time has passed
@@ -344,11 +353,11 @@ uInt8 CartridgeFA2::ramReadWrite()
       {
         try
         {
-          serializer.getByteArray(myRAM, 256);
+          serializer.getByteArray(myRAM.data(), myRAM.size());
         }
         catch(...)
         {
-          memset(myRAM, 0, 256);
+          myRAM.fill(0);
         }
         myRamAccessTimeout += 500;  // Add 0.5 ms delay for read
       }
@@ -356,7 +365,7 @@ uInt8 CartridgeFA2::ramReadWrite()
       {
         try
         {
-          serializer.putByteArray(myRAM, 256);
+          serializer.putByteArray(myRAM.data(), myRAM.size());
         }
         catch(...)
         {
@@ -372,7 +381,7 @@ uInt8 CartridgeFA2::ramReadWrite()
   else
   {
     // Have we reached the timeout value yet?
-    if(myOSystem.getTicks() >= myRamAccessTimeout)
+    if(TimerManager::getTicks() >= myRamAccessTimeout)
     {
       myRamAccessTimeout = 0;  // Turn off timer
       myRAM[255] = 0;          // Successful operation
@@ -396,9 +405,8 @@ void CartridgeFA2::flash(uInt8 operation)
     {
       try
       {
-        uInt8 buf[256];
-        memset(buf, 0, 256);
-        serializer.putByteArray(buf, 256);
+        std::array<uInt8, 256> buf = {};
+        serializer.putByteArray(buf.data(), buf.size());
       }
       catch(...)
       {
@@ -408,18 +416,18 @@ void CartridgeFA2::flash(uInt8 operation)
     {
       try
       {
-        serializer.getByteArray(myRAM, 256);
+        serializer.getByteArray(myRAM.data(), myRAM.size());
       }
       catch(...)
       {
-        memset(myRAM, 0, 256);
+        myRAM.fill(0);
       }
     }
     else if(operation == 2)  // write
     {
       try
       {
-        serializer.putByteArray(myRAM, 256);
+        serializer.putByteArray(myRAM.data(), myRAM.size());
       }
       catch(...)
       {
