@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -19,24 +19,22 @@
 #include "CartMNetwork.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeMNetwork::CartridgeMNetwork(const BytePtr& image, uInt32 size,
+CartridgeMNetwork::CartridgeMNetwork(const ByteBuffer& image, size_t size,
                                      const string& md5, const Settings& settings)
   : Cartridge(settings, md5),
-    mySize(size),
-    myCurrentRAM(0),
-    myRAMSlice(0)
+    mySize(size)
 {
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeMNetwork::initialize(const BytePtr& image, uInt32 size)
+void CartridgeMNetwork::initialize(const ByteBuffer& image, size_t size)
 {
   // Allocate array for the ROM image
   myImage = make_unique<uInt8[]>(size);
 
   // Copy the ROM image into my buffer
-  memcpy(myImage.get(), image.get(), std::min(romSize(), size));
-  createCodeAccessBase(romSize() + RAM_SIZE);
+  std::copy_n(image.get(), std::min<size_t>(romSize(), size), myImage.get());
+  createCodeAccessBase(romSize() + myRAM.size());
 
   myRAMSlice = bankCount() - 1;
 }
@@ -44,7 +42,7 @@ void CartridgeMNetwork::initialize(const BytePtr& image, uInt32 size)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeMNetwork::reset()
 {
-  initializeRAM(myRAM, RAM_SIZE);
+  initializeRAM(myRAM.data(), myRAM.size());
 
   initializeStartBank(0);
   uInt32 ramBank = randomStartBank() ?
@@ -68,9 +66,9 @@ void CartridgeMNetwork::setAccess(uInt16 addrFrom, uInt16 size,
 
   for(uInt16 addr = addrFrom; addr < addrFrom + size; addr += System::PAGE_SIZE)
   {
-    if(type == System::PA_READ)
+    if(type == System::PageAccessType::READ)
       access.directPeekBase = &directData[directOffset + (addr & addrMask)];
-    else if(type == System::PA_WRITE)  // all RAM writes mapped to ::poke()
+    else if(type == System::PageAccessType::WRITE)  // all RAM writes mapped to ::poke()
       access.directPokeBase = nullptr;
     access.codeAccessBase = &myCodeAccessBase[codeOffset + (addr & addrMask)];
     mySystem->setPageAccess(addr, access);
@@ -82,7 +80,7 @@ void CartridgeMNetwork::install(System& system)
 {
   mySystem = &system;
 
-  System::PageAccess access(this, System::PA_READ);
+  System::PageAccess access(this, System::PageAccessType::READ);
 
   // Set the page accessing methods for the hot spots
   for(uInt16 addr = (0x1FE0 & ~System::PAGE_MASK); addr < 0x2000;
@@ -96,7 +94,7 @@ void CartridgeMNetwork::install(System& system)
 
   // Setup the second segment to always point to the last ROM slice
   setAccess(0x1A00, 0x1FE0U & (~System::PAGE_MASK - 0x1A00),
-            myRAMSlice * BANK_SIZE, myImage.get(), myRAMSlice * BANK_SIZE, System::PA_READ, BANK_SIZE - 1);
+            myRAMSlice * BANK_SIZE, myImage.get(), myRAMSlice * BANK_SIZE, System::PageAccessType::READ, BANK_SIZE - 1);
   myCurrentSlice[1] = myRAMSlice;
 
   // Install some default banks for the RAM and first segment
@@ -121,7 +119,7 @@ uInt8 CartridgeMNetwork::peek(uInt16 address)
   else if((address >= 0x0800) && (address <= 0x08FF))
   {
     // Reading from the 256B write port @ $1800 triggers an unwanted write
-    return peekRAM(myRAM[1024 + (myCurrentRAM << 8) + (address & 0x00FF)], peekAddress);
+    return peekRAM(myRAM[0x0400 + (myCurrentRAM << 8) + (address & 0x00FF)], peekAddress);
   }
   else
     return myImage[(myCurrentSlice[address >> 11] << 11) + (address & (BANK_SIZE - 1))];
@@ -144,7 +142,7 @@ bool CartridgeMNetwork::poke(uInt16 address, uInt8 value)
   }
   else if((address >= 0x0800) && (address <= 0x08FF))
   {
-    pokeRAM(myRAM[1024 + (myCurrentRAM << 8) + (address & 0x00FF)], pokeAddress, value);
+    pokeRAM(myRAM[0x0400 + (myCurrentRAM << 8) + (address & 0x00FF)], pokeAddress, value);
     return true;
   }
 
@@ -162,9 +160,9 @@ void CartridgeMNetwork::bankRAM(uInt16 bank)
 
   // Setup the page access methods for the current bank
   // Set the page accessing method for the 256 bytes of RAM reading pages
-  setAccess(0x1800, 0x100, 1024 + offset, myRAM, romSize() + BANK_SIZE / 2, System::PA_WRITE);
+  setAccess(0x1800, 0x100, 0x0400 + offset, myRAM.data(), romSize() + BANK_SIZE / 2, System::PageAccessType::WRITE);
   // Set the page accessing method for the 256 bytes of RAM reading pages
-  setAccess(0x1900, 0x100, 1024 + offset, myRAM, romSize() + BANK_SIZE / 2, System::PA_READ);
+  setAccess(0x1900, 0x100, 0x0400 + offset, myRAM.data(), romSize() + BANK_SIZE / 2, System::PageAccessType::READ);
 
   myBankChanged = true;
 }
@@ -183,22 +181,22 @@ bool CartridgeMNetwork::bank(uInt16 slice)
     uInt16 offset = slice << 11; // * BANK_SIZE (2048)
 
     // Map ROM image into first segment
-    setAccess(0x1000, BANK_SIZE, offset, myImage.get(), offset, System::PA_READ);
+    setAccess(0x1000, BANK_SIZE, offset, myImage.get(), offset, System::PageAccessType::READ);
   }
   else
   {
     // Set the page accessing method for the 1K slice of RAM writing pages
-    setAccess(0x1000,                 BANK_SIZE / 2, 0, myRAM, romSize(), System::PA_WRITE);
+    setAccess(0x1000,                 BANK_SIZE / 2, 0, myRAM.data(), romSize(), System::PageAccessType::WRITE);
     // Set the page accessing method for the 1K slice of RAM reading pages
-    setAccess(0x1000 + BANK_SIZE / 2, BANK_SIZE / 2, 0, myRAM, romSize(), System::PA_READ);
+    setAccess(0x1000 + BANK_SIZE / 2, BANK_SIZE / 2, 0, myRAM.data(), romSize(), System::PageAccessType::READ);
   }
   return myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt16 CartridgeMNetwork::getBank() const
+uInt16 CartridgeMNetwork::getBank(uInt16 address) const
 {
-  return myCurrentSlice[0];
+  return myCurrentSlice[(address & 0xFFF) >> 11]; // 2K slices
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -223,7 +221,7 @@ bool CartridgeMNetwork::patch(uInt16 address, uInt8 value)
     // Normally, a write to the read port won't do anything
     // However, the patch command is special in that ignores such
     // cart restrictions
-    myRAM[1024 + (myCurrentRAM << 8) + (address & 0x00FF)] = value;
+    myRAM[0x0400 + (myCurrentRAM << 8) + (address & 0x00FF)] = value;
   }
   else
     myImage[(myCurrentSlice[address >> 11] << 11) + (address & (BANK_SIZE-1))] = value;
@@ -232,7 +230,7 @@ bool CartridgeMNetwork::patch(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* CartridgeMNetwork::getImage(uInt32& size) const
+const uInt8* CartridgeMNetwork::getImage(size_t& size) const
 {
   size = bankCount() * BANK_SIZE;
   return myImage.get();
@@ -243,9 +241,9 @@ bool CartridgeMNetwork::save(Serializer& out) const
 {
   try
   {
-    out.putShortArray(myCurrentSlice, NUM_SEGMENTS);
+    out.putShortArray(myCurrentSlice.data(), myCurrentSlice.size());
     out.putShort(myCurrentRAM);
-    out.putByteArray(myRAM, RAM_SIZE);
+    out.putByteArray(myRAM.data(), myRAM.size());
   }
   catch(...)
   {
@@ -261,9 +259,9 @@ bool CartridgeMNetwork::load(Serializer& in)
 {
   try
   {
-    in.getShortArray(myCurrentSlice, NUM_SEGMENTS);
+    in.getShortArray(myCurrentSlice.data(), myCurrentSlice.size());
     myCurrentRAM = in.getShort();
-    in.getByteArray(myRAM, RAM_SIZE);
+    in.getByteArray(myRAM.data(), myRAM.size());
   }
   catch(...)
   {
@@ -281,11 +279,11 @@ bool CartridgeMNetwork::load(Serializer& in)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt16 CartridgeMNetwork::bankCount() const
 {
-  return mySize >> 11;
+  return uInt16(mySize >> 11);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt32 CartridgeMNetwork::romSize() const
+uInt16 CartridgeMNetwork::romSize() const
 {
   return bankCount() * BANK_SIZE;
 }
