@@ -8,7 +8,7 @@
 // MM     MM 66  66 55  55 00  00 22
 // MM     MM  6666   5555   0000  222222
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -19,7 +19,6 @@
   #include "Debugger.hxx"
   #include "Expression.hxx"
   #include "CartDebug.hxx"
-  #include "PackedBitArray.hxx"
   #include "Base.hxx"
 
   // Flags for disassembly types
@@ -54,35 +53,8 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 M6502::M6502(const Settings& settings)
-  : myExecutionStatus(0),
-    mySystem(nullptr),
-    mySettings(settings),
-    A(0), X(0), Y(0), SP(0), IR(0), PC(0),
-    N(false), V(false), B(false), D(false), I(false), notZ(false), C(false),
-    icycles(0),
-    myNumberOfDistinctAccesses(0),
-    myLastAddress(0),
-    myLastBreakCycle(ULLONG_MAX),
-    myLastPeekAddress(0),
-    myLastPokeAddress(0),
-    myLastPeekBaseAddress(0),
-    myLastPokeBaseAddress(0),
-    myFlags(DISASM_NONE),
-    myLastSrcAddressS(-1),
-    myLastSrcAddressA(-1),
-    myLastSrcAddressX(-1),
-    myLastSrcAddressY(-1),
-    myDataAddressForPoke(0),
-    myOnHaltCallback(nullptr),
-    myHaltRequested(false),
-    myGhostReadsTrap(false),
-    myReadFromWritePortBreak(false),
-    myStepStateByInstruction(false)
+  : mySettings(settings)
 {
-#ifdef DEBUGGER_SUPPORT
-  myDebugger = nullptr;
-  myJustHitReadTrapFlag = myJustHitWriteTrapFlag = false;
-#endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -125,7 +97,8 @@ void M6502::reset()
 
   myHaltRequested = false;
   myGhostReadsTrap = mySettings.getBool("dbg.ghostreadstrap");
-  myReadFromWritePortBreak = mySettings.getBool(devSettings ? "dev.rwportbreak" : "plr.rwportbreak");
+  myReadFromWritePortBreak = devSettings ? mySettings.getBool("dev.rwportbreak") : false;
+  myWriteToReadPortBreak = devSettings ? mySettings.getBool("dev.wrportbreak") : false;
 
   myLastBreakCycle = ULLONG_MAX;
 }
@@ -280,16 +253,34 @@ inline void M6502::_execute(uInt64 cycles, DispatchResult& result)
           return;
         }
 
-        if(myBreakPoints.isInitialized() && myBreakPoints.isSet(PC)) {
-          myLastBreakCycle = mySystem->cycles();
-          result.setDebugger(currentCycles, "BP: ", PC);
-          return;
+        if(myBreakPoints.isInitialized())
+        {
+          uInt8 bank = mySystem->cart().getBank(PC);
+
+          if(myBreakPoints.check(PC, bank))
+          {
+            myLastBreakCycle = mySystem->cycles();
+            // disable a one-shot breakpoint
+            if(myBreakPoints.get(PC, bank) & BreakpointMap::ONE_SHOT)
+            {
+              myBreakPoints.erase(PC, bank);
+            }
+            else
+            {
+              ostringstream msg;
+
+              msg << "BP: $" << Common::Base::HEX4 << PC << ", bank #" << std::dec << int(bank);
+              result.setDebugger(currentCycles, msg.str());
+            }
+            return;
+          }
         }
 
         int cond = evalCondBreaks();
         if(cond > -1)
         {
           ostringstream msg;
+
           msg << "CBP[" << Common::Base::HEX2 << cond << "]: " << myCondBreakNames[cond];
 
           myLastBreakCycle = mySystem->cycles();
@@ -309,13 +300,13 @@ inline void M6502::_execute(uInt64 cycles, DispatchResult& result)
       mySystem->cart().clearAllRAMAccesses();
   #endif  // DEBUGGER_SUPPORT
 
-      uInt16 operandAddress = 0, intermediateAddress = 0;
-      uInt8 operand = 0;
-
       // Reset the peek/poke address pointers
       myLastPeekAddress = myLastPokeAddress = myDataAddressForPoke = 0;
 
       try {
+        uInt16 operandAddress = 0, intermediateAddress = 0;
+        uInt8 operand = 0;
+
         icycles = 0;
     #ifdef DEBUGGER_SUPPORT
         uInt16 oldPC = PC;
@@ -337,7 +328,7 @@ inline void M6502::_execute(uInt64 cycles, DispatchResult& result)
     #ifdef DEBUGGER_SUPPORT
         if(myReadFromWritePortBreak)
         {
-          uInt16 rwpAddr = mySystem->cart().getIllegalRAMAccess();
+          uInt16 rwpAddr = mySystem->cart().getIllegalRAMReadAccess();
           if(rwpAddr)
           {
             ostringstream msg;
@@ -346,6 +337,18 @@ inline void M6502::_execute(uInt64 cycles, DispatchResult& result)
             return;
           }
         }
+
+        if (myWriteToReadPortBreak)
+        {
+          uInt16 wrpAddr = mySystem->cart().getIllegalRAMWriteAccess();
+          if (wrpAddr)
+          {
+            ostringstream msg;
+            msg << "WRP[@ $" << Common::Base::HEX4 << wrpAddr << "]: ";
+            result.setDebugger(currentCycles, msg.str(), oldPC);
+            return;
+          }
+        }
     #endif  // DEBUGGER_SUPPORT
       } catch (const FatalEmulationError& e) {
         myExecutionStatus |= FatalErrorBit;
@@ -536,7 +539,7 @@ void M6502::attach(Debugger& debugger)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt32 M6502::addCondBreak(Expression* e, const string& name)
+uInt32 M6502::addCondBreak(Expression* e, const string& name, bool oneShot)
 {
   myCondBreaks.emplace_back(e);
   myCondBreakNames.push_back(name);
