@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -16,18 +16,31 @@
 //============================================================================
 
 #include <cassert>
+#include <functional>
 
 #include "bspf.hxx"
+#include "Logger.hxx"
 
 #include "MediaFactory.hxx"
 #include "Sound.hxx"
 
+#ifdef CHEATCODE_SUPPORT
+  #include "CheatManager.hxx"
+#endif
 #ifdef DEBUGGER_SUPPORT
   #include "Debugger.hxx"
 #endif
-
-#ifdef CHEATCODE_SUPPORT
-  #include "CheatManager.hxx"
+#ifdef GUI_SUPPORT
+  #include "Menu.hxx"
+  #include "CommandMenu.hxx"
+  #include "MessageMenu.hxx"
+  #include "Launcher.hxx"
+  #include "TimeMachine.hxx"
+  #include "Widget.hxx"
+#endif
+#ifdef SQLITE_SUPPORT
+  #include "KeyValueRepositorySqlite.hxx"
+  #include "SettingsDb.hxx"
 #endif
 
 #include "FSNode.hxx"
@@ -40,15 +53,9 @@
 #include "Settings.hxx"
 #include "PropsSet.hxx"
 #include "EventHandler.hxx"
-#include "Menu.hxx"
-#include "CommandMenu.hxx"
-#include "Launcher.hxx"
-#include "TimeMachine.hxx"
 #include "PNGLibrary.hxx"
-#include "Widget.hxx"
 #include "Console.hxx"
 #include "Random.hxx"
-#include "SerialPort.hxx"
 #include "StateManager.hxx"
 #include "TimerManager.hxx"
 #include "Version.hxx"
@@ -56,20 +63,16 @@
 #include "DispatchResult.hxx"
 #include "EmulationWorker.hxx"
 #include "AudioSettings.hxx"
+#include "repository/KeyValueRepositoryNoop.hxx"
+#include "repository/KeyValueRepositoryConfigfile.hxx"
+#include "M6532.hxx"
 
 #include "OSystem.hxx"
 
 using namespace std::chrono;
 
-namespace {
-  constexpr uInt32 FPS_METER_QUEUE_SIZE = 100;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 OSystem::OSystem()
-  : myLauncherUsed(false),
-    myQuitLoop(false),
-    myFpsMeter(FPS_METER_QUEUE_SIZE)
 {
   // Get built-in features
   #ifdef SOUND_SUPPORT
@@ -82,17 +85,26 @@ OSystem::OSystem()
     myFeatures += "Debugger ";
   #endif
   #ifdef CHEATCODE_SUPPORT
-    myFeatures += "Cheats";
+    myFeatures += "Cheats ";
+  #endif
+  #ifdef PNG_SUPPORT
+    myFeatures += "PNG ";
+  #endif
+  #ifdef ZIP_SUPPORT
+    myFeatures += "ZIP";
   #endif
 
   // Get build info
   ostringstream info;
-
   info << "Build " << STELLA_BUILD << ", using " << MediaFactory::backendName()
        << " [" << BSPF::ARCH << "]";
   myBuildInfo = info.str();
 
-  mySettings = MediaFactory::createSettings(*this);
+  mySettings = MediaFactory::createSettings();
+
+  myPropSet = make_unique<PropertiesSet>();
+
+  Logger::instance().setLogParameters(Logger::Level::MAX, false);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -103,19 +115,28 @@ OSystem::~OSystem()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool OSystem::create()
 {
-  // Get updated paths for all configuration files
-  setConfigPaths();
   ostringstream buf;
   buf << "Stella " << STELLA_VERSION << endl
       << "  Features: " << myFeatures << endl
       << "  " << myBuildInfo << endl << endl
-      << "Base directory:       '"
+      << "Base directory:     '"
       << FilesystemNode(myBaseDir).getShortPath() << "'" << endl
-      << "Configuration file:   '"
-      << FilesystemNode(myConfigFile).getShortPath() << "'" << endl
-      << "User game properties: '"
-      << FilesystemNode(myPropertiesFile).getShortPath() << "'" << endl;
-  logMessage(buf.str(), 1);
+      << "State directory:    '"
+      << FilesystemNode(myStateDir).getShortPath() << "'" << endl
+      << "NVRam directory:    '"
+      << FilesystemNode(myNVRamDir).getShortPath() << "'" << endl;
+
+  if(!myConfigFile.empty())
+    buf << "Configuration file: '"
+        << FilesystemNode(myConfigFile).getShortPath() << "'" << endl;
+
+  buf << "Game properties:    '"
+      << FilesystemNode(myPropertiesFile).getShortPath() << "'" << endl
+      << "Cheat file:         '"
+      << FilesystemNode(myCheatFile).getShortPath() << "'" << endl
+      << "Palette file:       '"
+      << FilesystemNode(myPaletteFile).getShortPath() << "'" << endl;
+  Logger::info(buf.str());
 
   // NOTE: The framebuffer MUST be created before any other object!!!
   // Get relevant information about the video hardware
@@ -130,46 +151,85 @@ bool OSystem::create()
   myEventHandler = MediaFactory::createEventHandler(*this);
   myEventHandler->initialize();
 
-  // Create the ROM properties database
-  myPropSet = make_unique<PropertiesSet>(myPropertiesFile);
-
-#ifdef CHEATCODE_SUPPORT
-  myCheatManager = make_unique<CheatManager>(*this);
-  myCheatManager->loadCheatDatabase();
-#endif
-
-  // Create various subsystems (menu and launcher GUI objects, etc)
-  myMenu = make_unique<Menu>(*this);
-  myCommandMenu = make_unique<CommandMenu>(*this);
-  myTimeMachine = make_unique<TimeMachine>(*this);
-  myLauncher = make_unique<Launcher>(*this);
   myStateManager = make_unique<StateManager>(*this);
   myTimerManager = make_unique<TimerManager>();
   myAudioSettings = make_unique<AudioSettings>(*mySettings);
 
   // Create the sound object; the sound subsystem isn't actually
   // opened until needed, so this is non-blocking (on those systems
-  // that only have a single sound device (no hardware mixing)
+  // that only have a single sound device (no hardware mixing))
   createSound();
 
-  // Create the serial port object
-  // This is used by any controller that wants to directly access
-  // a real serial port on the system
-  mySerialPort = MediaFactory::createSerialPort();
-
   // Create random number generator
-  myRandom = make_unique<Random>(uInt32(getTicks()));
+  myRandom = make_unique<Random>(uInt32(TimerManager::getTicks()));
 
+#ifdef CHEATCODE_SUPPORT
+  myCheatManager = make_unique<CheatManager>(*this);
+  myCheatManager->loadCheatDatabase();
+#endif
+
+#ifdef GUI_SUPPORT
+  // Create various subsystems (menu and launcher GUI objects, etc)
+  myMenu = make_unique<Menu>(*this);
+  myCommandMenu = make_unique<CommandMenu>(*this);
+  myMessageMenu = make_unique<MessageMenu>(*this);
+  myTimeMachine = make_unique<TimeMachine>(*this);
+  myLauncher = make_unique<Launcher>(*this);
+#endif
+
+#ifdef PNG_SUPPORT
   // Create PNG handler
   myPNGLib = make_unique<PNGLibrary>(*this);
+#endif
+
+  myPropSet->load(myPropertiesFile);
 
   return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void OSystem::loadConfig()
+void OSystem::loadConfig(const Settings::Options& options)
 {
-  mySettings->loadConfig();
+  // Get base directory and config file from derived class
+  // It will decide whether it can override its default location
+  getBaseDirAndConfig(myBaseDir, myConfigFile,
+      myDefaultSaveDir, myDefaultLoadDir,
+      ourOverrideBaseDirWithApp, ourOverrideBaseDir);
+
+  // Get fully-qualified pathnames, and make directories when needed
+  FilesystemNode node(myBaseDir);
+  if(!node.isDirectory())
+    node.makeDir();
+  myBaseDir = node.getPath();
+  if(!myConfigFile.empty())
+    myConfigFile = FilesystemNode(myConfigFile).getPath();
+
+  FilesystemNode save(myDefaultSaveDir);
+  if(!save.isDirectory())
+    save.makeDir();
+  myDefaultSaveDir = save.getShortPath();
+
+  FilesystemNode load(myDefaultLoadDir);
+  if(!load.isDirectory())
+    load.makeDir();
+  myDefaultLoadDir = load.getShortPath();
+
+#ifdef SQLITE_SUPPORT
+  mySettingsDb = make_shared<SettingsDb>(myBaseDir, "settings");
+  if(!mySettingsDb->initialize())
+    mySettingsDb.reset();
+#endif
+
+  mySettings->setRepository(createSettingsRepository());
+
+  mySettings->load(options);
+
+  Logger::instance().setLogParameters(mySettings->getInt("loglevel"),
+                                      mySettings->getBool("logtoconsole"));
+  Logger::debug("Loading config options ...");
+
+  // Get updated paths for all configuration files
+  setConfigPaths();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -177,63 +237,91 @@ void OSystem::saveConfig()
 {
   // Ask all subsystems to save their settings
   if(myFrameBuffer)
-    myFrameBuffer->tiaSurface().ntsc().saveConfig(*mySettings);
+  {
+    // Save the last windowed position and display on system shutdown
+    myFrameBuffer->updateWindowedPos();
+    settings().setValue("display", myFrameBuffer->getCurrentDisplayIndex());
 
-  if(mySettings)
-    mySettings->saveConfig();
+    Logger::debug("Saving TV effects options ...");
+    myFrameBuffer->tiaSurface().ntsc().saveConfig(settings());
+  }
 
-  if(myPropSet)
-    myPropSet->save(myPropertiesFile);
+  Logger::debug("Saving config options ...");
+  mySettings->save();
+
+  if(myPropSet && myPropSet->save(myPropertiesFile))
+    Logger::debug("Saving properties set ...");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void OSystem::setConfigPaths()
 {
-  // Paths are saved with special characters preserved ('~' or '.')
-  // We do some error checking here, so the rest of the codebase doesn't
-  // have to worry about it
-  FilesystemNode node;
-  string s;
+  // Make sure all required directories actually exist
+  auto buildDirIfRequired = [](string& path, const string& pathToBuild)
+  {
+    FilesystemNode node(pathToBuild);
+    if(!node.isDirectory())
+      node.makeDir();
 
-  validatePath(myStateDir, "statedir", myBaseDir + "state");
-  validatePath(mySnapshotSaveDir, "snapsavedir", defaultSaveDir());
-  validatePath(mySnapshotLoadDir, "snaploaddir", defaultLoadDir());
-  validatePath(myNVRamDir, "nvramdir", myBaseDir + "nvram");
-  validatePath(myCfgDir, "cfgdir", myBaseDir + "cfg");
+    path = node.getPath();
+  };
 
-  s = mySettings->getString("cheatfile");
-  if(s == "") s = myBaseDir + "stella.cht";
-  node = FilesystemNode(s);
-  myCheatFile = node.getPath();
-  mySettings->setValue("cheatfile", node.getShortPath());
+  buildDirIfRequired(myStateDir, myBaseDir + "state");
+  buildDirIfRequired(myNVRamDir, myBaseDir + "nvram");
 
-  s = mySettings->getString("palettefile");
-  if(s == "") s = myBaseDir + "stella.pal";
-  node = FilesystemNode(s);
-  myPaletteFile = node.getPath();
-  mySettings->setValue("palettefile", node.getShortPath());
+#ifdef PNG_SUPPORT
+  mySnapshotSaveDir = mySettings->getString("snapsavedir");
+  if(mySnapshotSaveDir == "") mySnapshotSaveDir = defaultSaveDir();
+  buildDirIfRequired(mySnapshotSaveDir, mySnapshotSaveDir);
 
-  s = mySettings->getString("propsfile");
-  if(s == "") s = myBaseDir + "stella.pro";
-  node = FilesystemNode(s);
-  myPropertiesFile = node.getPath();
-  mySettings->setValue("propsfile", node.getShortPath());
+  mySnapshotLoadDir = mySettings->getString("snaploaddir");
+  if(mySnapshotLoadDir == "") mySnapshotLoadDir = defaultLoadDir();
+  buildDirIfRequired(mySnapshotLoadDir, mySnapshotLoadDir);
+#endif
+
+  myCheatFile = FilesystemNode(myBaseDir + "stella.cht").getPath();
+  myPaletteFile = FilesystemNode(myBaseDir + "stella.pal").getPath();
+  myPropertiesFile = FilesystemNode(myBaseDir + "stella.pro").getPath();
+
+#if 0
+  // Debug code
+  auto dbgPath = [](const string& desc, const string& location)
+  {
+    cerr << desc << ": " << location << endl;
+  };
+  dbgPath("base dir  ", myBaseDir);
+  dbgPath("state dir ", myStateDir);
+  dbgPath("nvram dir ", myNVRamDir);
+  dbgPath("ssave dir ", mySnapshotSaveDir);
+  dbgPath("sload dir ", mySnapshotLoadDir);
+  dbgPath("cheat file", myCheatFile);
+  dbgPath("pal file  ", myPaletteFile);
+  dbgPath("pro file  ", myPropertiesFile);
+  dbgPath("INI file  ", myConfigFile);
+#endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void OSystem::setBaseDir(const string& basedir)
+bool OSystem::checkUserPalette(bool outputError) const
 {
-  FilesystemNode node(basedir);
-  if(!node.isDirectory())
-    node.makeDir();
+  const string& palette = paletteFile();
+  ifstream in(palette, std::ios::binary);
+  if (!in)
+    return false;
 
-  myBaseDir = node.getPath();
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void OSystem::setConfigFile(const string& file)
-{
-  myConfigFile = FilesystemNode(file).getPath();
+  // Make sure the contains enough data for the NTSC, PAL and SECAM palettes
+  // This means 128 colours each for NTSC and PAL, at 3 bytes per pixel
+  // and 8 colours for SECAM at 3 bytes per pixel
+  in.seekg(0, std::ios::end);
+  std::streampos length = in.tellg();
+  in.seekg(0, std::ios::beg);
+  if (length < 128 * 3 * 2 + 8 * 3)
+  {
+    if (outputError)
+      cerr << "ERROR: invalid palette file " << palette << endl;
+    return false;
+  }
+  return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -245,27 +333,32 @@ FBInitStatus OSystem::createFrameBuffer()
   {
     case EventHandlerState::EMULATION:
     case EventHandlerState::PAUSE:
+  #ifdef GUI_SUPPORT
     case EventHandlerState::OPTIONSMENU:
     case EventHandlerState::CMDMENU:
     case EventHandlerState::TIMEMACHINE:
+  #endif
       if((fbstatus = myConsole->initializeVideo()) != FBInitStatus::Success)
         return fbstatus;
       break;
 
+  #ifdef GUI_SUPPORT
     case EventHandlerState::LAUNCHER:
       if((fbstatus = myLauncher->initializeVideo()) != FBInitStatus::Success)
         return fbstatus;
       break;
+  #endif
 
-    case EventHandlerState::DEBUGGER:
   #ifdef DEBUGGER_SUPPORT
+    case EventHandlerState::DEBUGGER:
       if((fbstatus = myDebugger->initializeVideo()) != FBInitStatus::Success)
         return fbstatus;
-  #endif
       break;
+  #endif
 
     case EventHandlerState::NONE:  // Should never happen
-      logMessage("ERROR: Unknown emulation state in createFrameBuffer()", 0);
+    default:
+      Logger::error("ERROR: Unknown emulation state in createFrameBuffer()");
       break;
   }
   return fbstatus;
@@ -305,6 +398,9 @@ string OSystem::createConsole(const FilesystemNode& rom, const string& md5sum,
 
   // Create an instance of the 2600 game console
   ostringstream buf;
+
+  myEventHandler->handleConsoleStartupEvents();
+
   try
   {
     closeConsole();
@@ -313,7 +409,7 @@ string OSystem::createConsole(const FilesystemNode& rom, const string& md5sum,
   catch(const runtime_error& e)
   {
     buf << "ERROR: Couldn't create console (" << e.what() << ")";
-    logMessage(buf.str(), 0);
+    Logger::error(buf.str());
     return buf.str();
   }
 
@@ -331,12 +427,19 @@ string OSystem::createConsole(const FilesystemNode& rom, const string& md5sum,
     myEventHandler->setMouseControllerMode(mySettings->getString("usemouse"));
     if(createFrameBuffer() != FBInitStatus::Success)  // Takes care of initializeVideo()
     {
-      logMessage("ERROR: Couldn't create framebuffer for console", 0);
+      Logger::error("ERROR: Couldn't create framebuffer for console");
       myEventHandler->reset(EventHandlerState::LAUNCHER);
       return "ERROR: Couldn't create framebuffer for console";
     }
     myConsole->initializeAudio();
 
+    string saveOnExit = settings().getString("saveonexit");
+    bool activeTM = settings().getBool(
+      settings().getBool("dev.settings") ? "dev.timemachine" : "plr.timemachine");
+
+    if (saveOnExit == "all" && activeTM)
+      myEventHandler->handleEvent(Event::LoadAllStates);
+
     if(showmessage)
     {
       const string& id = myConsole->cartridge().multiCartID();
@@ -349,13 +452,17 @@ string OSystem::createConsole(const FilesystemNode& rom, const string& md5sum,
     buf << "Game console created:" << endl
         << "  ROM file: " << myRomFile.getShortPath() << endl << endl
         << getROMInfo(*myConsole);
-    logMessage(buf.str(), 1);
+    Logger::info(buf.str());
 
     myFrameBuffer->setCursorState();
 
-    // Also check if certain virtual buttons should be held down
-    // These must be checked each time a new console is being created
     myEventHandler->handleConsoleStartupEvents();
+    myConsole->riot().update();
+
+    #ifdef DEBUGGER_SUPPORT
+      if(mySettings->getBool("debug"))
+        myEventHandler->enterDebugMode();
+    #endif
   }
   return EmptyString;
 }
@@ -384,6 +491,7 @@ bool OSystem::createLauncher(const string& startdir)
   mySettings->setValue("tmpromdir", startdir);
   bool status = false;
 
+#ifdef GUI_SUPPORT
   myEventHandler->reset(EventHandlerState::LAUNCHER);
   if(createFrameBuffer() == FBInitStatus::Success)
   {
@@ -393,7 +501,8 @@ bool OSystem::createLauncher(const string& startdir)
     status = true;
   }
   else
-    logMessage("ERROR: Couldn't create launcher", 0);
+    Logger::error("ERROR: Couldn't create launcher");
+#endif
 
   myLauncherUsed = myLauncherUsed || status;
   return status;
@@ -418,22 +527,6 @@ string OSystem::getROMInfo(const FilesystemNode& romfile)
   return getROMInfo(*console);
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void OSystem::logMessage(const string& message, uInt8 level)
-{
-  if(level == 0)
-  {
-    cout << message << endl << std::flush;
-    myLogMessages += message + "\n";
-  }
-  else if(level <= uInt8(mySettings->getInt("loglevel")))
-  {
-    if(mySettings->getBool("logtoconsole"))
-      cout << message << endl << std::flush;
-    myLogMessages += message + "\n";
-  }
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void OSystem::resetFps()
 {
@@ -446,8 +539,8 @@ unique_ptr<Console> OSystem::openConsole(const FilesystemNode& romfile, string&
   unique_ptr<Console> console;
 
   // Open the cartridge image and read it in
-  BytePtr image;
-  uInt32 size  = 0;
+  ByteBuffer image;
+  size_t size = 0;
   if((image = openROM(romfile, md5, size)) != nullptr)
   {
     // Get a valid set of properties, including any entered on the commandline
@@ -456,50 +549,57 @@ unique_ptr<Console> OSystem::openConsole(const FilesystemNode& romfile, string&
     myPropSet->getMD5(md5, props);
 
     // Local helper method
-    auto CMDLINE_PROPS_UPDATE = [&](const string& name, PropertyType prop)
+    auto CMDLINE_PROPS_UPDATE = [&](const string& name, PropType prop)
     {
       const string& s = mySettings->getString(name);
       if(s != "") props.set(prop, s);
     };
 
-    CMDLINE_PROPS_UPDATE("bs", Cartridge_Type);
-    CMDLINE_PROPS_UPDATE("type", Cartridge_Type);
+    CMDLINE_PROPS_UPDATE("bs", PropType::Cart_Type);
+    CMDLINE_PROPS_UPDATE("type", PropType::Cart_Type);
+    CMDLINE_PROPS_UPDATE("startbank", PropType::Cart_StartBank);
 
     // Now create the cartridge
     string cartmd5 = md5;
-    const string& type = props.get(Cartridge_Type);
+    const string& type = props.get(PropType::Cart_Type);
     unique_ptr<Cartridge> cart =
-      CartDetector::create(romfile, image, size, cartmd5, type, *this);
+      CartDetector::create(romfile, image, size, cartmd5, type, *mySettings);
+
+    // Some properties may not have a name set; we can't leave it blank
+    if(props.get(PropType::Cart_Name) == EmptyString)
+      props.set(PropType::Cart_Name, romfile.getNameWithExt(""));
 
     // It's possible that the cart created was from a piece of the image,
     // and that the md5 (and hence the cart) has changed
-    if(props.get(Cartridge_MD5) != cartmd5)
+    if(props.get(PropType::Cart_MD5) != cartmd5)
     {
       if(!myPropSet->getMD5(cartmd5, props))
       {
         // Cart md5 wasn't found, so we create a new props for it
-        props.set(Cartridge_MD5, cartmd5);
-        props.set(Cartridge_Name, props.get(Cartridge_Name)+cart->multiCartID());
+        props.set(PropType::Cart_MD5, cartmd5);
+        props.set(PropType::Cart_Name, props.get(PropType::Cart_Name)+cart->multiCartID());
         myPropSet->insert(props, false);
       }
     }
 
-    CMDLINE_PROPS_UPDATE("channels", Cartridge_Sound);
-    CMDLINE_PROPS_UPDATE("ld", Console_LeftDifficulty);
-    CMDLINE_PROPS_UPDATE("rd", Console_RightDifficulty);
-    CMDLINE_PROPS_UPDATE("tv", Console_TelevisionType);
-    CMDLINE_PROPS_UPDATE("sp", Console_SwapPorts);
-    CMDLINE_PROPS_UPDATE("lc", Controller_Left);
-    CMDLINE_PROPS_UPDATE("rc", Controller_Right);
+    CMDLINE_PROPS_UPDATE("sp", PropType::Console_SwapPorts);
+    CMDLINE_PROPS_UPDATE("lc", PropType::Controller_Left);
+    CMDLINE_PROPS_UPDATE("rc", PropType::Controller_Right);
     const string& s = mySettings->getString("bc");
-    if(s != "") { props.set(Controller_Left, s); props.set(Controller_Right, s); }
-    CMDLINE_PROPS_UPDATE("cp", Controller_SwapPaddles);
-    CMDLINE_PROPS_UPDATE("ma", Controller_MouseAxis);
-    CMDLINE_PROPS_UPDATE("format", Display_Format);
-    CMDLINE_PROPS_UPDATE("ystart", Display_YStart);
-    CMDLINE_PROPS_UPDATE("height", Display_Height);
-    CMDLINE_PROPS_UPDATE("pp", Display_Phosphor);
-    CMDLINE_PROPS_UPDATE("ppblend", Display_PPBlend);
+    if(s != "") {
+      props.set(PropType::Controller_Left, s);
+      props.set(PropType::Controller_Right, s);
+    }
+    CMDLINE_PROPS_UPDATE("cp", PropType::Controller_SwapPaddles);
+    CMDLINE_PROPS_UPDATE("ma", PropType::Controller_MouseAxis);
+    CMDLINE_PROPS_UPDATE("channels", PropType::Cart_Sound);
+    CMDLINE_PROPS_UPDATE("ld", PropType::Console_LeftDiff);
+    CMDLINE_PROPS_UPDATE("rd", PropType::Console_RightDiff);
+    CMDLINE_PROPS_UPDATE("tv", PropType::Console_TVType);
+    CMDLINE_PROPS_UPDATE("format", PropType::Display_Format);
+    CMDLINE_PROPS_UPDATE("vcenter", PropType::Display_VCenter);
+    CMDLINE_PROPS_UPDATE("pp", PropType::Display_Phosphor);
+    CMDLINE_PROPS_UPDATE("ppblend", PropType::Display_PPBlend);
 
     // Finally, create the cart with the correct properties
     if(cart)
@@ -516,21 +616,21 @@ void OSystem::closeConsole()
   {
   #ifdef CHEATCODE_SUPPORT
     // If a previous console existed, save cheats before creating a new one
-    myCheatManager->saveCheats(myConsole->properties().get(Cartridge_MD5));
+    myCheatManager->saveCheats(myConsole->properties().get(PropType::Cart_MD5));
   #endif
     myConsole.reset();
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-BytePtr OSystem::openROM(const FilesystemNode& rom, string& md5, uInt32& size)
+ByteBuffer OSystem::openROM(const FilesystemNode& rom, string& md5, size_t& size)
 {
   // This method has a documented side-effect:
   // It not only loads a ROM and creates an array with its contents,
   // but also adds a properties entry if the one for the ROM doesn't
   // contain a valid name
 
-  BytePtr image;
+  ByteBuffer image;
   if((size = rom.read(image)) == 0)
     return nullptr;
 
@@ -565,26 +665,6 @@ string OSystem::getROMInfo(const Console& console)
   return buf.str();
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void OSystem::validatePath(string& path, const string& setting,
-                           const string& defaultpath)
-{
-  const string& s = mySettings->getString(setting) == "" ? defaultpath :
-                    mySettings->getString(setting);
-  FilesystemNode node(s);
-  if(!node.isDirectory())
-    node.makeDir();
-
-  path = node.getPath();
-  mySettings->setValue(setting, node.getShortPath());
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt64 OSystem::getTicks() const
-{
-  return duration_cast<duration<uInt64, std::ratio<1, 1000000> > >(system_clock::now().time_since_epoch()).count();
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 float OSystem::frameRate() const
 {
@@ -645,10 +725,10 @@ double OSystem::dispatchEmulation(EmulationWorker& emulationWorker)
     case DispatchResult::Status::fatal:
       #ifdef DEBUGGER_SUPPORT
         myDebugger->startWithFatalError(dispatchResult.getMessage());
-        break;
       #else
-        throw runtime_error(dispatchResult.getMessage());
+        cerr << dispatchResult.getMessage() << endl;
       #endif
+        break;
 
     default:
       throw runtime_error("invalid emulation dispatch result");
@@ -676,7 +756,7 @@ void OSystem::mainLoop()
   {
     bool wasEmulation = myEventHandler->state() == EventHandlerState::EMULATION;
 
-    myEventHandler->poll(getTicks());
+    myEventHandler->poll(TimerManager::getTicks());
     if(myQuitLoop) break;  // Exit if the user wants to quit
 
     if (!wasEmulation && myEventHandler->state() == EventHandlerState::EMULATION) {
@@ -719,8 +799,27 @@ void OSystem::mainLoop()
   // Cleanup time
 #ifdef CHEATCODE_SUPPORT
   if(myConsole)
-    myCheatManager->saveCheats(myConsole->properties().get(Cartridge_MD5));
+    myCheatManager->saveCheats(myConsole->properties().get(PropType::Cart_MD5));
 
   myCheatManager->saveCheatDatabase();
 #endif
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+shared_ptr<KeyValueRepository> OSystem::createSettingsRepository()
+{
+  #ifdef SQLITE_SUPPORT
+    return mySettingsDb
+      ? shared_ptr<KeyValueRepository>(mySettingsDb, &mySettingsDb->settingsRepository())
+      : make_shared<KeyValueRepositoryNoop>();
+  #else
+    if (myConfigFile.empty())
+      return make_shared<KeyValueRepositoryNoop>();
+
+    return make_shared<KeyValueRepositoryConfigfile>(myConfigFile);
+  #endif
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string OSystem::ourOverrideBaseDir = "";
+bool OSystem::ourOverrideBaseDirWithApp = false;
