@@ -8,28 +8,27 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
+#include <cmath>
+
 #include "Event.hxx"
 #include "Paddles.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Paddles::Paddles(Jack jack, const Event& event, const System& system,
                  bool swappaddle, bool swapaxis, bool swapdir)
-  : Controller(jack, event, system, Controller::Paddles),
-    myMPaddleID(-1),
-    myMPaddleIDX(-1),
-    myMPaddleIDY(-1)
+  : Controller(jack, event, system, Controller::Type::Paddles)
 {
   // We must start with minimum resistance; see commit
   // 38b452e1a047a0dca38c5bcce7c271d40f76736e for more information
-  updateAnalogPin(Five, MIN_RESISTANCE);
-  updateAnalogPin(Nine, MIN_RESISTANCE);
+  setPin(AnalogPin::Five, MIN_RESISTANCE);
+  setPin(AnalogPin::Nine, MIN_RESISTANCE);
 
   // The following logic reflects that mapping paddles to different
   // devices can be extremely complex
@@ -46,78 +45,58 @@ Paddles::Paddles(Jack jack, const Event& event, const System& system,
   // a given port; this will speed up processing in update()
 
   // Consider whether this is the left or right port
-  if(myJack == Left)
+  if(myJack == Jack::Left)
   {
     if(!swappaddle)  // First paddle is 0, second is 1
     {
       // These aren't affected by changes in axis orientation
-      myP0AxisValue  = Event::SALeftAxis0Value;
-      myP1AxisValue  = Event::SALeftAxis1Value;
-      myP0FireEvent1 = Event::PaddleZeroFire;
-      myP0FireEvent2 = Event::JoystickZeroFire;
-      myP1FireEvent1 = Event::PaddleOneFire;
-      myP1FireEvent2 = Event::JoystickZeroFire9;
+      myP0AxisValue  = Event::PaddleZeroAnalog;
+      myP1AxisValue  = Event::PaddleOneAnalog;
+      myP0FireEvent = Event::PaddleZeroFire;
+      myP1FireEvent = Event::PaddleOneFire;
 
       // Direction of movement is swapped
       // That is, moving in a certain direction on an axis can
       // result in either increasing or decreasing paddle movement
       if(!swapdir)
       {
-        myP0DecEvent1 = Event::PaddleZeroDecrease;
-        myP0DecEvent2 = Event::JoystickZeroRight;
-        myP0IncEvent1 = Event::PaddleZeroIncrease;
-        myP0IncEvent2 = Event::JoystickZeroLeft;
-        myP1DecEvent1 = Event::PaddleOneDecrease;
-        myP1DecEvent2 = Event::JoystickZeroDown;
-        myP1IncEvent1 = Event::PaddleOneIncrease;
-        myP1IncEvent2 = Event::JoystickZeroUp;
+        myP0DecEvent = Event::PaddleZeroDecrease;
+        myP0IncEvent = Event::PaddleZeroIncrease;
+        myP1DecEvent = Event::PaddleOneDecrease;
+        myP1IncEvent = Event::PaddleOneIncrease;
       }
       else
       {
-        myP0DecEvent1 = Event::PaddleZeroIncrease;
-        myP0DecEvent2 = Event::JoystickZeroLeft;
-        myP0IncEvent1 = Event::PaddleZeroDecrease;
-        myP0IncEvent2 = Event::JoystickZeroRight;
-        myP1DecEvent1 = Event::PaddleOneIncrease;
-        myP1DecEvent2 = Event::JoystickZeroUp;
-        myP1IncEvent1 = Event::PaddleOneDecrease;
-        myP1IncEvent2 = Event::JoystickZeroDown;
+        myP0DecEvent = Event::PaddleZeroIncrease;
+        myP0IncEvent = Event::PaddleZeroDecrease;
+        myP1DecEvent = Event::PaddleOneIncrease;
+        myP1IncEvent = Event::PaddleOneDecrease;
       }
     }
     else           // First paddle is 1, second is 0
     {
       // These aren't affected by changes in axis orientation
-      myP0AxisValue  = Event::SALeftAxis1Value;
-      myP1AxisValue  = Event::SALeftAxis0Value;
-      myP0FireEvent1 = Event::PaddleOneFire;
-      myP0FireEvent2 = Event::JoystickZeroFire9;
-      myP1FireEvent1 = Event::PaddleZeroFire;
-      myP1FireEvent2 = Event::JoystickZeroFire;
+      myP0AxisValue  = Event::PaddleOneAnalog;
+      myP1AxisValue  = Event::PaddleZeroAnalog;
+      myP0FireEvent = Event::PaddleOneFire;
+      myP1FireEvent = Event::PaddleZeroFire;
 
       // Direction of movement is swapped
       // That is, moving in a certain direction on an axis can
       // result in either increasing or decreasing paddle movement
       if(!swapdir)
       {
-        myP0DecEvent1 = Event::PaddleOneDecrease;
-        myP0DecEvent2 = Event::JoystickZeroDown;
-        myP0IncEvent1 = Event::PaddleOneIncrease;
-        myP0IncEvent2 = Event::JoystickZeroUp;
-        myP1DecEvent1 = Event::PaddleZeroDecrease;
-        myP1DecEvent2 = Event::JoystickZeroRight;
-        myP1IncEvent1 = Event::PaddleZeroIncrease;
-        myP1IncEvent2 = Event::JoystickZeroLeft;
+        myP0DecEvent = Event::PaddleOneDecrease;
+        myP0IncEvent = Event::PaddleOneIncrease;
+        myP1DecEvent = Event::PaddleZeroDecrease;
+        myP1IncEvent = Event::PaddleZeroIncrease;
       }
       else
       {
-        myP0DecEvent1 = Event::PaddleOneIncrease;
-        myP0DecEvent2 = Event::JoystickZeroUp;
-        myP0IncEvent1 = Event::PaddleOneDecrease;
-        myP0IncEvent2 = Event::JoystickZeroDown;
-        myP1DecEvent1 = Event::PaddleZeroIncrease;
-        myP1DecEvent2 = Event::JoystickZeroLeft;
-        myP1IncEvent1 = Event::PaddleZeroDecrease;
-        myP1IncEvent2 = Event::JoystickZeroRight;
+        myP0DecEvent = Event::PaddleOneIncrease;
+        myP0IncEvent = Event::PaddleOneDecrease;
+        myP1DecEvent = Event::PaddleZeroIncrease;
+        myP1IncEvent = Event::PaddleZeroDecrease;
       }
     }
   }
@@ -126,73 +105,53 @@ Paddles::Paddles(Jack jack, const Event& event, const System& system,
     if(!swappaddle)  // First paddle is 2, second is 3
     {
       // These aren't affected by changes in axis orientation
-      myP0AxisValue  = Event::SARightAxis0Value;
-      myP1AxisValue  = Event::SARightAxis1Value;
-      myP0FireEvent1 = Event::PaddleTwoFire;
-      myP0FireEvent2 = Event::JoystickOneFire;
-      myP1FireEvent1 = Event::PaddleThreeFire;
-      myP1FireEvent2 = Event::JoystickOneFire9;
+      myP0AxisValue  = Event::PaddleTwoAnalog;
+      myP1AxisValue  = Event::PaddleThreeAnalog;
+      myP0FireEvent = Event::PaddleTwoFire;
+      myP1FireEvent = Event::PaddleThreeFire;
 
       // Direction of movement is swapped
       // That is, moving in a certain direction on an axis can
       // result in either increasing or decreasing paddle movement
       if(!swapdir)
       {
-        myP0DecEvent1 = Event::PaddleTwoDecrease;
-        myP0DecEvent2 = Event::JoystickOneRight;
-        myP0IncEvent1 = Event::PaddleTwoIncrease;
-        myP0IncEvent2 = Event::JoystickOneLeft;
-        myP1DecEvent1 = Event::PaddleThreeDecrease;
-        myP1DecEvent2 = Event::JoystickOneDown;
-        myP1IncEvent1 = Event::PaddleThreeIncrease;
-        myP1IncEvent2 = Event::JoystickOneUp;
+        myP0DecEvent = Event::PaddleTwoDecrease;
+        myP0IncEvent = Event::PaddleTwoIncrease;
+        myP1DecEvent = Event::PaddleThreeDecrease;
+        myP1IncEvent = Event::PaddleThreeIncrease;
       }
       else
       {
-        myP0DecEvent1 = Event::PaddleTwoIncrease;
-        myP0DecEvent2 = Event::JoystickOneLeft;
-        myP0IncEvent1 = Event::PaddleTwoDecrease;
-        myP0IncEvent2 = Event::JoystickOneRight;
-        myP1DecEvent1 = Event::PaddleThreeIncrease;
-        myP1DecEvent2 = Event::JoystickOneUp;
-        myP1IncEvent1 = Event::PaddleThreeDecrease;
-        myP1IncEvent2 = Event::JoystickOneDown;
+        myP0DecEvent = Event::PaddleTwoIncrease;
+        myP0IncEvent = Event::PaddleTwoDecrease;
+        myP1DecEvent = Event::PaddleThreeIncrease;
+        myP1IncEvent = Event::PaddleThreeDecrease;
       }
     }
     else           // First paddle is 3, second is 2
     {
       // These aren't affected by changes in axis orientation
-      myP0AxisValue  = Event::SARightAxis1Value;
-      myP1AxisValue  = Event::SARightAxis0Value;
-      myP0FireEvent1 = Event::PaddleThreeFire;
-      myP0FireEvent2 = Event::JoystickOneFire9;
-      myP1FireEvent1 = Event::PaddleTwoFire;
-      myP1FireEvent2 = Event::JoystickOneFire;
+      myP0AxisValue  = Event::PaddleThreeAnalog;
+      myP1AxisValue  = Event::PaddleTwoAnalog;
+      myP0FireEvent = Event::PaddleThreeFire;
+      myP1FireEvent = Event::PaddleTwoFire;
 
       // Direction of movement is swapped
       // That is, moving in a certain direction on an axis can
       // result in either increasing or decreasing paddle movement
       if(!swapdir)
       {
-        myP0DecEvent1 = Event::PaddleThreeDecrease;
-        myP0DecEvent2 = Event::JoystickOneDown;
-        myP0IncEvent1 = Event::PaddleThreeIncrease;
-        myP0IncEvent2 = Event::JoystickOneUp;
-        myP1DecEvent1 = Event::PaddleTwoDecrease;
-        myP1DecEvent2 = Event::JoystickOneRight;
-        myP1IncEvent1 = Event::PaddleTwoIncrease;
-        myP1IncEvent2 = Event::JoystickOneLeft;
+        myP0DecEvent = Event::PaddleThreeDecrease;
+        myP0IncEvent = Event::PaddleThreeIncrease;
+        myP1DecEvent = Event::PaddleTwoDecrease;
+        myP1IncEvent = Event::PaddleTwoIncrease;
       }
       else
       {
-        myP0DecEvent1 = Event::PaddleThreeIncrease;
-        myP0DecEvent2 = Event::JoystickOneUp;
-        myP0IncEvent1 = Event::PaddleThreeDecrease;
-        myP0IncEvent2 = Event::JoystickOneDown;
-        myP1DecEvent1 = Event::PaddleTwoIncrease;
-        myP1DecEvent2 = Event::JoystickOneLeft;
-        myP1IncEvent1 = Event::PaddleTwoDecrease;
-        myP1IncEvent2 = Event::JoystickOneRight;
+        myP0DecEvent = Event::PaddleThreeIncrease;
+        myP0IncEvent = Event::PaddleThreeDecrease;
+        myP1DecEvent = Event::PaddleTwoIncrease;
+        myP1IncEvent = Event::PaddleTwoDecrease;
       }
     }
   }
@@ -203,40 +162,32 @@ Paddles::Paddles(Jack jack, const Event& event, const System& system,
                                  abs(MOUSE_SENSITIVITY);
   if(!swapaxis)
   {
-    myAxisMouseMotion = Event::MouseAxisXValue;
+    myAxisMouseMotion = Event::MouseAxisXMove;
     myAxisDigitalZero = 0;
     myAxisDigitalOne  = 1;
   }
   else
   {
-    myAxisMouseMotion = Event::MouseAxisYValue;
+    myAxisMouseMotion = Event::MouseAxisYMove;
     myAxisDigitalZero = 1;
     myAxisDigitalOne  = 0;
   }
 
   // Digital pins 1, 2 and 6 are not connected
-  myDigitalPinState[One] =
-  myDigitalPinState[Two] =
-  myDigitalPinState[Six] = true;
-
-  // Digital emulation of analog paddle movement
-  myKeyRepeat0 = myKeyRepeat1 = false;
-  myPaddleRepeat0 = myPaddleRepeat1 = myLastAxisX = myLastAxisY = 0;
-
-  myCharge[0] = myCharge[1] = TRIGRANGE / 2;
-  myLastCharge[0] = myLastCharge[1] = 0;
+  setPin(DigitalPin::One, true);
+  setPin(DigitalPin::Two, true);
+  setPin(DigitalPin::Six, true);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Paddles::update()
 {
-  myDigitalPinState[Three] = myDigitalPinState[Four] = true;
+  setPin(DigitalPin::Three, true);
+  setPin(DigitalPin::Four, true);
 
   // Digital events (from keyboard or joystick hats & buttons)
-  myDigitalPinState[Three] =
-    (myEvent.get(myP1FireEvent1) == 0 && myEvent.get(myP1FireEvent2) == 0);
-  myDigitalPinState[Four]  =
-    (myEvent.get(myP0FireEvent1) == 0 && myEvent.get(myP0FireEvent2) == 0);
+  setPin(DigitalPin::Three, myEvent.get(myP1FireEvent) == 0);
+  setPin(DigitalPin::Four, myEvent.get(myP0FireEvent) == 0);
 
   // Paddle movement is a very difficult thing to accurately emulate,
   // since it originally came from an analog device that had very
@@ -259,14 +210,47 @@ void Paddles::update()
   bool sa_changed = false;
   int sa_xaxis = myEvent.get(myP0AxisValue);
   int sa_yaxis = myEvent.get(myP1AxisValue);
+  int new_val;
+
+  static constexpr std::array<double, MAX_DEJITTER - MIN_DEJITTER + 1> bFac = {
+    // higher values mean more dejitter strength
+    0, // off
+    0.50, 0.59, 0.67, 0.74, 0.80,
+    0.85, 0.89, 0.92, 0.94, 0.95
+  };
+  static constexpr std::array<double, MAX_DEJITTER - MIN_DEJITTER + 1> dFac = {
+    // lower values mean more dejitter strength
+    1, // off
+    1.0 /  181, 1.0 /  256, 1.0 /  362, 1.0 /  512, 1.0 /  724,
+    1.0 / 1024, 1.0 / 1448, 1.0 / 2048, 1.0 / 2896, 1.0 / 4096
+  };
+  const double baseFactor = bFac[DEJITTER_BASE];
+  const double diffFactor = dFac[DEJITTER_DIFF];
+
   if(abs(myLastAxisX - sa_xaxis) > 10)
   {
-    updateAnalogPin(Nine, Int32(MAX_RESISTANCE * ((32767 - Int16(sa_xaxis)) / 65536.0)));
+    // dejitter, suppress small changes only
+    double dejitter = std::pow(baseFactor, abs(sa_xaxis - myLastAxisX) * diffFactor);
+    new_val = sa_xaxis * (1 - dejitter) + myLastAxisX * dejitter;
+
+    // only use new dejittered value for larger differences
+    if (abs(new_val - sa_xaxis) > 10)
+      sa_xaxis = new_val;
+
+    setPin(AnalogPin::Nine, Int32(MAX_RESISTANCE * ((32767 - Int16(sa_xaxis)) / 65536.0)));
     sa_changed = true;
   }
   if(abs(myLastAxisY - sa_yaxis) > 10)
   {
-    updateAnalogPin(Five, Int32(MAX_RESISTANCE * ((32767 - Int16(sa_yaxis)) / 65536.0)));
+    // dejitter, suppress small changes only
+    double dejitter = std::pow(baseFactor, abs(sa_yaxis - myLastAxisY) * diffFactor);
+    new_val = sa_yaxis * (1 - dejitter) + myLastAxisY * dejitter;
+
+    // only use new dejittered value for larger differences
+    if (abs(new_val - sa_yaxis) > 10)
+      sa_yaxis = new_val;
+
+    setPin(AnalogPin::Five, Int32(MAX_RESISTANCE * ((32767 - Int16(sa_yaxis)) / 65536.0)));
     sa_changed = true;
   }
   myLastAxisX = sa_xaxis;
@@ -284,7 +268,7 @@ void Paddles::update()
         TRIGMIN, TRIGRANGE);
     if(myEvent.get(Event::MouseButtonLeftValue) ||
        myEvent.get(Event::MouseButtonRightValue))
-      myDigitalPinState[ourButtonPin[myMPaddleID]] = false;
+      setPin(ourButtonPin[myMPaddleID], false);
   }
   else
   {
@@ -293,18 +277,18 @@ void Paddles::update()
     if(myMPaddleIDX > -1)
     {
       myCharge[myMPaddleIDX] = BSPF::clamp(myCharge[myMPaddleIDX] -
-          (myEvent.get(Event::MouseAxisXValue) * MOUSE_SENSITIVITY),
+          (myEvent.get(Event::MouseAxisXMove) * MOUSE_SENSITIVITY),
           TRIGMIN, TRIGRANGE);
       if(myEvent.get(Event::MouseButtonLeftValue))
-        myDigitalPinState[ourButtonPin[myMPaddleIDX]] = false;
+        setPin(ourButtonPin[myMPaddleIDX], false);
     }
     if(myMPaddleIDY > -1)
     {
       myCharge[myMPaddleIDY] = BSPF::clamp(myCharge[myMPaddleIDY] -
-          (myEvent.get(Event::MouseAxisYValue) * MOUSE_SENSITIVITY),
+          (myEvent.get(Event::MouseAxisYMove) * MOUSE_SENSITIVITY),
           TRIGMIN, TRIGRANGE);
       if(myEvent.get(Event::MouseButtonRightValue))
-        myDigitalPinState[ourButtonPin[myMPaddleIDY]] = false;
+        setPin(ourButtonPin[myMPaddleIDY], false);
     }
   }
 
@@ -326,25 +310,25 @@ void Paddles::update()
   myKeyRepeat0 = false;
   myKeyRepeat1 = false;
 
-  if(myEvent.get(myP0DecEvent1) || myEvent.get(myP0DecEvent2))
+  if(myEvent.get(myP0DecEvent))
   {
     myKeyRepeat0 = true;
     if(myCharge[myAxisDigitalZero] > myPaddleRepeat0)
       myCharge[myAxisDigitalZero] -= myPaddleRepeat0;
   }
-  if(myEvent.get(myP0IncEvent1) || myEvent.get(myP0IncEvent2))
+  if(myEvent.get(myP0IncEvent))
   {
     myKeyRepeat0 = true;
     if((myCharge[myAxisDigitalZero] + myPaddleRepeat0) < TRIGRANGE)
       myCharge[myAxisDigitalZero] += myPaddleRepeat0;
   }
-  if(myEvent.get(myP1DecEvent1) || myEvent.get(myP1DecEvent2))
+  if(myEvent.get(myP1DecEvent))
   {
     myKeyRepeat1 = true;
     if(myCharge[myAxisDigitalOne] > myPaddleRepeat1)
       myCharge[myAxisDigitalOne] -= myPaddleRepeat1;
   }
-  if(myEvent.get(myP1IncEvent1) || myEvent.get(myP1IncEvent2))
+  if(myEvent.get(myP1IncEvent))
   {
     myKeyRepeat1 = true;
     if((myCharge[myAxisDigitalOne] + myPaddleRepeat1) < TRIGRANGE)
@@ -353,9 +337,9 @@ void Paddles::update()
 
   // Only change state if the charge has actually changed
   if(myCharge[1] != myLastCharge[1])
-    updateAnalogPin(Five, Int32(MAX_RESISTANCE * (myCharge[1] / float(TRIGMAX))));
+    setPin(AnalogPin::Five, Int32(MAX_RESISTANCE * (myCharge[1] / double(TRIGMAX))));
   if(myCharge[0] != myLastCharge[0])
-    updateAnalogPin(Nine, Int32(MAX_RESISTANCE * (myCharge[0] / float(TRIGMAX))));
+    setPin(AnalogPin::Nine, Int32(MAX_RESISTANCE * (myCharge[0] / double(TRIGMAX))));
 
   myLastCharge[1] = myCharge[1];
   myLastCharge[0] = myCharge[0];
@@ -368,10 +352,10 @@ bool Paddles::setMouseControl(
   // In 'automatic' mode, both axes on the mouse map to a single paddle,
   // and the paddle axis and direction settings are taken into account
   // This overrides any other mode
-  if(xtype == Controller::Paddles && ytype == Controller::Paddles && xid == yid)
+  if(xtype == Controller::Type::Paddles && ytype == Controller::Type::Paddles && xid == yid)
   {
-    myMPaddleID = ((myJack == Left && (xid == 0 || xid == 1)) ||
-                   (myJack == Right && (xid == 2 || xid == 3))
+    myMPaddleID = ((myJack == Jack::Left && (xid == 0 || xid == 1)) ||
+                   (myJack == Jack::Right && (xid == 2 || xid == 3))
                   ) ? xid & 0x01 : -1;
     myMPaddleIDX = myMPaddleIDY = -1;
   }
@@ -380,12 +364,12 @@ bool Paddles::setMouseControl(
     // The following is somewhat complex, but we need to pre-process as much
     // as possible, so that ::update() can run quickly
     myMPaddleID = -1;
-    if(myJack == Left && xtype == Controller::Paddles)
+    if(myJack == Jack::Left && xtype == Controller::Type::Paddles)
     {
       myMPaddleIDX = (xid == 0 || xid == 1) ? xid & 0x01 : -1;
       myMPaddleIDY = (yid == 0 || yid == 1) ? yid & 0x01 : -1;
     }
-    else if(myJack == Right && ytype == Controller::Paddles)
+    else if(myJack == Jack::Right && ytype == Controller::Type::Paddles)
     {
       myMPaddleIDX = (xid == 2 || xid == 3) ? xid & 0x01 : -1;
       myMPaddleIDY = (yid == 2 || yid == 3) ? yid & 0x01 : -1;
@@ -395,6 +379,18 @@ bool Paddles::setMouseControl(
   return true;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Paddles::setDejitterBase(int strength)
+{
+  DEJITTER_BASE = BSPF::clamp(strength, MIN_DEJITTER, MAX_DEJITTER);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Paddles::setDejitterDiff(int strength)
+{
+  DEJITTER_DIFF = BSPF::clamp(strength, MIN_DEJITTER, MAX_DEJITTER);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Paddles::setDigitalSensitivity(int sensitivity)
 {
@@ -420,6 +416,10 @@ int Paddles::TRIGRANGE = Paddles::TRIGMAX;
 int Paddles::DIGITAL_SENSITIVITY = -1;
 int Paddles::DIGITAL_DISTANCE = -1;
 int Paddles::MOUSE_SENSITIVITY = -1;
+int Paddles::DEJITTER_BASE = 0;
+int Paddles::DEJITTER_DIFF = 0;
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const Controller::DigitalPin Paddles::ourButtonPin[2] = { Four, Three };
+const std::array<Controller::DigitalPin, 2> Paddles::ourButtonPin = {
+  DigitalPin::Four, DigitalPin::Three
+};
