@@ -8,15 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include <cctype>
-#include <algorithm>
 #include <sstream>
 
 #include "bspf.hxx"
@@ -35,39 +33,39 @@ Properties::Properties(const Properties& properties)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Properties::set(PropertyType key, const string& value)
+void Properties::set(PropType key, const string& value)
 {
-  if(key != LastPropType)
+  size_t pos = static_cast<size_t>(key);
+  if(pos < myProperties.size())
   {
-    myProperties[key] = value;
-    if(BSPF::equalsIgnoreCase(myProperties[key], "AUTO-DETECT"))
-      myProperties[key] = "AUTO";
+    myProperties[pos] = value;
+    if(BSPF::equalsIgnoreCase(myProperties[pos], "AUTO-DETECT"))
+      myProperties[pos] = "AUTO";
 
     switch(key)
     {
-      case Cartridge_Type:
-      case Display_Format:
-      case Cartridge_Sound:
-      case Console_LeftDifficulty:
-      case Console_RightDifficulty:
-      case Console_TelevisionType:
-      case Console_SwapPorts:
-      case Controller_Left:
-      case Controller_Right:
-      case Controller_SwapPaddles:
-      case Controller_MouseAxis:
-      case Display_Phosphor:
+      case PropType::Cart_Sound:
+      case PropType::Cart_Type:
+      case PropType::Console_LeftDiff:
+      case PropType::Console_RightDiff:
+      case PropType::Console_TVType:
+      case PropType::Console_SwapPorts:
+      case PropType::Controller_Left:
+      case PropType::Controller_Right:
+      case PropType::Controller_SwapPaddles:
+      case PropType::Controller_MouseAxis:
+      case PropType::Display_Format:
+      case PropType::Display_Phosphor:
       {
-        transform(myProperties[key].begin(), myProperties[key].end(),
-                  myProperties[key].begin(), ::toupper);
+        BSPF::toUpperCase(myProperties[pos]);
         break;
       }
 
-      case Display_PPBlend:
+      case PropType::Display_PPBlend:
       {
-        int blend = atoi(myProperties[key].c_str());
-        if(blend < 1 || blend > 100)
-          myProperties[key] = ourDefaultProperties[key];
+        int blend = BSPF::stringToInt(myProperties[pos]);
+        if(blend < 0 || blend > 100)
+          myProperties[pos] = ourDefaultProperties[pos];
         break;
       }
 
@@ -105,7 +103,7 @@ istream& operator>>(istream& is, Properties& p)
       return is;
 
     // Set the property
-    PropertyType type = Properties::getPropertyType(key);
+    PropType type = Properties::getPropType(key);
     p.set(type, value);
   }
 
@@ -117,7 +115,7 @@ ostream& operator<<(ostream& os, const Properties& p)
 {
   // Write out each of the key and value pairs
   bool changed = false;
-  for(int i = 0; i < LastPropType; ++i)
+  for(size_t i = 0; i < static_cast<size_t>(PropType::NumTypes); ++i)
   {
     // Try to save some space by only saving the items that differ from default
     if(p.myProperties[i] != Properties::ourDefaultProperties[i])
@@ -194,7 +192,7 @@ void Properties::writeQuotedString(ostream& out, const string& s)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Properties::operator==(const Properties& properties) const
 {
-  for(int i = 0; i < LastPropType; ++i)
+  for(size_t i = 0; i < myProperties.size(); ++i)
     if(myProperties[i] != properties.myProperties[i])
       return false;
 
@@ -221,141 +219,139 @@ Properties& Properties::operator=(const Properties& properties)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Properties::setDefault(PropertyType key, const string& value)
+void Properties::setDefault(PropType key, const string& value)
 {
-  ourDefaultProperties[key] = value;
+  ourDefaultProperties[static_cast<size_t>(key)] = value;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Properties::copy(const Properties& properties)
 {
   // Now, copy each property from properties
-  for(int i = 0; i < LastPropType; ++i)
+  for(size_t i = 0; i < myProperties.size(); ++i)
     myProperties[i] = properties.myProperties[i];
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Properties::print() const
 {
-  cout << get(Cartridge_MD5)          << "|"
-       << get(Cartridge_Name)         << "|"
-       << get(Cartridge_Manufacturer) << "|"
-       << get(Cartridge_ModelNo)      << "|"
-       << get(Cartridge_Note)         << "|"
-       << get(Cartridge_Rarity)       << "|"
-       << get(Cartridge_Sound)        << "|"
-       << get(Cartridge_StartBank)    << "|"
-       << get(Cartridge_Type)         << "|"
-       << get(Console_LeftDifficulty) << "|"
-       << get(Console_RightDifficulty)<< "|"
-       << get(Console_TelevisionType) << "|"
-       << get(Console_SwapPorts)      << "|"
-       << get(Controller_Left)        << "|"
-       << get(Controller_Right)       << "|"
-       << get(Controller_SwapPaddles) << "|"
-       << get(Controller_MouseAxis)   << "|"
-       << get(Display_Format)         << "|"
-       << get(Display_YStart)         << "|"
-       << get(Display_Height)         << "|"
-       << get(Display_Phosphor)       << "|"
-       << get(Display_PPBlend)
+  cout << get(PropType::Cart_MD5)               << "|"
+       << get(PropType::Cart_Name)              << "|"
+       << get(PropType::Cart_Manufacturer)      << "|"
+       << get(PropType::Cart_ModelNo)           << "|"
+       << get(PropType::Cart_Note)              << "|"
+       << get(PropType::Cart_Rarity)            << "|"
+       << get(PropType::Cart_Sound)             << "|"
+       << get(PropType::Cart_StartBank)         << "|"
+       << get(PropType::Cart_Type)              << "|"
+       << get(PropType::Console_LeftDiff)       << "|"
+       << get(PropType::Console_RightDiff)      << "|"
+       << get(PropType::Console_TVType)         << "|"
+       << get(PropType::Console_SwapPorts)      << "|"
+       << get(PropType::Controller_Left)        << "|"
+       << get(PropType::Controller_Right)       << "|"
+       << get(PropType::Controller_SwapPaddles) << "|"
+       << get(PropType::Controller_MouseAxis)   << "|"
+       << get(PropType::Display_Format)         << "|"
+       << get(PropType::Display_VCenter)        << "|"
+       << get(PropType::Display_Phosphor)       << "|"
+       << get(PropType::Display_PPBlend)
        << endl;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Properties::setDefaults()
 {
-  for(int i = 0; i < LastPropType; ++i)
+  for(size_t i = 0; i < myProperties.size(); ++i)
     myProperties[i] = ourDefaultProperties[i];
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-PropertyType Properties::getPropertyType(const string& name)
+PropType Properties::getPropType(const string& name)
 {
-  for(int i = 0; i < LastPropType; ++i)
+  for(size_t i = 0; i < NUM_PROPS; ++i)
     if(ourPropertyNames[i] == name)
-      return PropertyType(i);
+      return static_cast<PropType>(i);
 
   // Otherwise, indicate that the item wasn't found
-  return LastPropType;
+  return PropType::NumTypes;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Properties::printHeader()
 {
-  cout << "Cartridge_MD5|"
-       << "Cartridge_Name|"
-       << "Cartridge_Manufacturer|"
-       << "Cartridge_ModelNo|"
-       << "Cartridge_Note|"
-       << "Cartridge_Rarity|"
-       << "Cartridge_Sound|"
-       << "Cartridge_StartBank|"
-       << "Cartridge_Type|"
-       << "Console_LeftDifficulty|"
-       << "Console_RightDifficulty|"
-       << "Console_TelevisionType|"
+  cout << "Cart_MD5|"
+       << "Cart_Name|"
+       << "Cart_Manufacturer|"
+       << "Cart_ModelNo|"
+       << "Cart_Note|"
+       << "Cart_Rarity|"
+       << "Cart_Sound|"
+       << "Cart_StartBank|"
+       << "Cart_Type|"
+       << "Console_LeftDiff|"
+       << "Console_RightDiff|"
+       << "Console_TVType|"
        << "Console_SwapPorts|"
        << "Controller_Left|"
        << "Controller_Right|"
        << "Controller_SwapPaddles|"
        << "Controller_MouseAxis|"
        << "Display_Format|"
-       << "Display_YStart|"
-       << "Display_Height|"
+       << "Display_VCenter|"
        << "Display_Phosphor|"
        << "Display_PPBlend"
        << endl;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string Properties::ourDefaultProperties[LastPropType] = {
-  "",          // Cartridge.MD5
-  "",          // Cartridge.Manufacturer
-  "",          // Cartridge.ModelNo
-  "Untitled",  // Cartridge.Name
-  "",          // Cartridge.Note
-  "",          // Cartridge.Rarity
-  "MONO",      // Cartridge.Sound
-  "",          // Cartridge.StartBank
-  "AUTO",      // Cartridge.Type
-  "B",         // Console.LeftDifficulty
-  "B",         // Console.RightDifficulty
-  "COLOR",     // Console.TelevisionType
-  "NO",        // Console.SwapPorts
-  "JOYSTICK",  // Controller.Left
-  "JOYSTICK",  // Controller.Right
-  "NO",        // Controller.SwapPaddles
-  "AUTO",      // Controller.MouseAxis
-  "AUTO",      // Display.Format
-  "0",         // Display.YStart
-  "0",         // Display.Height
-  "NO",        // Display.Phosphor
-  "0"          // Display.PPBlend
+std::array<string, Properties::NUM_PROPS> Properties::ourDefaultProperties =
+{
+  "",       // Cart.MD5
+  "",       // Cart.Manufacturer
+  "",       // Cart.ModelNo
+  "",       // Cart.Name
+  "",       // Cart.Note
+  "",       // Cart.Rarity
+  "MONO",   // Cart.Sound
+  "AUTO",   // Cart.StartBank
+  "AUTO",   // Cart.Type
+  "B",      // Console.LeftDiff
+  "B",      // Console.RightDiff
+  "COLOR",  // Console.TVType
+  "NO",     // Console.SwapPorts
+  "AUTO",   // Controller.Left
+  "AUTO",   // Controller.Right
+  "NO",     // Controller.SwapPaddles
+  "AUTO",   // Controller.MouseAxis
+  "AUTO",   // Display.Format
+  "0",      // Display.VCenter
+  "NO",     // Display.Phosphor
+  "0"       // Display.PPBlend
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* const Properties::ourPropertyNames[LastPropType] = {
-  "Cartridge.MD5",
-  "Cartridge.Manufacturer",
-  "Cartridge.ModelNo",
-  "Cartridge.Name",
-  "Cartridge.Note",
-  "Cartridge.Rarity",
-  "Cartridge.Sound",
-  "Cartridge.StartBank",
-  "Cartridge.Type",
-  "Console.LeftDifficulty",
-  "Console.RightDifficulty",
-  "Console.TelevisionType",
+std::array<string, Properties::NUM_PROPS> Properties::ourPropertyNames =
+{
+  "Cart.MD5",
+  "Cart.Manufacturer",
+  "Cart.ModelNo",
+  "Cart.Name",
+  "Cart.Note",
+  "Cart.Rarity",
+  "Cart.Sound",
+  "Cart.StartBank",
+  "Cart.Type",
+  "Console.LeftDiff",
+  "Console.RightDiff",
+  "Console.TVType",
   "Console.SwapPorts",
   "Controller.Left",
   "Controller.Right",
   "Controller.SwapPaddles",
   "Controller.MouseAxis",
   "Display.Format",
-  "Display.YStart",
-  "Display.Height",
+  "Display.VCenter",
   "Display.Phosphor",
   "Display.PPBlend"
 };
