@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -17,60 +17,77 @@
 
 #include <cmath>
 
-#include "FrameBuffer.hxx"
 #include "FBSurface.hxx"
 #include "Settings.hxx"
 #include "OSystem.hxx"
 #include "Console.hxx"
 #include "TIA.hxx"
-    #include "PNGLibrary.hxx"
+#include "PNGLibrary.hxx"
 #include "TIASurface.hxx"
 
+namespace {
+  FrameBuffer::ScalingInterpolation interpolationModeFromSettings(const Settings& settings)
+  {
+#ifdef RETRON77
+  // Witv TV / and or scanline interpolation, the image has a height of ~480px. THe R77 runs at 720p, so there
+  // is no benefit from QIS in y-direction. In addition, QIS on the R77 has performance issues if TV effects are
+  // enabled.
+  return settings.getBool("tia.inter") || settings.getInt("tv.filter") != 0
+    ? FrameBuffer::ScalingInterpolation::blur
+    : FrameBuffer::ScalingInterpolation::sharp;
+#else
+    return settings.getBool("tia.inter") ?
+      FrameBuffer::ScalingInterpolation::blur :
+      FrameBuffer::ScalingInterpolation::sharp;
+#endif
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 TIASurface::TIASurface(OSystem& system)
   : myOSystem(system),
-    myFB(system.frameBuffer()),
-    myTIA(nullptr),
-    myFilter(Filter::Normal),
-    myUsePhosphor(false),
-    myPhosphorPercent(0.60f),
-    myScanlinesEnabled(false),
-    myPalette(nullptr),
-    mySaveSnapFlag(false)
+    myFB(system.frameBuffer())
 {
   // Load NTSC filter settings
   myNTSCFilter.loadConfig(myOSystem.settings());
 
   // Create a surface for the TIA image and scanlines; we'll need them eventually
-  myTiaSurface = myFB.allocateSurface(AtariNTSC::outWidth(kTIAW), kTIAH);
+  myTiaSurface = myFB.allocateSurface(
+    AtariNTSC::outWidth(TIAConstants::frameBufferWidth),
+    TIAConstants::frameBufferHeight,
+    interpolationModeFromSettings(myOSystem.settings())
+  );
 
   // Generate scanline data, and a pre-defined scanline surface
-  uInt32 scanData[kScanH];
-  for(int i = 0; i < kScanH; i+=2)
+  constexpr uInt32 scanHeight = TIAConstants::frameBufferHeight * 2;
+  std::array<uInt32, scanHeight> scanData;
+  for(uInt32 i = 0; i < scanHeight; i += 2)
   {
     scanData[i]   = 0x00000000;
     scanData[i+1] = 0xff000000;
   }
-  mySLineSurface = myFB.allocateSurface(1, kScanH, scanData);
+  mySLineSurface = myFB.allocateSurface(1, scanHeight, interpolationModeFromSettings(myOSystem.settings()), scanData.data());
 
   // Base TIA surface for use in taking snapshots in 1x mode
-  myBaseTiaSurface = myFB.allocateSurface(kTIAW*2, kTIAH);
+  myBaseTiaSurface = myFB.allocateSurface(TIAConstants::frameBufferWidth*2,
+                                          TIAConstants::frameBufferHeight);
 
-  memset(myRGBFramebuffer, 0, sizeof(myRGBFramebuffer));
+  myRGBFramebuffer.fill(0);
 
   // Enable/disable threading in the NTSC TV effects renderer
   myNTSCFilter.enableThreading(myOSystem.settings().getBool("threads"));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void TIASurface::initialize(const Console& console, const VideoMode& mode)
+void TIASurface::initialize(const Console& console,
+                            const FrameBuffer::VideoMode& mode)
 {
   myTIA = &(console.tia());
 
   myTiaSurface->setDstPos(mode.image.x(), mode.image.y());
-  myTiaSurface->setDstSize(mode.image.width(), mode.image.height());
+  myTiaSurface->setDstSize(mode.image.w(), mode.image.h());
   mySLineSurface->setDstPos(mode.image.x(), mode.image.y());
-  mySLineSurface->setDstSize(mode.image.width(), mode.image.height());
+  mySLineSurface->setDstSize(mode.image.w(), mode.image.h());
 
   // Phosphor mode can be enabled either globally or per-ROM
   int p_blend = 0;
@@ -83,20 +100,13 @@ void TIASurface::initialize(const Console& console, const VideoMode& mode)
   }
   else
   {
-    p_blend = atoi(console.properties().get(Display_PPBlend).c_str());
-    enable = console.properties().get(Display_Phosphor) == "YES";
+    p_blend = BSPF::stringToInt(console.properties().get(PropType::Display_PPBlend));
+    enable = console.properties().get(PropType::Display_Phosphor) == "YES";
   }
   enablePhosphor(enable, p_blend);
 
   setNTSC(NTSCFilter::Preset(myOSystem.settings().getInt("tv.filter")), false);
 
-  // Scanline repeating is sensitive to non-integral vertical resolution,
-  // so rounding is performed to eliminate it
-  // This won't be 100% accurate, but non-integral scaling isn't 100%
-  // accurate anyway
-  mySLineSurface->setSrcSize(1, int(2 * float(mode.image.height()) /
-    floor((float(mode.image.height()) / myTIA->height()) + 0.5)));
-
 #if 0
 cerr << "INITIALIZE:\n"
      << "TIA:\n"
@@ -111,17 +121,18 @@ cerr << "SLine:\n"
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void TIASurface::setPalette(const uInt32* tia_palette, const uInt32* rgb_palette)
+void TIASurface::setPalette(const PaletteArray& tia_palette,
+                            const PaletteArray& rgb_palette)
 {
   myPalette = tia_palette;
 
   // The NTSC filtering needs access to the raw RGB data, since it calculates
   // its own internal palette
-  myNTSCFilter.setTIAPalette(rgb_palette);
+  myNTSCFilter.setPalette(rgb_palette);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const FBSurface& TIASurface::baseSurface(GUI::Rect& rect) const
+const FBSurface& TIASurface::baseSurface(Common::Rect& rect) const
 {
   uInt32 tiaw = myTIA->width(), width = tiaw * 2, height = myTIA->height();
   rect.setBounds(0, 0, width, height);
@@ -161,7 +172,7 @@ uInt32 TIASurface::mapIndexedPixel(uInt8 indexedColor, uInt8 shift)
 void TIASurface::setNTSC(NTSCFilter::Preset preset, bool show)
 {
   ostringstream buf;
-  if(preset == NTSCFilter::PRESET_OFF)
+  if(preset == NTSCFilter::Preset::OFF)
   {
     enableNTSC(false);
     buf << "TV filtering disabled";
@@ -181,31 +192,9 @@ void TIASurface::setNTSC(NTSCFilter::Preset preset, bool show)
 void TIASurface::setScanlineIntensity(int amount)
 {
   ostringstream buf;
-  if(ntscEnabled())
-  {
-    uInt32 intensity = enableScanlines(amount);
-    buf << "Scanline intensity at " << intensity  << "%";
-    myOSystem.settings().setValue("tv.scanlines", intensity);
-  }
-  else
-    buf << "Scanlines only available in TV filtering mode";
-
-  myFB.showMessage(buf.str());
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void TIASurface::toggleScanlineInterpolation()
-{
-  ostringstream buf;
-  if(ntscEnabled())
-  {
-    bool enable = !myOSystem.settings().getBool("tv.scaninter");
-    enableScanlineInterpolation(enable);
-    buf << "Scanline interpolation " << (enable ? "enabled" : "disabled");
-    myOSystem.settings().setValue("tv.scaninter", enable);
-  }
-  else
-    buf << "Scanlines only available in TV filtering mode";
+  uInt32 intensity = enableScanlines(amount);
+  buf << "Scanline intensity at " << intensity  << "%";
+  myOSystem.settings().setValue("tv.scanlines", intensity);
 
   myFB.showMessage(buf.str());
 }
@@ -216,83 +205,47 @@ uInt32 TIASurface::enableScanlines(int relative, int absolute)
   FBSurface::Attributes& attr = mySLineSurface->attributes();
   if(relative == 0)  attr.blendalpha = absolute;
   else               attr.blendalpha += relative;
-  attr.blendalpha = std::min(100u, attr.blendalpha);
+  attr.blendalpha = std::max(0, Int32(attr.blendalpha));
+  attr.blendalpha = std::min(100U, attr.blendalpha);
 
   mySLineSurface->applyAttributes();
 
   return attr.blendalpha;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void TIASurface::enableScanlineInterpolation(bool enable)
-{
-  FBSurface::Attributes& attr = mySLineSurface->attributes();
-  attr.smoothing = enable;
-  mySLineSurface->applyAttributes();
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIASurface::enablePhosphor(bool enable, int blend)
 {
-  if(myUsePhosphor == enable && myPhosphorPercent == blend / 100.0f)
-    return;
-
-  myUsePhosphor = enable;
-  if(blend >= 0)
-    myPhosphorPercent = blend / 100.0f;
-  myFilter = Filter(enable ? uInt8(myFilter) | 0x01 : uInt8(myFilter) & 0x10);
-
-  memset(myRGBFramebuffer, 0, sizeof(myRGBFramebuffer));
-
-  // Precalculate the average colors for the 'phosphor' effect
-  if(myUsePhosphor)
+  if(myPhosphorHandler.initialize(enable, blend))
   {
-    for(int c = 255; c >= 0; c--)
-      for(int p = 255; p >= 0; p--)
-        myPhosphorPalette[c][p] = getPhosphor(uInt8(c), uInt8(p));
-
-    myNTSCFilter.setPhosphorPalette(myPhosphorPalette);
+    myFilter = Filter(enable ? uInt8(myFilter) | 0x01 : uInt8(myFilter) & 0x10);
+    myRGBFramebuffer.fill(0);
   }
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-inline uInt32 TIASurface::getRGBPhosphor(const uInt32 c, const uInt32 p) const
-{
-  #define TO_RGB(color, red, green, blue) \
-    const uInt8 red = color >> 16; const uInt8 green = color >> 8; const uInt8 blue = color;
-
-  TO_RGB(c, rc, gc, bc);
-  TO_RGB(p, rp, gp, bp);
-
-  // Mix current calculated frame with previous displayed frame
-  const uInt8 rn = myPhosphorPalette[rc][rp];
-  const uInt8 gn = myPhosphorPalette[gc][gp];
-  const uInt8 bn = myPhosphorPalette[bc][bp];
-
-  return (rn << 16) | (gn << 8) | bn;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIASurface::enableNTSC(bool enable)
 {
   myFilter = Filter(enable ? uInt8(myFilter) | 0x10 : uInt8(myFilter) & 0x01);
 
-  // Normal vs NTSC mode uses different source widths
-  myTiaSurface->setSrcSize(enable ?
-      AtariNTSC::outWidth(kTIAW) : uInt32(kTIAW), myTIA->height());
+  uInt32 surfaceWidth = enable ?
+    AtariNTSC::outWidth(TIAConstants::frameBufferWidth) : TIAConstants::frameBufferWidth;
 
-  FBSurface::Attributes& tia_attr = myTiaSurface->attributes();
-  tia_attr.smoothing = myOSystem.settings().getBool("tia.inter");
-  myTiaSurface->applyAttributes();
+  if (surfaceWidth != myTiaSurface->srcRect().w() || myTIA->height() != myTiaSurface->srcRect().h()) {
+    myTiaSurface->setSrcSize(surfaceWidth, myTIA->height());
 
-  myScanlinesEnabled = enable;
+    myTiaSurface->invalidate();
+  }
+
+  mySLineSurface->setSrcSize(1, 2 * myTIA->height());
+
+  myScanlinesEnabled = myOSystem.settings().getInt("tv.scanlines") > 0;
   FBSurface::Attributes& sl_attr = mySLineSurface->attributes();
-  sl_attr.smoothing  = myOSystem.settings().getBool("tv.scaninter");
   sl_attr.blending   = myScanlinesEnabled;
   sl_attr.blendalpha = myOSystem.settings().getInt("tv.scanlines");
   mySLineSurface->applyAttributes();
 
-  memset(myRGBFramebuffer, 0, sizeof(myRGBFramebuffer));
+  myRGBFramebuffer.fill(0);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -310,26 +263,32 @@ string TIASurface::effectsInfo() const
       buf << "Disabled, phosphor mode";
       break;
     case Filter::BlarggNormal:
-      buf << myNTSCFilter.getPreset() << ", scanlines=" << attr.blendalpha << "/"
-          << (attr.smoothing ? "inter" : "nointer");
+      buf << myNTSCFilter.getPreset() << ", scanlines=" << attr.blendalpha;
       break;
     case Filter::BlarggPhosphor:
       buf << myNTSCFilter.getPreset() << ", phosphor, scanlines="
-          << attr.blendalpha << "/" << (attr.smoothing ? "inter" : "nointer");
+          << attr.blendalpha;
       break;
   }
+
+  buf << ", inter=" << (myOSystem.settings().getBool("tia.inter") ? "enabled" : "disabled");
+
   return buf.str();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 inline uInt32 TIASurface::averageBuffers(uInt32 bufOfs)
 {
-  uInt32 c = myRGBFramebuffer[bufOfs];
-  uInt32 p = myPrevRGBFramebuffer[bufOfs];
+  const uInt32 c = myRGBFramebuffer[bufOfs];
+  const uInt32 p = myPrevRGBFramebuffer[bufOfs];
 
   // Split into RGB values
-  TO_RGB(c, rc, gc, bc);
-  TO_RGB(p, rp, gp, bp);
+  const uInt8 rc = static_cast<uInt8>(c >> 16),
+              gc = static_cast<uInt8>(c >> 8),
+              bc = static_cast<uInt8>(c),
+              rp = static_cast<uInt8>(p >> 16),
+              gp = static_cast<uInt8>(p >> 8),
+              bp = static_cast<uInt8>(p);
 
   // Mix current calculated buffer with previous calculated buffer (50:50)
   const uInt8 rn = (rc + rp) / 2;
@@ -337,14 +296,13 @@ inline uInt32 TIASurface::averageBuffers(uInt32 bufOfs)
   const uInt8 bn = (bc + bp) / 2;
 
   // return averaged value
-  return  (rn << 16) | (gn << 8) | bn;
+  return (rn << 16) | (gn << 8) | bn;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIASurface::render()
 {
-  uInt32 width  = myTIA->width();
-  uInt32 height = myTIA->height();
+  uInt32 width = myTIA->width(), height = myTIA->height();
 
   uInt32 *out, outPitch;
   myTiaSurface->basePtr(out, outPitch);
@@ -372,10 +330,11 @@ void TIASurface::render()
     case Filter::Phosphor:
     {
       uInt8*  tiaIn = myTIA->frameBuffer();
-      uInt32* rgbIn = myRGBFramebuffer;
+      uInt32* rgbIn = myRGBFramebuffer.data();
 
       if (mySaveSnapFlag)
-        memcpy(myPrevRGBFramebuffer, myRGBFramebuffer, width * height * sizeof(uInt32));
+        std::copy_n(myRGBFramebuffer.begin(), width * height,
+                    myPrevRGBFramebuffer.begin());
 
       uInt32 bufofs = 0, screenofsY = 0, pos;
       for(uInt32 y = height; y ; --y)
@@ -384,9 +343,9 @@ void TIASurface::render()
         for(uInt32 x = width / 2; x ; --x)
         {
           // Store back into displayed frame buffer (for next frame)
-          rgbIn[bufofs] = out[pos++] = getRGBPhosphor(myPalette[tiaIn[bufofs]], rgbIn[bufofs]);
+          rgbIn[bufofs] = out[pos++] = PhosphorHandler::getPixel(myPalette[tiaIn[bufofs]], rgbIn[bufofs]);
           ++bufofs;
-          rgbIn[bufofs] = out[pos++] = getRGBPhosphor(myPalette[tiaIn[bufofs]], rgbIn[bufofs]);
+          rgbIn[bufofs] = out[pos++] = PhosphorHandler::getPixel(myPalette[tiaIn[bufofs]], rgbIn[bufofs]);
           ++bufofs;
         }
         screenofsY += outPitch;
@@ -403,9 +362,10 @@ void TIASurface::render()
     case Filter::BlarggPhosphor:
     {
       if(mySaveSnapFlag)
-        memcpy(myPrevRGBFramebuffer, myRGBFramebuffer, height * outPitch * sizeof(uInt32));
+        std::copy_n(myRGBFramebuffer.begin(), height * outPitch,
+                    myPrevRGBFramebuffer.begin());
 
-      myNTSCFilter.render(myTIA->frameBuffer(), width, height, out, outPitch << 2, myRGBFramebuffer);
+      myNTSCFilter.render(myTIA->frameBuffer(), width, height, out, outPitch << 2, myRGBFramebuffer.data());
       break;
     }
   }
@@ -419,8 +379,10 @@ void TIASurface::render()
 
   if(mySaveSnapFlag)
   {
-    myOSystem.png().takeSnapshot();
     mySaveSnapFlag = false;
+  #ifdef PNG_SUPPORT
+    myOSystem.png().takeSnapshot();
+  #endif
   }
 }
 
@@ -474,7 +436,7 @@ void TIASurface::renderForSnapshot()
       break;
   }
 
-  if(myUsePhosphor)
+  if(myPhosphorHandler.phosphorEnabled())
   {
     // Draw TIA image
     myTiaSurface->render();
@@ -484,3 +446,12 @@ void TIASurface::renderForSnapshot()
       mySLineSurface->render();
   }
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIASurface::updateSurfaceSettings()
+{
+  myTiaSurface->setScalingInterpolation(interpolationModeFromSettings(myOSystem.settings()));
+  mySLineSurface->setScalingInterpolation(
+      interpolationModeFromSettings(myOSystem.settings())
+  );
+}
