@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -58,10 +58,8 @@ Thumbulator::Thumbulator(const uInt16* rom_ptr, uInt16* ram_ptr, uInt16 rom_size
                          Cartridge* cartridge)
   : rom(rom_ptr),
     romSize(rom_size),
-    decodedRom(new Op[romSize / 2]),
+    decodedRom(make_unique<Op[]>(romSize / 2)),  // NOLINT
     ram(ram_ptr),
-    T1TCR(0),
-    T1TC(0),
     configuration(configurefor),
     myCartridge(cartridge)
 {
@@ -146,7 +144,7 @@ inline int Thumbulator::fatalError(const char* opcode, uInt32 v1, uInt32 v2,
   statusMsg << "Thumb ARM emulation fatal error: " << endl
             << opcode << "(" << Base::HEX8 << v1 << "," << v2 << "), " << msg << endl;
   dump_regs();
-  if(trapOnFatal)
+    if(trapOnFatal)
     throw runtime_error(statusMsg.str());
   return 0;
 }
@@ -165,10 +163,11 @@ void Thumbulator::dump_counters()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Thumbulator::dump_regs()
 {
-  for (int cnt = 1; cnt < 14; cnt++)
+  for (int cnt = 0; cnt <= 12; cnt++)
   {
-    statusMsg << "R" << cnt << " = " << Base::HEX8 << reg_norm[cnt-1] << "  ";
-    if(cnt % 4 == 0) statusMsg << endl;
+    statusMsg << "R" << cnt << " = " << Base::HEX8 << reg_norm[cnt] << "  ";
+    if((cnt + 1) % 4 == 0)
+      statusMsg << endl;
   }
   statusMsg << endl
             << "SP = " << Base::HEX8 << reg_norm[13] << "  "
@@ -315,6 +314,9 @@ void Thumbulator::write32(uInt32 addr, uInt32 data)
         case 0xE000E01C:
           systick_calibrate = data & 0x00FFFFFF;
           break;
+
+        default:
+          break;
       }
       return;
 
@@ -334,6 +336,9 @@ void Thumbulator::write32(uInt32 addr, uInt32 data)
         case 0x20:
           statusMsg << Base::HEX8 << data << endl;
           return;
+
+        default:
+          break;
       }
 #endif
       return;
@@ -939,7 +944,7 @@ int Thumbulator::execute()
       rb <<= 2;
       DO_DISS(statusMsg << "add r" << dec << rd << ",PC,#0x" << Base::HEX2 << rb << endl);
       ra = read_register(15);
-      rc = (ra & (~3u)) + rb;
+      rc = (ra & (~3U)) + rb;
       write_register(rd, rc);
       return 0;
     }
@@ -993,7 +998,7 @@ int Thumbulator::execute()
         if(rc & 0x80000000)
         {
           do_cflag_bit(1);
-          rc = ~0u;
+          rc = ~0U;
         }
         else
         {
@@ -1007,7 +1012,7 @@ int Thumbulator::execute()
         ra = rc & 0x80000000;
         rc >>= rb;
         if(ra) //asr, sign is shifted in
-          rc |= (~0u) << (32-rb);
+          rc |= (~0U) << (32-rb);
       }
       write_register(rd, rc);
       do_nflag(rc);
@@ -1033,7 +1038,7 @@ int Thumbulator::execute()
         rc >>= rb;
         if(ra) //asr, sign is shifted in
         {
-          rc |= (~0u) << (32-rb);
+          rc |= (~0U) << (32-rb);
         }
       }
       else
@@ -1041,7 +1046,7 @@ int Thumbulator::execute()
         if(rc & 0x80000000)
         {
           do_cflag_bit(1);
-          rc = (~0u);
+          rc = (~0U);
         }
         else
         {
@@ -1059,7 +1064,7 @@ int Thumbulator::execute()
     case Op::b1: {
       rb = (inst >> 0) & 0xFF;
       if(rb & 0x80)
-        rb |= (~0u) << 8;
+        rb |= (~0U) << 8;
       op=(inst >> 8) & 0xF;
       rb <<= 1;
       rb += pc;
@@ -1165,8 +1170,10 @@ int Thumbulator::execute()
         case 0xF:
           //swi
           break;
-      }
 
+        default:
+          break;
+      }
       break;
     }
 
@@ -1174,7 +1181,7 @@ int Thumbulator::execute()
     case Op::b2: {
       rb = (inst >> 0) & 0x7FF;
       if(rb & (1 << 10))
-        rb |= (~0u) << 11;
+        rb |= (~0U) << 11;
       rb <<= 1;
       rb += pc;
       rb += 2;
@@ -1692,7 +1699,7 @@ int Thumbulator::execute()
       DO_DISS(statusMsg << "ldrb r" << dec << rd << ",[r" << dec << rn << ",#0x" << Base::HEX2 << rb << "]" << endl);
       rb = read_register(rn) + rb;
 #ifndef UNSAFE_OPTIMIZATIONS
-      rc = read16(rb & (~1u));
+      rc = read16(rb & (~1U));
 #else
       rc = read16(rb);
 #endif
@@ -1715,7 +1722,7 @@ int Thumbulator::execute()
       DO_DISS(statusMsg << "ldrb r" << dec << rd << ",[r" << dec << rn << ",r" << dec << rm << "]" << endl);
       rb = read_register(rn) + read_register(rm);
 #ifndef UNSAFE_OPTIMIZATIONS
-      rc = read16(rb & (~1u));
+      rc = read16(rb & (~1U));
 #else
       rc = read16(rb);
 #endif
@@ -1760,7 +1767,7 @@ int Thumbulator::execute()
       DO_DISS(statusMsg << "ldrsb r" << dec << rd << ",[r" << dec << rn << ",r" << dec << rm << "]" << endl);
       rb = read_register(rn) + read_register(rm);
 #ifndef UNSAFE_OPTIMIZATIONS
-      rc = read16(rb & (~1u));
+      rc = read16(rb & (~1U));
 #else
       rc = read16(rb);
 #endif
@@ -1770,7 +1777,7 @@ int Thumbulator::execute()
       }
       rc &= 0xFF;
       if(rc & 0x80)
-        rc |= ((~0u) << 8);
+        rc |= ((~0U) << 8);
       write_register(rd, rc);
       return 0;
     }
@@ -1785,7 +1792,7 @@ int Thumbulator::execute()
       rc = read16(rb);
       rc &= 0xFFFF;
       if(rc & 0x8000)
-        rc |= ((~0u) << 16);
+        rc |= ((~0U) << 16);
       write_register(rd, rc);
       return 0;
     }
@@ -2285,7 +2292,7 @@ int Thumbulator::execute()
       rb = read_register(rn) + rb;
       rc = read_register(rd);
 #ifndef UNSAFE_OPTIMIZATIONS
-      ra = read16(rb & (~1u));
+      ra = read16(rb & (~1U));
 #else
       ra = read16(rb);
 #endif
@@ -2299,7 +2306,7 @@ int Thumbulator::execute()
         ra &= 0xFF00;
         ra |= rc & 0x00FF;
       }
-      write16(rb & (~1u), ra & 0xFFFF);
+      write16(rb & (~1U), ra & 0xFFFF);
       return 0;
     }
 
@@ -2312,7 +2319,7 @@ int Thumbulator::execute()
       rb = read_register(rn) + read_register(rm);
       rc = read_register(rd);
 #ifndef UNSAFE_OPTIMIZATIONS
-      ra = read16(rb & (~1u));
+      ra = read16(rb & (~1U));
 #else
       ra = read16(rb);
 #endif
@@ -2326,7 +2333,7 @@ int Thumbulator::execute()
         ra &= 0xFF00;
         ra |= rc & 0x00FF;
       }
-      write16(rb & (~1u), ra & 0xFFFF);
+      write16(rb & (~1U), ra & 0xFFFF);
       return 0;
     }
 
@@ -2419,7 +2426,7 @@ int Thumbulator::execute()
       rb = inst & 0xFF;
       DO_DISS(statusMsg << "swi 0x" << Base::HEX2 << rb << endl);
 
-      if((inst & 0xFF) == 0xCC)
+      if(rb == 0xCC)
       {
         write_register(0, cpsr);
         return 0;
@@ -2441,7 +2448,7 @@ int Thumbulator::execute()
       ra = read_register(rm);
       rc = ra & 0xFF;
       if(rc & 0x80)
-        rc |= (~0u) << 8;
+        rc |= (~0U) << 8;
       write_register(rd, rc);
       return 0;
     }
@@ -2454,7 +2461,7 @@ int Thumbulator::execute()
       ra = read_register(rm);
       rc = ra & 0xFFFF;
       if(rc & 0x8000)
-        rc |= (~0u) << 16;
+        rc |= (~0U) << 16;
       write_register(rd, rc);
       return 0;
     }
@@ -2512,7 +2519,7 @@ int Thumbulator::execute()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int Thumbulator::reset()
 {
-  std::fill(reg_norm, reg_norm+12, 0);
+  reg_norm.fill(0);
   reg_norm[13] = 0x40001FB4;
 
   switch(configuration)
