@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -17,7 +17,6 @@
 
 #include "TIA.hxx"
 #include "M6502.hxx"
-#include "Console.hxx"
 #include "Control.hxx"
 #include "Paddles.hxx"
 #include "DelayQueueIteratorImpl.hxx"
@@ -64,25 +63,21 @@ enum ResxCounter: uInt8 {
 // This parameter still has room for tuning. If we go lower than 73, long005 will show
 // a slight artifact (still have to crosscheck on real hardware), if we go lower than
 // 70, the G.I. Joe will show an artifact (hole in roof).
-static constexpr uInt8 resxLateHblankThreshold = 73;
+static constexpr uInt8 resxLateHblankThreshold = TIAConstants::H_CYCLES - 3;
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-TIA::TIA(Console& console, Settings& settings)
+TIA::TIA(ConsoleIO& console, const ConsoleTimingProvider& timingProvider,
+         Settings& settings)
   : myConsole(console),
+    myTimingProvider(timingProvider),
     mySettings(settings),
-    myFrameManager(nullptr),
     myPlayfield(~CollisionMask::playfield & 0x7FFF),
     myMissile0(~CollisionMask::missile0 & 0x7FFF),
     myMissile1(~CollisionMask::missile1 & 0x7FFF),
     myPlayer0(~CollisionMask::player0 & 0x7FFF),
     myPlayer1(~CollisionMask::player1 & 0x7FFF),
-    myBall(~CollisionMask::ball & 0x7FFF),
-    mySpriteEnabledBits(0xFF),
-    myCollisionsEnabledBits(0xFF)
+    myBall(~CollisionMask::ball & 0x7FFF)
 {
-  bool devSettings = mySettings.getBool("dev.settings");
-  myTIAPinsDriven = mySettings.getBool(devSettings ? "dev.tiadriven" : "plr.tiadriven");
-
   myBackground.setTIA(this);
   myPlayfield.setTIA(this);
   myPlayer0.setTIA(this);
@@ -91,14 +86,11 @@ TIA::TIA(Console& console, Settings& settings)
   myMissile1.setTIA(this);
   myBall.setTIA(this);
 
-  myEnableJitter = mySettings.getBool(devSettings ? "dev.tv.jitter" : "plr.tv.jitter");
-  myJitterFactor = mySettings.getInt(devSettings ? "dev.tv.jitter_recovery" : "plr.tv.jitter_recovery");
-
-  reset();
+  initialize();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void TIA::setFrameManager(AbstractFrameManager *frameManager)
+void TIA::setFrameManager(AbstractFrameManager* frameManager)
 {
   clearFrameManager();
 
@@ -118,7 +110,7 @@ void TIA::setFrameManager(AbstractFrameManager *frameManager)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void TIA::setAudioQueue(shared_ptr<AudioQueue> queue)
+void TIA::setAudioQueue(const shared_ptr<AudioQueue>& queue)
 {
   myAudio.setAudioQueue(queue);
 }
@@ -134,7 +126,7 @@ void TIA::clearFrameManager()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void TIA::reset()
+void TIA::initialize()
 {
   myHctr = 0;
   myMovementInProgress = false;
@@ -152,7 +144,7 @@ void TIA::reset()
   myHctrDelta = 0;
   myXAtRenderingStart = 0;
 
-  memset(myShadowRegisters, 0, 64);
+  myShadowRegisters.fill(0);
 
   myBackground.reset();
   myPlayfield.reset();
@@ -176,29 +168,37 @@ void TIA::reset()
   myCyclesAtFrameStart = 0;
 
   if (myFrameManager)
-  {
     myFrameManager->reset();
-    enableColorLoss(mySettings.getBool(mySettings.getBool("dev.settings") ? "dev.colorloss" : "plr.colorloss"));
-  }
 
   myFrontBufferScanlines = myFrameBufferScanlines = 0;
 
   myFramesSinceLastRender = 0;
 
-  // Must be done last, after all other items have reset
-  enableFixedColors(mySettings.getBool(mySettings.getBool("dev.settings") ? "dev.debugcolors" : "plr.debugcolors"));
-  setFixedColorPalette(mySettings.getString("tia.dbgcolors"));
-
   // Blank the various framebuffers; they may contain graphical garbage
-  memset(myBackBuffer, 0, 160 * TIAConstants::frameBufferHeight);
-  memset(myFrontBuffer, 0, 160 * TIAConstants::frameBufferHeight);
-  memset(myFramebuffer, 0, 160 * TIAConstants::frameBufferHeight);
+  myBackBuffer.fill(0);
+  myFrontBuffer.fill(0);
+  myFramebuffer.fill(0);
+
+  applyDeveloperSettings();
+
+  // Must be done last, after all other items have reset
+  bool devSettings = mySettings.getBool("dev.settings");
+  setFixedColorPalette(mySettings.getString("tia.dbgcolors"));
+  enableFixedColors(mySettings.getBool(devSettings ? "dev.debugcolors" : "plr.debugcolors"));
 
 #ifdef DEBUGGER_SUPPORT
   createAccessBase();
 #endif // DEBUGGER_SUPPORT
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::reset()
+{
+  // Simply call initialize(); mostly to get around calling a virtual method
+  // from the constructor
+  initialize();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::install(System& system)
 {
@@ -212,7 +212,7 @@ void TIA::installDelegate(System& system, Device& device)
   mySystem = &system;
 
   // All accesses are to the given device
-  System::PageAccess access(&device, System::PA_READWRITE);
+  System::PageAccess access(&device, System::PageAccessType::READWRITE);
 
   // Map all peek/poke to mirrors of TIA address space to this class
   // That is, all mirrors of ($00 - $3F) in the lower 4K of the 2600
@@ -251,8 +251,6 @@ bool TIA::save(Serializer& out) const
     if(!myInput0.save(out)) return false;
     if(!myInput1.save(out)) return false;
 
-    out.putBool(myTIAPinsDriven);
-
     out.putInt(int(myHstate));
 
     out.putInt(myHctr);
@@ -281,12 +279,16 @@ bool TIA::save(Serializer& out) const
 
     out.putLong(myTimestamp);
 
-    out.putByteArray(myShadowRegisters, 64);
+    out.putByteArray(myShadowRegisters.data(), myShadowRegisters.size());
 
     out.putLong(myCyclesAtFrameStart);
 
     out.putInt(myFrameBufferScanlines);
     out.putInt(myFrontBufferScanlines);
+
+    out.putByte(myPFBitsDelay);
+    out.putByte(myPFColorDelay);
+    out.putByte(myPlSwapDelay);
   }
   catch(...)
   {
@@ -320,8 +322,6 @@ bool TIA::load(Serializer& in)
     if(!myInput0.load(in)) return false;
     if(!myInput1.load(in)) return false;
 
-    myTIAPinsDriven = in.getBool();
-
     myHstate = HState(in.getInt());
 
     myHctr = in.getInt();
@@ -350,12 +350,19 @@ bool TIA::load(Serializer& in)
 
     myTimestamp = in.getLong();
 
-    in.getByteArray(myShadowRegisters, 64);
+    in.getByteArray(myShadowRegisters.data(), myShadowRegisters.size());
 
     myCyclesAtFrameStart = in.getLong();
 
     myFrameBufferScanlines = in.getInt();
     myFrontBufferScanlines = in.getInt();
+
+    myPFBitsDelay = in.getByte();
+    myPFColorDelay = in.getByte();
+    myPlSwapDelay = in.getByte();
+
+    // Re-apply dev settings
+    applyDeveloperSettings();
   }
   catch(...)
   {
@@ -474,13 +481,13 @@ uInt8 TIA::peek(uInt16 address)
 
     case INPT4:
       result =
-        myInput0.inpt(!myConsole.leftController().read(Controller::Six)) |
+        myInput0.inpt(!myConsole.leftController().read(Controller::DigitalPin::Six)) |
         (lastDataBusValue & 0x40);
       break;
 
     case INPT5:
       result =
-        myInput1.inpt(!myConsole.rightController().read(Controller::Six)) |
+        myInput1.inpt(!myConsole.rightController().read(Controller::DigitalPin::Six)) |
         (lastDataBusValue & 0x40);
       break;
 
@@ -561,7 +568,8 @@ bool TIA::poke(uInt16 address, uInt8 value)
       break;
 
     case COLUBK:
-      myBackground.setColor(value & 0xFE);
+      value &= 0xFE;
+      myBackground.setColor(value);
       myShadowRegisters[address] = value;
       break;
 
@@ -593,14 +601,19 @@ bool TIA::poke(uInt16 address, uInt8 value)
     case COLUPF:
       flushLineCache();
       value &= 0xFE;
-      myPlayfield.setColor(value);
-      myBall.setColor(value);
-      myShadowRegisters[address] = value;
+      if (myPFColorDelay)
+        myDelayQueue.push(COLUPF, value, 1);
+      else
+      {
+        myPlayfield.setColor(value);
+        myBall.setColor(value);
+        myShadowRegisters[address] = value;
+      }
       break;
 
     case PF0:
     {
-      myDelayQueue.push(PF0, value, Delay::pf);
+      myDelayQueue.push(PF0, value, myPFBitsDelay);
     #ifdef DEBUGGER_SUPPORT
       uInt16 dataAddr = mySystem->m6502().lastDataAddressForPoke();
       if(dataAddr)
@@ -611,7 +624,7 @@ bool TIA::poke(uInt16 address, uInt8 value)
 
     case PF1:
     {
-      myDelayQueue.push(PF1, value, Delay::pf);
+      myDelayQueue.push(PF1, value, myPFBitsDelay);
     #ifdef DEBUGGER_SUPPORT
       uInt16 dataAddr = mySystem->m6502().lastDataAddressForPoke();
       if(dataAddr)
@@ -622,7 +635,7 @@ bool TIA::poke(uInt16 address, uInt8 value)
 
     case PF2:
     {
-      myDelayQueue.push(PF2, value, Delay::pf);
+      myDelayQueue.push(PF2, value, myPFBitsDelay);
     #ifdef DEBUGGER_SUPPORT
       uInt16 dataAddr = mySystem->m6502().lastDataAddressForPoke();
       if(dataAddr)
@@ -690,7 +703,7 @@ bool TIA::poke(uInt16 address, uInt8 value)
     case GRP0:
     {
       myDelayQueue.push(GRP0, value, Delay::grp);
-      myDelayQueue.push(DummyRegisters::shuffleP1, 0, Delay::shufflePlayer);
+      myDelayQueue.push(DummyRegisters::shuffleP1, 0, myPlSwapDelay);
     #ifdef DEBUGGER_SUPPORT
       uInt16 dataAddr = mySystem->m6502().lastDataAddressForPoke();
       if(dataAddr)
@@ -702,7 +715,7 @@ bool TIA::poke(uInt16 address, uInt8 value)
     case GRP1:
     {
       myDelayQueue.push(GRP1, value, Delay::grp);
-      myDelayQueue.push(DummyRegisters::shuffleP0, 0, Delay::shufflePlayer);
+      myDelayQueue.push(DummyRegisters::shuffleP0, 0, myPlSwapDelay);
       myDelayQueue.push(DummyRegisters::shuffleBL, 0, Delay::shuffleBall);
     #ifdef DEBUGGER_SUPPORT
       uInt16 dataAddr = mySystem->m6502().lastDataAddressForPoke();
@@ -774,6 +787,9 @@ bool TIA::poke(uInt16 address, uInt8 value)
       myCollisionMask = 0;
       myShadowRegisters[address] = value;
       break;
+
+    default:
+      break;
   }
 
   return true;
@@ -784,9 +800,9 @@ bool TIA::saveDisplay(Serializer& out) const
 {
   try
   {
-    out.putByteArray(myFramebuffer, 160* TIAConstants::frameBufferHeight);
-    out.putByteArray(myBackBuffer, 160 * TIAConstants::frameBufferHeight);
-    out.putByteArray(myFrontBuffer, 160 * TIAConstants::frameBufferHeight);
+    out.putByteArray(myFramebuffer.data(), myFramebuffer.size());
+    out.putByteArray(myBackBuffer.data(), myBackBuffer.size());
+    out.putByteArray(myFrontBuffer.data(), myFrontBuffer.size());
     out.putInt(myFramesSinceLastRender);
   }
   catch(...)
@@ -804,9 +820,9 @@ bool TIA::loadDisplay(Serializer& in)
   try
   {
     // Reset frame buffer pointer and data
-    in.getByteArray(myFramebuffer, 160 * TIAConstants::frameBufferHeight);
-    in.getByteArray(myBackBuffer, 160 * TIAConstants::frameBufferHeight);
-    in.getByteArray(myFrontBuffer, 160 * TIAConstants::frameBufferHeight);
+    in.getByteArray(myFramebuffer.data(), myFramebuffer.size());
+    in.getByteArray(myBackBuffer.data(), myBackBuffer.size());
+    in.getByteArray(myFrontBuffer.data(), myFrontBuffer.size());
     myFramesSinceLastRender = in.getInt();
   }
   catch(...)
@@ -818,6 +834,52 @@ bool TIA::loadDisplay(Serializer& in)
   return true;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::applyDeveloperSettings()
+{
+  bool devSettings = mySettings.getBool("dev.settings");
+  if(devSettings)
+  {
+    bool custom = BSPF::equalsIgnoreCase("custom", mySettings.getString("dev.tia.type"));
+
+    setPlInvertedPhaseClock(custom
+                            ? mySettings.getBool("dev.tia.plinvphase")
+                            : BSPF::equalsIgnoreCase("koolaidman", mySettings.getString("dev.tia.type")));
+    setMsInvertedPhaseClock(custom
+                            ? mySettings.getBool("dev.tia.msinvphase")
+                            : BSPF::equalsIgnoreCase("cosmicark", mySettings.getString("dev.tia.type")));
+    setBlInvertedPhaseClock(custom ? mySettings.getBool("dev.tia.blinvphase") : false);
+    setPFBitsDelay(custom
+                   ? mySettings.getBool("dev.tia.delaypfbits")
+                   : BSPF::equalsIgnoreCase("pesco", mySettings.getString("dev.tia.type")));
+    setPFColorDelay(custom
+                    ? mySettings.getBool("dev.tia.delaypfcolor")
+                    : BSPF::equalsIgnoreCase("quickstep", mySettings.getString("dev.tia.type")));
+    setPlSwapDelay(custom
+                   ? mySettings.getBool("dev.tia.delayplswap")
+                   : BSPF::equalsIgnoreCase("heman", mySettings.getString("dev.tia.type")));
+    setBlSwapDelay(custom ? mySettings.getBool("dev.tia.delayblswap") : false);
+  }
+  else
+  {
+    setPlInvertedPhaseClock(false);
+    setMsInvertedPhaseClock(false);
+    setBlInvertedPhaseClock(false);
+    setPFBitsDelay(false);
+    setPFColorDelay(false);
+    setPlSwapDelay(false);
+    setBlSwapDelay(false);
+  }
+
+  myTIAPinsDriven = devSettings ? mySettings.getBool("dev.tiadriven") : false;
+
+  myEnableJitter = mySettings.getBool(devSettings ? "dev.tv.jitter" : "plr.tv.jitter");
+  myJitterFactor = mySettings.getInt(devSettings ? "dev.tv.jitter_recovery" : "plr.tv.jitter_recovery");
+
+  if(myFrameManager)
+    enableColorLoss(mySettings.getBool(devSettings ? "dev.colorloss" : "plr.colorloss"));
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::update(DispatchResult& result, uInt64 maxCycles)
 {
@@ -833,11 +895,18 @@ void TIA::renderToFrameBuffer()
 
   myFramesSinceLastRender = 0;
 
-  memcpy(myFramebuffer, myFrontBuffer, 160 * TIAConstants::frameBufferHeight);
+  myFramebuffer = myFrontBuffer;
 
   myFrameBufferScanlines = myFrontBufferScanlines;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::clearFrameBuffer()
+{
+  myFramebuffer.fill(0);
+  myFrontBuffer.fill(0);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::update(uInt64 maxCycles)
 {
@@ -849,7 +918,7 @@ void TIA::update(uInt64 maxCycles)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIA::enableColorLoss(bool enabled)
 {
-  bool allowColorLoss = consoleTiming() == ConsoleTiming::pal;
+  bool allowColorLoss = myTimingProvider() == ConsoleTiming::pal;
 
   if(allowColorLoss && enabled)
   {
@@ -877,7 +946,7 @@ bool TIA::electronBeamPos(uInt32& x, uInt32& y) const
 {
   uInt8 clocks = clocksThisLine();
 
-  x = (clocks < 68) ? 0 : clocks - 68;
+  x = (clocks < TIAConstants::H_BLANK_CLOCKS) ? 0 : clocks - TIAConstants::H_BLANK_CLOCKS;
   y = myFrameManager->getY();
 
   return isRendering();
@@ -964,8 +1033,8 @@ bool TIA::toggleCollisions()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIA::enableFixedColors(bool enable)
 {
-  int timing = consoleTiming() == ConsoleTiming::ntsc ? 0
-    : consoleTiming() == ConsoleTiming::pal ? 1 : 2;
+  int timing = myTimingProvider() == ConsoleTiming::ntsc ? 0
+    : myTimingProvider() == ConsoleTiming::pal ? 1 : 2;
 
   myMissile0.setDebugColor(myFixedColorPalette[timing][FixedObject::M0]);
   myMissile1.setDebugColor(myFixedColorPalette[timing][FixedObject::M1]);
@@ -1054,10 +1123,8 @@ bool TIA::driveUnusedPinsRandom(uInt8 mode)
   // If mode is 0 or 1, use it as a boolean (off or on)
   // Otherwise, return the state
   if (mode == 0 || mode == 1)
-  {
     myTIAPinsDriven = bool(mode);
-    mySettings.setValue(mySettings.getBool("dev.settings") ? "dev.tiadriven" : "plr.tiadriven", myTIAPinsDriven);
-  }
+
   return myTIAPinsDriven;
 }
 
@@ -1097,8 +1164,8 @@ void TIA::setJitterRecoveryFactor(Int32 factor)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 shared_ptr<DelayQueueIterator> TIA::delayQueueIterator() const
 {
-  return shared_ptr<DelayQueueIterator>(
-    new DelayQueueIteratorImpl<delayQueueLength, delayQueueSize>(myDelayQueue)
+  return make_shared<DelayQueueIteratorImpl<delayQueueLength, delayQueueSize>>(
+    myDelayQueue
   );
 }
 
@@ -1121,16 +1188,6 @@ TIA& TIA::updateScanlineByStep()
   return *this;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-TIA& TIA::updateScanlineByTrace(int target)
-{
-  uInt32 count = 10000;  // only try up to 100 steps
-  while (mySystem->m6502().getPC() != target && count-- &&
-         mySystem->m6502().execute(1));
-
-  return *this;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIA::registerValue(uInt8 reg) const
 {
@@ -1142,10 +1199,10 @@ void TIA::updateEmulation()
 {
   const uInt64 systemCycles = mySystem->cycles();
 
-  if (mySubClock > 2)
+  if (mySubClock > TIAConstants::CYCLE_CLOCKS - 1)
     throw runtime_error("subclock exceeds range");
 
-  const uInt32 cyclesToRun = 3 * uInt32(systemCycles - myLastCycle) + mySubClock;
+  const uInt32 cyclesToRun = TIAConstants::CYCLE_CLOCKS * uInt32(systemCycles - myLastCycle) + mySubClock;
 
   mySubClock = 0;
   myLastCycle = systemCycles;
@@ -1185,14 +1242,14 @@ void TIA::onFrameComplete()
   myCyclesAtFrameStart = mySystem->cycles();
 
   if (myXAtRenderingStart > 0)
-    memset(myBackBuffer, 0, myXAtRenderingStart);
+    std::fill_n(myBackBuffer.begin(), myXAtRenderingStart, 0);
 
   // Blank out any extra lines not drawn this frame
   const Int32 missingScanlines = myFrameManager->missingScanlines();
   if (missingScanlines > 0)
-    memset(myBackBuffer + 160 * myFrameManager->getY(), 0, missingScanlines * 160);
+    std::fill_n(myBackBuffer.begin() + TIAConstants::H_PIXEL * myFrameManager->getY(), missingScanlines * TIAConstants::H_PIXEL, 0);
 
-  memcpy(myFrontBuffer, myBackBuffer, 160 * TIAConstants::frameBufferHeight);
+  myFrontBuffer = myBackBuffer;
 
   myFrontBufferScanlines = scanlinesLastFrame();
 
@@ -1202,9 +1259,9 @@ void TIA::onFrameComplete()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::onHalt()
 {
-  mySubClock += (228 - myHctr) % 228;
-  mySystem->incrementCycles(mySubClock / 3);
-  mySubClock %= 3;
+  mySubClock += (TIAConstants::H_CLOCKS - myHctr) % TIAConstants::H_CLOCKS;
+  mySystem->incrementCycles(mySubClock / TIAConstants::CYCLE_CLOCKS);
+  mySubClock %= TIAConstants::CYCLE_CLOCKS;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1230,7 +1287,7 @@ void TIA::cycle(uInt32 colorClocks)
       if (myCollisionUpdateRequired && !myFrameManager->vblank()) updateCollision();
     }
 
-    if (++myHctr >= 228)
+    if (++myHctr >= TIAConstants::H_CLOCKS)
       nextLine();
 
     #ifdef SOUND_SUPPORT
@@ -1247,18 +1304,23 @@ void TIA::tickMovement()
   if (!myMovementInProgress) return;
 
   if ((myHctr & 0x03) == 0) {
-    const bool apply = myHstate == HState::blank;
-    bool m = false;
+    const bool hblank = myHstate == HState::blank;
     uInt8 movementCounter = myMovementClock > 15 ? 0 : myMovementClock;
 
-    m = myMissile0.movementTick(movementCounter, myHctr, apply) || m;
-    m = myMissile1.movementTick(movementCounter, myHctr, apply) || m;
-    m = myPlayer0.movementTick(movementCounter, apply) || m;
-    m = myPlayer1.movementTick(movementCounter, apply) || m;
-    m = myBall.movementTick(movementCounter, apply) || m;
+    myMissile0.movementTick(movementCounter, myHctr, hblank);
+    myMissile1.movementTick(movementCounter, myHctr, hblank);
+    myPlayer0.movementTick(movementCounter, hblank);
+    myPlayer1.movementTick(movementCounter, hblank);
+    myBall.movementTick(movementCounter, hblank);
 
-    myMovementInProgress = m;
-    myCollisionUpdateRequired = m;
+    myMovementInProgress =
+      myMissile0.isMoving ||
+      myMissile1.isMoving ||
+      myPlayer0.isMoving  ||
+      myPlayer1.isMoving  ||
+      myBall.isMoving;
+
+    myCollisionUpdateRequired = myCollisionUpdateRequired || myMovementInProgress;
 
     ++myMovementClock;
   }
@@ -1272,23 +1334,23 @@ void TIA::tickHblank()
       myExtendedHblank = false;
       break;
 
-    case 67:
+    case TIAConstants::H_BLANK_CLOCKS - 1:
       if (!myExtendedHblank) myHstate = HState::frame;
       break;
 
-    case 75:
+    case TIAConstants::H_BLANK_CLOCKS + 7:
       if (myExtendedHblank) myHstate = HState::frame;
       break;
   }
 
-  if (myExtendedHblank && myHctr > 67) myPlayfield.tick(myHctr - 68 - myHctrDelta);
+  if (myExtendedHblank && myHctr > TIAConstants::H_BLANK_CLOCKS - 1) myPlayfield.tick(myHctr - TIAConstants::H_BLANK_CLOCKS - myHctrDelta);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::tickHframe()
 {
   const uInt32 y = myFrameManager->getY();
-  const uInt32 x = myHctr - 68 - myHctrDelta;
+  const uInt32 x = myHctr - TIAConstants::H_BLANK_CLOCKS - myHctrDelta;
 
   myCollisionUpdateRequired = true;
 
@@ -1306,13 +1368,13 @@ void TIA::tickHframe()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::applyRsync()
 {
-  const uInt32 x = myHctr > 68 ? myHctr - 68 : 0;
+  const uInt32 x = myHctr > TIAConstants::H_BLANK_CLOCKS ? myHctr - TIAConstants::H_BLANK_CLOCKS : 0;
 
-  myHctrDelta = 225 - myHctr;
+  myHctrDelta = TIAConstants::H_CLOCKS - 3 - myHctr;
   if (myFrameManager->isRendering())
-    memset(myBackBuffer + myFrameManager->getY() * 160 + x, 0, 160 - x);
+    std::fill_n(myBackBuffer.begin() + myFrameManager->getY() * TIAConstants::H_PIXEL + x, TIAConstants::H_PIXEL - x, 0);
 
-  myHctr = 225;
+  myHctr = TIAConstants::H_CLOCKS - 3;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1349,9 +1411,8 @@ void TIA::cloneLastLine()
 
   if (!myFrameManager->isRendering() || y == 0) return;
 
-  uInt8* buffer = myBackBuffer;
-
-  memcpy(buffer + y * 160, buffer + (y-1) * 160, 160);
+  std::copy_n(myBackBuffer.begin() + (y-1) * TIAConstants::H_PIXEL, TIAConstants::H_PIXEL,
+      myBackBuffer.begin() + y * TIAConstants::H_PIXEL);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1376,7 +1437,7 @@ void TIA::updateCollision()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::renderPixel(uInt32 x, uInt32 y)
 {
-  if (x >= 160) return;
+  if (x >= TIAConstants::H_PIXEL) return;
 
   uInt8 color = 0;
 
@@ -1428,7 +1489,7 @@ void TIA::renderPixel(uInt32 x, uInt32 y)
     }
   }
 
-  myBackBuffer[y * 160 + x] = color;
+  myBackBuffer[y * TIAConstants::H_PIXEL + x] = color;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1454,7 +1515,51 @@ void TIA::flushLineCache()
 void TIA::clearHmoveComb()
 {
   if (myFrameManager->isRendering() && myHstate == HState::blank)
-    memset(myBackBuffer + myFrameManager->getY() * 160, myColorHBlank, 8);
+    std::fill_n(myBackBuffer.begin() + myFrameManager->getY() * TIAConstants::H_PIXEL, 8, myColorHBlank);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::setPFBitsDelay(bool delayed)
+{
+  myPFBitsDelay = delayed ? Delay::pf + 1 : Delay::pf;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::setPFColorDelay(bool delayed)
+{
+  myPFColorDelay = delayed ? 1 : 0;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::setPlSwapDelay(bool delayed)
+{
+  myPlSwapDelay = delayed ? Delay::shufflePlayer + 1 : Delay::shufflePlayer;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::setBlSwapDelay(bool delayed)
+{
+  myBlSwapDelay = delayed ? Delay::shuffleBall + 1 : Delay::shuffleBall;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::setPlInvertedPhaseClock(bool enable)
+{
+  myPlayer0.setInvertedPhaseClock(enable);
+  myPlayer1.setInvertedPhaseClock(enable);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::setMsInvertedPhaseClock(bool enable)
+{
+  myMissile0.setInvertedPhaseClock(enable);
+  myMissile1.setInvertedPhaseClock(enable);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::setBlInvertedPhaseClock(bool enable)
+{
+  myBall.setInvertedPhaseClock(enable);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1500,6 +1605,11 @@ void TIA::delayedWrite(uInt8 address, uInt8 value)
       myPlayfield.pf2(value);
       break;
 
+    case COLUPF:
+      myPlayfield.setColor(value);
+      myBall.setColor(value);
+      break;
+
     case HMM0:
       myMissile0.hmm(value);
       break;
@@ -1568,6 +1678,9 @@ void TIA::delayedWrite(uInt8 address, uInt8 value)
     case ENAM1:
       myMissile1.enam(value);
       break;
+
+    default:
+      break;
   }
 }
 
@@ -1577,19 +1690,19 @@ void TIA::updatePaddle(uInt8 idx)
   Int32 resistance;
   switch (idx) {
     case 0:
-      resistance = myConsole.leftController().read(Controller::Nine);
+      resistance = myConsole.leftController().read(Controller::AnalogPin::Nine);
       break;
 
     case 1:
-      resistance = myConsole.leftController().read(Controller::Five);
+      resistance = myConsole.leftController().read(Controller::AnalogPin::Five);
       break;
 
     case 2:
-      resistance = myConsole.rightController().read(Controller::Nine);
+      resistance = myConsole.rightController().read(Controller::AnalogPin::Nine);
       break;
 
     case 3:
-      resistance = myConsole.rightController().read(Controller::Five);
+      resistance = myConsole.rightController().read(Controller::AnalogPin::Five);
       break;
 
     default:
@@ -1599,7 +1712,7 @@ void TIA::updatePaddle(uInt8 idx)
   myPaddleReaders[idx].update(
     (resistance == Controller::MAX_RESISTANCE) ? -1 : (double(resistance) / Paddles::MAX_RESISTANCE),
     myTimestamp,
-    consoleTiming()
+    myTimingProvider()
   );
 }
 
@@ -1773,10 +1886,8 @@ void TIA::toggleCollBLPF()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::createAccessBase()
 {
-  myAccessBase = make_unique<uInt8[]>(TIA_SIZE);
-  memset(myAccessBase.get(), CartDebug::NONE, TIA_SIZE);
-  myAccessDelay = make_unique<uInt8[]>(TIA_SIZE);
-  memset(myAccessDelay.get(), TIA_DELAY, TIA_SIZE);
+  myAccessBase.fill(CartDebug::NONE);
+  myAccessDelay.fill(TIA_DELAY);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
