@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -25,10 +25,7 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 AboutDialog::AboutDialog(OSystem& osystem, DialogContainer& parent,
                          const GUI::Font& font)
-  : Dialog(osystem, parent, font, "About Stella"),
-    myPage(1),
-    myNumPages(4),
-    myLinesPerPage(13)
+  : Dialog(osystem, parent, font, "About Stella")
 {
   const int lineHeight   = font.getLineHeight(),
             fontWidth    = font.getMaxCharWidth(),
@@ -47,7 +44,7 @@ AboutDialog::AboutDialog(OSystem& osystem, DialogContainer& parent,
   myPrevButton =
     new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
                      "Previous", GuiObject::kPrevCmd);
-  myPrevButton->clearFlags(WIDGET_ENABLED);
+  myPrevButton->clearFlags(Widget::FLAG_ENABLED);
   wid.push_back(myPrevButton);
 
   xpos += buttonWidth + 8;
@@ -73,7 +70,7 @@ AboutDialog::AboutDialog(OSystem& osystem, DialogContainer& parent,
   {
     myDesc.push_back(new StaticTextWidget(this, font, xpos, ypos, _w - xpos * 2,
                                           fontHeight, "", TextAlign::Left));
-    myDescStr.push_back("");
+    myDescStr.emplace_back("");
     ypos += fontHeight;
   }
 
@@ -104,7 +101,7 @@ void AboutDialog::updateStrings(int page, int lines, string& title)
       ADD_ATEXT(string("\\C\\c2Features: ") + instance().features());
       ADD_ATEXT(string("\\C\\c2") + instance().buildInfo());
       ADD_ALINE();
-      ADD_ATEXT("\\CCopyright (C) 1995-2019 The Stella Team");
+      ADD_ATEXT("\\CCopyright (c) 1995-2020 The Stella Team");
       ADD_ATEXT("\\C(https://stella-emu.github.io)");
       ADD_ALINE();
       ADD_ATEXT("\\CStella is now DonationWare!");
@@ -152,6 +149,9 @@ void AboutDialog::updateStrings(int page, int lines, string& title)
       ADD_ATEXT("\\L\\c0""VCS team for giving us the magic, and to the");
       ADD_ATEXT("\\L\\c0""homebrew developers for keeping the magic alive.");
       break;
+
+    default:
+      return;
   }
 
   while(i < lines)
@@ -237,9 +237,9 @@ void AboutDialog::handleCommand(CommandSender* sender, int cmd, int data, int id
     case GuiObject::kNextCmd:
       myPage++;
       if(myPage >= myNumPages)
-        myNextButton->clearFlags(WIDGET_ENABLED);
+        myNextButton->clearFlags(Widget::FLAG_ENABLED);
       if(myPage >= 2)
-        myPrevButton->setFlags(WIDGET_ENABLED);
+        myPrevButton->setFlags(Widget::FLAG_ENABLED);
 
       displayInfo();
       break;
@@ -247,9 +247,9 @@ void AboutDialog::handleCommand(CommandSender* sender, int cmd, int data, int id
     case GuiObject::kPrevCmd:
       myPage--;
       if(myPage <= myNumPages)
-        myNextButton->setFlags(WIDGET_ENABLED);
+        myNextButton->setFlags(Widget::FLAG_ENABLED);
       if(myPage <= 1)
-        myPrevButton->clearFlags(WIDGET_ENABLED);
+        myPrevButton->clearFlags(Widget::FLAG_ENABLED);
 
       displayInfo();
       break;
