@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -20,6 +20,8 @@
 #include "bspf.hxx"
 
 #include "Console.hxx"
+#include "Cart.hxx"
+#include "CartDPC.hxx"
 #include "Control.hxx"
 #include "Dialog.hxx"
 #include "Font.hxx"
@@ -53,7 +55,7 @@ AudioDialog::AudioDialog(OSystem& osystem, DialogContainer& parent,
 
   // Set real dimensions
   _w = 48 * fontWidth + HBORDER * 2;
-  _h = 11 * (lineHeight + VGAP) + VBORDER + _th;
+  _h = 12 * (lineHeight + VGAP) + VBORDER + _th;
 
   xpos = HBORDER;  ypos = VBORDER + _th;
 
@@ -68,7 +70,7 @@ AudioDialog::AudioDialog(OSystem& osystem, DialogContainer& parent,
   myVolumeSlider = new SliderWidget(this, font, xpos, ypos,
                                     "Volume", lwidth, 0, 4 * fontWidth, "%");
   myVolumeSlider->setMinValue(1); myVolumeSlider->setMaxValue(100);
-  myVolumeSlider->setTickmarkInterval(4);
+  myVolumeSlider->setTickmarkIntervals(4);
   wid.push_back(myVolumeSlider);
   ypos += lineHeight + VGAP;
 
@@ -77,7 +79,7 @@ AudioDialog::AudioDialog(OSystem& osystem, DialogContainer& parent,
   VarList::push_back(items, "Low quality, medium lag", static_cast<int>(AudioSettings::Preset::lowQualityMediumLag));
   VarList::push_back(items, "High quality, medium lag", static_cast<int>(AudioSettings::Preset::highQualityMediumLag));
   VarList::push_back(items, "High quality, low lag", static_cast<int>(AudioSettings::Preset::highQualityLowLag));
-  VarList::push_back(items, "Ultra quality, minimal lag", static_cast<int>(AudioSettings::Preset::veryHighQualityVeryLowLag));
+  VarList::push_back(items, "Ultra quality, minimal lag", static_cast<int>(AudioSettings::Preset::ultraQualityMinimalLag));
   VarList::push_back(items, "Custom", static_cast<int>(AudioSettings::Preset::custom));
   myModePopup = new PopUpWidget(this, font, xpos, ypos,
                                    font.getStringWidth("Ultry quality, minimal lag"), lineHeight,
@@ -87,7 +89,7 @@ AudioDialog::AudioDialog(OSystem& osystem, DialogContainer& parent,
   xpos += INDENT;
 
   // Fragment size
-  pwidth = font.getStringWidth("512 samples");
+  pwidth = font.getStringWidth("512 samples") + 7;
   lwidth = font.getStringWidth("Resampling quality ");
   items.clear();
   VarList::push_back(items, "128 samples", 128);
@@ -129,7 +131,7 @@ AudioDialog::AudioDialog(OSystem& osystem, DialogContainer& parent,
   myHeadroomSlider = new SliderWidget(this, font, xpos, ypos, swidth, lineHeight,
                                       "Headroom           ", 0, kHeadroomChanged, 10 * fontWidth);
   myHeadroomSlider->setMinValue(0); myHeadroomSlider->setMaxValue(AudioSettings::MAX_HEADROOM);
-  myHeadroomSlider->setTickmarkInterval(5);
+  myHeadroomSlider->setTickmarkIntervals(5);
   wid.push_back(myHeadroomSlider);
   ypos += lineHeight + VGAP;
 
@@ -137,7 +139,7 @@ AudioDialog::AudioDialog(OSystem& osystem, DialogContainer& parent,
   myBufferSizeSlider = new SliderWidget(this, font, xpos, ypos, swidth, lineHeight,
                                         "Buffer size        ", 0, kBufferSizeChanged, 10 * fontWidth);
   myBufferSizeSlider->setMinValue(0); myBufferSizeSlider->setMaxValue(AudioSettings::MAX_BUFFER_SIZE);
-  myBufferSizeSlider->setTickmarkInterval(5);
+  myBufferSizeSlider->setTickmarkIntervals(5);
   wid.push_back(myBufferSizeSlider);
   ypos += lineHeight + VGAP;
 
@@ -146,6 +148,14 @@ AudioDialog::AudioDialog(OSystem& osystem, DialogContainer& parent,
   myStereoSoundCheckbox = new CheckboxWidget(this, font, xpos, ypos,
                                              "Stereo for all ROMs");
   wid.push_back(myStereoSoundCheckbox);
+  ypos += lineHeight + VGAP;
+
+  myDpcPitch = new SliderWidget(this, font, xpos, ypos, swidth - 16, lineHeight,
+                                "Pitfall II music pitch ", 0, 0, 5 * fontWidth);
+  myDpcPitch->setMinValue(10000); myDpcPitch->setMaxValue(30000);
+  myDpcPitch->setStepValue(100);
+  myDpcPitch->setTickmarkIntervals(2);
+  wid.push_back(myDpcPitch);
 
   // Add Defaults, OK and Cancel buttons
   addDefaultsOKCancelBGroup(wid, font);
@@ -167,6 +177,9 @@ void AudioDialog::loadConfig()
   // Stereo
   myStereoSoundCheckbox->setState(audioSettings.stereo());
 
+  // DPC Pitch
+  myDpcPitch->setValue(audioSettings.dpcPitch());
+
   // Preset / mode
   myModePopup->setSelected(static_cast<int>(audioSettings.preset()));
 
@@ -210,6 +223,15 @@ void AudioDialog::saveConfig()
   // Stereo
   audioSettings.setStereo(myStereoSoundCheckbox->getState());
 
+  // DPC Pitch
+  audioSettings.setDpcPitch(myDpcPitch->getValue());
+  // update if current cart is Pitfall II
+  if (instance().hasConsole() && instance().console().cartridge().name() == "CartridgeDPC")
+  {
+    CartridgeDPC& cart = static_cast<CartridgeDPC&>(instance().console().cartridge());
+    cart.setDpcPitch(myDpcPitch->getValue());
+  }
+
   AudioSettings::Preset preset = static_cast<AudioSettings::Preset>(myModePopup->getSelectedTag().toInt());
   audioSettings.setPreset(preset);
 
@@ -234,6 +256,7 @@ void AudioDialog::setDefaults()
   mySoundEnableCheckbox->setState(AudioSettings::DEFAULT_ENABLED);
   myVolumeSlider->setValue(AudioSettings::DEFAULT_VOLUME);
   myStereoSoundCheckbox->setState(AudioSettings::DEFAULT_STEREO);
+  myDpcPitch->setValue(AudioSettings::DEFAULT_DPC_PITCH);
   myModePopup->setSelected(static_cast<int>(AudioSettings::DEFAULT_PRESET));
 
   if (AudioSettings::DEFAULT_PRESET == AudioSettings::Preset::custom) {
@@ -258,6 +281,8 @@ void AudioDialog::updateEnabledState()
   myVolumeSlider->setEnabled(active);
   myStereoSoundCheckbox->setEnabled(active);
   myModePopup->setEnabled(active);
+  // enable only for Pitfall II cart
+  myDpcPitch->setEnabled(active && instance().hasConsole() && instance().console().cartridge().name() == "CartridgeDPC");
 
   myFragsizePopup->setEnabled(active && userMode);
   myFreqPopup->setEnabled(active && userMode);
@@ -280,7 +305,6 @@ void AudioDialog::updatePreset()
   updateSettingsWithPreset(audioSettings);
 }
 
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void AudioDialog::handleCommand(CommandSender* sender, int cmd,
                                 int data, int id)
