@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -35,9 +35,7 @@
 BrowserDialog::BrowserDialog(GuiObject* boss, const GUI::Font& font,
                              int max_w, int max_h, const string& title)
   : Dialog(boss->instance(), boss->parent(), font, title),
-    CommandSender(boss),
-    _cmd(0),
-    _mode(FileSave)
+    CommandSender(boss)
 {
   // Set real dimensions
   _w = max_w;
@@ -114,31 +112,36 @@ void BrowserDialog::show(const string& startpath,
   switch(_mode)
   {
     case FileLoad:
-      _fileList->setFileListMode(FilesystemNode::kListAll);
-      _fileList->setFileExtension(ext);
+      _fileList->setListMode(FilesystemNode::ListMode::All);
+      _fileList->setNameFilter([&ext](const FilesystemNode& node) {
+        return BSPF::endsWithIgnoreCase(node.getName(), ext);
+      });
       _selected->setEditable(false);
-      _selected->clearFlags(WIDGET_INVISIBLE);
-      _type->clearFlags(WIDGET_INVISIBLE);
+      _selected->clearFlags(Widget::FLAG_INVISIBLE);
+      _type->clearFlags(Widget::FLAG_INVISIBLE);
       break;
 
     case FileSave:
-      _fileList->setFileListMode(FilesystemNode::kListAll);
-      _fileList->setFileExtension(ext);
+      _fileList->setListMode(FilesystemNode::ListMode::All);
+      _fileList->setNameFilter([&ext](const FilesystemNode& node) {
+        return BSPF::endsWithIgnoreCase(node.getName(), ext);
+      });
       _selected->setEditable(false);  // FIXME - disable user input for now
-      _selected->clearFlags(WIDGET_INVISIBLE);
-      _type->clearFlags(WIDGET_INVISIBLE);
+      _selected->clearFlags(Widget::FLAG_INVISIBLE);
+      _type->clearFlags(Widget::FLAG_INVISIBLE);
       break;
 
     case Directories:
-      _fileList->setFileListMode(FilesystemNode::kListDirectoriesOnly);
+      _fileList->setListMode(FilesystemNode::ListMode::DirectoriesOnly);
+      _fileList->setNameFilter([](const FilesystemNode&) { return true; });
       _selected->setEditable(false);
-      _selected->setFlags(WIDGET_INVISIBLE);
-      _type->setFlags(WIDGET_INVISIBLE);
+      _selected->setFlags(Widget::FLAG_INVISIBLE);
+      _type->setFlags(Widget::FLAG_INVISIBLE);
       break;
   }
 
   // Set start path
-  _fileList->setLocation(FilesystemNode(startpath));
+  _fileList->setDirectory(FilesystemNode(startpath));
 
   updateUI();
 
@@ -193,7 +196,7 @@ void BrowserDialog::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kBaseDirCmd:
-      _fileList->setLocation(FilesystemNode(instance().baseDir()));
+      _fileList->setDirectory(FilesystemNode(instance().baseDir()));
       break;
 
     case FileListWidget::ItemChanged:
