@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -23,7 +23,7 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CheckListWidget::CheckListWidget(GuiObject* boss, const GUI::Font& font,
                                  int x, int y, int w, int h)
-  : ListWidget(boss, font, x, y, w, h, false)  // disable quick select
+  : ListWidget(boss, font, x, y, w, h)
 {
   int ypos = _y + 2;
 
@@ -49,14 +49,14 @@ CheckListWidget::CheckListWidget(GuiObject* boss, const GUI::Font& font,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CheckListWidget::handleMouseEntered()
 {
-  setFlags(WIDGET_HILITED);
+  setFlags(Widget::FLAG_HILITED);
   setDirty();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CheckListWidget::handleMouseLeft()
 {
-  clearFlags(WIDGET_HILITED);
+  clearFlags(Widget::FLAG_HILITED);
   setDirty();
 }
 
@@ -70,12 +70,12 @@ void CheckListWidget::setList(const StringList& list, const BoolArray& state)
 
   // Enable all checkboxes
   for(int i = 0; i < _rows; ++i)
-    _checkList[i]->setFlags(WIDGET_ENABLED);
+    _checkList[i]->setFlags(Widget::FLAG_ENABLED);
 
   // Then turn off any extras
   if(int(_stateList.size()) < _rows)
     for(int i = int(_stateList.size()); i < _rows; ++i)
-      _checkList[i]->clearFlags(WIDGET_ENABLED);
+      _checkList[i]->clearFlags(Widget::FLAG_ENABLED);
 
   ListWidget::recalc();
 }
@@ -113,30 +113,30 @@ void CheckListWidget::drawWidget(bool hilite)
     const int y = _y + 2 + _fontHeight * i + 2;
     ColorId textColor = kTextColor;
 
-    GUI::Rect r(getEditRect());
+    Common::Rect r(getEditRect());
 
     // Draw the selected item inverted, on a highlighted background.
     if (_selectedItem == pos)
     {
       if(_hasFocus && !_editMode)
       {
-        s.fillRect(_x + r.left - 3, _y + 1 + _fontHeight * i,
-                   _w - r.left, _fontHeight, kTextColorHi);
+        s.fillRect(_x + r.x() - 3, _y + 1 + _fontHeight * i,
+                   _w - r.x(), _fontHeight, kTextColorHi);
         textColor = kTextColorInv;
       }
       else
-        s.frameRect(_x + r.left - 3, _y + 1 + _fontHeight * i,
-                    _w - r.left, _fontHeight, onTop ? kTextColorHi : kColor);
+        s.frameRect(_x + r.x() - 3, _y + 1 + _fontHeight * i,
+                    _w - r.x(), _fontHeight, onTop ? kTextColorHi : kColor);
     }
 
     if (_selectedItem == pos && _editMode)
     {
       adjustOffset();
-      s.drawString(_font, editString(), _x + r.left, y, r.width(), onTop ? kTextColor : kColor,
+      s.drawString(_font, editString(), _x + r.x(), y, r.w(), onTop ? kTextColor : kColor,
                    TextAlign::Left, -_editScrollOffset, false);
     }
     else
-      s.drawString(_font, _list[pos], _x + r.left, y, r.width(),
+      s.drawString(_font, _list[pos], _x + r.x(), y, r.w(),
                    onTop ? textColor : kColor);
   }
 
@@ -147,17 +147,13 @@ void CheckListWidget::drawWidget(bool hilite)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-GUI::Rect CheckListWidget::getEditRect() const
+Common::Rect CheckListWidget::getEditRect() const
 {
-  GUI::Rect r(2, 1, _w, _fontHeight);
   const int yoffset = (_selectedItem - _currentPos) * _fontHeight,
             xoffset = CheckboxWidget::boxSize() + 10;
-  r.top    += yoffset;
-  r.bottom += yoffset;
-  r.left  += xoffset;
-  r.right -= xoffset - 15;
 
-  return r;
+  return Common::Rect(2 + xoffset, 1 + yoffset,
+                      _w - (xoffset - 15), _fontHeight + yoffset);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -172,37 +168,29 @@ bool CheckListWidget::getState(int line)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CheckListWidget::handleEvent(Event::Type e)
 {
-  switch(e)
+  if(e == Event::UISelect)
   {
-    case Event::UISelect:
-      // Simulate a mouse button click
-      _checkList[ListWidget::getSelected()]->handleMouseUp(0, 0, MouseButton::LEFT, 0);
-      return true;
-
-    default:
-      return ListWidget::handleEvent(e);
+    // Simulate a mouse button click
+    _checkList[ListWidget::getSelected()]->handleMouseUp(0, 0, MouseButton::LEFT, 0);
+    return true;
   }
+  else
+    return ListWidget::handleEvent(e);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CheckListWidget::handleCommand(CommandSender* sender, int cmd,
                                     int data, int id)
 {
-  switch(cmd)
+  if(cmd == CheckboxWidget::kCheckActionCmd)
   {
-    case CheckboxWidget::kCheckActionCmd:
-    {
-      // Figure out which line has been checked
-      int line = _currentPos + id;
-      _stateList[line] = bool(data);
+    // Figure out which line has been checked
+    int line = _currentPos + id;
+    _stateList[line] = bool(data);
 
-      // Let the boss know about it
-      sendCommand(CheckListWidget::kListItemChecked, line, _id);
-      break;
-    }
-
-    default:
-      ListWidget::handleCommand(sender, cmd, data, id);
-      break;
+    // Let the boss know about it
+    sendCommand(CheckListWidget::kListItemChecked, line, _id);
   }
+  else
+    ListWidget::handleCommand(sender, cmd, data, id);
 }
