@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -29,26 +29,26 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ComboDialog::ComboDialog(GuiObject* boss, const GUI::Font& font,
                          const VariantList& combolist)
-  : Dialog(boss->instance(), boss->parent(), font, "Add..."),
-    myComboEvent(Event::NoType)
+  : Dialog(boss->instance(), boss->parent(), font, "Add...")
 {
   const int lineHeight   = font.getLineHeight(),
             fontWidth    = font.getMaxCharWidth();
   int xpos, ypos;
   WidgetArray wid;
 
+  // Get maximum width of popupwidget
+  int pwidth = 0;
+  for (const auto& s : combolist)
+    pwidth = std::max(font.getStringWidth(s.first), pwidth);
+
   // Set real dimensions
-  _w = 33 * fontWidth + 10*2;
+  _w = 11 * fontWidth + pwidth-4 + 10 * 2;
   _h = 10 * (lineHeight + 4) + 10 + _th;
   xpos = 10;
   ypos = 10 + _th;
 
-  // Get maximum width of popupwidget
-  int pwidth = 0;
-  for(const auto& s: combolist)
-    pwidth = std::max(font.getStringWidth(s.first), pwidth);
-
   // Add event popup for 8 events
+  myEvents.fill(nullptr);
   auto ADD_EVENT_POPUP = [&](int idx, const string& label)
   {
     myEvents[idx] = new PopUpWidget(this, font, xpos, ypos,
@@ -56,15 +56,14 @@ ComboDialog::ComboDialog(GuiObject* boss, const GUI::Font& font,
     wid.push_back(myEvents[idx]);
     ypos += lineHeight + 4;
   };
-
-  myEvents[0] = nullptr;  ADD_EVENT_POPUP(0, "Event 1 ");
-  myEvents[1] = nullptr;  ADD_EVENT_POPUP(1, "Event 2 ");
-  myEvents[2] = nullptr;  ADD_EVENT_POPUP(2, "Event 3 ");
-  myEvents[3] = nullptr;  ADD_EVENT_POPUP(3, "Event 4 ");
-  myEvents[4] = nullptr;  ADD_EVENT_POPUP(4, "Event 5 ");
-  myEvents[5] = nullptr;  ADD_EVENT_POPUP(5, "Event 6 ");
-  myEvents[6] = nullptr;  ADD_EVENT_POPUP(6, "Event 7 ");
-  myEvents[7] = nullptr;  ADD_EVENT_POPUP(7, "Event 8 ");
+  ADD_EVENT_POPUP(0, "Event 1 ");
+  ADD_EVENT_POPUP(1, "Event 2 ");
+  ADD_EVENT_POPUP(2, "Event 3 ");
+  ADD_EVENT_POPUP(3, "Event 4 ");
+  ADD_EVENT_POPUP(4, "Event 5 ");
+  ADD_EVENT_POPUP(5, "Event 6 ");
+  ADD_EVENT_POPUP(6, "Event 7 ");
+  ADD_EVENT_POPUP(7, "Event 8 ");
 
   // Add Defaults, OK and Cancel buttons
   addDefaultsOKCancelBGroup(wid, font);
@@ -91,13 +90,13 @@ void ComboDialog::loadConfig()
 {
   StringList events = instance().eventHandler().getComboListForEvent(myComboEvent);
 
-  uInt32 size = std::min(uInt32(events.size()), 8u);
-  for(uInt32 i = 0; i < size; ++i)
+  size_t size = std::min<size_t>(events.size(), 8);
+  for(size_t i = 0; i < size; ++i)
     myEvents[i]->setSelected("", events[i]);
 
   // Fill any remaining items to 'None'
   if(size < 8)
-    for(uInt32 i = size; i < 8; ++i)
+    for(size_t i = size; i < 8; ++i)
       myEvents[i]->setSelected("None", "-1");
 }
 
