@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -17,11 +17,27 @@
 
 #include "Dialog.hxx"
 #include "CommandDialog.hxx"
+#include "MinUICommandDialog.hxx"
 #include "CommandMenu.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CommandMenu::CommandMenu(OSystem& osystem)
   : DialogContainer(osystem)
 {
-  myBaseDialog = new CommandDialog(myOSystem, *this);
+  if (osystem.settings().getBool("minimal_ui"))
+    myBaseDialog = new MinUICommandDialog(myOSystem, *this);
+  else
+    myBaseDialog = new CommandDialog(myOSystem, *this);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+CommandMenu::~CommandMenu()
+{
+  delete myBaseDialog;  myBaseDialog = nullptr;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Dialog* CommandMenu::baseDialog()
+{
+  return myBaseDialog;
 }
