@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -31,17 +31,7 @@ ContextMenu::ContextMenu(GuiObject* boss, const GUI::Font& font,
   : Dialog(boss->instance(), boss->parent(), font),
     CommandSender(boss),
     _rowHeight(font.getLineHeight()),
-    _firstEntry(0),
-    _numEntries(0),
-    _selectedOffset(0),
-    _selectedItem(-1),
-    _showScroll(false),
-    _isScrolling(false),
-    _scrollUpColor(kColor),
-    _scrollDnColor(kColor),
     _cmd(cmd),
-    _xorig(0),
-    _yorig(0),
     _maxWidth(width)
 {
   addItems(items);
@@ -68,10 +58,15 @@ void ContextMenu::addItems(const VariantList& items)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ContextMenu::show(uInt32 x, uInt32 y, int item)
+void ContextMenu::show(uInt32 x, uInt32 y, const Common::Rect& bossRect, int item)
 {
-  _xorig = x;
-  _yorig = y;
+  uInt32 scale = instance().frameBuffer().hidpiScaleFactor();
+  _xorig = bossRect.x() + x * scale;
+  _yorig = bossRect.y() + y * scale;
+
+  // Only show menu if we're inside the visible area
+  if(!bossRect.contains(_xorig, _yorig))
+    return;
 
   recalc(instance().frameBuffer().imageRect());
   open();
@@ -82,26 +77,22 @@ void ContextMenu::show(uInt32 x, uInt32 y, int item)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ContextMenu::center()
 {
-  // Make sure the menu is exactly where it should be, in case the image
-  // offset has changed
-  const GUI::Rect& image = instance().frameBuffer().imageRect();
-  recalc(image);
-  uInt32 x = image.x() + _xorig;
-  uInt32 y = image.y() + _yorig;
-  uInt32 tx = image.x() + image.width();
-  uInt32 ty = image.y() + image.height();
-  if(x + _w > tx) x -= (x + _w - tx);
-  if(y + _h > ty) y -= (y + _h - ty);
+  // First set position according to original coordinates
+  surface().setDstPos(_xorig, _yorig);
 
-  surface().setDstPos(x, y);
+  // Now make sure that the entire menu can fit inside the screen bounds
+  // If not, we reset its position
+  if(!instance().frameBuffer().screenRect().contains(
+      _xorig, _yorig, surface().dstRect()))
+    surface().setDstPos(_xorig, _yorig);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ContextMenu::recalc(const GUI::Rect& image)
+void ContextMenu::recalc(const Common::Rect& image)
 {
   // Now is the time to adjust the height
   // If it's higher than the screen, we need to scroll through
-  uInt32 maxentries = std::min(18u, (image.height() - 2) / _rowHeight);
+  uInt32 maxentries = std::min<uInt32>(18, (image.h() - 2) / _rowHeight);
   if(_entries.size() > maxentries)
   {
     // We show two less than the max, so we have room for two scroll buttons
@@ -279,28 +270,28 @@ void ContextMenu::handleMouseWheel(int x, int y, int direction)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ContextMenu::handleKeyDown(StellaKey key, StellaMod mod)
+void ContextMenu::handleKeyDown(StellaKey key, StellaMod mod, bool repeated)
 {
-  handleEvent(instance().eventHandler().eventForKey(key, kMenuMode));
+  handleEvent(instance().eventHandler().eventForKey(EventMode::kMenuMode, key, mod));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ContextMenu::handleJoyDown(int stick, int button)
+void ContextMenu::handleJoyUp(int stick, int button)
 {
-  handleEvent(instance().eventHandler().eventForJoyButton(stick, button, kMenuMode));
+  handleEvent(instance().eventHandler().eventForJoyButton(EventMode::kMenuMode, stick, button));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ContextMenu::handleJoyAxis(int stick, int axis, int value)
+void ContextMenu::handleJoyAxis(int stick, JoyAxis axis, JoyDir adir, int button)
 {
-  if(value != 0)  // we don't care about 'axis up' events
-    handleEvent(instance().eventHandler().eventForJoyAxis(stick, axis, value, kMenuMode));
+  if(adir != JoyDir::NONE)  // we don't care about 'axis off' events
+    handleEvent(instance().eventHandler().eventForJoyAxis(EventMode::kMenuMode, stick, axis, adir, button));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool ContextMenu::handleJoyHat(int stick, int hat, JoyHat value)
+bool ContextMenu::handleJoyHat(int stick, int hat, JoyHatDir hdir, int button)
 {
-  handleEvent(instance().eventHandler().eventForJoyHat(stick, hat, value, kMenuMode));
+  handleEvent(instance().eventHandler().eventForJoyHat(EventMode::kMenuMode, stick, hat, hdir, button));
   return true;
 }
 
@@ -520,7 +511,7 @@ void ContextMenu::scrollDown(int distance)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ContextMenu::drawDialog()
 {
-  static uInt32 up_arrow[8] = {
+  static constexpr std::array<uInt32, 8> up_arrow = {
     0b00011000,
     0b00011000,
     0b00111100,
@@ -530,7 +521,7 @@ void ContextMenu::drawDialog()
     0b11111111,
     0b11111111
   };
-  static uInt32 down_arrow[8] = {
+  static constexpr std::array<uInt32, 8> down_arrow = {
     0b11111111,
     0b11111111,
     0b01111110,
@@ -558,7 +549,7 @@ void ContextMenu::drawDialog()
   if(_showScroll)
   {
     s.hLine(x, y+_rowHeight-1, w+2, kColor);
-    s.drawBitmap(up_arrow, ((_w-_x)>>1)-4, (_rowHeight>>1)+y-4, _scrollUpColor, 8);
+    s.drawBitmap(up_arrow.data(), ((_w-_x)>>1)-4, (_rowHeight>>1)+y-4, _scrollUpColor, 8);
     y += _rowHeight;
     offset--;
   }
@@ -576,7 +567,7 @@ void ContextMenu::drawDialog()
   if(_showScroll)
   {
     s.hLine(x, y, w+2, kColor);
-    s.drawBitmap(down_arrow, ((_w-_x)>>1)-4, (_rowHeight>>1)+y-4, _scrollDnColor, 8);
+    s.drawBitmap(down_arrow.data(), ((_w-_x)>>1)-4, (_rowHeight>>1)+y-4, _scrollDnColor, 8);
   }
 
   setDirty();
