@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -33,6 +33,7 @@
 #include "Console.hxx"
 #include "TIA.hxx"
 #include "OSystem.hxx"
+#include "EventHandler.hxx"
 #include "StateManager.hxx"
 #include "RewindManager.hxx"
 #include "M6502.hxx"
@@ -55,8 +56,7 @@ DeveloperDialog::DeveloperDialog(OSystem& osystem, DialogContainer& parent,
   int xpos, ypos;
 
   // Set real dimensions
-  _w = std::min(53 * fontWidth + 10, max_w);
-  _h = std::min(15 * (lineHeight + VGAP) + 14 + _th, max_h);
+  setSize(54 * fontWidth + 10, 16 * (lineHeight + VGAP) + 14 + _th, max_w, max_h);
 
   // The tab widget
   xpos = 2; ypos = 4;
@@ -64,6 +64,7 @@ DeveloperDialog::DeveloperDialog(OSystem& osystem, DialogContainer& parent,
   addTabWidget(myTab);
 
   addEmulationTab(font);
+  addTiaTab(font);
   addVideoTab(font);
   addTimeMachineTab(font);
   addDebuggerTab(font);
@@ -87,16 +88,16 @@ void DeveloperDialog::addEmulationTab(const GUI::Font& font)
   int lineHeight = font.getLineHeight();
   WidgetArray wid;
   VariantList items;
-  int tabID = myTab->addTab("Emulation");
+  int tabID = myTab->addTab(" Emulation ", TabWidget::AUTO_WIDTH);
 
   // settings set
-  mySettingsGroup0 = new RadioButtonGroup();
+  mySettingsGroupEmulation = new RadioButtonGroup();
   RadioButtonWidget* r = new RadioButtonWidget(myTab, font, HBORDER, ypos + 1,
-                                               "Player settings", mySettingsGroup0, kPlrSettings);
+                                               "Player settings", mySettingsGroupEmulation, kPlrSettings);
   wid.push_back(r);
   ypos += lineHeight + VGAP;
   r = new RadioButtonWidget(myTab, font, HBORDER, ypos + 1,
-                            "Developer settings", mySettingsGroup0, kDevSettings);
+                            "Developer settings", mySettingsGroupEmulation, kDevSettings);
   wid.push_back(r);
   ypos += lineHeight + VGAP * 1;
 
@@ -137,11 +138,10 @@ void DeveloperDialog::addEmulationTab(const GUI::Font& font)
   wid.push_back(myRandomizeCPULabel);
 
   int xpos = myRandomizeCPULabel->getRight() + 10;
-  const char* const cpuregs[] = { "SP", "A", "X", "Y", "PS" };
   for(int i = 0; i < 5; ++i)
   {
     myRandomizeCPUWidget[i] = new CheckboxWidget(myTab, font, xpos, ypos + 1,
-                                           cpuregs[i], kRandCPUID);
+                                           ourCPUregs[i], kRandCPUID);
     wid.push_back(myRandomizeCPUWidget[i]);
     xpos += CheckboxWidget::boxSize() + font.getStringWidth("XX") + 20;
   }
@@ -158,6 +158,11 @@ void DeveloperDialog::addEmulationTab(const GUI::Font& font)
                                            "Break on reads from write ports");
   wid.push_back(myRWPortBreakWidget);
   ypos += lineHeight + VGAP;
+
+  myWRPortBreakWidget = new CheckboxWidget(myTab, font, HBORDER + INDENT * 1, ypos + 1,
+                                           "Break on writes to read ports");
+  wid.push_back(myWRPortBreakWidget);
+  ypos += lineHeight + VGAP;
 #endif
 
   // Thumb ARM emulation exception
@@ -175,6 +180,92 @@ void DeveloperDialog::addEmulationTab(const GUI::Font& font)
   addToFocusList(wid, myTab, tabID);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DeveloperDialog::addTiaTab(const GUI::Font& font)
+{
+  const int HBORDER = 10;
+  const int INDENT = 16 + 4;
+  const int VBORDER = 8;
+  const int VGAP = 4;
+  int ypos = VBORDER;
+  int lineHeight = font.getLineHeight();
+  int pwidth = font.getStringWidth("Faulty Cosmic Ark stars");
+  WidgetArray wid;
+  VariantList items;
+  int tabID = myTab->addTab("  TIA  ", TabWidget::AUTO_WIDTH);
+
+  wid.clear();
+
+  // settings set
+  mySettingsGroupTia = new RadioButtonGroup();
+  RadioButtonWidget* r = new RadioButtonWidget(myTab, font, HBORDER, ypos + 1,
+                                               "Player settings", mySettingsGroupTia, kPlrSettings);
+  wid.push_back(r);
+  ypos += lineHeight + VGAP;
+  r = new RadioButtonWidget(myTab, font, HBORDER, ypos + 1,
+                            "Developer settings", mySettingsGroupTia, kDevSettings);
+  wid.push_back(r);
+  ypos += lineHeight + VGAP * 1;
+
+  items.clear();
+  VarList::push_back(items, "Standard", "standard");
+  VarList::push_back(items, "Faulty Kool-Aid Man", "koolaidman");
+  VarList::push_back(items, "Faulty Cosmic Ark stars", "cosmicark");
+  VarList::push_back(items, "Glitched Pesco", "pesco");
+  VarList::push_back(items, "Glitched Quick Step!", "quickstep");
+  VarList::push_back(items, "Glitched He-Man title", "heman");
+  VarList::push_back(items, "Custom", "custom");
+  myTIATypeWidget = new PopUpWidget(myTab, font, HBORDER + INDENT, ypos - 1,
+                                    pwidth, lineHeight, items, "Chip type ", 0, kTIAType);
+  wid.push_back(myTIATypeWidget);
+  ypos += lineHeight + VGAP * 1;
+
+  myInvPhaseLabel = new StaticTextWidget(myTab, font, HBORDER + INDENT * 2, ypos + 1,
+                                         "Inverted HMOVE clock phase for");
+  wid.push_back(myInvPhaseLabel);
+  ypos += lineHeight + VGAP * 1;
+
+  myPlInvPhaseWidget = new CheckboxWidget(myTab, font, HBORDER + INDENT * 3, ypos + 1,
+                                          "Players");
+  wid.push_back(myPlInvPhaseWidget);
+
+  myMsInvPhaseWidget = new CheckboxWidget(myTab, font, myPlInvPhaseWidget->getRight() + 20, ypos + 1,
+                                          "Missiles");
+  wid.push_back(myMsInvPhaseWidget);
+
+  myBlInvPhaseWidget = new CheckboxWidget(myTab, font, myMsInvPhaseWidget->getRight() + 20, ypos + 1,
+                                          "Ball");
+  wid.push_back(myBlInvPhaseWidget);
+  ypos += lineHeight + VGAP * 1;
+
+  myPlayfieldLabel = new StaticTextWidget(myTab, font, HBORDER + INDENT * 2, ypos + 1,
+                                         "Delayed playfield");
+  wid.push_back(myPlayfieldLabel);
+  ypos += lineHeight + VGAP * 1;
+
+  myPFBitsWidget = new CheckboxWidget(myTab, font, HBORDER + INDENT * 3, ypos + 1, "Bits");
+  wid.push_back(myPFBitsWidget);
+  //ypos += lineHeight + VGAP * 1;
+
+  myPFColorWidget = new CheckboxWidget(myTab, font, myPFBitsWidget->getRight() + 20, ypos + 1, "Color");
+  wid.push_back(myPFColorWidget);
+  ypos += lineHeight + VGAP * 1;
+
+  mySwapLabel = new StaticTextWidget(myTab, font, HBORDER + INDENT * 2, ypos + 1,
+                                     "Delayed VDEL" + ELLIPSIS + " swap for");
+  wid.push_back(mySwapLabel);
+  ypos += lineHeight + VGAP * 1;
+
+  myPlSwapWidget = new CheckboxWidget(myTab, font, HBORDER + INDENT * 3, ypos + 1, "Players");
+  wid.push_back(myPlSwapWidget);
+
+  myBlSwapWidget = new CheckboxWidget(myTab, font, myPlSwapWidget->getRight() + 20, ypos + 1, "Ball");
+  wid.push_back(myBlSwapWidget);
+
+  // Add items for tab 2
+  addToFocusList(wid, myTab, tabID);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DeveloperDialog::addVideoTab(const GUI::Font& font)
 {
@@ -189,18 +280,18 @@ void DeveloperDialog::addVideoTab(const GUI::Font& font)
   int pwidth = font.getMaxCharWidth() * 6;
   WidgetArray wid;
   VariantList items;
-  int tabID = myTab->addTab("Video");
+  int tabID = myTab->addTab(" Video ", TabWidget::AUTO_WIDTH);
 
   wid.clear();
 
   // settings set
-  mySettingsGroup1 = new RadioButtonGroup();
+  mySettingsGroupVideo = new RadioButtonGroup();
   RadioButtonWidget* r = new RadioButtonWidget(myTab, font, HBORDER, ypos + 1,
-                                               "Player settings", mySettingsGroup1, kPlrSettings);
+                                               "Player settings", mySettingsGroupVideo, kPlrSettings);
   wid.push_back(r);
   ypos += lineHeight + VGAP;
   r = new RadioButtonWidget(myTab, font, HBORDER, ypos + 1,
-                            "Developer settings", mySettingsGroup1, kDevSettings);
+                            "Developer settings", mySettingsGroupVideo, kDevSettings);
   wid.push_back(r);
   ypos += lineHeight + VGAP * 1;
 
@@ -212,7 +303,7 @@ void DeveloperDialog::addVideoTab(const GUI::Font& font)
                                          myTVJitterWidget->getRight() + fontWidth * 3, ypos - 1,
                                          "Recovery ", 0, kTVJitterChanged);
   myTVJitterRecWidget->setMinValue(1); myTVJitterRecWidget->setMaxValue(20);
-  myTVJitterRecWidget->setTickmarkInterval(5);
+  myTVJitterRecWidget->setTickmarkIntervals(5);
   wid.push_back(myTVJitterRecWidget);
   myTVJitterRecLabelWidget = new StaticTextWidget(myTab, font,
                                                   myTVJitterRecWidget->getRight() + 4,
@@ -239,7 +330,7 @@ void DeveloperDialog::addVideoTab(const GUI::Font& font)
   VarList::push_back(items, "Purple", "p");
   VarList::push_back(items, "Blue", "b");
 
-  static constexpr int dbg_cmds[DEBUG_COLORS] = {
+  static constexpr std::array<int, DEBUG_COLORS> dbg_cmds = {
     kP0ColourChangedCmd,  kM0ColourChangedCmd,  kP1ColourChangedCmd,
     kM1ColourChangedCmd,  kPFColourChangedCmd,  kBLColourChangedCmd
   };
@@ -275,7 +366,7 @@ void DeveloperDialog::addVideoTab(const GUI::Font& font)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DeveloperDialog::addTimeMachineTab(const GUI::Font& font)
 {
-  const string INTERVALS[NUM_INTERVALS] = {
+  const std::array<string, NUM_INTERVALS> INTERVALS = {
     " 1 frame",
     " 3 frames",
     "10 frames",
@@ -284,7 +375,7 @@ void DeveloperDialog::addTimeMachineTab(const GUI::Font& font)
     " 3 seconds",
     "10 seconds"
   };
-  const string INT_SETTINGS[NUM_INTERVALS] = {
+  const std::array<string, NUM_INTERVALS> INT_SETTINGS = {
     "1f",
     "3f",
     "10f",
@@ -293,7 +384,7 @@ void DeveloperDialog::addTimeMachineTab(const GUI::Font& font)
     "3s",
     "10s"
   };
-  const string HORIZONS[NUM_HORIZONS] = {
+  const std::array<string, NUM_HORIZONS> HORIZONS = {
     " 3 seconds",
     "10 seconds",
     "30 seconds",
@@ -303,7 +394,7 @@ void DeveloperDialog::addTimeMachineTab(const GUI::Font& font)
     "30 minutes",
     "60 minutes"
   };
-  const string HOR_SETTINGS[NUM_HORIZONS] = {
+  const std::array<string, NUM_HORIZONS> HOR_SETTINGS = {
     "3s",
     "10s",
     "30s",
@@ -324,16 +415,16 @@ void DeveloperDialog::addTimeMachineTab(const GUI::Font& font)
     lwidth = fontWidth * 11;
   WidgetArray wid;
   VariantList items;
-  int tabID = myTab->addTab("Time Machine");
+  int tabID = myTab->addTab(" Time Machine ", TabWidget::AUTO_WIDTH);
 
   // settings set
-  mySettingsGroup2 = new RadioButtonGroup();
+  mySettingsGroupTM = new RadioButtonGroup();
   RadioButtonWidget* r = new RadioButtonWidget(myTab, font, HBORDER, ypos + 1,
-                                               "Player settings", mySettingsGroup2, kPlrSettings);
+                                               "Player settings", mySettingsGroupTM, kPlrSettings);
   wid.push_back(r);
   ypos += lineHeight + VGAP;
   r = new RadioButtonWidget(myTab, font, HBORDER, ypos + 1,
-                            "Developer settings", mySettingsGroup2, kDevSettings);
+                            "Developer settings", mySettingsGroupTM, kDevSettings);
   wid.push_back(r);
   ypos += lineHeight + VGAP * 1;
 
@@ -346,18 +437,26 @@ void DeveloperDialog::addTimeMachineTab(const GUI::Font& font)
   myStateSizeWidget = new SliderWidget(myTab, font, HBORDER + INDENT * 2, ypos - 1, swidth, lineHeight,
                                        "Buffer size (*)   ", 0, kSizeChanged, lwidth, " states");
   myStateSizeWidget->setMinValue(20);
+#ifdef RETRON77
+  myStateSizeWidget->setMaxValue(100);
+#else
   myStateSizeWidget->setMaxValue(1000);
+#endif
   myStateSizeWidget->setStepValue(20);
-  myStateSizeWidget->setTickmarkInterval(5);
+  myStateSizeWidget->setTickmarkIntervals(5);
   wid.push_back(myStateSizeWidget);
   ypos += lineHeight + VGAP;
 
   myUncompressedWidget = new SliderWidget(myTab, font, HBORDER + INDENT * 2, ypos - 1, swidth, lineHeight,
                                           "Uncompressed size ", 0, kUncompressedChanged, lwidth, " states");
   myUncompressedWidget->setMinValue(0);
+#ifdef RETRON77
+  myUncompressedWidget->setMaxValue(100);
+#else
   myUncompressedWidget->setMaxValue(1000);
+#endif
   myUncompressedWidget->setStepValue(20);
-  myUncompressedWidget->setTickmarkInterval(5);
+  myUncompressedWidget->setTickmarkIntervals(5);
   wid.push_back(myUncompressedWidget);
   ypos += lineHeight + VGAP;
 
@@ -377,6 +476,29 @@ void DeveloperDialog::addTimeMachineTab(const GUI::Font& font)
                                          lineHeight, items, "Horizon         ~ ", 0, kHorizonChanged);
   wid.push_back(myStateHorizonWidget);
 
+  ypos += lineHeight + VGAP * 2;
+  new StaticTextWidget(myTab, font, HBORDER, ypos + 1,
+    "When entering/exiting emulation:");
+  ypos += lineHeight + VGAP;
+  mySaveOnExitGroup = new RadioButtonGroup();
+  r = new RadioButtonWidget(myTab, font, HBORDER + INDENT, ypos + 1,
+    "Do nothing", mySaveOnExitGroup);
+  wid.push_back(r);
+  ypos += lineHeight + VGAP;
+  r = new RadioButtonWidget(myTab, font, HBORDER + INDENT, ypos + 1,
+    "Save current state in current slot", mySaveOnExitGroup);
+  wid.push_back(r);
+  ypos += lineHeight + VGAP;
+  r = new RadioButtonWidget(myTab, font, HBORDER + INDENT, ypos + 1,
+    "Load/save all Time Machine states", mySaveOnExitGroup);
+  wid.push_back(r);
+  ypos += lineHeight + VGAP;
+
+
+  myAutoSlotWidget = new CheckboxWidget(myTab, font, HBORDER, ypos + 1, "Automatically change save state slots");
+  wid.push_back(myAutoSlotWidget);
+  ypos += lineHeight + VGAP;
+
   // Add message concerning usage
   const GUI::Font& infofont = instance().frameBuffer().infoFont();
   ypos = myTab->getHeight() - 5 - fontHeight - infofont.getFontHeight() - 10;
@@ -388,7 +510,7 @@ void DeveloperDialog::addTimeMachineTab(const GUI::Font& font)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DeveloperDialog::addDebuggerTab(const GUI::Font& font)
 {
-  int tabID = myTab->addTab("Debugger");
+  int tabID = myTab->addTab(" Debugger ", TabWidget::AUTO_WIDTH);
   WidgetArray wid;
 
 #ifdef DEBUGGER_SUPPORT
@@ -401,7 +523,7 @@ void DeveloperDialog::addDebuggerTab(const GUI::Font& font)
     fontHeight = font.getFontHeight(),
     lineHeight = font.getLineHeight();
   int xpos, ypos, pwidth;
-  const GUI::Size& ds = instance().frameBuffer().desktopSize();
+  const Common::Size& ds = instance().frameBuffer().desktopSize();
 
   xpos = HBORDER;
   ypos = VBORDER;
@@ -439,7 +561,7 @@ void DeveloperDialog::addDebuggerTab(const GUI::Font& font)
   myDebuggerWidthSlider->setMaxValue(ds.w);
   myDebuggerWidthSlider->setStepValue(10);
   // one tickmark every ~100 pixel
-  myDebuggerWidthSlider->setTickmarkInterval((ds.w - DebuggerDialog::kSmallFontMinW + 50) / 100);
+  myDebuggerWidthSlider->setTickmarkIntervals((ds.w - DebuggerDialog::kSmallFontMinW + 50) / 100);
   wid.push_back(myDebuggerWidthSlider);
   ypos += lineHeight + VGAP;
 
@@ -449,7 +571,7 @@ void DeveloperDialog::addDebuggerTab(const GUI::Font& font)
   myDebuggerHeightSlider->setMaxValue(ds.h);
   myDebuggerHeightSlider->setStepValue(10);
   // one tickmark every ~100 pixel
-  myDebuggerHeightSlider->setTickmarkInterval((ds.h - DebuggerDialog::kSmallFontMinH + 50) / 100);
+  myDebuggerHeightSlider->setTickmarkIntervals((ds.h - DebuggerDialog::kSmallFontMinH + 50) / 100);
   wid.push_back(myDebuggerHeightSlider);
   ypos += lineHeight + VGAP * 4;
 
@@ -462,19 +584,15 @@ void DeveloperDialog::addDebuggerTab(const GUI::Font& font)
   ypos = myTab->getHeight() - 5 - fontHeight - infofont.getFontHeight() - 10;
   new StaticTextWidget(myTab, infofont, HBORDER, ypos, "(*) Changes require a ROM reload");
 
+#if defined(DEBUGGER_SUPPORT) && defined(WINDOWED_SUPPORT)
   // Debugger is only realistically available in windowed modes 800x600 or greater
   // (and when it's actually been compiled into the app)
-  bool debuggerAvailable =
-#if defined(DEBUGGER_SUPPORT) && defined(WINDOWED_SUPPORT)
-    (ds.w >= 800 && ds.h >= 600);  // TODO - maybe this logic can disappear?
-#else
-    false;
-#endif
-  if(!debuggerAvailable)
+  if(ds.w < 800 || ds.h < 600)  // TODO - maybe this logic can disappear?
   {
-    myDebuggerWidthSlider->clearFlags(WIDGET_ENABLED);
-    myDebuggerHeightSlider->clearFlags(WIDGET_ENABLED);
+    myDebuggerWidthSlider->clearFlags(Widget::FLAG_ENABLED);
+    myDebuggerHeightSlider->clearFlags(Widget::FLAG_ENABLED);
   }
+#endif
 #else
   new StaticTextWidget(myTab, font, 0, 20, _w - 20, font.getFontHeight(),
                        "Debugger support not included", TextAlign::Center);
@@ -486,7 +604,8 @@ void DeveloperDialog::addDebuggerTab(const GUI::Font& font)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DeveloperDialog::loadSettings(SettingsSet set)
 {
-  const string& prefix = set == SettingsSet::player ? "plr." : "dev.";
+  bool devSettings = set == SettingsSet::developer;
+  const string& prefix = devSettings ? "dev." : "plr.";
 
   myFrameStats[set] = instance().settings().getBool(prefix + "stats");
   myConsole[set] = instance().settings().getString(prefix + "console") == "7800" ? 1 : 0;
@@ -495,16 +614,28 @@ void DeveloperDialog::loadSettings(SettingsSet set)
   myRandomizeRAM[set] = instance().settings().getBool(prefix + "ramrandom");
   myRandomizeCPU[set] = instance().settings().getString(prefix + "cpurandom");
   // Undriven TIA pins
-  myUndrivenPins[set] = instance().settings().getBool(prefix + "tiadriven");
+  myUndrivenPins[set] = devSettings ? instance().settings().getBool("dev.tiadriven") : false;
 #ifdef DEBUGGER_SUPPORT
   // Read from write ports break
-  myRWPortBreak[set] = instance().settings().getBool(prefix + "rwportbreak");
+  myRWPortBreak[set] = devSettings ? instance().settings().getBool("dev.rwportbreak") : false;
+  // Write to read ports break
+  myWRPortBreak[set] = devSettings ? instance().settings().getBool("dev.wrportbreak") : false;
 #endif
   // Thumb ARM emulation exception
-  myThumbException[set] = instance().settings().getBool(prefix + "thumb.trapfatal");
+  myThumbException[set] = devSettings ? instance().settings().getBool("dev.thumb.trapfatal") : false;
   // AtariVox/SaveKey EEPROM access
   myEEPROMAccess[set] = instance().settings().getBool(prefix + "eepromaccess");
 
+  // TIA tab
+  myTIAType[set] = devSettings ? instance().settings().getString("dev.tia.type") : "standard";
+  myPlInvPhase[set] = devSettings ? instance().settings().getBool("dev.tia.plinvphase") : false;
+  myMsInvPhase[set] = devSettings ? instance().settings().getBool("dev.tia.msinvphase") : false;
+  myBlInvPhase[set] = devSettings ? instance().settings().getBool("dev.tia.blinvphase") : false;
+  myPFBits[set] = devSettings ? instance().settings().getBool("dev.tia.delaypfbits") : false;
+  myPFColor[set] = devSettings ? instance().settings().getBool("dev.tia.delaypfcolor") : false;
+  myPlSwap[set] = devSettings ? instance().settings().getBool("dev.tia.delayplswap") : false;
+  myBlSwap[set] = devSettings ? instance().settings().getBool("dev.tia.delayblswap") : false;
+
   // Debug colors
   myDebugColors[set] = instance().settings().getBool(prefix + "debugcolors");
   // PAL color-loss effect
@@ -519,30 +650,59 @@ void DeveloperDialog::loadSettings(SettingsSet set)
   myUncompressed[set] = instance().settings().getInt(prefix + "tm.uncompressed");
   myStateInterval[set] = instance().settings().getString(prefix + "tm.interval");
   myStateHorizon[set] = instance().settings().getString(prefix + "tm.horizon");
+
+
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DeveloperDialog::saveSettings(SettingsSet set)
 {
-  const string& prefix = set == SettingsSet::player ? "plr." : "dev.";
+  bool devSettings = set == SettingsSet::developer;
+  const string& prefix = devSettings ? "dev." : "plr.";
 
   instance().settings().setValue(prefix + "stats", myFrameStats[set]);
   instance().settings().setValue(prefix + "console", myConsole[set] == 1 ? "7800" : "2600");
+  if(instance().hasConsole())
+    instance().eventHandler().set7800Mode();
+
   // Randomization
   instance().settings().setValue(prefix + "bankrandom", myRandomBank[set]);
   instance().settings().setValue(prefix + "ramrandom", myRandomizeRAM[set]);
   instance().settings().setValue(prefix + "cpurandom", myRandomizeCPU[set]);
   // Undriven TIA pins
-  instance().settings().setValue(prefix + "tiadriven", myUndrivenPins[set]);
+  if(devSettings)
+    instance().settings().setValue("dev.tiadriven", myUndrivenPins[set]);
 #ifdef DEBUGGER_SUPPORT
-  // Read from write ports break
-  instance().settings().setValue(prefix + "rwportbreak", myRWPortBreak[set]);
+  if (devSettings)
+  {
+    // Read from write ports break
+    instance().settings().setValue("dev.rwportbreak", myRWPortBreak[set]);
+    // Write to read ports break
+    instance().settings().setValue("dev.wrportbreak", myWRPortBreak[set]);
+  }
 #endif
-  // Thumb ARM emulation exception
-  instance().settings().setValue(prefix + "thumb.trapfatal", myThumbException[set]);
+  if(devSettings)
+    // Thumb ARM emulation exception
+    instance().settings().setValue("dev.thumb.trapfatal", myThumbException[set]);
   // AtariVox/SaveKey EEPROM access
   instance().settings().setValue(prefix + "eepromaccess", myEEPROMAccess[set]);
 
+  // TIA tab
+  if (devSettings)
+  {
+    instance().settings().setValue("dev.tia.type", myTIAType[set]);
+    if (BSPF::equalsIgnoreCase("custom", myTIAType[set]))
+    {
+      instance().settings().setValue("dev.tia.plinvphase", myPlInvPhase[set]);
+      instance().settings().setValue("dev.tia.msinvphase", myMsInvPhase[set]);
+      instance().settings().setValue("dev.tia.blinvphase", myBlInvPhase[set]);
+      instance().settings().setValue("dev.tia.delaypfbits", myPFBits[set]);
+      instance().settings().setValue("dev.tia.delaypfcolor", myPFColor[set]);
+      instance().settings().setValue("dev.tia.delayplswap", myPlSwap[set]);
+      instance().settings().setValue("dev.tia.delayblswap", myBlSwap[set]);
+    }
+  }
+
   // Debug colors
   instance().settings().setValue(prefix + "debugcolors", myDebugColors[set]);
   // PAL color loss
@@ -568,22 +728,32 @@ void DeveloperDialog::getWidgetStates(SettingsSet set)
   myRandomBank[set] = myRandomBankWidget->getState();
   myRandomizeRAM[set] = myRandomizeRAMWidget->getState();
   string cpurandom;
-  const char* const cpuregs[] = { "S", "A", "X", "Y", "P" };
   for(int i = 0; i < 5; ++i)
     if(myRandomizeCPUWidget[i]->getState())
-      cpurandom += cpuregs[i];
+      cpurandom += ourCPUregs[i];
   myRandomizeCPU[set] = cpurandom;
   // Undriven TIA pins
   myUndrivenPins[set] = myUndrivenPinsWidget->getState();
 #ifdef DEBUGGER_SUPPORT
   // Read from write ports break
   myRWPortBreak[set] = myRWPortBreakWidget->getState();
+  myWRPortBreak[set] = myWRPortBreakWidget->getState();
 #endif
   // Thumb ARM emulation exception
   myThumbException[set] = myThumbExceptionWidget->getState();
   // AtariVox/SaveKey EEPROM access
   myEEPROMAccess[set] = myEEPROMAccessWidget->getState();
 
+  // TIA tab
+  myTIAType[set] = myTIATypeWidget->getSelectedTag().toString();
+  myPlInvPhase[set] = myPlInvPhaseWidget->getState();
+  myMsInvPhase[set] = myMsInvPhaseWidget->getState();
+  myBlInvPhase[set] = myBlInvPhaseWidget->getState();
+  myPFBits[set] = myPFBitsWidget->getState();
+  myPFColor[set] = myPFColorWidget->getState();
+  myPlSwap[set] = myPlSwapWidget->getState();
+  myBlSwap[set] = myBlSwapWidget->getState();
+
   // Debug colors
   myDebugColors[set] = myDebugColorsWidget->getState();
   // PAL color-loss effect
@@ -596,7 +766,6 @@ void DeveloperDialog::getWidgetStates(SettingsSet set)
   myTimeMachine[set] = myTimeMachineWidget->getState();
   myStateSize[set] = myStateSizeWidget->getValue();
   myUncompressed[set] = myUncompressedWidget->getValue();
-  myStateInterval[set] = myStateIntervalWidget->getSelected();
   myStateInterval[set] = myStateIntervalWidget->getSelectedTag().toString();
   myStateHorizon[set] = myStateHorizonWidget->getSelectedTag().toString();
 }
@@ -611,22 +780,25 @@ void DeveloperDialog::setWidgetStates(SettingsSet set)
   myRandomizeRAMWidget->setState(myRandomizeRAM[set]);
 
   const string& cpurandom = myRandomizeCPU[set];
-  const char* const cpuregs[] = { "S", "A", "X", "Y", "P" };
   for(int i = 0; i < 5; ++i)
-    myRandomizeCPUWidget[i]->setState(BSPF::containsIgnoreCase(cpurandom, cpuregs[i]));
+    myRandomizeCPUWidget[i]->setState(BSPF::containsIgnoreCase(cpurandom, ourCPUregs[i]));
   // Undriven TIA pins
   myUndrivenPinsWidget->setState(myUndrivenPins[set]);
 #ifdef DEBUGGER_SUPPORT
   // Read from write ports break
   myRWPortBreakWidget->setState(myRWPortBreak[set]);
+  myWRPortBreakWidget->setState(myWRPortBreak[set]);
 #endif
   // Thumb ARM emulation exception
   myThumbExceptionWidget->setState(myThumbException[set]);
   // AtariVox/SaveKey EEPROM access
   myEEPROMAccessWidget->setState(myEEPROMAccess[set]);
-
   handleConsole();
 
+  // TIA tab
+  myTIATypeWidget->setSelected(myTIAType[set], "standard");
+  handleTia();
+
   // Debug colors
   myDebugColorsWidget->setState(myDebugColors[set]);
   // PAL color-loss effect
@@ -636,7 +808,6 @@ void DeveloperDialog::setWidgetStates(SettingsSet set)
   myTVJitterRecWidget->setValue(myTVJitterRec[set]);
 
   handleTVJitterChange(myTVJitterWidget->getState());
-  handleEnableDebugColors();
 
   // States
   myTimeMachineWidget->setState(myTimeMachine[set]);
@@ -656,25 +827,33 @@ void DeveloperDialog::setWidgetStates(SettingsSet set)
 void DeveloperDialog::loadConfig()
 {
   bool devSettings = instance().settings().getBool("dev.settings");
+  handleSettings(devSettings);
   mySettings = devSettings;
-  mySettingsGroup0->setSelected(devSettings ? 1 : 0);
-  mySettingsGroup1->setSelected(devSettings ? 1 : 0);
-  mySettingsGroup2->setSelected(devSettings ? 1 : 0);
+  mySettingsGroupEmulation->setSelected(devSettings ? 1 : 0);
+  mySettingsGroupTia->setSelected(devSettings ? 1 : 0);
+  mySettingsGroupVideo->setSelected(devSettings ? 1 : 0);
+  mySettingsGroupTM->setSelected(devSettings ? 1 : 0);
 
   // load both setting sets...
   loadSettings(SettingsSet::player);
   loadSettings(SettingsSet::developer);
   // ...and select the current one
-  setWidgetStates(SettingsSet(mySettingsGroup0->getSelected()));
+  setWidgetStates(SettingsSet(mySettingsGroupEmulation->getSelected()));
 
   // Debug colours
   handleDebugColours(instance().settings().getString("tia.dbgcolors"));
 
+  // Save on exit
+  string saveOnExit = instance().settings().getString("saveonexit");
+  mySaveOnExitGroup->setSelected(saveOnExit == "all" ? 2 : saveOnExit == "current" ? 1 : 0);
+  // Automatically change save state slots
+  myAutoSlotWidget->setState(instance().settings().getBool("autoslot"));
+
 #ifdef DEBUGGER_SUPPORT
   uInt32 w, h;
 
   // Debugger size
-  const GUI::Size& ds = instance().settings().getSize("dbg.res");
+  const Common::Size& ds = instance().settings().getSize("dbg.res");
   w = ds.w; h = ds.h;
 
   myDebuggerWidthSlider->setValue(w);
@@ -700,9 +879,9 @@ void DeveloperDialog::loadConfig()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DeveloperDialog::saveConfig()
 {
-  instance().settings().setValue("dev.settings", mySettingsGroup0->getSelected() == SettingsSet::developer);
+  instance().settings().setValue("dev.settings", mySettingsGroupEmulation->getSelected() == SettingsSet::developer);
   // copy current widget status into set...
-  getWidgetStates(SettingsSet(mySettingsGroup0->getSelected()));
+  getWidgetStates(SettingsSet(mySettingsGroupEmulation->getSelected()));
   // ...and save both sets
   saveSettings(SettingsSet::player);
   saveSettings(SettingsSet::developer);
@@ -715,6 +894,19 @@ void DeveloperDialog::saveConfig()
     instance().console().tia().toggleJitter(myTVJitterWidget->getState() ? 1 : 0);
     instance().console().tia().setJitterRecoveryFactor(myTVJitterRecWidget->getValue());
   }
+
+  // TIA tab
+  if(instance().hasConsole())
+  {
+    instance().console().tia().setPlInvertedPhaseClock(myPlInvPhaseWidget->getState());
+    instance().console().tia().setMsInvertedPhaseClock(myMsInvPhaseWidget->getState());
+    instance().console().tia().setBlInvertedPhaseClock(myBlInvPhaseWidget->getState());
+    instance().console().tia().setPFBitsDelay(myPFBitsWidget->getState());
+    instance().console().tia().setPFColorDelay(myPFColorWidget->getState());
+    instance().console().tia().setPlSwapDelay(myPlSwapWidget->getState());
+    instance().console().tia().setBlSwapDelay(myBlSwapWidget->getState());
+  }
+
   handleEnableDebugColors();
   // PAL color loss
   if(instance().hasConsole())
@@ -733,13 +925,20 @@ void DeveloperDialog::saveConfig()
   instance().state().setRewindMode(myTimeMachineWidget->getState() ?
                                    StateManager::Mode::TimeMachine : StateManager::Mode::Off);
 
+  // Save on exit
+  int saveOnExit = mySaveOnExitGroup->getSelected();
+  instance().settings().setValue("saveonexit",
+    saveOnExit == 0 ? "none" : saveOnExit == 1 ? "current" : "all");
+  // Automatically change save state slots
+  instance().settings().setValue("autoslot", myAutoSlotWidget->getState());
+
 #ifdef DEBUGGER_SUPPORT
   // Debugger font style
   instance().settings().setValue("dbg.fontstyle",
                                  myDebuggerFontStyle->getSelectedTag().toString());
   // Debugger size
   instance().settings().setValue("dbg.res",
-                                 GUI::Size(myDebuggerWidthSlider->getValue(),
+                                 Common::Size(myDebuggerWidthSlider->getValue(),
                                  myDebuggerHeightSlider->getValue()));
   // Debugger font size
   instance().settings().setValue("dbg.fontsize", myDebuggerFontSize->getSelectedTag().toString());
@@ -749,17 +948,20 @@ void DeveloperDialog::saveConfig()
   if(instance().hasConsole())
     instance().console().system().m6502().setGhostReadsTrap(myGhostReadsTrapWidget->getState());
 
-  // Read from write ports break
-  if(instance().hasConsole())
+  // Read from write ports and write to read ports breaks
+  if (instance().hasConsole())
+  {
     instance().console().system().m6502().setReadFromWritePortBreak(myRWPortBreakWidget->getState());
+    instance().console().system().m6502().setWriteToReadPortBreak(myWRPortBreakWidget->getState());
+  }
 #endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DeveloperDialog::setDefaults()
 {
-  bool devSettings = mySettingsGroup0->getSelected() == 1;
-  SettingsSet set = SettingsSet(mySettingsGroup0->getSelected());
+  bool devSettings = mySettings;
+  SettingsSet set = SettingsSet(mySettingsGroupEmulation->getSelected());
 
   switch(myTab->getActiveTab())
   {
@@ -775,6 +977,7 @@ void DeveloperDialog::setDefaults()
     #ifdef DEBUGGER_SUPPORT
       // Reads from write ports
       myRWPortBreak[set] = devSettings ? true : false;
+      myWRPortBreak[set] = devSettings ? true : false;
     #endif
       // Thumb ARM emulation exception
       myThumbException[set] = devSettings ? true : false;
@@ -784,7 +987,21 @@ void DeveloperDialog::setDefaults()
       setWidgetStates(set);
       break;
 
-    case 1: // Video
+    case 1: // TIA
+      myTIAType[set] = "standard";
+      // reset "custom" mode
+      myPlInvPhase[set] = devSettings ? true : false;
+      myMsInvPhase[set] = devSettings ? true : false;
+      myBlInvPhase[set] = devSettings ? true : false;
+      myPFBits[set] = devSettings ? true : false;
+      myPFColor[set] = devSettings ? true : false;
+      myPlSwap[set] = devSettings ? true : false;
+      myBlSwap[set] = devSettings ? true : false;
+
+      setWidgetStates(set);
+      break;
+
+    case 2: // Video
       // Jitter
       myTVJitter[set] = true;
       myTVJitterRec[set] = devSettings ? 2 : 10;
@@ -797,17 +1014,19 @@ void DeveloperDialog::setDefaults()
       setWidgetStates(set);
       break;
 
-    case 2: // States
-      myTimeMachine[set] = devSettings ? true : false;
-      myStateSize[set] = 100;
-      myUncompressed[set] = devSettings ? 60 : 30;
+    case 3: // States
+      myTimeMachine[set] = true;
+      myStateSize[set] = devSettings ? 1000 : 200;
+      myUncompressed[set] = devSettings ? 600 : 60;
       myStateInterval[set] = devSettings ? "1f" : "30f";
-      myStateHorizon[set] = devSettings ? "10s" : "10m";
+      myStateHorizon[set] = devSettings ? "30s" : "10m";
 
       setWidgetStates(set);
+      mySaveOnExitGroup->setSelected(0);
+      myAutoSlotWidget->setState(false);
       break;
 
-    case 3: // Debugger options
+    case 4: // Debugger options
     {
 #ifdef DEBUGGER_SUPPORT
       uInt32 w = std::min(instance().frameBuffer().desktopSize().w, uInt32(DebuggerDialog::kMediumFontMinW));
@@ -842,6 +1061,10 @@ void DeveloperDialog::handleCommand(CommandSender* sender, int cmd, int data, in
       handleSettings(true);
       break;
 
+    case kTIAType:
+      handleTia();
+      break;
+
     case kConsole:
       handleConsole();
         break;
@@ -930,14 +1153,23 @@ void DeveloperDialog::handleCommand(CommandSender* sender, int cmd, int data, in
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DeveloperDialog::handleSettings(bool devSettings)
 {
+  myUndrivenPinsWidget->setEnabled(devSettings);
+#ifdef DEBUGGER_SUPPORT
+  myRWPortBreakWidget->setEnabled(devSettings);
+  myWRPortBreakWidget->setEnabled(devSettings);
+#endif
+  myThumbExceptionWidget->setEnabled(devSettings);
+
   if (mySettings != devSettings)
   {
     mySettings = devSettings; // block redundant events first!
-    mySettingsGroup0->setSelected(devSettings ? 1 : 0);
-    mySettingsGroup1->setSelected(devSettings ? 1 : 0);
-    mySettingsGroup2->setSelected(devSettings ? 1 : 0);
+    SettingsSet set = devSettings ? SettingsSet::developer : SettingsSet::player;
+    mySettingsGroupEmulation->setSelected(set);
+    mySettingsGroupTia->setSelected(set);
+    mySettingsGroupVideo->setSelected(set);
+    mySettingsGroupTM->setSelected(set);
     getWidgetStates(devSettings ? SettingsSet::player : SettingsSet::developer);
-    setWidgetStates(devSettings ? SettingsSet::developer : SettingsSet::player);
+    setWidgetStates(set);
   }
 }
 
@@ -969,6 +1201,45 @@ void DeveloperDialog::handleConsole()
     myRandomizeRAMWidget->setState(false);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DeveloperDialog::handleTia()
+{
+  bool enable = BSPF::equalsIgnoreCase("custom", myTIATypeWidget->getSelectedTag().toString());
+
+  myTIATypeWidget->setEnabled(mySettings);
+  myInvPhaseLabel->setEnabled(enable);
+  myPlInvPhaseWidget->setEnabled(enable);
+  myMsInvPhaseWidget->setEnabled(enable);
+  myBlInvPhaseWidget->setEnabled(enable);
+  myPlayfieldLabel->setEnabled(enable);
+  myPFBitsWidget->setEnabled(enable);
+  myPFColorWidget->setEnabled(enable);
+  mySwapLabel->setEnabled(enable);
+  myPlSwapWidget->setEnabled(enable);
+  myBlSwapWidget->setEnabled(enable);
+
+  if(BSPF::equalsIgnoreCase("custom", myTIATypeWidget->getSelectedTag().toString()))
+  {
+    myPlInvPhaseWidget->setState(myPlInvPhase[SettingsSet::developer]);
+    myMsInvPhaseWidget->setState(myMsInvPhase[SettingsSet::developer]);
+    myBlInvPhaseWidget->setState(myBlInvPhase[SettingsSet::developer]);
+    myPFBitsWidget->setState(myPFBits[SettingsSet::developer]);
+    myPFColorWidget->setState(myPFColor[SettingsSet::developer]);
+    myPlSwapWidget->setState(myPlSwap[SettingsSet::developer]);
+    myBlSwapWidget->setState(myBlSwap[SettingsSet::developer]);
+  }
+  else
+  {
+    myPlInvPhaseWidget->setState(BSPF::equalsIgnoreCase("koolaidman", myTIATypeWidget->getSelectedTag().toString()));
+    myMsInvPhaseWidget->setState(BSPF::equalsIgnoreCase("cosmicark", myTIATypeWidget->getSelectedTag().toString()));
+    myBlInvPhaseWidget->setState(false);
+    myPFBitsWidget->setState(BSPF::equalsIgnoreCase("pesco", myTIATypeWidget->getSelectedTag().toString()));
+    myPFColorWidget->setState(BSPF::equalsIgnoreCase("quickstep", myTIATypeWidget->getSelectedTag().toString()));
+    myPlSwapWidget->setState(BSPF::equalsIgnoreCase("heman", myTIATypeWidget->getSelectedTag().toString()));
+    myBlSwapWidget->setState(false);
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DeveloperDialog::handleTimeMachine()
 {
@@ -1118,12 +1389,12 @@ void DeveloperDialog::handleDebugColours(int idx, int color)
 
   if(!instance().hasConsole())
   {
-    myDbgColour[idx]->clearFlags(WIDGET_ENABLED);
-    myDbgColourSwatch[idx]->clearFlags(WIDGET_ENABLED);
+    myDbgColour[idx]->clearFlags(Widget::FLAG_ENABLED);
+    myDbgColourSwatch[idx]->clearFlags(Widget::FLAG_ENABLED);
     return;
   }
 
-  static constexpr ColorId dbg_color[3][DEBUG_COLORS] = {
+  static constexpr BSPF::array2D<ColorId, 3, DEBUG_COLORS> dbg_color = {{
     {
       TIA::FixedColor::NTSC_RED,
       TIA::FixedColor::NTSC_ORANGE,
@@ -1148,16 +1419,16 @@ void DeveloperDialog::handleDebugColours(int idx, int color)
       TIA::FixedColor::SECAM_PURPLE,
       TIA::FixedColor::SECAM_BLUE
     }
-  };
+  }};
 
-  int timing = instance().console().tia().consoleTiming() == ConsoleTiming::ntsc ? 0
-    : instance().console().tia().consoleTiming() == ConsoleTiming::pal ? 1 : 2;
+  int timing = instance().console().timing() == ConsoleTiming::ntsc ? 0
+    : instance().console().timing() == ConsoleTiming::pal ? 1 : 2;
 
   myDbgColourSwatch[idx]->setColor(dbg_color[timing][color]);
   myDbgColour[idx]->setSelectedIndex(color);
 
   // make sure the selected debug colors are all different
-  bool usedCol[DEBUG_COLORS];
+  std::array<bool, DEBUG_COLORS> usedCol;
 
   // identify used colors
   for(int i = 0; i < DEBUG_COLORS; ++i)
@@ -1198,13 +1469,13 @@ void DeveloperDialog::handleDebugColours(const string& colors)
   {
     switch(colors[i])
     {
-      case 'r': handleDebugColours(i, 0);  break;
-      case 'o': handleDebugColours(i, 1);  break;
-      case 'y': handleDebugColours(i, 2);  break;
-      case 'g': handleDebugColours(i, 3);  break;
-      case 'p': handleDebugColours(i, 4);  break;
-      case 'b': handleDebugColours(i, 5);  break;
-      default:                              break;
+      case 'r': handleDebugColours(i, 0); break;
+      case 'o': handleDebugColours(i, 1); break;
+      case 'y': handleDebugColours(i, 2); break;
+      case 'g': handleDebugColours(i, 3); break;
+      case 'p': handleDebugColours(i, 4); break;
+      case 'b': handleDebugColours(i, 5); break;
+      default:                            break;
     }
   }
 }
@@ -1243,3 +1514,8 @@ void DeveloperDialog::handleFontSize()
     myDebuggerHeightSlider->setValue(minH);
 #endif
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+const std::array<string, 5> DeveloperDialog::ourCPUregs = {
+  "SP", "A", "X", "Y", "PS"
+};
