@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -49,23 +49,10 @@ Dialog::Dialog(OSystem& instance, DialogContainer& parent, const GUI::Font& font
                const string& title, int x, int y, int w, int h)
   : GuiObject(instance, parent, *this, x, y, w, h),
     _font(font),
-    _mouseWidget(nullptr),
-    _focusedWidget(nullptr),
-    _dragWidget(nullptr),
-    _defaultWidget(nullptr),
-    _okWidget(nullptr),
-    _cancelWidget(nullptr),
-    _visible(false),
-    _onTop(true),
-    _processCancel(false),
     _title(title),
-    _th(0),
-    _surface(nullptr),
-    _tabID(0),
-    _flags(WIDGET_ENABLED | WIDGET_BORDER | WIDGET_CLEARBG)
+    _flags(Widget::FLAG_ENABLED | Widget::FLAG_BORDER | Widget::FLAG_CLEARBG)
 {
   setTitle(title);
-  setDirty();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -93,16 +80,34 @@ void Dialog::open()
   // Make sure we have a valid surface to draw into
   // Technically, this shouldn't be needed until drawDialog(), but some
   // dialogs cause drawing to occur within loadConfig()
-  if(_surface == nullptr)
+  if (_surface == nullptr)
     _surface = instance().frameBuffer().allocateSurface(_w, _h);
-  parent().addDialog(this);
+  else if (uInt32(_w) > _surface->width() || uInt32(_h) > _surface->height())
+    _surface->resize(_w, _h);
+  _surface->setSrcSize(_w, _h);
+  _layer = parent().addDialog(this);
+
+  // Take hidpi scaling into account
+  const uInt32 scale = instance().frameBuffer().hidpiScaleFactor();
+  _surface->setDstSize(_w * scale, _h * scale);
 
   center();
-  loadConfig();
 
-  // (Re)-build the focus list to use for the widgets which are currently
-  // onscreen
-  buildCurrentFocusList();
+  if(_myTabList.size())
+    // (Re)-build the focus list to use for all widgets of all tabs
+    for(auto& tabfocus : _myTabList)
+      buildCurrentFocusList(tabfocus.widget->getID());
+  else
+    buildCurrentFocusList();
+
+  /*if (!_surface->attributes().blending)
+  {
+    _surface->attributes().blending = true;
+    _surface->attributes().blendalpha = 90;
+    _surface->applyAttributes();
+  }*/
+
+  loadConfig(); // has to be done AFTER (re)building the focus list
 
   _visible = true;
 
@@ -112,7 +117,7 @@ void Dialog::open()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Dialog::close()
 {
-  if (_mouseWidget)
+  if(_mouseWidget)
   {
     _mouseWidget->handleMouseLeft();
     _mouseWidget = nullptr;
@@ -136,16 +141,52 @@ void Dialog::setTitle(const string& title)
   else
     _th = _font.getLineHeight() + 4;
   _h += _th;
-
-  setDirty();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Dialog::center()
 {
-  const GUI::Size& screen = instance().frameBuffer().screenSize();
-  const GUI::Rect& dst = _surface->dstRect();
-  _surface->setDstPos((screen.w - dst.width()) >> 1, (screen.h - dst.height()) >> 1);
+  positionAt(instance().settings().getInt("dialogpos"));
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Dialog::positionAt(uInt32 pos)
+{
+  const bool fullscreen = instance().settings().getBool("fullscreen");
+  const double overscan = fullscreen ? instance().settings().getInt("tia.fs_overscan") / 200.0 : 0.0;
+  const Common::Size& screen = instance().frameBuffer().screenSize();
+  const Common::Rect& dst = _surface->dstRect();
+  // shift stacked dialogs
+  Int32 hgap = (screen.w >> 6) * _layer + screen.w * overscan;
+  Int32 vgap = (screen.w >> 6) * _layer + screen.h * overscan;
+  int top = std::min(std::max(0, Int32(screen.h - dst.h())), vgap);
+  int btm = std::max(0, Int32(screen.h - dst.h() - vgap));
+  int left = std::min(std::max(0, Int32(screen.w - dst.w())), hgap);
+  int right = std::max(0, Int32(screen.w - dst.w() - hgap));
+
+  switch (pos)
+  {
+    case 1:
+      _surface->setDstPos(left, top);
+      break;
+
+    case 2:
+      _surface->setDstPos(right, top);
+      break;
+
+    case 3:
+      _surface->setDstPos(right, btm);
+      break;
+
+    case 4:
+      _surface->setDstPos(left, btm);
+      break;
+
+    default:
+      // center
+      _surface->setDstPos((screen.w - dst.w()) >> 1, (screen.h - dst.h()) >> 1);
+      break;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -176,8 +217,12 @@ void Dialog::releaseFocus()
 {
   if(_focusedWidget)
   {
-    _focusedWidget->lostFocus();
-    _focusedWidget = nullptr;
+    // remember focus of all tabs for when dialog is reopened again
+    for(auto& tabfocus : _myTabList)
+      tabfocus.saveCurrentFocus(_focusedWidget);
+
+    //_focusedWidget->lostFocus();
+    //_focusedWidget = nullptr;
   }
 }
 
@@ -188,7 +233,7 @@ void Dialog::addFocusWidget(Widget* w)
     return;
 
   // All focusable widgets should retain focus
-  w->setFlags(WIDGET_RETAIN_FOCUS);
+  w->setFlags(Widget::FLAG_RETAIN_FOCUS);
 
   _myFocus.widget = w;
   _myFocus.list.push_back(w);
@@ -199,7 +244,7 @@ void Dialog::addToFocusList(WidgetArray& list)
 {
   // All focusable widgets should retain focus
   for(const auto& w: list)
-    w->setFlags(WIDGET_RETAIN_FOCUS);
+    w->setFlags(Widget::FLAG_RETAIN_FOCUS);
 
   Vec::append(_myFocus.list, list);
   _focusList = _myFocus.list;
@@ -212,14 +257,14 @@ void Dialog::addToFocusList(WidgetArray& list)
 void Dialog::addToFocusList(WidgetArray& list, TabWidget* w, int tabId)
 {
   // Only add the list if the tab actually exists
-  if(!w || w->getID() < 0 || uInt32(w->getID()) >= _myTabList.size())
+  if(!w || w->getID() >= _myTabList.size())
     return;
 
   assert(w == _myTabList[w->getID()].widget);
 
   // All focusable widgets should retain focus
   for(const auto& fw: list)
-    fw->setFlags(WIDGET_RETAIN_FOCUS);
+    fw->setFlags(Widget::FLAG_RETAIN_FOCUS);
 
   // First get the appropriate focus list
   FocusList& focus = _myTabList[w->getID()].focus;
@@ -244,7 +289,7 @@ void Dialog::addToFocusList(WidgetArray& list, TabWidget* w, int tabId)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Dialog::addTabWidget(TabWidget* w)
 {
-  if(!w || w->getID() < 0)
+  if(!w)
     return;
 
   // Make sure the array is large enough
@@ -313,7 +358,7 @@ void Dialog::buildCurrentFocusList(int tabID)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Dialog::addSurface(shared_ptr<FBSurface> surface)
+void Dialog::addSurface(const shared_ptr<FBSurface>& surface)
 {
   mySurfaceStack.push(surface);
 }
@@ -331,7 +376,7 @@ void Dialog::drawDialog()
     || (parent().myDialogStack.get(parent().myDialogStack.size() - 2) == this
     && !parent().myDialogStack.top()->hasTitle());
 
-  if(_flags & WIDGET_CLEARBG)
+  if(_flags & Widget::FLAG_CLEARBG)
   {
     //    cerr << "Dialog::drawDialog(): w = " << _w << ", h = " << _h << " @ " << &s << endl << endl;
     s.fillRect(_x, _y + _th, _w, _h - _th, _onTop ? kDlgColor : kBGColorLo);
@@ -344,7 +389,7 @@ void Dialog::drawDialog()
   }
   else
     s.invalidate();
-  if(_flags & WIDGET_BORDER) // currently only used by Dialog itself
+  if(_flags & Widget::FLAG_BORDER) // currently only used by Dialog itself
     s.frameRect(_x, _y, _w, _h, _onTop ? kColor : kShadowColor);
 
   // Make all child widget dirty
@@ -363,8 +408,12 @@ void Dialog::drawDialog()
   // Don't change focus, since this will trigger lost and received
   // focus events
   if(_focusedWidget)
+  {
     _focusedWidget = Widget::setFocusForChain(this, getFocusList(),
-                        _focusedWidget, 0, false);
+      _focusedWidget, 0, false);
+    if(_focusedWidget)
+      _focusedWidget->draw(); // make sure the highlight color is drawn initially
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -376,44 +425,27 @@ void Dialog::handleText(char text)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Dialog::handleKeyDown(StellaKey key, StellaMod mod)
+void Dialog::handleKeyDown(StellaKey key, StellaMod mod, bool repeated)
 {
-  // Test for TAB character
-  // Control-Tab selects next tab
-  // Shift-Control-Tab selects previous tab
-  // Tab sets next widget in current tab
-  // Shift-Tab sets previous widget in current tab
   Event::Type e = Event::NoType;
 
-  // Detect selection of previous and next tab headers and objects
-  if(key == KBDK_TAB)
-  {
-    if(StellaModTest::isControl(mod))
-    {
-      // tab header navigation
-      if(StellaModTest::isShift(mod) && cycleTab(-1))
-        return;
-      else if(!StellaModTest::isShift(mod) && cycleTab(+1))
-        return;
-    }
-    else
-    {
-      // object navigation
-      if(StellaModTest::isShift(mod))
-        e = Event::UINavPrev;
-      else
-        e = Event::UINavNext;
-    }
-  }
+// FIXME - I don't think this will compile!
+#if defined(RETRON77)
+  // special keys used for R77
+  if (key == KBDK_F13)
+    e = Event::UITabPrev;
+  else if (key == KBDK_BACKSPACE)
+    e = Event::UITabNext;
+#endif
 
   // Check the keytable now, since we might get one of the above events,
   // which must always be processed before any widget sees it.
   if(e == Event::NoType)
-    e = instance().eventHandler().eventForKey(key, kMenuMode);
+    e = instance().eventHandler().eventForKey(EventMode::kMenuMode, key, mod);
 
   // Unless a widget has claimed all responsibility for data, we assume
   // that if an event exists for the given data, it should have priority.
-  if(!handleNavEvent(e) && _focusedWidget)
+  if(!handleNavEvent(e, repeated) && _focusedWidget)
   {
     if(_focusedWidget->wantsRaw() || e == Event::NoType)
       _focusedWidget->handleKeyDown(key, mod);
@@ -449,7 +481,7 @@ void Dialog::handleMouseUp(int x, int y, MouseButton b, int clickCount)
   if(_focusedWidget)
   {
     // Lose focus on mouseup unless the widget requested to retain the focus
-    if(! (_focusedWidget->getFlags() & WIDGET_RETAIN_FOCUS ))
+    if(! (_focusedWidget->getFlags() & Widget::FLAG_RETAIN_FOCUS ))
       releaseFocus();
   }
 
@@ -472,7 +504,7 @@ void Dialog::handleMouseWheel(int x, int y, int direction)
   if(!w)
     w = _focusedWidget;
   if(w)
-    w->handleMouseWheel(x, y, direction);
+    w->handleMouseWheel(x - (w->getAbsX() - _x), y - (w->getAbsY() - _y), direction);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -521,7 +553,7 @@ void Dialog::handleMouseMoved(int x, int y)
     _mouseWidget = w;
   }
 
-  if (w && (w->getFlags() & WIDGET_TRACK_MOUSE))
+  if (w && (w->getFlags() & Widget::FLAG_TRACK_MOUSE))
     w->handleMouseMoved(x - (w->getAbsX() - _x), y - (w->getAbsY() - _y));
 }
 
@@ -538,59 +570,70 @@ bool Dialog::handleMouseClicks(int x, int y, MouseButton b)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Dialog::handleJoyDown(int stick, int button)
+void Dialog::handleJoyDown(int stick, int button, bool longPress)
 {
-  Event::Type e =
-    instance().eventHandler().eventForJoyButton(stick, button, kMenuMode);
-
-  // Unless a widget has claimed all responsibility for data, we assume
-  // that if an event exists for the given data, it should have priority.
-  if(!handleNavEvent(e) && _focusedWidget)
+  // Focused widget receives joystick events
+  if(_focusedWidget)
   {
+    Event::Type e =
+      instance().eventHandler().eventForJoyButton(EventMode::kMenuMode, stick, button);
+
     if(_focusedWidget->wantsRaw() || e == Event::NoType)
-      _focusedWidget->handleJoyDown(stick, button);
-    else
-      _focusedWidget->handleEvent(e);
+      _focusedWidget->handleJoyDown(stick, button, longPress);
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Dialog::handleJoyUp(int stick, int button)
-{
-  // Focused widget receives joystick events
-  if(_focusedWidget)
-    _focusedWidget->handleJoyUp(stick, button);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Dialog::handleJoyAxis(int stick, int axis, int value)
 {
   Event::Type e =
-    instance().eventHandler().eventForJoyAxis(stick, axis, value, kMenuMode);
+    instance().eventHandler().eventForJoyButton(EventMode::kMenuMode, stick, button);
 
   // Unless a widget has claimed all responsibility for data, we assume
   // that if an event exists for the given data, it should have priority.
-  if(!handleNavEvent(e) && _focusedWidget)
+  if (!handleNavEvent(e) && _focusedWidget)
   {
-    if(_focusedWidget->wantsRaw() || e == Event::NoType)
-      _focusedWidget->handleJoyAxis(stick, axis, value);
-    else if(value != 0)
+    if (_focusedWidget->wantsRaw() || e == Event::NoType)
+      _focusedWidget->handleJoyUp(stick, button);
+    else
       _focusedWidget->handleEvent(e);
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Dialog::handleJoyHat(int stick, int hat, JoyHat value)
+Event::Type Dialog::getJoyAxisEvent(int stick, JoyAxis axis, JoyDir adir, int button)
 {
-  Event::Type e =
-    instance().eventHandler().eventForJoyHat(stick, hat, value, kMenuMode);
+  return instance().eventHandler().eventForJoyAxis(EventMode::kMenuMode, stick, axis, adir, button);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Dialog::handleJoyAxis(int stick, JoyAxis axis, JoyDir adir, int button)
+{
+  Event::Type e = getJoyAxisEvent(stick, axis, adir, button);
 
   // Unless a widget has claimed all responsibility for data, we assume
   // that if an event exists for the given data, it should have priority.
   if(!handleNavEvent(e) && _focusedWidget)
   {
     if(_focusedWidget->wantsRaw() || e == Event::NoType)
-      return _focusedWidget->handleJoyHat(stick, hat, value);
+      _focusedWidget->handleJoyAxis(stick, axis, adir, button);
+    else if(adir != JoyDir::NONE)
+      _focusedWidget->handleEvent(e);
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Dialog::handleJoyHat(int stick, int hat, JoyHatDir hdir, int button)
+{
+  Event::Type e =
+    instance().eventHandler().eventForJoyHat(EventMode::kMenuMode, stick, hat, hdir, button);
+
+  // Unless a widget has claimed all responsibility for data, we assume
+  // that if an event exists for the given data, it should have priority.
+  if(!handleNavEvent(e) && _focusedWidget)
+  {
+    if(_focusedWidget->wantsRaw() || e == Event::NoType)
+      return _focusedWidget->handleJoyHat(stick, hat, hdir, button);
     else
       return _focusedWidget->handleEvent(e);
   }
@@ -598,10 +641,20 @@ bool Dialog::handleJoyHat(int stick, int hat, JoyHat value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Dialog::handleNavEvent(Event::Type e)
+bool Dialog::handleNavEvent(Event::Type e, bool repeated)
 {
   switch(e)
   {
+    case Event::UITabPrev:
+      if (cycleTab(-1))
+        return true;
+      break;
+
+    case Event::UITabNext:
+      if (cycleTab(1))
+        return true;
+      break;
+
     case Event::UINavPrev:
       if(_focusedWidget && !_focusedWidget->wantsTab())
       {
@@ -627,7 +680,7 @@ bool Dialog::handleNavEvent(Event::Type e)
       break;
 
     case Event::UIOK:
-      if(_okWidget && _okWidget->isEnabled())
+      if(_okWidget && _okWidget->isEnabled() && !repeated)
       {
         // Receiving 'OK' is the same as getting the 'Select' event
         _okWidget->handleEvent(Event::UISelect);
@@ -636,7 +689,7 @@ bool Dialog::handleNavEvent(Event::Type e)
       break;
 
     case Event::UICancel:
-      if(_cancelWidget && _cancelWidget->isEnabled())
+      if(_cancelWidget && _cancelWidget->isEnabled() && !repeated)
       {
         // Receiving 'Cancel' is the same as getting the 'Select' event
         _cancelWidget->handleEvent(Event::UISelect);
@@ -646,7 +699,7 @@ bool Dialog::handleNavEvent(Event::Type e)
       {
         // Some dialogs want the ability to cancel without actually having
         // a corresponding cancel button
-        close();
+        processCancel();
         return true;
       }
       break;
@@ -697,6 +750,9 @@ void Dialog::handleCommand(CommandSender* sender, int cmd, int data, int id)
     case GuiObject::kCloseCmd:
       close();
       break;
+
+    default:
+      break;
   }
 }
 
@@ -802,18 +858,40 @@ Widget* Dialog::TabFocus::getNewFocus()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Dialog::getDynamicBounds(uInt32& w, uInt32& h) const
 {
-  const GUI::Rect& r = instance().frameBuffer().imageRect();
+  const Common::Rect& r = instance().frameBuffer().imageRect();
+  const uInt32 scale = instance().frameBuffer().hidpiScaleFactor();
 
-  if(r.width() <= FrameBuffer::kFBMinW || r.height() <= FrameBuffer::kFBMinH)
+  if(r.w() <= FBMinimum::Width || r.h() <= FBMinimum::Height)
   {
-    w = r.width();
-    h = r.height();
+    w = r.w() / scale;
+    h = r.h() / scale;
     return false;
   }
   else
   {
-    w = uInt32(0.95 * r.width());
-    h = uInt32(0.95 * r.height());
+    w = uInt32(0.95 * r.w() / scale);
+    h = uInt32(0.95 * r.h() / scale);
     return true;
   }
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Dialog::setSize(uInt32 w, uInt32 h, uInt32 max_w, uInt32 max_h)
+{
+  _w = std::min(w, max_w);
+  _max_w = w;
+  _h = std::min(h, max_h);
+  _max_h = h;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Dialog::shouldResize(uInt32& w, uInt32& h) const
+{
+  getDynamicBounds(w, h);
+
+  // returns true if the current size is larger than the allowed size or
+  //  if the current size is smaller than the allowed and wanted size
+  return (uInt32(_w) > w || uInt32(_h) > h ||
+          (uInt32(_w) < w && uInt32(_w) < _max_w) ||
+          (uInt32(_h) < h && uInt32(_h) < _max_h));
+}
