@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -27,24 +27,14 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 DialogContainer::DialogContainer(OSystem& osystem)
-  : myOSystem(osystem),
-    myBaseDialog(nullptr),
-    myTime(0),
-    myKeyRepeatTime(0),
-    myClickRepeatTime(0),
-    myButtonRepeatTime(0),
-    myAxisRepeatTime(0),
-    myHatRepeatTime(0)
+  : myOSystem(osystem)
 {
+  _DOUBLE_CLICK_DELAY = osystem.settings().getInt("mdouble");
+  _REPEAT_INITIAL_DELAY = osystem.settings().getInt("ctrldelay");
+  setControllerRate(osystem.settings().getInt("ctrlrate"));
   reset();
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-DialogContainer::~DialogContainer()
-{
-  delete myBaseDialog;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DialogContainer::updateTime(uInt64 time)
 {
@@ -57,43 +47,44 @@ void DialogContainer::updateTime(uInt64 time)
   // Check for pending continuous events and send them to the active dialog box
   Dialog* activeDialog = myDialogStack.top();
 
-  // Key still pressed
-  if(myCurrentKeyDown.key != KBDK_UNKNOWN && myKeyRepeatTime < myTime)
-  {
-    activeDialog->handleKeyDown(myCurrentKeyDown.key, myCurrentKeyDown.mod);
-    myKeyRepeatTime = myTime + kRepeatSustainDelay;
-  }
-
   // Mouse button still pressed
   if(myCurrentMouseDown.b != MouseButton::NONE && myClickRepeatTime < myTime)
   {
     activeDialog->handleMouseDown(myCurrentMouseDown.x - activeDialog->_x,
                                   myCurrentMouseDown.y - activeDialog->_y,
                                   myCurrentMouseDown.b, 1);
-    myClickRepeatTime = myTime + kRepeatSustainDelay;
+    myClickRepeatTime = myTime + _REPEAT_SUSTAIN_DELAY;
   }
 
   // Joystick button still pressed
   if(myCurrentButtonDown.stick != -1 && myButtonRepeatTime < myTime)
   {
     activeDialog->handleJoyDown(myCurrentButtonDown.stick, myCurrentButtonDown.button);
-    myButtonRepeatTime = myTime + kRepeatSustainDelay;
+    myButtonRepeatTime = myTime + _REPEAT_SUSTAIN_DELAY;
+  }
+
+  // Joystick has been pressed long
+  if(myCurrentButtonDown.stick != -1 && myButtonLongPressTime < myTime)
+  {
+    myButtonLongPress = true;
+    activeDialog->handleJoyDown(myCurrentButtonDown.stick, myCurrentButtonDown.button, true);
+    myButtonLongPressTime = myButtonRepeatTime = myTime + _REPEAT_NONE;
   }
 
   // Joystick axis still pressed
   if(myCurrentAxisDown.stick != -1 && myAxisRepeatTime < myTime)
   {
     activeDialog->handleJoyAxis(myCurrentAxisDown.stick, myCurrentAxisDown.axis,
-                                myCurrentAxisDown.value);
-    myAxisRepeatTime = myTime + kRepeatSustainDelay;
+                                myCurrentAxisDown.adir);
+    myAxisRepeatTime = myTime + _REPEAT_SUSTAIN_DELAY;
   }
 
   // Joystick hat still pressed
   if(myCurrentHatDown.stick != -1 && myHatRepeatTime < myTime)
   {
     activeDialog->handleJoyHat(myCurrentHatDown.stick, myCurrentHatDown.hat,
-                                myCurrentHatDown.value);
-    myHatRepeatTime = myTime + kRepeatSustainDelay;
+                                myCurrentHatDown.hdir);
+    myHatRepeatTime = myTime + _REPEAT_SUSTAIN_DELAY;
   }
 }
 
@@ -108,7 +99,7 @@ bool DialogContainer::draw(bool full)
     myDialogStack.top()->setDirty();
 
   // If the top dialog is dirty, then all below it must be redrawn too
-  bool dirty = needsRedraw();
+  const bool dirty = needsRedraw();
 
   myDialogStack.applyAll([&](Dialog*& d){
     if(dirty)
@@ -126,10 +117,18 @@ bool DialogContainer::needsRedraw() const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DialogContainer::addDialog(Dialog* d)
+bool DialogContainer::baseDialogIsActive() const
 {
-  const GUI::Rect& r = myOSystem.frameBuffer().imageRect();
-  if(uInt32(d->getWidth()) > r.width() || uInt32(d->getHeight()) > r.height())
+  return myDialogStack.size() == 1;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int DialogContainer::addDialog(Dialog* d)
+{
+  const Common::Rect& r = myOSystem.frameBuffer().imageRect();
+  const uInt32 scale = myOSystem.frameBuffer().hidpiScaleFactor();
+
+  if(uInt32(d->getWidth() * scale) > r.w() || uInt32(d->getHeight() * scale) > r.h())
     myOSystem.frameBuffer().showMessage(
         "Unable to show dialog box; FIX THE CODE");
   else
@@ -137,6 +136,7 @@ void DialogContainer::addDialog(Dialog* d)
     d->setDirty();
     myDialogStack.push(d);
   }
+  return myDialogStack.size();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -157,7 +157,7 @@ void DialogContainer::reStack()
   while(!myDialogStack.empty())
     myDialogStack.top()->close();
 
-  myBaseDialog->open();
+  baseDialog()->open();
 
   // Reset all continuous events
   reset();
@@ -175,29 +175,17 @@ void DialogContainer::handleTextEvent(char text)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DialogContainer::handleKeyEvent(StellaKey key, StellaMod mod, bool state)
+void DialogContainer::handleKeyEvent(StellaKey key, StellaMod mod, bool pressed, bool repeated)
 {
   if(myDialogStack.empty())
     return;
 
   // Send the event to the dialog box on the top of the stack
   Dialog* activeDialog = myDialogStack.top();
-  if(state)
-  {
-    myCurrentKeyDown.key = key;
-    myCurrentKeyDown.mod = mod;
-    myKeyRepeatTime = myTime + kRepeatInitialDelay;
-
-    activeDialog->handleKeyDown(key, mod);
-  }
+  if(pressed)
+    activeDialog->handleKeyDown(key, mod, repeated);
   else
-  {
     activeDialog->handleKeyUp(key, mod);
-
-    // Only stop firing events if it's the current key
-    if(key == myCurrentKeyDown.key)
-      myCurrentKeyDown.key = KBDK_UNKNOWN;
-  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -237,7 +225,7 @@ void DialogContainer::handleMouseButtonEvent(MouseButton b, bool pressed,
           myLastClick.count = 0;
         }
 
-        if(myLastClick.count && (myTime < myLastClick.time + kDoubleClickDelay)
+        if(myLastClick.count && (myTime < myLastClick.time + _DOUBLE_CLICK_DELAY)
            && std::abs(myLastClick.x - x) < 3
            && std::abs(myLastClick.y - y) < 3)
         {
@@ -258,7 +246,7 @@ void DialogContainer::handleMouseButtonEvent(MouseButton b, bool pressed,
           myCurrentMouseDown.x = x;
           myCurrentMouseDown.y = y;
           myCurrentMouseDown.b = b;
-          myClickRepeatTime = myTime + kRepeatInitialDelay;
+          myClickRepeatTime = myTime + _REPEAT_INITIAL_DELAY;
         }
         else
           myCurrentMouseDown.b = MouseButton::NONE;
@@ -290,7 +278,7 @@ void DialogContainer::handleMouseButtonEvent(MouseButton b, bool pressed,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DialogContainer::handleJoyBtnEvent(int stick, int button, uInt8 state)
+void DialogContainer::handleJoyBtnEvent(int stick, int button, bool pressed)
 {
   if(myDialogStack.empty())
     return;
@@ -298,11 +286,12 @@ void DialogContainer::handleJoyBtnEvent(int stick, int button, uInt8 state)
   // Send the event to the dialog box on the top of the stack
   Dialog* activeDialog = myDialogStack.top();
 
-  if(state == 1)
+  if(pressed && myButtonRepeatTime < myTime) // prevent pending repeats after enabling repeat again
   {
     myCurrentButtonDown.stick  = stick;
     myCurrentButtonDown.button = button;
-    myButtonRepeatTime = myTime + kRepeatInitialDelay;
+    myButtonRepeatTime = myTime + (activeDialog->repeatEnabled() ? _REPEAT_INITIAL_DELAY : _REPEAT_NONE);
+    myButtonLongPressTime = myTime + _LONG_PRESS_DELAY;
 
     activeDialog->handleJoyDown(stick, button);
   }
@@ -310,67 +299,89 @@ void DialogContainer::handleJoyBtnEvent(int stick, int button, uInt8 state)
   {
     // Only stop firing events if it's the current button
     if(stick == myCurrentButtonDown.stick)
+    {
       myCurrentButtonDown.stick = myCurrentButtonDown.button = -1;
-
-    activeDialog->handleJoyUp(stick, button);
+      myButtonRepeatTime = myButtonLongPressTime = 0;
+    }
+    if (myButtonLongPress)
+      myButtonLongPress = false;
+    else
+      activeDialog->handleJoyUp(stick, button);
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DialogContainer::handleJoyAxisEvent(int stick, int axis, int value)
+void DialogContainer::handleJoyAxisEvent(int stick, JoyAxis axis, JoyDir adir, int button)
 {
   if(myDialogStack.empty())
     return;
 
+  // Send the event to the dialog box on the top of the stack
+  Dialog* activeDialog = myDialogStack.top();
+
+  // Prevent long button press in button/axis combinations
+  myButtonLongPressTime = myTime + _REPEAT_NONE;
+
   // Only stop firing events if it's the current stick
-  if(myCurrentAxisDown.stick == stick && value == 0)
+  if(myCurrentAxisDown.stick == stick && adir == JoyDir::NONE)
   {
-    myCurrentAxisDown.stick = myCurrentAxisDown.axis = -1;
+    myCurrentAxisDown.stick = -1;
+    myCurrentAxisDown.axis = JoyAxis::NONE;
+    myAxisRepeatTime = 0;
   }
-  else if(value != 0)  // never repeat the 'off' event
+  else if(adir != JoyDir::NONE && myAxisRepeatTime < myTime)  // never repeat the 'off' event; prevent pending repeats after enabling repeat again
   {
     // Now account for repeated axis events (press and hold)
     myCurrentAxisDown.stick = stick;
     myCurrentAxisDown.axis  = axis;
-    myCurrentAxisDown.value = value;
-    myAxisRepeatTime = myTime + kRepeatInitialDelay;
+    myCurrentAxisDown.adir = adir;
+    myAxisRepeatTime = myTime + (activeDialog->repeatEnabled() ? _REPEAT_INITIAL_DELAY : _REPEAT_NONE);
   }
-  myDialogStack.top()->handleJoyAxis(stick, axis, value);
+  activeDialog->handleJoyAxis(stick, axis, adir, button);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DialogContainer::handleJoyHatEvent(int stick, int hat, JoyHat value)
+void DialogContainer::handleJoyHatEvent(int stick, int hat, JoyHatDir hdir, int button)
 {
   if(myDialogStack.empty())
     return;
 
+  // Send the event to the dialog box on the top of the stack
+  Dialog* activeDialog = myDialogStack.top();
+
+  // Prevent long button press in button/hat combinations
+  myButtonLongPressTime = myTime + _REPEAT_NONE;
+
   // Only stop firing events if it's the current stick
-  if(myCurrentHatDown.stick == stick && value == JoyHat::CENTER)
+  if(myCurrentHatDown.stick == stick && hdir == JoyHatDir::CENTER)
   {
     myCurrentHatDown.stick = myCurrentHatDown.hat = -1;
+    myHatRepeatTime = 0;
   }
-  else if(value != JoyHat::CENTER)  // never repeat the 'center' direction
+  else if(hdir != JoyHatDir::CENTER && myHatRepeatTime < myTime)  // never repeat the 'center' direction; prevent pending repeats after enabling repeat again
   {
     // Now account for repeated hat events (press and hold)
     myCurrentHatDown.stick = stick;
     myCurrentHatDown.hat  = hat;
-    myCurrentHatDown.value = value;
-    myHatRepeatTime = myTime + kRepeatInitialDelay;
+    myCurrentHatDown.hdir = hdir;
+    myHatRepeatTime = myTime + (activeDialog->repeatEnabled() ? _REPEAT_INITIAL_DELAY : _REPEAT_NONE);
   }
-  myDialogStack.top()->handleJoyHat(stick, hat, value);
+  activeDialog->handleJoyHat(stick, hat, hdir, button);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DialogContainer::reset()
 {
-  myCurrentKeyDown.key = KBDK_UNKNOWN;
-  myCurrentKeyDown.mod = KBDM_NONE;
-  myCurrentMouseDown.b = MouseButton::NONE;
-  myLastClick.x = myLastClick.y = 0;
-  myLastClick.time = 0;
-  myLastClick.count = 0;
+  myCurrentMouseDown  = { 0, 0, MouseButton::NONE };
+  myCurrentButtonDown = { -1, -1 };
+  myCurrentAxisDown   = { -1, JoyAxis::NONE, JoyDir::NONE };
+  myCurrentHatDown    = { -1, -1, JoyHatDir::CENTER };
 
-  myCurrentButtonDown.stick = myCurrentButtonDown.button = -1;
-  myCurrentAxisDown.stick = myCurrentAxisDown.axis = -1;
-  myCurrentHatDown.stick = myCurrentHatDown.hat = -1;
+  myLastClick = { 0, 0, 0, 0 };
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt64 DialogContainer::_DOUBLE_CLICK_DELAY = 500;
+uInt64 DialogContainer::_REPEAT_INITIAL_DELAY = 400;
+uInt64 DialogContainer::_REPEAT_SUSTAIN_DELAY = 50;
+
