@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -24,12 +24,11 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 EditTextWidget::EditTextWidget(GuiObject* boss, const GUI::Font& font,
                                int x, int y, int w, int h, const string& text)
-  : EditableWidget(boss, font, x, y, w, h + 2, text),
-    _changed(false)
+  : EditableWidget(boss, font, x, y, w, h + 2, text)
 {
-  _flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS;
+  _flags = Widget::FLAG_ENABLED | Widget::FLAG_CLEARBG | Widget::FLAG_RETAIN_FOCUS;
 
-  startEditMode();  // We're always in edit mode
+  EditableWidget::startEditMode();  // We're always in edit mode
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -43,14 +42,14 @@ void EditTextWidget::setText(const string& str, bool changed)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EditTextWidget::handleMouseEntered()
 {
-  setFlags(WIDGET_HILITED);
+  setFlags(Widget::FLAG_HILITED);
   setDirty();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EditTextWidget::handleMouseLeft()
 {
-  clearFlags(WIDGET_HILITED);
+  clearFlags(Widget::FLAG_HILITED);
   setDirty();
 }
 
@@ -93,21 +92,20 @@ void EditTextWidget::drawWidget(bool hilite)
 
   // Draw the text
   adjustOffset();
-  s.drawString(_font, editString(), _x + 2, _y + 2, getEditRect().width(),
+  s.drawString(_font, editString(), _x + 2, _y + 2, getEditRect().w(), getEditRect().h(),
                _changed && onTop && isEnabled()
                ? kDbgChangedTextColor
                : onTop && isEnabled() ? _textcolor : kColor,
-               TextAlign::Left, -_editScrollOffset, false);
+               TextAlign::Left, isEditable() ? -_editScrollOffset : 0, !isEditable());
 
   // Draw the caret
   drawCaret();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-GUI::Rect EditTextWidget::getEditRect() const
+Common::Rect EditTextWidget::getEditRect() const
 {
-  GUI::Rect r(2, 1, _w - 2, _h);
-  return r;
+  return Common::Rect(2, 1, _w - 2, _h);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
