@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -19,6 +19,10 @@
 
 #include "bspf.hxx"
 
+#include "OSystem.hxx"
+#include "GuiObject.hxx"
+#include "PopUpWidget.hxx"
+#include "FrameBuffer.hxx"
 #include "EventHandler.hxx"
 #include "Event.hxx"
 #include "OSystem.hxx"
@@ -33,18 +37,10 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 EventMappingWidget::EventMappingWidget(GuiObject* boss, const GUI::Font& font,
                                        int x, int y, int w, int h,
-                                       const StringList& actions, EventMode mode)
+                                       EventMode mode)
   : Widget(boss, font, x, y, w, h),
     CommandSender(boss),
-    myComboDialog(nullptr),
-    myEventMode(mode),
-    myActionSelected(-1),
-    myRemapStatus(false),
-    myLastStick(0),
-    myLastAxis(0),
-    myLastHat(0),
-    myLastValue(0),
-    myFirstTime(true)
+    myEventMode(mode)
 {
   const int fontHeight   = font.getFontHeight(),
             lineHeight   = font.getLineHeight(),
@@ -52,13 +48,39 @@ EventMappingWidget::EventMappingWidget(GuiObject* boss, const GUI::Font& font,
             buttonHeight = font.getLineHeight() + 4;
   const int HBORDER = 8;
   const int VBORDER = 8;
+  const int ACTION_LINES = 2;
   int xpos = HBORDER, ypos = VBORDER;
+  const int listWidth = _w - buttonWidth - HBORDER * 2 - 8;
+  int listHeight = _h - (2 + ACTION_LINES) * lineHeight - VBORDER + 2;
 
-  myActionsList = new StringListWidget(boss, font, xpos, ypos,
-                                       _w - buttonWidth - HBORDER * 2 - 8, _h - 3*lineHeight - VBORDER);
+  if(mode == EventMode::kEmulationMode)
+  {
+    VariantList items;
+
+    items.clear();
+    VarList::push_back(items, "All", Event::Group::Emulation);
+    VarList::push_back(items, "Miscellaneous", Event::Group::Misc);
+    VarList::push_back(items, "Video & Audio", Event::Group::AudioVideo);
+    VarList::push_back(items, "States", Event::Group::States);
+    VarList::push_back(items, "Console", Event::Group::Console);
+    VarList::push_back(items, "Joystick", Event::Group::Joystick);
+    VarList::push_back(items, "Paddles", Event::Group::Paddles);
+    VarList::push_back(items, "Keyboard", Event::Group::Keyboard);
+    VarList::push_back(items, "Combo", Event::Group::Combo);
+    VarList::push_back(items, "Debug", Event::Group::Debug);
+
+    myFilterPopup = new PopUpWidget(boss, font, xpos, ypos,
+                                    listWidth - font.getStringWidth("Events ") - 23, lineHeight,
+                                    items, "Events ", 0, kFilterCmd);
+    myFilterPopup->setTarget(this);
+    addFocusWidget(myFilterPopup);
+    ypos += lineHeight + 8;
+    listHeight -= lineHeight + 8;
+  }
+
+  myActionsList = new StringListWidget(boss, font, xpos, ypos, listWidth, listHeight);
   myActionsList->setTarget(this);
   myActionsList->setEditable(false);
-  myActionsList->setList(actions);
   addFocusWidget(myActionsList);
 
   // Add remap, erase, cancel and default buttons
@@ -74,7 +96,7 @@ EventMappingWidget::EventMappingWidget(GuiObject* boss, const GUI::Font& font,
                                        buttonWidth, buttonHeight,
                                        "Cancel", kStopMapCmd);
   myCancelMapButton->setTarget(this);
-  myCancelMapButton->clearFlags(WIDGET_ENABLED);
+  myCancelMapButton->clearFlags(Widget::FLAG_ENABLED);
   addFocusWidget(myCancelMapButton);
 
   ypos += lineHeight + 20;
@@ -91,7 +113,7 @@ EventMappingWidget::EventMappingWidget(GuiObject* boss, const GUI::Font& font,
   myResetButton->setTarget(this);
   addFocusWidget(myResetButton);
 
-  if(mode == kEmulationMode)
+  if(mode == EventMode::kEmulationMode)
   {
     ypos += lineHeight + 20;
     myComboButton = new ButtonWidget(boss, font, xpos, ypos,
@@ -103,19 +125,19 @@ EventMappingWidget::EventMappingWidget(GuiObject* boss, const GUI::Font& font,
     VariantList combolist = instance().eventHandler().getComboList(mode);
     myComboDialog = new ComboDialog(boss, font, combolist);
   }
-  else
-    myComboButton = nullptr;
 
   // Show message for currently selected event
-  xpos = HBORDER;  ypos = VBORDER + myActionsList->getHeight() + 8;
+  xpos = HBORDER;
+  ypos = myActionsList->getBottom() + 8;
   StaticTextWidget* t;
   t = new StaticTextWidget(boss, font, xpos, ypos+2, font.getStringWidth("Action"),
                            fontHeight, "Action", TextAlign::Left);
 
   myKeyMapping = new EditTextWidget(boss, font, xpos + t->getWidth() + 8, ypos,
-                                    _w - xpos - t->getWidth() - 8 - HBORDER, lineHeight, "");
+                                    _w - xpos - t->getWidth() - 8 - HBORDER,
+                                    lineHeight + font.getFontHeight() * (ACTION_LINES - 1), "");
   myKeyMapping->setEditable(false, true);
-  myKeyMapping->clearFlags(WIDGET_RETAIN_FOCUS);
+  myKeyMapping->clearFlags(Widget::FLAG_RETAIN_FOCUS);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -123,7 +145,9 @@ void EventMappingWidget::loadConfig()
 {
   if(myFirstTime)
   {
-    myActionsList->setSelected(0);
+    if(myFilterPopup)
+      myFilterPopup->setSelectedIndex(0);
+
     myFirstTime = false;
   }
 
@@ -131,6 +155,8 @@ void EventMappingWidget::loadConfig()
   // exit last time
   if(myRemapStatus)
     stopRemapping();
+
+  updateActions();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -138,6 +164,21 @@ void EventMappingWidget::saveConfig()
 {
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void EventMappingWidget::updateActions()
+{
+  if(myFilterPopup)
+    myEventGroup = Event::Group(myFilterPopup->getSelectedTag().toInt());
+  else
+    myEventGroup = Event::Group::Menu;
+  StringList actions = instance().eventHandler().getActionList(myEventGroup);
+
+  myActionsList->setList(actions);
+  myActionSelected = myActionsList->getSelected();
+  drawKeyMapping();
+  enableButtons(true);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EventMappingWidget::setDefaults()
 {
@@ -155,7 +196,15 @@ void EventMappingWidget::startRemapping()
   myRemapStatus = true;
 
   // Reset all previous events for determining correct axis/hat values
-  myLastStick = myLastAxis = myLastHat = myLastValue = -1;
+  myLastStick = -1;
+  myLastButton = JOY_CTRL_NONE;
+  myLastAxis = JoyAxis::NONE;
+  myLastDir = JoyDir::NONE;
+  myLastHat = -1;
+  myLastHatDir = JoyHatDir::CENTER;
+
+  // Reset the previously aggregated key mappings
+  myMod = myLastKey = 0;
 
   // Disable all other widgets while in remap mode, except enable 'Cancel'
   enableButtons(false);
@@ -163,14 +212,14 @@ void EventMappingWidget::startRemapping()
   // And show a message indicating which key is being remapped
   ostringstream buf;
   buf << "Select action for '"
-      << instance().eventHandler().actionAtIndex(myActionSelected, myEventMode)
+      << instance().eventHandler().actionAtIndex(myActionSelected, myEventGroup)
       << "' event";
   myKeyMapping->setTextColor(kTextColorEm);
   myKeyMapping->setText(buf.str());
 
   // Make sure that this widget receives all raw data, before any
   // pre-processing occurs
-  myActionsList->setFlags(WIDGET_WANTS_RAWDATA);
+  myActionsList->setFlags(Widget::FLAG_WANTS_RAWDATA);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -180,7 +229,7 @@ void EventMappingWidget::eraseRemapping()
     return;
 
   Event::Type event =
-    instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
+    instance().eventHandler().eventAtIndex(myActionSelected, myEventGroup);
   instance().eventHandler().eraseMapping(event, myEventMode);
 
   drawKeyMapping();
@@ -193,7 +242,7 @@ void EventMappingWidget::resetRemapping()
     return;
 
   Event::Type event =
-    instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
+    instance().eventHandler().eventAtIndex(myActionSelected, myEventGroup);
   instance().eventHandler().setDefaultMapping(event, myEventMode);
 
   drawKeyMapping();
@@ -206,7 +255,12 @@ void EventMappingWidget::stopRemapping()
   myRemapStatus = false;
 
   // Reset all previous events for determining correct axis/hat values
-  myLastStick = myLastAxis = myLastHat = myLastValue = -1;
+  myLastStick = -1;
+  myLastButton = JOY_CTRL_NONE;
+  myLastAxis = JoyAxis::NONE;
+  myLastDir = JoyDir::NONE;
+  myLastHat = -1;
+  myLastHatDir = JoyHatDir::CENTER;
 
   // And re-enable all the widgets
   enableButtons(true);
@@ -215,7 +269,7 @@ void EventMappingWidget::stopRemapping()
   drawKeyMapping();
 
   // Widget is now free to process events normally
-  myActionsList->clearFlags(WIDGET_WANTS_RAWDATA);
+  myActionsList->clearFlags(Widget::FLAG_WANTS_RAWDATA);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -224,7 +278,7 @@ void EventMappingWidget::drawKeyMapping()
   if(myActionSelected >= 0)
   {
     myKeyMapping->setTextColor(kTextColor);
-    myKeyMapping->setText(instance().eventHandler().keyAtIndex(myActionSelected, myEventMode));
+    myKeyMapping->setText(instance().eventHandler().keyAtIndex(myActionSelected, myEventGroup));
   }
 }
 
@@ -239,7 +293,7 @@ void EventMappingWidget::enableButtons(bool state)
   if(myComboButton)
   {
     Event::Type e =
-      instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
+      instance().eventHandler().eventAtIndex(myActionSelected, myEventGroup);
 
     myComboButton->setEnabled(state && e >= Event::Combo1 && e <= Event::Combo16);
   }
@@ -249,31 +303,84 @@ void EventMappingWidget::enableButtons(bool state)
 bool EventMappingWidget::handleKeyDown(StellaKey key, StellaMod mod)
 {
   // Remap keys in remap mode
-  if(myRemapStatus && myActionSelected >= 0)
+  if (myRemapStatus && myActionSelected >= 0)
+  {
+    // Mod keys are only recorded if no other key has been recorded before
+    if (key < KBDK_LCTRL || key > KBDK_RGUI
+      || (!myLastKey || (myLastKey >= KBDK_LCTRL && myLastKey <= KBDK_RGUI)))
+    {
+      myLastKey = key;
+    }
+    myMod |= mod;
+  }
+  return true;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool EventMappingWidget::handleKeyUp(StellaKey key, StellaMod mod)
+{
+  // Remap keys in remap mode
+  if (myRemapStatus && myActionSelected >= 0
+    && (mod & (KBDM_CTRL | KBDM_SHIFT | KBDM_ALT | KBDM_GUI)) == 0)
   {
     Event::Type event =
-      instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
-    if(instance().eventHandler().addKeyMapping(event, myEventMode, key))
+      instance().eventHandler().eventAtIndex(myActionSelected, myEventGroup);
+
+    // if not pressed alone, map left and right modifier keys
+    if(myLastKey < KBDK_LCTRL || myLastKey > KBDK_RGUI)
+    {
+      if(myMod & KBDM_CTRL)
+        myMod |= KBDM_CTRL;
+      if(myMod & KBDM_SHIFT)
+        myMod |= KBDM_SHIFT;
+      if(myMod & KBDM_ALT)
+        myMod |= KBDM_ALT;
+      if(myMod & KBDM_GUI)
+        myMod |= KBDM_GUI;
+    }
+    if (instance().eventHandler().addKeyMapping(event, myEventMode, StellaKey(myLastKey), StellaMod(myMod)))
       stopRemapping();
   }
   return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void EventMappingWidget::handleJoyDown(int stick, int button)
+void EventMappingWidget::handleJoyDown(int stick, int button, bool longPress)
 {
   // Remap joystick buttons in remap mode
   if(myRemapStatus && myActionSelected >= 0)
   {
-    Event::Type event =
-      instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
-    if(instance().eventHandler().addJoyButtonMapping(event, myEventMode, stick, button))
-      stopRemapping();
+    myLastStick = stick;
+    myLastButton = button;
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void EventMappingWidget::handleJoyAxis(int stick, int axis, int value)
+void EventMappingWidget::handleJoyUp(int stick, int button)
+{
+  // Remap joystick buttons in remap mode
+  if (myRemapStatus && myActionSelected >= 0)
+  {
+    if (myLastStick == stick && myLastButton == button)
+    {
+      EventHandler& eh = instance().eventHandler();
+      Event::Type event = eh.eventAtIndex(myActionSelected, myEventGroup);
+
+      // map either button/hat, solo button or button/axis combinations
+      if(myLastHat != -1)
+      {
+        if(eh.addJoyHatMapping(event, myEventMode, stick, button, myLastHat, myLastHatDir))
+          stopRemapping();
+      }
+      else
+        if (eh.addJoyMapping(event, myEventMode, stick, button, myLastAxis, myLastDir))
+          stopRemapping();
+    }
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void EventMappingWidget::handleJoyAxis(int stick, JoyAxis axis, JoyDir adir, int button)
 {
   // Remap joystick axes in remap mode
   // There are two phases to detection:
@@ -282,29 +389,27 @@ void EventMappingWidget::handleJoyAxis(int stick, int axis, int value)
   if(myRemapStatus && myActionSelected >= 0)
   {
     // Detect the first axis event that represents 'on'
-    if(myLastStick == -1 && myLastAxis == -1 && value != 0)
+    if((myLastStick == -1 || myLastStick == stick) && myLastAxis == JoyAxis::NONE && adir != JoyDir::NONE)
     {
       myLastStick = stick;
       myLastAxis = axis;
-      myLastValue = value;
+      myLastDir = adir;
     }
     // Detect the first axis event that matches a previously set
     // stick and axis, but turns the axis 'off'
-    else if(myLastStick == stick && axis == myLastAxis && value == 0)
+    else if(myLastStick == stick && axis == myLastAxis && adir == JoyDir::NONE)
     {
-      value = myLastValue;
+      EventHandler& eh = instance().eventHandler();
+      Event::Type event = eh.eventAtIndex(myActionSelected, myEventGroup);
 
-      Event::Type event =
-        instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
-      if(instance().eventHandler().addJoyAxisMapping(event, myEventMode,
-                                                     stick, axis, value))
+      if (eh.addJoyMapping(event, myEventMode, stick, myLastButton, axis, myLastDir))
         stopRemapping();
     }
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool EventMappingWidget::handleJoyHat(int stick, int hat, JoyHat value)
+bool EventMappingWidget::handleJoyHat(int stick, int hat, JoyHatDir hdir, int button)
 {
   // Remap joystick hats in remap mode
   // There are two phases to detection:
@@ -313,24 +418,22 @@ bool EventMappingWidget::handleJoyHat(int stick, int hat, JoyHat value)
   if(myRemapStatus && myActionSelected >= 0)
   {
     // Detect the first hat event that represents a valid direction
-    if(myLastStick == -1 && myLastHat == -1 && value != JoyHat::CENTER)
+    if((myLastStick == -1 || myLastStick == stick) && myLastHat == -1 && hdir != JoyHatDir::CENTER)
     {
       myLastStick = stick;
       myLastHat = hat;
-      myLastValue = int(value);
+      myLastHatDir = hdir;
 
       return true;
     }
     // Detect the first hat event that matches a previously set
     // stick and hat, but centers the hat
-    else if(myLastStick == stick && hat == myLastHat && value == JoyHat::CENTER)
+    else if(myLastStick == stick && hat == myLastHat && hdir == JoyHatDir::CENTER)
     {
-      value = JoyHat(myLastValue);
+      EventHandler& eh = instance().eventHandler();
+      Event::Type event = eh.eventAtIndex(myActionSelected, myEventGroup);
 
-      Event::Type event =
-        instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
-      if(instance().eventHandler().addJoyHatMapping(event, myEventMode,
-                                                    stick, hat, value))
+      if (eh.addJoyHatMapping(event, myEventMode, stick, myLastButton, hat, myLastHatDir))
       {
         stopRemapping();
         return true;
@@ -347,6 +450,10 @@ void EventMappingWidget::handleCommand(CommandSender* sender, int cmd,
 {
   switch(cmd)
   {
+    case kFilterCmd:
+      updateActions();
+      break;
+
     case ListWidget::kSelectionChangedCmd:
       if(myActionsList->getSelected() >= 0)
       {
@@ -356,7 +463,6 @@ void EventMappingWidget::handleCommand(CommandSender* sender, int cmd,
       }
       break;
 
-/*
     case ListWidget::kDoubleClickedCmd:
       if(myActionsList->getSelected() >= 0)
       {
@@ -364,7 +470,6 @@ void EventMappingWidget::handleCommand(CommandSender* sender, int cmd,
         startRemapping();
       }
       break;
-*/
 
     case kStartMapCmd:
       startRemapping();
@@ -385,8 +490,11 @@ void EventMappingWidget::handleCommand(CommandSender* sender, int cmd,
     case kComboCmd:
       if(myComboDialog)
         myComboDialog->show(
-          instance().eventHandler().eventAtIndex(myActionSelected, myEventMode),
-          instance().eventHandler().actionAtIndex(myActionSelected, myEventMode));
+          instance().eventHandler().eventAtIndex(myActionSelected, myEventGroup),
+          instance().eventHandler().actionAtIndex(myActionSelected, myEventGroup));
+      break;
+
+    default:
       break;
   }
 }
