@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -60,7 +60,7 @@ GlobalPropsDialog::GlobalPropsDialog(GuiObject* boss, const GUI::Font& font)
   wid.push_back(myBSType);
   ypos += lineHeight + VGAP * 3;
 
-  pwidth = font.getStringWidth("Debugger");
+  pwidth = font.getStringWidth("A (Expert)");
 
   // TV type
   new StaticTextWidget(this, font, xpos, ypos + 1, "TV type");
@@ -74,18 +74,18 @@ GlobalPropsDialog::GlobalPropsDialog(GuiObject* boss, const GUI::Font& font)
   ypos += lineHeight + VGAP;
 
   // Left difficulty
-  new StaticTextWidget(this, font, xpos, ypos+1, "Left difficulty");
+  new StaticTextWidget(this, font, xpos, ypos+1, GUI::LEFT_DIFFICULTY);
   items.clear();
   VarList::push_back(items, "Default", "DEFAULT");
-  VarList::push_back(items, "B", "B");
-  VarList::push_back(items, "A", "A");
+  VarList::push_back(items, "A (Expert)", "A");
+  VarList::push_back(items, "B (Novice)", "B");
   myLeftDiff = new PopUpWidget(this, font, xpos+lwidth, ypos,
                                pwidth, lineHeight, items, "");
   wid.push_back(myLeftDiff);
   ypos += lineHeight + VGAP;
 
   // Right difficulty
-  new StaticTextWidget(this, font, xpos, ypos+1, "Right difficulty");
+  new StaticTextWidget(this, font, xpos, ypos+1, GUI::RIGHT_DIFFICULTY);
   // ... use same items as above
   myRightDiff = new PopUpWidget(this, font, xpos+lwidth, ypos,
                                 pwidth, lineHeight, items, "");
@@ -96,7 +96,9 @@ GlobalPropsDialog::GlobalPropsDialog(GuiObject* boss, const GUI::Font& font)
   new StaticTextWidget(this, font, xpos, ypos+1, "Startup mode");
   items.clear();
   VarList::push_back(items, "Console", "false");
+#ifdef DEBUGGER_SUPPORT
   VarList::push_back(items, "Debugger", "true");
+#endif
   myDebug = new PopUpWidget(this, font, xpos+lwidth, ypos,
                             pwidth, lineHeight, items, "");
   wid.push_back(myDebug);
@@ -173,12 +175,16 @@ int GlobalPropsDialog::addHoldWidgets(const GUI::Font& font, int x, int y,
   // Console Select/Reset
   t = new StaticTextWidget(this, font, xpos, ypos+2, "Console");
   ypos += t->getHeight() + VGAP;
-  myHoldSelect = new CheckboxWidget(this, font, xpos, ypos, "Select");
+  myHoldSelect = new CheckboxWidget(this, font, xpos, ypos, GUI::SELECT);
   ypos += myHoldSelect->getHeight() + VGAP;
   myHoldReset = new CheckboxWidget(this, font, xpos, ypos, "Reset");
 
+  static constexpr std::array<int, 10> TAB_ORDER = {
+    kJ0Up, kJ0Left, kJ0Right, kJ0Down, kJ0Fire,
+    kJ1Up, kJ1Left, kJ1Right, kJ1Down, kJ1Fire
+  };
   for(int i = kJ0Up; i <= kJ1Fire; ++i)
-    wid.push_back(myJoy[i]);
+    wid.push_back(myJoy[TAB_ORDER[i]]);
 
   wid.push_back(myHoldSelect);
   wid.push_back(myHoldReset);
@@ -286,6 +292,6 @@ void GlobalPropsDialog::handleCommand(CommandSender* sender, int cmd,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* const GlobalPropsDialog::ourJoyState[10] = {
+const std::array<string, 10> GlobalPropsDialog::ourJoyState = {
   "U", "D", "L", "R", "F", "U", "D", "L", "R", "F"
 };
