@@ -8,25 +8,25 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include "bspf.hxx"
+#include "OSystem.hxx"
+#include "EventHandler.hxx"
 #include "Dialog.hxx"
 #include "Widget.hxx"
 #include "Font.hxx"
+
 #include "HelpDialog.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 HelpDialog::HelpDialog(OSystem& osystem, DialogContainer& parent,
                        const GUI::Font& font)
-  : Dialog(osystem, parent, font, "Help"),
-    myPage(1),
-    myNumPages(5)
+  : Dialog(osystem, parent, font, "Help")
 {
   const int lineHeight   = font.getLineHeight(),
             fontWidth    = font.getMaxCharWidth(),
@@ -45,7 +45,7 @@ HelpDialog::HelpDialog(OSystem& osystem, DialogContainer& parent,
   myPrevButton =
     new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
                      "Previous", GuiObject::kPrevCmd);
-  myPrevButton->clearFlags(WIDGET_ENABLED);
+  myPrevButton->clearFlags(Widget::FLAG_ENABLED);
   wid.push_back(myPrevButton);
 
   xpos += buttonWidth + 8;
@@ -65,9 +65,9 @@ HelpDialog::HelpDialog(OSystem& osystem, DialogContainer& parent,
   myTitle = new StaticTextWidget(this, font, xpos, ypos, _w - 10, fontHeight,
                                  "", TextAlign::Center);
 
-  int lwidth = 12 * fontWidth;
+  int lwidth = 13 * fontWidth;
   xpos += 5;  ypos += lineHeight + 4;
-  for(uInt8 i = 0; i < kLINES_PER_PAGE; ++i)
+  for(uInt8 i = 0; i < LINES_PER_PAGE; ++i)
   {
     myKey[i] =
       new StaticTextWidget(this, font, xpos, ypos, lwidth,
@@ -84,17 +84,16 @@ HelpDialog::HelpDialog(OSystem& osystem, DialogContainer& parent,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void HelpDialog::updateStrings(uInt8 page, uInt8 lines, string& title)
 {
-#ifdef BSPF_MACOS
-  #define ALT_ "Cmd"
-#else
-  #define ALT_ "Alt"
-#endif
-
   int i = 0;
   auto ADD_BIND = [&](const string& k, const string& d)
   {
     myKeyStr[i] = k;  myDescStr[i] = d;  i++;
   };
+  auto ADD_EVENT = [&](const Event::Type e, const string & d)
+  {
+    string desc = instance().eventHandler().getMappingDesc(e, EventMode::kEmulationMode);
+    ADD_BIND(desc.length() ? desc : "None", d);
+  };
   auto ADD_TEXT = [&](const string& d) { ADD_BIND("", d); };
   auto ADD_LINE = [&]() { ADD_BIND("", ""); };
 
@@ -102,70 +101,72 @@ void HelpDialog::updateStrings(uInt8 page, uInt8 lines, string& title)
   {
     case 1:
       title = "Common commands";
-#ifndef BSPF_MACOS
-      ADD_BIND("Ctrl Q",    "Quit emulation");
-#else
-      ADD_BIND("Cmd Q",     "Quit emulation");
-#endif
-      ADD_BIND("Escape",     "Exit current game");
-      ADD_BIND("Tab",        "Enter 'Options' menu");
-      ADD_BIND("\\",         "Toggle command menu");
-      ADD_BIND(ALT_" =",     "Increase window size");
-      ADD_BIND(ALT_" -",     "Decrease window size");
-      ADD_BIND(ALT_" Enter", "Toggle fullscreen /");
-      ADD_BIND("",           "  windowed mode");
-      ADD_BIND(ALT_" ]",     "Increase volume by 2%");
-      ADD_BIND(ALT_" [",     "Decrease volume by 2%");
+      ADD_EVENT(Event::Quit,                "Quit emulation");
+      ADD_EVENT(Event::ExitMode,            "Exit current mode/menu");
+      ADD_EVENT(Event::OptionsMenuMode,     "Enter Options menu");
+      ADD_EVENT(Event::CmdMenuMode,         "Toggle Command menu");
+      ADD_EVENT(Event::VidmodeIncrease,     "Increase window size");
+      ADD_EVENT(Event::VidmodeDecrease,     "Decrease window size");
+      ADD_EVENT(Event::ToggleFullScreen,    "Toggle fullscreen /");
+      ADD_BIND("",                          "  windowed mode");
+      ADD_EVENT(Event::OverscanIncrease,    "Increase overscan in FS mode");
+      ADD_EVENT(Event::OverscanDecrease,    "Decrease overscan in FS mode");
       break;
 
     case 2:
       title = "Special commands";
-      ADD_BIND("Ctrl g", "Grab mouse (keep in window)");
-      ADD_BIND("Ctrl f", "Switch between NTSC/PAL/SECAM");
-      ADD_BIND("Ctrl s", "Save game properties to a");
-      ADD_BIND("",       "  new file");
+      ADD_EVENT(Event::FormatIncrease,      "Switch between NTSC/PAL/SECAM");
+      ADD_EVENT(Event::TogglePalette,       "Switch palette");
+      ADD_EVENT(Event::TogglePhosphor,      "Toggle 'phosphor' effect");
       ADD_LINE();
-      ADD_BIND("Ctrl 0", "Toggle controller for Mouse");
-      ADD_BIND("Ctrl 1", "Toggle Stelladaptor left/right");
+      ADD_EVENT(Event::ToggleGrabMouse,     "Grab mouse (keep in window)");
+      ADD_EVENT(Event::HandleMouseControl,  "Toggle controller for mouse");
+      ADD_EVENT(Event::ToggleSAPortOrder,   "Toggle Stelladaptor left/right");
+      ADD_LINE();
+      ADD_EVENT(Event::VolumeIncrease,      "Increase volume by 2%");
+      ADD_EVENT(Event::VolumeDecrease,      "Decrease volume by 2%");
       break;
 
     case 3:
-      title = "TV Filters";
-      ADD_BIND(ALT_" 1", "Disable filtering");
-      ADD_BIND(ALT_" 2", "Enable 'Composite' mode");
-      ADD_BIND(ALT_" 3", "Enable 'S-video' mode");
-      ADD_BIND(ALT_" 4", "Enable 'RGB' mode");
-      ADD_BIND(ALT_" 5", "Enable 'Bad Adjust' mode");
-      ADD_BIND(ALT_" 6", "Enable 'Custom' mode");
-      ADD_BIND(ALT_" 7", "Adjust scanline intensity");
-      ADD_BIND(ALT_" 8", "Toggle scanline interpol.");
-      ADD_BIND(ALT_" 9", "Select 'Custom' adjustable");
-      ADD_BIND(ALT_" 0", "Modify 'Custom' adjustable");
+      title = "TV effects";
+      ADD_EVENT(Event::VidmodeStd,          "Disable TV effects");
+      ADD_EVENT(Event::VidmodeRGB,          "Enable 'RGB' mode");
+      ADD_EVENT(Event::VidmodeSVideo,       "Enable 'S-Video' mode");
+      ADD_EVENT(Event::VidModeComposite,    "Enable 'Composite' mode");
+      ADD_EVENT(Event::VidModeBad,          "Enable 'Badly adjusted' mode");
+      ADD_EVENT(Event::VidModeCustom,       "Enable 'Custom' mode");
+      ADD_EVENT(Event::NextAttribute,       "Select 'Custom' attribute");
+      ADD_EVENT(Event::IncreaseAttribute,   "Modify 'Custom' attribute");
+      ADD_EVENT(Event::PhosphorIncrease,    "Adjust phosphor blend");
+      ADD_EVENT(Event::ScanlinesIncrease,   "Adjust scanline intensity");
       break;
 
     case 4:
       title = "Developer commands";
-      ADD_BIND("`",         "Enter/exit debugger");
+      ADD_EVENT(Event::DebuggerMode,        "Toggle debugger mode");
+      ADD_EVENT(Event::ToggleFrameStats,    "Toggle frame stats");
+      ADD_EVENT(Event::ToggleJitter,        "Toggle TV 'jitter'");
+      ADD_EVENT(Event::ToggleColorLoss,     "Toggle PAL color loss");
+      ADD_EVENT(Event::ToggleCollisions,    "Toggle collisions");
+      ADD_EVENT(Event::ToggleFixedColors,   "Toggle 'Debug colors' mode");
       ADD_LINE();
-      ADD_BIND(ALT_" PgUp", "Increase Display.YStart");
-      ADD_BIND(ALT_" PgDn", "Decrease Display.YStart");
-      ADD_BIND("Ctrl PgUp", "Increase Display.Height");
-      ADD_BIND("Ctrl PgDn", "Decrease Display.Height");
-      ADD_LINE();
-      ADD_BIND(ALT_" L", "Toggle frame stats");
-      ADD_BIND(ALT_" ,", "Toggle 'Debug Colors' mode");
-      ADD_BIND(ALT_" t", "Toggle 'Time Machine' mode");
+      ADD_EVENT(Event::ToggleTimeMachine,   "Toggle 'Time Machine' mode");
+      ADD_EVENT(Event::SaveAllStates,       "Save all 'Time Machine' states");
+      ADD_EVENT(Event::LoadAllStates,       "Load all 'Time Machine' states");
       break;
 
     case 5:
       title = "All other commands";
       ADD_LINE();
-      ADD_BIND("Remapped Eve", "nts");
+      ADD_BIND("Remapped Even", "ts");
       ADD_TEXT("Most other commands can be");
       ADD_TEXT("remapped. Please consult the");
       ADD_TEXT("'Options/Input" + ELLIPSIS + "' dialog for");
       ADD_TEXT("more information.");
       break;
+
+    default:
+      break;
   }
 
   while(i < lines)
@@ -176,10 +177,10 @@ void HelpDialog::updateStrings(uInt8 page, uInt8 lines, string& title)
 void HelpDialog::displayInfo()
 {
   string titleStr;
-  updateStrings(myPage, kLINES_PER_PAGE, titleStr);
+  updateStrings(myPage, LINES_PER_PAGE, titleStr);
 
   myTitle->setLabel(titleStr);
-  for(uInt8 i = 0; i < kLINES_PER_PAGE; ++i)
+  for(uInt8 i = 0; i < LINES_PER_PAGE; ++i)
   {
     myKey[i]->setLabel(myKeyStr[i]);
     myDesc[i]->setLabel(myDescStr[i]);
@@ -195,9 +196,9 @@ void HelpDialog::handleCommand(CommandSender* sender, int cmd,
     case GuiObject::kNextCmd:
       ++myPage;
       if(myPage >= myNumPages)
-        myNextButton->clearFlags(WIDGET_ENABLED);
+        myNextButton->clearFlags(Widget::FLAG_ENABLED);
       if(myPage >= 2)
-        myPrevButton->setFlags(WIDGET_ENABLED);
+        myPrevButton->setFlags(Widget::FLAG_ENABLED);
 
       displayInfo();
       break;
@@ -205,9 +206,9 @@ void HelpDialog::handleCommand(CommandSender* sender, int cmd,
     case GuiObject::kPrevCmd:
       --myPage;
       if(myPage <= myNumPages)
-        myNextButton->setFlags(WIDGET_ENABLED);
+        myNextButton->setFlags(Widget::FLAG_ENABLED);
       if(myPage <= 1)
-        myPrevButton->clearFlags(WIDGET_ENABLED);
+        myPrevButton->clearFlags(Widget::FLAG_ENABLED);
 
       displayInfo();
       break;
