@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -39,7 +39,6 @@
 InputDialog::InputDialog(OSystem& osystem, DialogContainer& parent,
                          const GUI::Font& font, int max_w, int max_h)
   : Dialog(osystem, parent, font, "Input settings"),
-    myConfirmMsg(nullptr),
     myMaxWidth(max_w),
     myMaxHeight(max_h)
 {
@@ -48,11 +47,9 @@ InputDialog::InputDialog(OSystem& osystem, DialogContainer& parent,
             buttonHeight = font.getLineHeight() + 4;
   const int vBorder = 4;
   int xpos, ypos, tabID;
-  StringList actions;
 
   // Set real dimensions
-  _w = std::min(50 * fontWidth + 10, max_w);
-  _h = std::min(16 * (lineHeight + 4) + 16 + _th, max_h);
+  setSize(51 * fontWidth + 10, 17 * (lineHeight + 4) + 16 + _th, max_w, max_h);
 
   // The tab widget
   xpos = 2; ypos = vBorder + _th;
@@ -60,22 +57,20 @@ InputDialog::InputDialog(OSystem& osystem, DialogContainer& parent,
   addTabWidget(myTab);
 
   // 1) Event mapper for emulation actions
-  tabID = myTab->addTab("Emul. Events");
-  actions = instance().eventHandler().getActionList(kEmulationMode);
+  tabID = myTab->addTab(" Emulation Events ", TabWidget::AUTO_WIDTH);
   myEmulEventMapper = new EventMappingWidget(myTab, font, 2, 2,
                                              myTab->getWidth(),
                                              myTab->getHeight() - 4,
-                                             actions, kEmulationMode);
+                                             EventMode::kEmulationMode);
   myTab->setParentWidget(tabID, myEmulEventMapper);
   addToFocusList(myEmulEventMapper->getFocusList(), myTab, tabID);
 
   // 2) Event mapper for UI actions
-  tabID = myTab->addTab("UI Events");
-  actions = instance().eventHandler().getActionList(kMenuMode);
+  tabID = myTab->addTab("  UI Events  ", TabWidget::AUTO_WIDTH);
   myMenuEventMapper = new EventMappingWidget(myTab, font, 2, 2,
                                              myTab->getWidth(),
                                              myTab->getHeight() - 4,
-                                             actions, kMenuMode);
+                                             EventMode::kMenuMode);
   myTab->setParentWidget(tabID, myMenuEventMapper);
   addToFocusList(myMenuEventMapper->getFocusList(), myTab, tabID);
 
@@ -111,7 +106,7 @@ void InputDialog::addDevicePortTab(const GUI::Font& font)
   const int HBORDER = 8;
 
   // Devices/ports
-  tabID = myTab->addTab("Devices & Ports");
+  tabID = myTab->addTab(" Devices & Ports ", TabWidget::AUTO_WIDTH);
 
   ypos = VBORDER;
   lwidth = font.getStringWidth("Digital paddle sensitivity "); // was: "Use mouse as a controller "
@@ -134,53 +129,71 @@ void InputDialog::addDevicePortTab(const GUI::Font& font)
   VarList::push_back(items, "+UI, -Emulation", "2");
   VarList::push_back(items, "+UI, +Emulation", "3");
   myCursorState = new PopUpWidget(myTab, font, HBORDER, ypos, pwidth, lineHeight, items,
-                                  "Mouse cursor visibility ", lwidth);
+                                  "Mouse cursor visibility ", lwidth, kCursorStateChanged);
   wid.push_back(myCursorState);
 #ifndef WINDOWED_SUPPORT
-  myCursorState->clearFlags(WIDGET_ENABLED);
+  myCursorState->clearFlags(Widget::FLAG_ENABLED);
 #endif
 
   lwidth = font.getStringWidth("Digital paddle sensitivity ");
 
   // Add joystick deadzone setting
   ypos += lineHeight + VGAP*3;
-  myDeadzone = new SliderWidget(myTab, font, HBORDER, ypos,
+  myDeadzone = new SliderWidget(myTab, font, HBORDER, ypos, 13 * fontWidth, lineHeight,
                                 "Joystick deadzone size ", lwidth, kDeadzoneChanged);
   myDeadzone->setMinValue(0); myDeadzone->setMaxValue(29);
-  myDeadzone->setTickmarkInterval(4);
+  myDeadzone->setTickmarkIntervals(4);
   xpos = HBORDER + myDeadzone->getWidth() + 5;
   myDeadzoneLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 5*fontWidth, lineHeight, "");
   wid.push_back(myDeadzone);
 
+  // Add dejitter (Stelladaptor emulation for now only)
+  ypos += lineHeight + VGAP;
+  myDejitterBase = new SliderWidget(myTab, font, HBORDER, ypos, 6 * fontWidth, lineHeight,
+                                    "Paddle dejitter strength", lwidth, kDejitterChanged);
+  myDejitterBase->setMinValue(Paddles::MIN_DEJITTER); myDejitterBase->setMaxValue(Paddles::MAX_DEJITTER);
+  myDejitterBase->setTickmarkIntervals(2);
+  xpos = HBORDER + myDejitterBase->getWidth() + fontWidth;
+  wid.push_back(myDejitterBase);
+
+  myDejitterDiff = new SliderWidget(myTab, font, xpos, ypos, 6 * fontWidth, lineHeight,
+                                    "", 0, kDejitterChanged);
+  myDejitterDiff->setMinValue(Paddles::MIN_DEJITTER); myDejitterDiff->setMaxValue(Paddles::MAX_DEJITTER);
+  myDejitterDiff->setTickmarkIntervals(2);
+  xpos += myDejitterDiff->getWidth() + 5;
+  wid.push_back(myDejitterDiff);
+
+  myDejitterLabel = new StaticTextWidget(myTab, font, xpos, ypos + 1, 7 * fontWidth, lineHeight, "");
+
   // Add paddle speed (digital emulation)
   ypos += lineHeight + VGAP;
-  myDPaddleSpeed = new SliderWidget(myTab, font, HBORDER, ypos,
+  myDPaddleSpeed = new SliderWidget(myTab, font, HBORDER, ypos, 13 * fontWidth, lineHeight,
                                     "Digital paddle sensitivity ",
                                     lwidth, kDPSpeedChanged);
   myDPaddleSpeed->setMinValue(1); myDPaddleSpeed->setMaxValue(20);
-  myDPaddleSpeed->setTickmarkInterval(4);
+  myDPaddleSpeed->setTickmarkIntervals(4);
   xpos = HBORDER + myDPaddleSpeed->getWidth() + 5;
   myDPaddleLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 24, lineHeight, "");
   wid.push_back(myDPaddleSpeed);
 
   // Add paddle speed (mouse emulation)
   ypos += lineHeight + VGAP;
-  myMPaddleSpeed = new SliderWidget(myTab, font, HBORDER, ypos,
+  myMPaddleSpeed = new SliderWidget(myTab, font, HBORDER, ypos, 13 * fontWidth, lineHeight,
                                     "Mouse paddle sensitivity ",
                                     lwidth, kMPSpeedChanged);
   myMPaddleSpeed->setMinValue(1); myMPaddleSpeed->setMaxValue(20);
-  myMPaddleSpeed->setTickmarkInterval(4);
+  myMPaddleSpeed->setTickmarkIntervals(4);
   xpos = HBORDER + myMPaddleSpeed->getWidth() + 5;
   myMPaddleLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 24, lineHeight, "");
   wid.push_back(myMPaddleSpeed);
 
   // Add trackball speed
   ypos += lineHeight + VGAP;
-  myTrackBallSpeed = new SliderWidget(myTab, font, HBORDER, ypos,
+  myTrackBallSpeed = new SliderWidget(myTab, font, HBORDER, ypos, 13 * fontWidth, lineHeight,
                                       "Trackball sensitivity ",
                                       lwidth, kTBSpeedChanged);
   myTrackBallSpeed->setMinValue(1); myTrackBallSpeed->setMaxValue(20);
-  myTrackBallSpeed->setTickmarkInterval(4);
+  myTrackBallSpeed->setTickmarkIntervals(4);
   xpos = HBORDER + myTrackBallSpeed->getWidth() + 5;
   myTrackBallLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 24, lineHeight, "");
   wid.push_back(myTrackBallSpeed);
@@ -197,14 +210,14 @@ void InputDialog::addDevicePortTab(const GUI::Font& font)
 	                "Grab mouse in emulation mode");
   wid.push_back(myGrabMouse);
 #ifndef WINDOWED_SUPPORT
-  myGrabMouse->clearFlags(WIDGET_ENABLED);
+  myGrabMouse->clearFlags(Widget::FLAG_ENABLED);
 #endif
 
-  // Enable/disable control key-combos
+  // Enable/disable modifier key-combos
   ypos += lineHeight + VGAP;
-  myCtrlCombo = new CheckboxWidget(myTab, font, HBORDER, ypos,
-	                "Use control key combos");
-  wid.push_back(myCtrlCombo);
+  myModCombo = new CheckboxWidget(myTab, font, HBORDER, ypos,
+	                "Use modifier key combos");
+  wid.push_back(myModCombo);
   ypos += lineHeight + VGAP;
 
   // Stelladaptor mappings
@@ -259,12 +272,16 @@ void InputDialog::loadConfig()
 
   // Mouse cursor state
   myCursorState->setSelected(instance().settings().getString("cursor"), "2");
+  handleCursorState();
 
   // Joystick deadzone
   myDeadzone->setValue(instance().settings().getInt("joydeadzone"));
   myDeadzoneLabel->setValue(Joystick::deadzone());
 
   // Paddle speed (digital and mouse)
+  myDejitterBase->setValue(instance().settings().getInt("dejitter.base"));
+  myDejitterDiff->setValue(instance().settings().getInt("dejitter.diff"));
+  updateDejitter();
   myDPaddleSpeed->setValue(instance().settings().getInt("dsense"));
   myDPaddleLabel->setLabel(instance().settings().getString("dsense"));
   myMPaddleSpeed->setValue(instance().settings().getInt("msense"));
@@ -283,8 +300,8 @@ void InputDialog::loadConfig()
     Controller& lport = instance().console().leftController();
     Controller& rport = instance().console().rightController();
 
-    myEraseEEPROMButton->setEnabled(lport.type() == Controller::SaveKey || lport.type() == Controller::AtariVox ||
-                                    rport.type() == Controller::SaveKey || rport.type() == Controller::AtariVox);
+    myEraseEEPROMButton->setEnabled(lport.type() == Controller::Type::SaveKey || lport.type() == Controller::Type::AtariVox ||
+                                    rport.type() == Controller::Type::SaveKey || rport.type() == Controller::Type::AtariVox);
   }
   else
     myEraseEEPROMButton->setEnabled(false);
@@ -295,8 +312,8 @@ void InputDialog::loadConfig()
   // Grab mouse
   myGrabMouse->setState(instance().settings().getBool("grabmouse"));
 
-  // Enable/disable control key-combos
-  myCtrlCombo->setState(instance().settings().getBool("ctrlcombo"));
+  // Enable/disable modifier key-combos
+  myModCombo->setState(instance().settings().getBool("modcombo"));
 
   myTab->loadConfig();
 }
@@ -318,9 +335,17 @@ void InputDialog::saveConfig()
   Joystick::setDeadZone(deadzone);
 
   // Paddle speed (digital and mouse)
+  int dejitter = myDejitterBase->getValue();
+  instance().settings().setValue("dejitter.base", dejitter);
+  Paddles::setDejitterBase(dejitter);
+  dejitter = myDejitterDiff->getValue();
+  instance().settings().setValue("dejitter.diff", dejitter);
+  Paddles::setDejitterDiff(dejitter);
+
   int sensitivity = myDPaddleSpeed->getValue();
   instance().settings().setValue("dsense", sensitivity);
   Paddles::setDigitalSensitivity(sensitivity);
+
   sensitivity = myMPaddleSpeed->getValue();
   instance().settings().setValue("msense", sensitivity);
   Paddles::setMouseSensitivity(sensitivity);
@@ -341,11 +366,19 @@ void InputDialog::saveConfig()
   // Grab mouse and hide cursor
   const string& cursor = myCursorState->getSelectedTag().toString();
   instance().settings().setValue("cursor", cursor);
-  instance().settings().setValue("grabmouse", myGrabMouse->getState());
-  instance().frameBuffer().enableGrabMouse(myGrabMouse->getState());
+  // only allow grab mouse if cursor is hidden in emulation
+  int state = myCursorState->getSelected();
+  bool enableGrab = state != 1 && state != 3;
+  bool grab = enableGrab ? myGrabMouse->getState() : false;
+  instance().settings().setValue("grabmouse", grab);
+  instance().frameBuffer().enableGrabMouse(grab);
 
-  // Enable/disable control key-combos
-  instance().settings().setValue("ctrlcombo", myCtrlCombo->getState());
+  // Enable/disable modifier key-combos
+  instance().settings().setValue("modcombo", myModCombo->getState());
+
+  instance().eventHandler().saveKeyMapping();
+  instance().eventHandler().saveJoyMapping();
+//  instance().saveConfig();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -381,6 +414,14 @@ void InputDialog::setDefaults()
       myDPaddleLabel->setLabel("10");
       myMPaddleSpeed->setValue(10);
       myMPaddleLabel->setLabel("10");
+    #if defined(RETRON77)
+      myDejitterBase->setValue(2);
+      myDejitterDiff->setValue(6);
+    #else
+      myDejitterBase->setValue(0);
+      myDejitterDiff->setValue(0);
+    #endif
+      updateDejitter();
       myTrackBallSpeed->setValue(10);
       myTrackBallLabel->setLabel("10");
 
@@ -393,8 +434,10 @@ void InputDialog::setDefaults()
       // Grab mouse
       myGrabMouse->setState(true);
 
-      // Enable/disable control key-combos
-      myCtrlCombo->setState(true);
+      // Enable/disable modifier key-combos
+      myModCombo->setState(true);
+
+      handleCursorState();
 
       break;
     }
@@ -405,19 +448,37 @@ void InputDialog::setDefaults()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void InputDialog::handleKeyDown(StellaKey key, StellaMod mod)
+bool InputDialog::repeatEnabled()
+{
+  return !myEmulEventMapper->isRemapping() && !myMenuEventMapper->isRemapping();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void InputDialog::handleKeyDown(StellaKey key, StellaMod mod, bool repeated)
 {
   // Remap key events in remap mode, otherwise pass to parent dialog
-  if(myEmulEventMapper->remapMode())
+  if (myEmulEventMapper->remapMode())
     myEmulEventMapper->handleKeyDown(key, mod);
-  else if(myMenuEventMapper->remapMode())
+  else if (myMenuEventMapper->remapMode())
     myMenuEventMapper->handleKeyDown(key, mod);
   else
     Dialog::handleKeyDown(key, mod);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void InputDialog::handleJoyDown(int stick, int button)
+void InputDialog::handleKeyUp(StellaKey key, StellaMod mod)
+{
+  // Remap key events in remap mode, otherwise pass to parent dialog
+  if (myEmulEventMapper->remapMode())
+    myEmulEventMapper->handleKeyUp(key, mod);
+  else if (myMenuEventMapper->remapMode())
+    myMenuEventMapper->handleKeyUp(key, mod);
+  else
+    Dialog::handleKeyUp(key, mod);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void InputDialog::handleJoyDown(int stick, int button, bool longPress)
 {
   // Remap joystick buttons in remap mode, otherwise pass to parent dialog
   if(myEmulEventMapper->remapMode())
@@ -429,27 +490,39 @@ void InputDialog::handleJoyDown(int stick, int button)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void InputDialog::handleJoyAxis(int stick, int axis, int value)
+void InputDialog::handleJoyUp(int stick, int button)
 {
-  // Remap joystick axis in remap mode, otherwise pass to parent dialog
-  if(myEmulEventMapper->remapMode())
-    myEmulEventMapper->handleJoyAxis(stick, axis, value);
-  else if(myMenuEventMapper->remapMode())
-    myMenuEventMapper->handleJoyAxis(stick, axis, value);
+  // Remap joystick buttons in remap mode, otherwise pass to parent dialog
+  if (myEmulEventMapper->remapMode())
+    myEmulEventMapper->handleJoyUp(stick, button);
+  else if (myMenuEventMapper->remapMode())
+    myMenuEventMapper->handleJoyUp(stick, button);
   else
-    Dialog::handleJoyAxis(stick, axis, value);
+    Dialog::handleJoyUp(stick, button);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool InputDialog::handleJoyHat(int stick, int hat, JoyHat value)
+void InputDialog::handleJoyAxis(int stick, JoyAxis axis, JoyDir adir, int button)
+{
+  // Remap joystick axis in remap mode, otherwise pass to parent dialog
+  if(myEmulEventMapper->remapMode())
+    myEmulEventMapper->handleJoyAxis(stick, axis, adir, button);
+  else if(myMenuEventMapper->remapMode())
+    myMenuEventMapper->handleJoyAxis(stick, axis, adir, button);
+  else
+    Dialog::handleJoyAxis(stick, axis, adir, button);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool InputDialog::handleJoyHat(int stick, int hat, JoyHatDir hdir, int button)
 {
   // Remap joystick hat in remap mode, otherwise pass to parent dialog
   if(myEmulEventMapper->remapMode())
-    return myEmulEventMapper->handleJoyHat(stick, hat, value);
+    return myEmulEventMapper->handleJoyHat(stick, hat, hdir, button);
   else if(myMenuEventMapper->remapMode())
-    return myMenuEventMapper->handleJoyHat(stick, hat, value);
+    return myMenuEventMapper->handleJoyHat(stick, hat, hdir, button);
   else
-    return Dialog::handleJoyHat(stick, hat, value);
+    return Dialog::handleJoyHat(stick, hat, hdir, button);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -460,13 +533,13 @@ void InputDialog::eraseEEPROM()
   Controller& lport = instance().console().leftController();
   Controller& rport = instance().console().rightController();
 
-  if(lport.type() == Controller::SaveKey || lport.type() == Controller::AtariVox)
+  if(lport.type() == Controller::Type::SaveKey || lport.type() == Controller::Type::AtariVox)
   {
     SaveKey& skey = static_cast<SaveKey&>(lport);
     skey.eraseCurrent();
   }
 
-  if(rport.type() == Controller::SaveKey || rport.type() == Controller::AtariVox)
+  if(rport.type() == Controller::Type::SaveKey || rport.type() == Controller::Type::AtariVox)
   {
     SaveKey& skey = static_cast<SaveKey&>(rport);
     skey.eraseCurrent();
@@ -493,6 +566,10 @@ void InputDialog::handleCommand(CommandSender* sender, int cmd,
       setDefaults();
       break;
 
+    case kCursorStateChanged:
+      handleCursorState();
+      break;
+
     case kDeadzoneChanged:
       myDeadzoneLabel->setValue(3200 + 1000*myDeadzone->getValue());
       break;
@@ -505,6 +582,10 @@ void InputDialog::handleCommand(CommandSender* sender, int cmd,
       myMPaddleLabel->setValue(myMPaddleSpeed->getValue());
       break;
 
+    case kDejitterChanged:
+      updateDejitter();
+      break;
+
     case kTBSpeedChanged:
       myTrackBallLabel->setValue(myTrackBallSpeed->getValue());
       break;
@@ -546,3 +627,35 @@ void InputDialog::handleCommand(CommandSender* sender, int cmd,
       Dialog::handleCommand(sender, cmd, data, 0);
   }
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void InputDialog::handleCursorState()
+{
+  int state = myCursorState->getSelected();
+  bool enableGrab = state != 1 && state != 3;
+
+  myGrabMouse->setEnabled(enableGrab);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void InputDialog::updateDejitter()
+{
+  int strength = myDejitterBase->getValue();
+  stringstream label;
+
+  if (strength)
+    label << myDejitterBase->getValue();
+  else
+    label << "Off";
+
+  label << " ";
+  strength = myDejitterDiff->getValue();
+
+  if (strength)
+    label << myDejitterDiff->getValue();
+  else
+    label << "Off";
+
+  myDejitterLabel->setLabel(label.str());
+}
+
