@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -31,11 +31,7 @@
 InputTextDialog::InputTextDialog(GuiObject* boss, const GUI::Font& font,
                                  const StringList& labels, const string& title)
   : Dialog(boss->instance(), boss->parent(), font, title),
-    CommandSender(boss),
-    myEnableCenter(false),
-    myErrorFlag(false),
-    myXOrig(0),
-    myYOrig(0)
+    CommandSender(boss)
 {
   initialize(font, font, labels);
 }
@@ -45,11 +41,7 @@ InputTextDialog::InputTextDialog(GuiObject* boss, const GUI::Font& lfont,
                                  const GUI::Font& nfont,
                                  const StringList& labels, const string& title)
   : Dialog(boss->instance(), boss->parent(), lfont, title),
-    CommandSender(boss),
-    myEnableCenter(false),
-    myErrorFlag(false),
-    myXOrig(0),
-    myYOrig(0)
+    CommandSender(boss)
 {
   initialize(lfont, nfont, labels);
 }
@@ -118,10 +110,16 @@ void InputTextDialog::show()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void InputTextDialog::show(uInt32 x, uInt32 y)
+void InputTextDialog::show(uInt32 x, uInt32 y, const Common::Rect& bossRect)
 {
-  myXOrig = x;
-  myYOrig = y;
+  uInt32 scale = instance().frameBuffer().hidpiScaleFactor();
+  myXOrig = bossRect.x() + x * scale;
+  myYOrig = bossRect.y() + y * scale;
+
+  // Only show dialog if we're inside the visible area
+  if(!bossRect.contains(myXOrig, myYOrig))
+    return;
+
   myEnableCenter = false;
   open();
 }
@@ -131,17 +129,14 @@ void InputTextDialog::center()
 {
   if(!myEnableCenter)
   {
-    // Make sure the menu is exactly where it should be, in case the image
-    // offset has changed
-    const GUI::Rect& image = instance().frameBuffer().imageRect();
-    uInt32 x = image.x() + myXOrig;
-    uInt32 y = image.y() + myYOrig;
-    uInt32 tx = image.x() + image.width();
-    uInt32 ty = image.y() + image.height();
-    if(x + _w > tx) x -= (x + _w - tx);
-    if(y + _h > ty) y -= (y + _h - ty);
+    // First set position according to original coordinates
+    surface().setDstPos(myXOrig, myYOrig);
 
-    surface().setDstPos(x, y);
+    // Now make sure that the entire menu can fit inside the screen bounds
+    // If not, we reset its position
+    if(!instance().frameBuffer().screenRect().contains(
+        myXOrig, myXOrig, surface().dstRect()))
+      surface().setDstPos(myXOrig, myYOrig);
   }
   else
     Dialog::center();
