@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -61,7 +61,7 @@ JoystickDialog::JoystickDialog(GuiObject* boss, const GUI::Font& font,
   xpos -= buttonWidth + 8;
   myRemoveBtn = new ButtonWidget(this, font, xpos, ypos,
       buttonWidth, buttonHeight, "Remove", kRemoveCmd);
-  myRemoveBtn->clearFlags(WIDGET_ENABLED);
+  myRemoveBtn->clearFlags(Widget::FLAG_ENABLED);
 
   // Now we can finally add the widgets to the focus list
   wid.push_back(myRemoveBtn);
