@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -29,23 +29,32 @@
 Launcher::Launcher(OSystem& osystem)
   : DialogContainer(osystem)
 {
-  const GUI::Size& s = myOSystem.settings().getSize("launcherres");
-  const GUI::Size& d = myOSystem.frameBuffer().desktopSize();
+  const Common::Size& s = myOSystem.settings().getSize("launcherres");
+  const Common::Size& d = myOSystem.frameBuffer().desktopSize();
+  double overscan = 1 - myOSystem.settings().getInt("tia.fs_overscan") / 100.0;
   myWidth = s.w;  myHeight = s.h;
 
   // The launcher dialog is resizable, within certain bounds
   // We check those bounds now
-  myWidth  = std::max(myWidth,  uInt32(FrameBuffer::kFBMinW));
-  myHeight = std::max(myHeight, uInt32(FrameBuffer::kFBMinH));
-  myWidth  = std::min(myWidth,  uInt32(d.w));
-  myHeight = std::min(myHeight, uInt32(d.h));
-
-  myOSystem.settings().setValue("launcherres",
-                                GUI::Size(myWidth, myHeight));
+  myWidth  = std::max(myWidth,  FBMinimum::Width);
+  myHeight = std::max(myHeight, FBMinimum::Height);
+  myWidth  = std::min(myWidth,  d.w);
+  myHeight = std::min(myHeight, d.h);
+  // do not include overscan when launcher saving size
+  myOSystem.settings().setValue("launcherres", Common::Size(myWidth, myHeight));
+  // now make overscan effective
+  myWidth  = std::min(myWidth,  uInt32(d.w * overscan));
+  myHeight = std::min(myHeight, uInt32(d.h * overscan));
 
   myBaseDialog = new LauncherDialog(myOSystem, *this, 0, 0, myWidth, myHeight);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Launcher::~Launcher()
+{
+  delete myBaseDialog;  myBaseDialog = nullptr;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 FBInitStatus Launcher::initializeVideo()
 {
@@ -53,6 +62,12 @@ FBInitStatus Launcher::initializeVideo()
   return myOSystem.frameBuffer().createDisplay(title, myWidth, myHeight);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+const string& Launcher::selectedRom()
+{
+  return (static_cast<LauncherDialog*>(myBaseDialog))->selectedRom();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 const string& Launcher::selectedRomMD5()
 {
@@ -60,9 +75,9 @@ const string& Launcher::selectedRomMD5()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const FilesystemNode& Launcher::currentNode() const
+const FilesystemNode& Launcher::currentDir() const
 {
-  return (static_cast<LauncherDialog*>(myBaseDialog))->currentNode();
+  return (static_cast<LauncherDialog*>(myBaseDialog))->currentDir();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -70,3 +85,9 @@ void Launcher::reload()
 {
   (static_cast<LauncherDialog*>(myBaseDialog))->reload();
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Dialog* Launcher::baseDialog()
+{
+  return myBaseDialog;
+}
