@@ -8,40 +8,28 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include <cctype>
-#include <algorithm>
-
 #include "OSystem.hxx"
 #include "Widget.hxx"
 #include "ScrollBarWidget.hxx"
 #include "Dialog.hxx"
 #include "FrameBuffer.hxx"
 #include "StellaKeys.hxx"
+#include "EventHandler.hxx"
 #include "ListWidget.hxx"
-#include "bspf.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ListWidget::ListWidget(GuiObject* boss, const GUI::Font& font,
-                       int x, int y, int w, int h, bool quickSelect)
-  : EditableWidget(boss, font, x, y, 16, 16),
-    _rows(0),
-    _cols(0),
-    _currentPos(0),
-    _selectedItem(-1),
-    _highlightedItem(-1),
-    _editMode(false),
-    _currentKeyDown(KBDK_UNKNOWN),
-    _quickSelect(quickSelect),
-    _quickSelectTime(0)
+                       int x, int y, int w, int h)
+  : EditableWidget(boss, font, x, y, 16, 16)
 {
-  _flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS;
+  _flags = Widget::FLAG_ENABLED | Widget::FLAG_CLEARBG | Widget::FLAG_RETAIN_FOCUS;
   _bgcolor = kWidColor;
   _bgcolorhi = kWidColor;
   _textcolor = kTextColor;
@@ -164,8 +152,13 @@ void ListWidget::recalc()
 {
   int size = int(_list.size());
 
-  if (_currentPos >= size)
-    _currentPos = size - 1;
+  if(_currentPos >= size)
+  {
+    if(size <= _rows)
+      _currentPos = 0;
+    else
+      _currentPos = size - _rows;
+  }
   if (_currentPos < 0)
     _currentPos = 0;
 
@@ -176,6 +169,8 @@ void ListWidget::recalc()
 
   _scrollBar->_numEntries     = int(_list.size());
   _scrollBar->_entriesPerPage = _rows;
+  // disable scrollbar if no longer necessary
+  scrollBarRecalc();
 
   // Reset to normal data entry
   abortEditMode();
@@ -246,53 +241,8 @@ int ListWidget::findItem(int x, int y) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool ListWidget::handleText(char text)
 {
-  bool handled = true;
-  int oldSelectedItem = _selectedItem;
-
-  if (!_editMode && _quickSelect)
-  {
-    // Quick selection mode: Go to first list item starting with this key
-    // (or a substring accumulated from the last couple key presses).
-    // Only works in a useful fashion if the list entries are sorted.
-    // TODO: Maybe this should be off by default, and instead we add a
-    // method "enableQuickSelect()" or so ?
-    uInt64 time = instance().getTicks() / 1000;
-    if (_quickSelectTime < time)
-      _quickSelectStr = text;
-    else
-      _quickSelectStr += text;
-    _quickSelectTime = time + _QUICK_SELECT_DELAY;
-
-    // FIXME: This is bad slow code (it scans the list linearly each time a
-    // key is pressed); it could be much faster. Only of importance if we have
-    // quite big lists to deal with -- so for now we can live with this lazy
-    // implementation :-)
-    int newSelectedItem = 0;
-    for(const auto& i: _list)
-    {
-      if(BSPF::startsWithIgnoreCase(i, _quickSelectStr))
-      {
-        _selectedItem = newSelectedItem;
-        break;
-      }
-      newSelectedItem++;
-    }
-  }
-  else if (_editMode)
-  {
-    // Class EditableWidget handles all text editing related key presses for us
-    handled = EditableWidget::handleText(text);
-  }
-
-  if (_selectedItem != oldSelectedItem)
-  {
-    _scrollBar->draw();
-    scrollToSelected();
-
-    sendCommand(ListWidget::kSelectionChangedCmd, _selectedItem, _id);
-  }
-
-  return handled;
+  // Class EditableWidget handles all text editing related key presses for us
+  return _editMode ? EditableWidget::handleText(text) : true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -321,16 +271,22 @@ bool ListWidget::handleKeyDown(StellaKey key, StellaMod mod)
     }
   }
 
-  _currentKeyDown = key;
   return handled;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool ListWidget::handleKeyUp(StellaKey key, StellaMod mod)
+void ListWidget::handleJoyDown(int stick, int button, bool longPress)
 {
-  if (key == _currentKeyDown)
-    _currentKeyDown = KBDK_UNKNOWN;
-  return true;
+  if (longPress)
+    sendCommand(ListWidget::kLongButtonPressCmd, _selectedItem, _id);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void ListWidget::handleJoyUp(int stick, int button)
+{
+  Event::Type e = _boss->instance().eventHandler().eventForJoyButton(EventMode::kMenuMode, stick, button);
+
+  handleEvent(e);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -366,12 +322,14 @@ bool ListWidget::handleEvent(Event::Type e)
       break;
 
     case Event::UIPgUp:
+    case Event::UILeft:
       _selectedItem -= _rows - 1;
       if (_selectedItem < 0)
         _selectedItem = 0;
       break;
 
     case Event::UIPgDown:
+    case Event::UIRight:
       _selectedItem += _rows - 1;
       if (_selectedItem >= size)
         _selectedItem = size - 1;
@@ -416,18 +374,16 @@ void ListWidget::lostFocusWidget()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ListWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
 {
-  switch (cmd)
+  if (cmd == GuiObject::kSetPositionCmd)
   {
-    case GuiObject::kSetPositionCmd:
-      if (_currentPos != data)
-      {
-        _currentPos = data;
-        setDirty();
+    if (_currentPos != data)
+    {
+      _currentPos = data;
+      setDirty();
 
-        // Let boss know the list has scrolled
-        sendCommand(ListWidget::kScrolledCmd, _currentPos, _id);
-      }
-      break;
+      // Let boss know the list has scrolled
+      sendCommand(ListWidget::kScrolledCmd, _currentPos, _id);
+    }
   }
 }
 
@@ -498,6 +454,3 @@ void ListWidget::abortEditMode()
   // Reset to normal data entry
   EditableWidget::abortEditMode();
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt64 ListWidget::_QUICK_SELECT_DELAY = 300;
