@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -27,14 +27,14 @@
 #include "StringParser.hxx"
 #include "Widget.hxx"
 #include "Font.hxx"
+#include "Logger.hxx"
 #include "LoggerDialog.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 LoggerDialog::LoggerDialog(OSystem& osystem, DialogContainer& parent,
                            const GUI::Font& font, int max_w, int max_h,
                            bool uselargefont)
-  : Dialog(osystem, parent, font, "System logs"),
-    myLogInfo(nullptr)
+  : Dialog(osystem, parent, font, "System logs")
 {
   const int lineHeight   = font.getLineHeight(),
             buttonWidth  = font.getStringWidth("Save log to disk") + 20,
@@ -44,8 +44,7 @@ LoggerDialog::LoggerDialog(OSystem& osystem, DialogContainer& parent,
 
   // Set real dimensions
   // This is one dialog that can take as much space as is available
-  _w = max_w;
-  _h = max_h;
+  setSize(4000, 4000, max_w, max_h);
 
   // Test listing of the log output
   xpos = 10;  ypos = 10 + _th;
@@ -58,9 +57,9 @@ LoggerDialog::LoggerDialog(OSystem& osystem, DialogContainer& parent,
 
   // Level of logging (how much info to print)
   VariantList items;
-  VarList::push_back(items, "None", "0");
-  VarList::push_back(items, "Basic", "1");
-  VarList::push_back(items, "Verbose", "2");
+  VarList::push_back(items, "None", static_cast<int>(Logger::Level::ERR));
+  VarList::push_back(items, "Basic", static_cast<int>(Logger::Level::INFO));
+  VarList::push_back(items, "Verbose", static_cast<int>(Logger::Level::DEBUG));
   myLogLevel =
     new PopUpWidget(this, font, xpos, ypos, font.getStringWidth("Verbose"),
                     lineHeight, items, "Log level ",
@@ -86,33 +85,41 @@ LoggerDialog::LoggerDialog(OSystem& osystem, DialogContainer& parent,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LoggerDialog::loadConfig()
 {
-  StringParser parser(instance().logMessages());
+  StringParser parser(Logger::instance().logMessages());
   myLogInfo->setList(parser.stringList());
   myLogInfo->setSelected(0);
   myLogInfo->scrollToEnd();
 
-  myLogLevel->setSelected(instance().settings().getString("loglevel"), "1");
+  myLogLevel->setSelected(instance().settings().getString("loglevel"),
+    static_cast<int>(Logger::Level::INFO));
   myLogToConsole->setState(instance().settings().getBool("logtoconsole"));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LoggerDialog::saveConfig()
 {
-  instance().settings().setValue("loglevel",
-    myLogLevel->getSelectedTag().toString());
-  instance().settings().setValue("logtoconsole", myLogToConsole->getState());
+  int loglevel = myLogLevel->getSelectedTag().toInt();
+  bool logtoconsole = myLogToConsole->getState();
+
+  instance().settings().setValue("loglevel", loglevel);
+  instance().settings().setValue("logtoconsole", logtoconsole);
+
+  Logger::instance().setLogParameters(loglevel, logtoconsole);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LoggerDialog::saveLogFile()
 {
   ostringstream path;
-  path << "~" << BSPF::PATH_SEPARATOR << "stella.log";
+  path << instance().defaultSaveDir() << "stella.log";
   FilesystemNode node(path.str());
 
   ofstream out(node.getPath());
   if(out.is_open())
-    out << instance().logMessages();
+  {
+    out << Logger::instance().logMessages();
+    instance().frameBuffer().showMessage("Saving log file to " + path.str());
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
