@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -27,13 +27,14 @@ namespace GUI {
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 MessageBox::MessageBox(GuiObject* boss, const GUI::Font& font,
-                       const StringList& text, int max_w, int max_h, int cmd,
+                       const StringList& text, int max_w, int max_h, int okCmd, int cancelCmd,
                        const string& okText, const string& cancelText,
                        const string& title,
                        bool focusOKButton)
   : Dialog(boss->instance(), boss->parent(), font, title, 0, 0, max_w, max_h),
     CommandSender(boss),
-    myCmd(cmd)
+    myOkCmd(okCmd),
+    myCancelCmd(cancelCmd)
 {
   addText(font, text);
 
@@ -44,20 +45,41 @@ MessageBox::MessageBox(GuiObject* boss, const GUI::Font& font,
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 MessageBox::MessageBox(GuiObject* boss, const GUI::Font& font,
-                       const string& text, int max_w, int max_h, int cmd,
+                       const StringList& text, int max_w, int max_h, int okCmd,
+                       const string& okText, const string& cancelText,
+                       const string& title,
+                       bool focusOKButton)
+  : MessageBox(boss, font, text, max_w, max_h,
+               okCmd, 0, okText, cancelText, title, focusOKButton)
+{
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+MessageBox::MessageBox(GuiObject* boss, const GUI::Font& font,
+                       const string& text, int max_w, int max_h, int okCmd,
                        const string& okText, const string& cancelText,
                        const string& title,
                        bool focusOKButton)
   : MessageBox(boss, font, StringParser(text).stringList(), max_w, max_h,
-               cmd, okText, cancelText, title, focusOKButton)
+               okCmd, okText, cancelText, title, focusOKButton)
+{
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+MessageBox::MessageBox(GuiObject* boss, const GUI::Font& font,
+                       const string& text, int max_w, int max_h, int okCmd, int cancelCmd,
+                       const string& okText, const string& cancelText,
+                       const string& title,
+                       bool focusOKButton)
+  : MessageBox(boss, font, StringParser(text).stringList(), max_w, max_h,
+               okCmd, cancelCmd, okText, cancelText, title, focusOKButton)
 {
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void MessageBox::addText(const GUI::Font& font, const StringList& text)
 {
-  const int lineHeight = font.getLineHeight(),
-            fontWidth  = font.getMaxCharWidth(),
+  const int fontWidth  = font.getMaxCharWidth(),
             fontHeight = font.getFontHeight();
   int xpos, ypos;
 
@@ -66,7 +88,7 @@ void MessageBox::addText(const GUI::Font& font, const StringList& text)
   for(const auto& s: text)
     str_w = std::max(int(s.length()), str_w);
   _w = std::min(str_w * fontWidth + 20, _w);
-  _h = std::min(uInt32((text.size() + 2) * lineHeight + 20 + _th), uInt32(_h));
+  _h = std::min(uInt32((text.size() + 2) * fontHeight + 20 + _th), uInt32(_h));
 
   xpos = 10;  ypos = 10 + _th;
   for(const auto& s: text)
@@ -80,24 +102,25 @@ void MessageBox::addText(const GUI::Font& font, const StringList& text)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void MessageBox::handleCommand(CommandSender* sender, int cmd, int data, int id)
 {
-  switch(cmd)
+  if(cmd == GuiObject::kOKCmd)
   {
-    case GuiObject::kOKCmd:
-    {
-      close();
+    close();
 
-      // Send a signal to the calling class that 'OK' has been selected
-      // Since we aren't derived from a widget, we don't have a 'data' or 'id'
-      if(myCmd)
-        sendCommand(myCmd, 0, 0);
-
-      break;
-    }
-
-    default:
-      Dialog::handleCommand(sender, cmd, data, id);
-      break;
+    // Send a signal to the calling class that 'OK' has been selected
+    // Since we aren't derived from a widget, we don't have a 'data' or 'id'
+    if(myOkCmd)
+      sendCommand(myOkCmd, 0, 0);
   }
+  else if (cmd == GuiObject::kCloseCmd)
+  {
+    close();
+
+    // Send a signal to the calling class that 'Cancel' has been selected
+    if (myCancelCmd)
+      sendCommand(myCancelCmd, 0, 0);
+  }
+  else
+    Dialog::handleCommand(sender, cmd, data, id);
 }
 
 }  // namespace GUI
