@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -28,7 +28,6 @@
 #include "InputDialog.hxx"
 #include "UIDialog.hxx"
 #include "SnapshotDialog.hxx"
-#include "ConfigPathDialog.hxx"
 #include "RomAuditDialog.hxx"
 #include "GameInfoDialog.hxx"
 #include "LoggerDialog.hxx"
@@ -38,6 +37,7 @@
 #include "OptionsDialog.hxx"
 #include "Launcher.hxx"
 #include "Settings.hxx"
+#include "Menu.hxx"
 
 #ifdef CHEATCODE_SUPPORT
   #include "CheatCodeDialog.hxx"
@@ -47,23 +47,35 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 OptionsDialog::OptionsDialog(OSystem& osystem, DialogContainer& parent,
-                             GuiObject* boss, int max_w, int max_h, stellaMode mode)
+                             GuiObject* boss, int max_w, int max_h, Menu::AppMode mode)
   : Dialog(osystem, parent, osystem.frameBuffer().font(), "Options"),
-    myMode(mode),
-    _boss(boss)
+    myMode(mode)
 {
-  const int buttonWidth = _font.getStringWidth("Game Properties" + ELLIPSIS) + 20,
-            buttonHeight = _font.getLineHeight() + 6,
-            rowHeight = _font.getLineHeight() + 10;
-  const int VBORDER = 10 + _th;
+  // do not show basic settings options in debugger
+  bool minSettings = osystem.settings().getBool("minimal_ui") && mode != Menu::AppMode::debugger;
+  const int buttonHeight = _font.getLineHeight() + 6,
+    GAP = buttonHeight > 26 ? 5 : 4,
+    rowHeight = buttonHeight + GAP;
+  const int VBORDER = GAP * 2 + 2;
+  const int HBORDER = GAP * 2 + 2;
+  int buttonWidth = _font.getStringWidth("Game Properties" + ELLIPSIS) + GAP * 5;
 
-  _w = 2 * buttonWidth + 30;
-  _h = 7 * rowHeight + 15 + _th;
+  _w = 2 * buttonWidth + HBORDER * 3;
+  _h = 7 * rowHeight + VBORDER * 2 - GAP + _th;
 
-  int xoffset = 10, yoffset = VBORDER;
+  int xoffset = HBORDER, yoffset = VBORDER + _th;
   WidgetArray wid;
   ButtonWidget* b = nullptr;
 
+  if (minSettings)
+  {
+    ButtonWidget* bw = new ButtonWidget(this, _font, xoffset, yoffset,
+      _w - HBORDER * 2, buttonHeight, "Use Basic Settings", kBasSetCmd);
+    wid.push_back(bw);
+    yoffset += rowHeight + GAP * 2;
+    _h += rowHeight + GAP * 2;
+  }
+
   auto ADD_OD_BUTTON = [&](const string& label, int cmd)
   {
     ButtonWidget* bw = new ButtonWidget(this, _font, xoffset, yoffset,
@@ -77,7 +89,7 @@ OptionsDialog::OptionsDialog(OSystem& osystem, DialogContainer& parent,
 
   b = ADD_OD_BUTTON("Audio" + ELLIPSIS, kAudCmd);
 #ifndef SOUND_SUPPORT
-  b->clearFlags(WIDGET_ENABLED);
+  b->clearFlags(Widget::FLAG_ENABLED);
 #endif
   wid.push_back(b);
 
@@ -90,21 +102,20 @@ OptionsDialog::OptionsDialog(OSystem& osystem, DialogContainer& parent,
   b = ADD_OD_BUTTON("Snapshots" + ELLIPSIS, kSnapCmd);
   wid.push_back(b);
 
-  b = ADD_OD_BUTTON("Paths" + ELLIPSIS, kCfgPathsCmd);
-  wid.push_back(b);
-
+  //yoffset += rowHeight;
   b = ADD_OD_BUTTON("Developer" + ELLIPSIS, kDevelopCmd);
   wid.push_back(b);
 
   // Move to second column
-  xoffset += buttonWidth + 10;  yoffset = VBORDER;
+  xoffset += buttonWidth + HBORDER;
+  yoffset = minSettings ? VBORDER + _th + rowHeight + GAP * 2 : VBORDER + _th;
 
   myGameInfoButton = ADD_OD_BUTTON("Game Properties" + ELLIPSIS, kInfoCmd);
   wid.push_back(myGameInfoButton);
 
   myCheatCodeButton = ADD_OD_BUTTON("Cheat Codes" + ELLIPSIS, kCheatCmd);
 #ifndef CHEATCODE_SUPPORT
-  myCheatCodeButton->clearFlags(WIDGET_ENABLED);
+  myCheatCodeButton->clearFlags(Widget::FLAG_ENABLED);
 #endif
   wid.push_back(myCheatCodeButton);
 
@@ -120,6 +131,8 @@ OptionsDialog::OptionsDialog(OSystem& osystem, DialogContainer& parent,
   b = ADD_OD_BUTTON("About" + ELLIPSIS, kAboutCmd);
   wid.push_back(b);
 
+  buttonWidth = _font.getStringWidth("   Close   ") + GAP * 5;
+  xoffset -= (buttonWidth + HBORDER) / 2;
   b = ADD_OD_BUTTON("Close", kExitCmd);
   wid.push_back(b);
   addCancelWidget(b);
@@ -128,28 +141,27 @@ OptionsDialog::OptionsDialog(OSystem& osystem, DialogContainer& parent,
   myVideoDialog    = make_unique<VideoDialog>(osystem, parent, _font, max_w, max_h);
   myAudioDialog    = make_unique<AudioDialog>(osystem, parent, _font);
   myInputDialog    = make_unique<InputDialog>(osystem, parent, _font, max_w, max_h);
-  myUIDialog       = make_unique<UIDialog>(osystem, parent, _font);
+  myUIDialog       = make_unique<UIDialog>(osystem, parent, _font, boss, max_w, max_h);
   mySnapshotDialog = make_unique<SnapshotDialog>(osystem, parent, _font, max_w, max_h);
-  myConfigPathDialog = make_unique<ConfigPathDialog>(osystem, parent, _font, boss, max_w, max_h);
-  myRomAuditDialog = make_unique<RomAuditDialog>(osystem, parent, _font, max_w, max_h);
-  myGameInfoDialog = make_unique<GameInfoDialog>(osystem, parent, _font, this);
+  myDeveloperDialog = make_unique<DeveloperDialog>(osystem, parent, _font, max_w, max_h);
+  myGameInfoDialog = make_unique<GameInfoDialog>(osystem, parent, _font, this, max_w, max_h);
 #ifdef CHEATCODE_SUPPORT
   myCheatCodeDialog = make_unique<CheatCodeDialog>(osystem, parent, _font);
 #endif
-  myDeveloperDialog = make_unique<DeveloperDialog>(osystem, parent, _font, max_w, max_h);
+  myRomAuditDialog = make_unique<RomAuditDialog>(osystem, parent, _font, max_w, max_h);
   myHelpDialog      = make_unique<HelpDialog>(osystem, parent, _font);
   myAboutDialog     = make_unique<AboutDialog>(osystem, parent, _font);
 
   addToFocusList(wid);
 
   // Certain buttons are disabled depending on mode
-  if(myMode == launcher)
+  if(myMode == Menu::AppMode::launcher)
   {
-    myCheatCodeButton->clearFlags(WIDGET_ENABLED);
+    myCheatCodeButton->clearFlags(Widget::FLAG_ENABLED);
   }
   else
   {
-    myRomAuditButton->clearFlags(WIDGET_ENABLED);
+    myRomAuditButton->clearFlags(Widget::FLAG_ENABLED);
   }
 }
 
@@ -167,13 +179,13 @@ void OptionsDialog::loadConfig()
   switch(instance().eventHandler().state())
   {
     case EventHandlerState::EMULATION:
-      myGameInfoButton->setFlags(WIDGET_ENABLED);
+      myGameInfoButton->setFlags(Widget::FLAG_ENABLED);
       break;
     case EventHandlerState::LAUNCHER:
       if(instance().launcher().selectedRomMD5() != "")
-        myGameInfoButton->setFlags(WIDGET_ENABLED);
+        myGameInfoButton->setFlags(Widget::FLAG_ENABLED);
       else
-        myGameInfoButton->clearFlags(WIDGET_ENABLED);
+        myGameInfoButton->clearFlags(Widget::FLAG_ENABLED);
       break;
     default:
       break;
@@ -186,18 +198,25 @@ void OptionsDialog::handleCommand(CommandSender* sender, int cmd,
 {
   switch(cmd)
   {
+    case kBasSetCmd:
+      // enable basic settings
+      instance().settings().setValue("basic_settings", true);
+      if (myMode != Menu::AppMode::emulator)
+        close();
+      else
+        instance().eventHandler().leaveMenuMode();
+      break;
+
     case kVidCmd:
     {
       // This dialog is resizable under certain conditions, so we need
       // to re-create it as necessary
       uInt32 w = 0, h = 0;
-      getDynamicBounds(w, h);
 
-      if(myVideoDialog == nullptr ||
-          uInt32(myVideoDialog->getWidth()) != w ||
-          uInt32(myVideoDialog->getHeight()) != h)
+      if(myVideoDialog == nullptr || myVideoDialog->shouldResize(w, h))
       {
-        myVideoDialog = make_unique<VideoDialog>(instance(), parent(), instance().frameBuffer().font(), w, h);
+        myVideoDialog = make_unique<VideoDialog>(instance(), parent(),
+                                                 instance().frameBuffer().font(), w, h);
       }
       myVideoDialog->open();
       break;
@@ -208,8 +227,20 @@ void OptionsDialog::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kInptCmd:
+    {
+      // This dialog is resizable under certain conditions, so we need
+      // to re-create it as necessary
+      uInt32 w = 0, h = 0;
+
+      if(myInputDialog == nullptr || myInputDialog->shouldResize(w, h))
+      {
+        myInputDialog = make_unique<InputDialog>(instance(), parent(),
+                                                 instance().frameBuffer().font(), w, h);
+      }
+
       myInputDialog->open();
       break;
+    }
 
     case kUsrIfaceCmd:
       myUIDialog->open();
@@ -220,43 +251,45 @@ void OptionsDialog::handleCommand(CommandSender* sender, int cmd,
       // This dialog is resizable under certain conditions, so we need
       // to re-create it as necessary
       uInt32 w = 0, h = 0;
-      getDynamicBounds(w, h);
 
-      if(mySnapshotDialog == nullptr ||
-          uInt32(mySnapshotDialog->getWidth()) != w ||
-          uInt32(mySnapshotDialog->getHeight()) != h)
+      if(mySnapshotDialog == nullptr || mySnapshotDialog->shouldResize(w, h))
       {
-        mySnapshotDialog = make_unique<SnapshotDialog>(instance(), parent(), instance().frameBuffer().font(), w, h);
+        mySnapshotDialog = make_unique<SnapshotDialog>(instance(), parent(),
+                                                       instance().frameBuffer().font(), w, h);
       }
       mySnapshotDialog->open();
       break;
     }
 
-    case kCfgPathsCmd:
+    case kDevelopCmd:
     {
       // This dialog is resizable under certain conditions, so we need
       // to re-create it as necessary
       uInt32 w = 0, h = 0;
-      getDynamicBounds(w, h);
 
-      if(myConfigPathDialog == nullptr ||
-          uInt32(myConfigPathDialog->getWidth()) != w ||
-          uInt32(myConfigPathDialog->getHeight()) != h)
+      if(myDeveloperDialog == nullptr || myDeveloperDialog->shouldResize(w, h))
       {
-        myConfigPathDialog = make_unique<ConfigPathDialog>(instance(), parent(),
-            instance().frameBuffer().font(), _boss, w, h);
+        myDeveloperDialog = make_unique<DeveloperDialog>(instance(), parent(),
+                                                         instance().frameBuffer().font(), w, h);
       }
-      myConfigPathDialog->open();
+      myDeveloperDialog->open();
       break;
     }
 
-    case kAuditCmd:
-      myRomAuditDialog->open();
-      break;
-
     case kInfoCmd:
+    {
+      // This dialog is resizable under certain conditions, so we need
+      // to re-create it as necessary
+      uInt32 w = 0, h = 0;
+
+      if(myGameInfoDialog == nullptr || myGameInfoDialog->shouldResize(w, h))
+      {
+        myGameInfoDialog = make_unique<GameInfoDialog>(instance(), parent(),
+                                                       instance().frameBuffer().font(), this, w, h);
+      }
       myGameInfoDialog->open();
       break;
+    }
 
 #ifdef CHEATCODE_SUPPORT
     case kCheatCmd:
@@ -264,6 +297,10 @@ void OptionsDialog::handleCommand(CommandSender* sender, int cmd,
       break;
 #endif
 
+    case kAuditCmd:
+      myRomAuditDialog->open();
+      break;
+
     case kLoggerCmd:
     {
       // This dialog is resizable under certain conditions, so we need
@@ -271,22 +308,15 @@ void OptionsDialog::handleCommand(CommandSender* sender, int cmd,
       uInt32 w = 0, h = 0;
       bool uselargefont = getDynamicBounds(w, h);
 
-      if(myLoggerDialog == nullptr ||
-          uInt32(myLoggerDialog->getWidth()) != w ||
-          uInt32(myLoggerDialog->getHeight()) != h)
+      if(myLoggerDialog == nullptr || myLoggerDialog->shouldResize(w, h))
       {
         myLoggerDialog = make_unique<LoggerDialog>(instance(), parent(),
             instance().frameBuffer().font(), w, h, uselargefont);
       }
-
       myLoggerDialog->open();
       break;
     }
 
-    case kDevelopCmd:
-      myDeveloperDialog->open();
-      break;
-
     case kHelpCmd:
       myHelpDialog->open();
       break;
@@ -296,7 +326,7 @@ void OptionsDialog::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kExitCmd:
-      if(myMode != emulator)
+      if(myMode != Menu::AppMode::emulator)
         close();
       else
         instance().eventHandler().leaveMenuMode();
