@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -23,18 +23,6 @@
 #include "DialogContainer.hxx"
 #include "PopUpWidget.hxx"
 
-// Little down arrow
-static uInt32 down_arrow[8] = {
-  0b100000001,
-  0b110000011,
-  0b111000111,
-  0b011101110,
-  0b001111100,
-  0b000111000,
-  0b000010000,
-  0b000000000
-};
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PopUpWidget::PopUpWidget(GuiObject* boss, const GUI::Font& font,
                          int x, int y, int w, int h, const VariantList& list,
@@ -42,10 +30,9 @@ PopUpWidget::PopUpWidget(GuiObject* boss, const GUI::Font& font,
   : Widget(boss, font, x, y - 1, w, h + 2),
     CommandSender(boss),
     _label(label),
-    _labelWidth(labelWidth),
-    _changed(false)
+    _labelWidth(labelWidth)
 {
-  _flags = WIDGET_ENABLED | WIDGET_RETAIN_FOCUS;
+  _flags = Widget::FLAG_ENABLED | Widget::FLAG_RETAIN_FOCUS;
   _bgcolor = kDlgColor;
   _bgcolorhi = kDlgColor;     // do not highlight the background
   _textcolor = kTextColor;
@@ -119,12 +106,8 @@ void PopUpWidget::handleMouseDown(int x, int y, MouseButton b, int clickCount)
   if(isEnabled() && !myMenu->isVisible())
   {
     // Add menu just underneath parent widget
-    const GUI::Rect& image = instance().frameBuffer().imageRect();
-    const GUI::Rect& srect = dialog().surface().dstRect();
-    uInt32 tx = srect.x(), ty = srect.y();
-    tx += getAbsX() + _labelWidth - image.x();
-    ty += getAbsY() + getHeight() - image.y();
-    myMenu->show(tx, ty, myMenu->getSelected());
+    myMenu->show(getAbsX() + _labelWidth, getAbsY() + getHeight(),
+                 dialog().surface().dstRect(), myMenu->getSelected());
   }
 }
 
@@ -143,14 +126,14 @@ void PopUpWidget::handleMouseWheel(int x, int y, int direction)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PopUpWidget::handleMouseEntered()
 {
-  setFlags(WIDGET_HILITED);
+  setFlags(Widget::FLAG_HILITED);
   setDirty();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PopUpWidget::handleMouseLeft()
 {
-  clearFlags(WIDGET_HILITED);
+  clearFlags(Widget::FLAG_HILITED);
   setDirty();
 }
 
@@ -196,6 +179,18 @@ void PopUpWidget::handleCommand(CommandSender* sender, int cmd, int data, int id
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PopUpWidget::drawWidget(bool hilite)
 {
+  // Little down arrow
+  static constexpr std::array<uInt32, 8> down_arrow = {
+    0b100000001,
+    0b110000011,
+    0b111000111,
+    0b011101110,
+    0b001111100,
+    0b000111000,
+    0b000010000,
+    0b000000000
+  };
+
 //cerr << "PopUpWidget::drawWidget\n";
   FBSurface& s = dialog().surface();
   bool onTop = _boss->dialog().isOnTop();
@@ -216,8 +211,8 @@ void PopUpWidget::drawWidget(bool hilite)
   s.fillRect(x + 1, _y + 1, w - 17, _h - 2, onTop ? _changed ? kDbgChangedColor : kWidColor : kDlgColor);
   s.fillRect(x + w - 15, _y + 2, 13, _h - 4, onTop ? isEnabled() && hilite ? kWidColor : kBGColorHi : kBGColorLo);
   // Draw an arrow pointing down at the right end to signal this is a dropdown/popup
-  s.drawBitmap(down_arrow, x + w - 13, _y + myArrowsY + 1,
-               !(isEnabled() && onTop) ? kColor : kTextColor, 9u, 8u);
+  s.drawBitmap(down_arrow.data(), x + w - 13, _y + myArrowsY + 1,
+               !(isEnabled() && onTop) ? kColor : kTextColor, 9U, 8U);
 
   // Draw the selected entry, if any
   const string& name = myMenu->getSelectedName();
