@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -36,7 +36,7 @@
 RomAuditDialog::RomAuditDialog(OSystem& osystem, DialogContainer& parent,
                                const GUI::Font& font, int max_w, int max_h)
   : Dialog(osystem, parent, font, "Audit ROMs"),
-    myConfirmMsg(nullptr),
+    myFont(font),
     myMaxWidth(max_w),
     myMaxHeight(max_h)
 {
@@ -88,9 +88,6 @@ RomAuditDialog::RomAuditDialog(OSystem& osystem, DialogContainer& parent,
   // Add OK and Cancel buttons
   addOKCancelBGroup(wid, font, "Audit", "Close");
   addBGroupToFocusList(wid);
-
-  // Create file browser dialog
-  myBrowser = make_unique<BrowserDialog>(this, font, myMaxWidth, myMaxHeight, "Select ROM directory to audit");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -102,7 +99,7 @@ RomAuditDialog::~RomAuditDialog()
 void RomAuditDialog::loadConfig()
 {
   const string& currentdir =
-    instance().launcher().currentNode().getShortPath();
+    instance().launcher().currentDir().getShortPath();
   const string& path = currentdir == "" ?
     instance().settings().getString("romdir") : currentdir;
 
@@ -121,7 +118,7 @@ void RomAuditDialog::auditRoms()
   FilesystemNode node(auditPath);
   FSList files;
   files.reserve(2048);
-  node.getChildren(files, FilesystemNode::kListFilesOnly);
+  node.getChildren(files, FilesystemNode::ListMode::FilesOnly);
 
   // Create a progress dialog box to show the progress of processing
   // the ROMs, since this is usually a time-consuming operation
@@ -129,9 +126,8 @@ void RomAuditDialog::auditRoms()
                           "Auditing ROM files ...");
   progress.setRange(0, int(files.size()) - 1, 5);
 
-  // Create a entry for the GameList for each file
   Properties props;
-  int renamed = 0, notfound = 0;
+  uInt32 renamed = 0, notfound = 0;
   for(uInt32 idx = 0; idx < files.size(); ++idx)
   {
     string extension;
@@ -145,7 +141,7 @@ void RomAuditDialog::auditRoms()
       const string& md5 = MD5::hash(files[idx]);
       if(instance().propSet().getMD5(md5, props))
       {
-        const string& name = props.get(Cartridge_Name);
+        const string& name = props.get(PropType::Cart_Name);
 
         // Only rename the file if we found a valid properties entry
         if(name != "" && name != files[idx].getName())
@@ -166,8 +162,8 @@ void RomAuditDialog::auditRoms()
   }
   progress.close();
 
-  myResults1->setText(Variant(renamed).toString());
-  myResults2->setText(Variant(notfound).toString());
+  myResults1->setText(std::to_string(renamed));
+  myResults2->setText(std::to_string(notfound));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -188,9 +184,8 @@ void RomAuditDialog::handleCommand(CommandSender* sender, int cmd,
         msg.push_back("If you're sure you want to proceed with the");
         msg.push_back("audit, click 'OK', otherwise click 'Cancel'.");
         myConfirmMsg = make_unique<GUI::MessageBox>
-                          (this, instance().frameBuffer().font(), msg,
-                          myMaxWidth, myMaxHeight, kConfirmAuditCmd,
-                           "OK", "Cancel", "ROM Audit", false);
+          (this, myFont, msg, myMaxWidth, myMaxHeight, kConfirmAuditCmd,
+          "OK", "Cancel", "ROM Audit", false);
       }
       myConfirmMsg->show();
       break;
@@ -201,6 +196,7 @@ void RomAuditDialog::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kChooseAuditDirCmd:
+      createBrowser("Select ROM directory to audit");
       myBrowser->show(myRomPath->getText(),
                       BrowserDialog::Directories, kAuditDirChosenCmd);
       break;
@@ -219,3 +215,17 @@ void RomAuditDialog::handleCommand(CommandSender* sender, int cmd,
       break;
   }
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void RomAuditDialog::createBrowser(const string& title)
+{
+  uInt32 w = 0, h = 0;
+  getDynamicBounds(w, h);
+
+  // Create file browser dialog
+  if(!myBrowser || uInt32(myBrowser->getWidth()) != w ||
+     uInt32(myBrowser->getHeight()) != h)
+    myBrowser = make_unique<BrowserDialog>(this, myFont, w, h, title);
+  else
+    myBrowser->setTitle(title);
+}
