@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -43,8 +43,7 @@ SnapshotDialog::SnapshotDialog(OSystem& osystem, DialogContainer& parent,
   ButtonWidget* b;
 
   // Set real dimensions
-  _w = std::min(max_w, 64 * fontWidth + HBORDER * 2);
-  _h = 9 * (lineHeight + 4) + VBORDER + _th;
+  setSize(64 * fontWidth + HBORDER * 2, 9 * (lineHeight + 4) + VBORDER + _th, max_w, max_h);
 
   xpos = HBORDER;  ypos = VBORDER + _th;
 
@@ -66,7 +65,7 @@ SnapshotDialog::SnapshotDialog(OSystem& osystem, DialogContainer& parent,
                                     font.getStringWidth("10 seconds"));
   mySnapInterval->setMinValue(1);
   mySnapInterval->setMaxValue(10);
-  mySnapInterval->setTickmarkInterval(3);
+  mySnapInterval->setTickmarkIntervals(3);
   wid.push_back(mySnapInterval);
 
   // Booleans for saving snapshots
