@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -25,8 +25,7 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 StringListWidget::StringListWidget(GuiObject* boss, const GUI::Font& font,
                                    int x, int y, int w, int h, bool hilite)
-  : ListWidget(boss, font, x, y, w, h,
-               boss->instance().settings().getInt("listdelay") >= 300),
+  : ListWidget(boss, font, x, y, w, h),
     _hilite(hilite)
 {
   _bgcolorlo = kDlgColor;
@@ -43,14 +42,14 @@ void StringListWidget::setList(const StringList& list)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void StringListWidget::handleMouseEntered()
 {
-  setFlags(WIDGET_HILITED);
+  setFlags(Widget::FLAG_HILITED);
   setDirty();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void StringListWidget::handleMouseLeft()
 {
-  clearFlags(WIDGET_HILITED);
+  clearFlags(Widget::FLAG_HILITED);
   setDirty();
 }
 
@@ -64,6 +63,9 @@ void StringListWidget::drawWidget(bool hilite)
   // Draw a thin frame around the list.
   s.frameRect(_x, _y, _w + 1, _h, onTop && hilite && _hilite ? kWidColorHi : kColor);
 
+  if (!isEnabled())
+    s.fillRect(_x + 1, _y + 1, _w - 1, _h - 2, onTop ? kDlgColor : kBGColorLo);
+
   // Draw the list items
   for (i = 0, pos = _currentPos; i < _rows && pos < len; i++, pos++)
   {
@@ -79,19 +81,19 @@ void StringListWidget::drawWidget(bool hilite)
         textColor = kTextColorInv;
       }
       else
-        s.frameRect(_x + 1, _y + 1 + _fontHeight * i, _w - 1, _fontHeight, onTop ? kWidColorHi : kBGColorLo);
+        s.frameRect(_x + 1, _y + 1 + _fontHeight * i, _w - 1, _fontHeight, kWidColorHi);
     }
 
-    GUI::Rect r(getEditRect());
+    Common::Rect r(getEditRect());
     if (_selectedItem == pos && _editMode)
     {
       adjustOffset();
 
-      s.drawString(_font, editString(), _x + r.left, y, r.width(), textColor,
+      s.drawString(_font, editString(), _x + r.x(), y, r.w(), textColor,
                    TextAlign::Left, -_editScrollOffset, false);
     }
     else
-      s.drawString(_font, _list[pos], _x + r.left, y, r.width(), textColor);
+      s.drawString(_font, _list[pos], _x + r.x(), y, r.w(), textColor);
   }
 
   // Only draw the caret while editing, and if it's in the current viewport
@@ -101,12 +103,8 @@ void StringListWidget::drawWidget(bool hilite)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-GUI::Rect StringListWidget::getEditRect() const
+Common::Rect StringListWidget::getEditRect() const
 {
-  GUI::Rect r(2, 1, _w - 2, _fontHeight);
   const int offset = std::max(0, (_selectedItem - _currentPos) * _fontHeight);
-  r.top += offset;
-  r.bottom += offset;
-
-  return r;
+  return Common::Rect(2, 1 + offset, _w - 2, _fontHeight + offset);
 }
