@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -32,6 +32,7 @@
 #include "TabWidget.hxx"
 #include "NTSCFilter.hxx"
 #include "TIASurface.hxx"
+
 #include "VideoDialog.hxx"
 
 namespace {
@@ -85,7 +86,7 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
             fontWidth    = font.getMaxCharWidth(),
             buttonHeight = font.getLineHeight() + 4;
   int xpos, ypos, tabID;
-  int lwidth = font.getStringWidth("TIA Palette "),
+  int lwidth = font.getStringWidth("V-Size adjust "),
     pwidth = font.getStringWidth("XXXXxXXXX"),
     swidth = font.getMaxCharWidth() * 10 - 2;
 
@@ -93,8 +94,7 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
   VariantList items;
 
   // Set real dimensions
-  _w = std::min(55 * fontWidth + HBORDER * 2, max_w);
-  _h = std::min(14 * (lineHeight + VGAP) + 14 + _th, max_h);
+  setSize(60 * fontWidth + HBORDER * 2, 14 * (lineHeight + VGAP) + 14 + _th, max_w, max_h);
 
   // The tab widget
   xpos = 2;  ypos = 4;
@@ -116,49 +116,33 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
   // TIA Palette
   items.clear();
   VarList::push_back(items, "Standard", "standard");
-  VarList::push_back(items, "Z26", "z26");
-  VarList::push_back(items, "User", "user");
+  VarList::push_back(items, "z26", "z26");
+  if (instance().checkUserPalette())
+    VarList::push_back(items, "User", "user");
   myTIAPalette = new PopUpWidget(myTab, font, xpos, ypos, pwidth,
-                                 lineHeight, items, "TIA palette ", lwidth);
+                                 lineHeight, items, "Palette ", lwidth);
   wid.push_back(myTIAPalette);
   ypos += lineHeight + VGAP;
 
-  // TIA filters (will be dynamically filled later)
-  myTIAZoom = new PopUpWidget(myTab, font, xpos, ypos, pwidth,
-                              lineHeight, items, "TIA zoom ", lwidth);
+  // TIA interpolation
+  myTIAInterpolate = new CheckboxWidget(myTab, font, xpos, ypos + 1, "Interpolation ");
+  wid.push_back(myTIAInterpolate);  ypos += lineHeight + VGAP;
+
+
+  // TIA zoom levels (will be dynamically filled later)
+  myTIAZoom = new SliderWidget(myTab, font, xpos, ypos - 1, swidth, lineHeight,
+    "Zoom ", lwidth, 0, fontWidth * 4, "%");
+  myTIAZoom->setMinValue(200); myTIAZoom->setStepValue(FrameBuffer::ZOOM_STEPS * 100);
   wid.push_back(myTIAZoom);
   ypos += lineHeight + VGAP;
 
-  /*SliderWidget* s = new SliderWidget(myTab, font, xpos, ypos - 1, swidth, lineHeight,
-                                     "TIA zoom", lwidth, 0, fontWidth * 4, "%");
-  s->setMinValue(200); s->setMaxValue(500);
-  s->setTickmarkInterval(3); // just for testing now; TODO: remove or redefine
-  wid.push_back(s);
-  ypos += lineHeight + VGAP;*/
-
-  // TIA interpolation
-  myTIAInterpolate = new CheckboxWidget(myTab, font, xpos, ypos + 1, "TIA interpolation ");
-  wid.push_back(myTIAInterpolate);
-  ypos += lineHeight + VGAP;
-
   // Aspect ratio (NTSC mode)
-  myNAspectRatio =
+  myVSizeAdjust =
     new SliderWidget(myTab, font, xpos, ypos-1, swidth, lineHeight,
-                     "NTSC aspect ", lwidth, 0,
-                     fontWidth * 4, "%");
-  myNAspectRatio->setMinValue(80); myNAspectRatio->setMaxValue(120);
-  myNAspectRatio->setTickmarkInterval(2);
-  wid.push_back(myNAspectRatio);
-  ypos += lineHeight + VGAP;
-
-  // Aspect ratio (PAL mode)
-  myPAspectRatio =
-    new SliderWidget(myTab, font, xpos, ypos-1, swidth, lineHeight,
-                     "PAL aspect ", lwidth, 0,
-                     fontWidth * 4, "%");
-  myPAspectRatio->setMinValue(80); myPAspectRatio->setMaxValue(120);
-  myPAspectRatio->setTickmarkInterval(2);
-  wid.push_back(myPAspectRatio);
+                     "V-Size adjust", lwidth, kVSizeChanged, fontWidth * 7, "%", 0, true);
+  myVSizeAdjust->setMinValue(-5); myVSizeAdjust->setMaxValue(5);
+  myVSizeAdjust->setTickmarkIntervals(2);
+  wid.push_back(myVSizeAdjust);
   ypos += lineHeight + VGAP;
 
   // Speed
@@ -167,7 +151,7 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
                      "Emul. speed ", lwidth, kSpeedupChanged, fontWidth * 5, "%");
   mySpeed->setMinValue(MIN_SPEED); mySpeed->setMaxValue(MAX_SPEED);
   mySpeed->setStepValue(SPEED_STEP);
-  mySpeed->setTickmarkInterval(2);
+  mySpeed->setTickmarkIntervals(2);
   wid.push_back(mySpeed);
   ypos += lineHeight + VGAP;
 
@@ -176,11 +160,11 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
   wid.push_back(myUseVSync);
 
   // Move over to the next column
-  xpos += mySpeed->getWidth() + 28;
+  xpos += mySpeed->getWidth() + 44;
   ypos = VBORDER;
 
   // Fullscreen
-  myFullscreen = new CheckboxWidget(myTab, font, xpos, ypos + 1, "Fullscreen");
+  myFullscreen = new CheckboxWidget(myTab, font, xpos, ypos + 1, "Fullscreen", kFullScreenChanged);
   wid.push_back(myFullscreen);
   ypos += lineHeight + VGAP;
 
@@ -191,8 +175,16 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
   ypos += lineHeight + VGAP;*/
 
   // FS stretch
-  myUseStretch = new CheckboxWidget(myTab, font, xpos, ypos + 1, "Fullscreen fill");
+  myUseStretch = new CheckboxWidget(myTab, font, xpos + INDENT, ypos + 1, "Stretch");
   wid.push_back(myUseStretch);
+  ypos += lineHeight + VGAP;
+
+  // FS overscan
+  myTVOverscan = new SliderWidget(myTab, font, xpos + INDENT, ypos - 1, swidth, lineHeight,
+    "Overscan", font.getStringWidth("Overscan "), kOverscanChanged, fontWidth * 3, "%");
+  myTVOverscan->setMinValue(0); myTVOverscan->setMaxValue(10);
+  myTVOverscan->setTickmarkIntervals(2);
+  wid.push_back(myTVOverscan);
   ypos += (lineHeight + VGAP) * 2;
 
   // Skip progress load bars for SuperCharger ROMs
@@ -228,12 +220,12 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
 
   // TV Mode
   items.clear();
-  VarList::push_back(items, "Disabled", NTSCFilter::PRESET_OFF);
-  VarList::push_back(items, "Composite", NTSCFilter::PRESET_COMPOSITE);
-  VarList::push_back(items, "S-Video", NTSCFilter::PRESET_SVIDEO);
-  VarList::push_back(items, "RGB", NTSCFilter::PRESET_RGB);
-  VarList::push_back(items, "Bad adjust", NTSCFilter::PRESET_BAD);
-  VarList::push_back(items, "Custom", NTSCFilter::PRESET_CUSTOM);
+  VarList::push_back(items, "Disabled", static_cast<uInt32>(NTSCFilter::Preset::OFF));
+  VarList::push_back(items, "RGB", static_cast<uInt32>(NTSCFilter::Preset::RGB));
+  VarList::push_back(items, "S-Video", static_cast<uInt32>(NTSCFilter::Preset::SVIDEO));
+  VarList::push_back(items, "Composite", static_cast<uInt32>(NTSCFilter::Preset::COMPOSITE));
+  VarList::push_back(items, "Bad adjust", static_cast<uInt32>(NTSCFilter::Preset::BAD));
+  VarList::push_back(items, "Custom", static_cast<uInt32>(NTSCFilter::Preset::CUSTOM));
   lwidth = font.getStringWidth("TV Mode ");
   pwidth = font.getStringWidth("Bad adjust");
   myTVMode =
@@ -246,25 +238,25 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
   xpos += INDENT - 2; ypos += 0;
   lwidth = font.getStringWidth("Saturation ");
 
-#define CREATE_CUSTOM_SLIDERS(obj, desc)                                 \
+#define CREATE_CUSTOM_SLIDERS(obj, desc, cmd)                            \
   myTV ## obj =                                                          \
     new SliderWidget(myTab, font, xpos, ypos-1, swidth, lineHeight,      \
-                     desc, lwidth, 0, fontWidth*4, "%");                 \
+                     desc, lwidth, cmd, fontWidth*4, "%");               \
   myTV ## obj->setMinValue(0); myTV ## obj->setMaxValue(100);            \
-  myTV ## obj->setTickmarkInterval(2);                                   \
+  myTV ## obj->setTickmarkIntervals(2);                                  \
   wid.push_back(myTV ## obj);                                            \
   ypos += lineHeight + VGAP;
 
-  CREATE_CUSTOM_SLIDERS(Contrast, "Contrast ");
-  CREATE_CUSTOM_SLIDERS(Bright, "Brightness ");
-  CREATE_CUSTOM_SLIDERS(Hue, "Hue ");
-  CREATE_CUSTOM_SLIDERS(Satur, "Saturation ");
-  CREATE_CUSTOM_SLIDERS(Gamma, "Gamma ");
-  CREATE_CUSTOM_SLIDERS(Sharp, "Sharpness ");
-  CREATE_CUSTOM_SLIDERS(Res, "Resolution ");
-  CREATE_CUSTOM_SLIDERS(Artifacts, "Artifacts ");
-  CREATE_CUSTOM_SLIDERS(Fringe, "Fringing ");
-  CREATE_CUSTOM_SLIDERS(Bleed, "Bleeding ");
+  CREATE_CUSTOM_SLIDERS(Contrast, "Contrast ", 0)
+  CREATE_CUSTOM_SLIDERS(Bright, "Brightness ", 0)
+  CREATE_CUSTOM_SLIDERS(Hue, "Hue ", 0)
+  CREATE_CUSTOM_SLIDERS(Satur, "Saturation ", 0)
+  CREATE_CUSTOM_SLIDERS(Gamma, "Gamma ", 0)
+  CREATE_CUSTOM_SLIDERS(Sharp, "Sharpness ", 0)
+  CREATE_CUSTOM_SLIDERS(Res, "Resolution ", 0)
+  CREATE_CUSTOM_SLIDERS(Artifacts, "Artifacts ", 0)
+  CREATE_CUSTOM_SLIDERS(Fringe, "Fringing ", 0)
+  CREATE_CUSTOM_SLIDERS(Bleed, "Bleeding ", 0)
 
   xpos += myTVContrast->getWidth() + 30;
   ypos = VBORDER;
@@ -274,25 +266,22 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
   // TV Phosphor effect
   myTVPhosphor = new CheckboxWidget(myTab, font, xpos, ypos + 1, "Phosphor for all ROMs", kPhosphorChanged);
   wid.push_back(myTVPhosphor);
-  ypos += lineHeight + VGAP;
+  ypos += lineHeight + VGAP / 2;
 
   // TV Phosphor blend level
   xpos += INDENT;
   swidth = font.getMaxCharWidth() * 10;
-  CREATE_CUSTOM_SLIDERS(PhosLevel, "Blend     ");
-  ypos += 6;
+  CREATE_CUSTOM_SLIDERS(PhosLevel, "Blend     ", kPhosBlendChanged)
+  ypos += 8;
 
   // Scanline intensity and interpolation
   xpos -= INDENT;
-  myTVScanLabel = new StaticTextWidget(myTab, font, xpos, ypos, "Scanline settings");
-  ypos += lineHeight;
+  myTVScanLabel = new StaticTextWidget(myTab, font, xpos, ypos, "Scanlines:");
+  ypos += lineHeight + VGAP / 2;
 
   xpos += INDENT;
-  CREATE_CUSTOM_SLIDERS(ScanIntense, "Intensity ");
-
-  myTVScanInterpolate = new CheckboxWidget(myTab, font, xpos, ypos, "Interpolation");
-  wid.push_back(myTVScanInterpolate);
-  ypos += lineHeight + 6;
+  CREATE_CUSTOM_SLIDERS(ScanIntense, "Intensity ", kScanlinesChanged)
+  ypos += lineHeight + 2;
 
   // Adjustable presets
   xpos -= INDENT;
@@ -302,14 +291,14 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
     new ButtonWidget(myTab, font, xpos, ypos, cloneWidth, buttonHeight,\
                      desc, kClone ## obj ##Cmd);                       \
   wid.push_back(myClone ## obj);                                       \
-  ypos += lineHeight + 4 + VGAP
+  ypos += lineHeight + 4 + VGAP;
 
   ypos += VGAP;
-  CREATE_CLONE_BUTTON(Composite, "Clone Composite");
-  CREATE_CLONE_BUTTON(Svideo, "Clone S-Video");
-  CREATE_CLONE_BUTTON(RGB, "Clone RGB");
-  CREATE_CLONE_BUTTON(Bad, "Clone Bad adjust");
-  CREATE_CLONE_BUTTON(Custom, "Revert");
+  CREATE_CLONE_BUTTON(RGB, "Clone RGB")
+  CREATE_CLONE_BUTTON(Svideo, "Clone S-Video")
+  CREATE_CLONE_BUTTON(Composite, "Clone Composite")
+  CREATE_CLONE_BUTTON(Bad, "Clone Bad adjust")
+  CREATE_CLONE_BUTTON(Custom, "Revert")
 
   // Add items for tab 2
   addToFocusList(wid, myTab, tabID);
@@ -324,8 +313,8 @@ VideoDialog::VideoDialog(OSystem& osystem, DialogContainer& parent,
 
   // Disable certain functions when we know they aren't present
 #ifndef WINDOWED_SUPPORT
-  myFullscreen->clearFlags(WIDGET_ENABLED);
-  myCenter->clearFlags(WIDGET_ENABLED);
+  myFullscreen->clearFlags(Widget::FLAG_ENABLED);
+  myCenter->clearFlags(Widget::FLAG_ENABLED);
 #endif
 }
 
@@ -335,12 +324,16 @@ void VideoDialog::loadConfig()
   // Renderer settings
   myRenderer->setSelected(instance().settings().getString("video"), "default");
 
-  // TIA Filter
+  // TIA zoom levels
   // These are dynamically loaded, since they depend on the size of
   // the desktop and which renderer we're using
-  const VariantList& items = instance().frameBuffer().supportedTIAZoomLevels();
-  myTIAZoom->addItems(items);
-  myTIAZoom->setSelected(instance().settings().getString("tia.zoom"), "3");
+  float minZoom = instance().frameBuffer().supportedTIAMinZoom(); // or 2 if we allow lower values
+  float maxZoom = instance().frameBuffer().supportedTIAMaxZoom();
+
+  myTIAZoom->setMinValue(minZoom * 100);
+  myTIAZoom->setMaxValue(maxZoom * 100);
+  myTIAZoom->setTickmarkIntervals((maxZoom - minZoom) * 2); // every ~50%
+  myTIAZoom->setValue(instance().settings().getFloat("tia.zoom") * 100);
 
   // TIA Palette
   myTIAPalette->setSelected(
@@ -350,8 +343,7 @@ void VideoDialog::loadConfig()
   myTIAInterpolate->setState(instance().settings().getBool("tia.inter"));
 
   // Aspect ratio setting (NTSC and PAL)
-  myNAspectRatio->setValue(instance().settings().getInt("tia.aspectn"));
-  myPAspectRatio->setValue(instance().settings().getInt("tia.aspectp"));
+  myVSizeAdjust->setValue(instance().settings().getInt("tia.vsizeadjust"));
 
   // Emulation speed
   int speed = mapSpeed(instance().settings().getFloat("speed"));
@@ -362,9 +354,11 @@ void VideoDialog::loadConfig()
   myFullscreen->setState(instance().settings().getBool("fullscreen"));
   /*string mode = instance().settings().getString("fullscreenmode");
   myFullScreenMode->setSelected(mode);*/
-
   // Fullscreen stretch setting
-  myUseStretch->setState(instance().settings().getBool("tia.fsfill"));
+  myUseStretch->setState(instance().settings().getBool("tia.fs_stretch"));
+  // Fullscreen overscan setting
+  myTVOverscan->setValue(instance().settings().getInt("tia.fs_overscan"));
+  handleFullScreenChange();
 
   // Use sync to vertical blank
   myUseVSync->setState(instance().settings().getBool("vsync"));
@@ -388,7 +382,7 @@ void VideoDialog::loadConfig()
   handleTVModeChange(NTSCFilter::Preset(preset));
 
   // TV Custom adjustables
-  loadTVAdjustables(NTSCFilter::PRESET_CUSTOM);
+  loadTVAdjustables(NTSCFilter::Preset::CUSTOM);
 
   // TV phosphor mode
   myTVPhosphor->setState(instance().settings().getString("tv.phosphor") == "always");
@@ -399,7 +393,6 @@ void VideoDialog::loadConfig()
 
   // TV scanline intensity and interpolation
   myTVScanIntense->setValue(instance().settings().getInt("tv.scanlines"));
-  myTVScanInterpolate->setState(instance().settings().getBool("tv.scaninter"));
 
   myTab->loadConfig();
 }
@@ -409,22 +402,24 @@ void VideoDialog::saveConfig()
 {
   // Renderer setting
   instance().settings().setValue("video",
-    myRenderer->getSelectedTag().toString());
+                                 myRenderer->getSelectedTag().toString());
 
-  // TIA Filter
-  instance().settings().setValue("tia.zoom",
-    myTIAZoom->getSelectedTag().toString());
+  // TIA zoom levels
+  instance().settings().setValue("tia.zoom", myTIAZoom->getValue() / 100.0);
 
   // TIA Palette
   instance().settings().setValue("palette",
-    myTIAPalette->getSelectedTag().toString());
+                                 myTIAPalette->getSelectedTag().toString());
 
   // TIA interpolation
   instance().settings().setValue("tia.inter", myTIAInterpolate->getState());
 
   // Aspect ratio setting (NTSC and PAL)
-  instance().settings().setValue("tia.aspectn", myNAspectRatio->getValueLabel());
-  instance().settings().setValue("tia.aspectp", myPAspectRatio->getValueLabel());
+  int oldAdjust = instance().settings().getInt("tia.vsizeadjust");
+  int newAdjust = myVSizeAdjust->getValue();
+  bool vsizeChanged = oldAdjust != newAdjust;
+
+  instance().settings().setValue("tia.vsizeadjust", newAdjust);
 
   // Speed
   int speedup = mySpeed->getValue();
@@ -433,10 +428,10 @@ void VideoDialog::saveConfig()
 
   // Fullscreen
   instance().settings().setValue("fullscreen", myFullscreen->getState());
-  /*instance().settings().setValue("fullscreenmode",
-                                 myFullScreenMode->getSelectedTag().toString());*/
   // Fullscreen stretch setting
-  instance().settings().setValue("tia.fsfill", myUseStretch->getState());
+  instance().settings().setValue("tia.fs_stretch", myUseStretch->getState());
+  // Fullscreen overscan
+  instance().settings().setValue("tia.fs_overscan", myTVOverscan->getValueLabel());
 
   // Use sync to vertical blank
   instance().settings().setValue("vsync", myUseVSync->getState());
@@ -452,39 +447,50 @@ void VideoDialog::saveConfig()
 
   // Multi-threaded rendering
   instance().settings().setValue("threads", myUseThreads->getState());
-  if(instance().hasConsole())
+  if (instance().hasConsole())
     instance().frameBuffer().tiaSurface().ntsc().enableThreading(myUseThreads->getState());
 
   // TV Mode
   instance().settings().setValue("tv.filter",
-    myTVMode->getSelectedTag().toString());
+                                 myTVMode->getSelectedTag().toString());
 
   // TV Custom adjustables
   NTSCFilter::Adjustable adj;
-  adj.hue         = myTVHue->getValue();
-  adj.saturation  = myTVSatur->getValue();
-  adj.contrast    = myTVContrast->getValue();
-  adj.brightness  = myTVBright->getValue();
-  adj.sharpness   = myTVSharp->getValue();
-  adj.gamma       = myTVGamma->getValue();
-  adj.resolution  = myTVRes->getValue();
-  adj.artifacts   = myTVArtifacts->getValue();
-  adj.fringing    = myTVFringe->getValue();
-  adj.bleed       = myTVBleed->getValue();
+  adj.hue = myTVHue->getValue();
+  adj.saturation = myTVSatur->getValue();
+  adj.contrast = myTVContrast->getValue();
+  adj.brightness = myTVBright->getValue();
+  adj.sharpness = myTVSharp->getValue();
+  adj.gamma = myTVGamma->getValue();
+  adj.resolution = myTVRes->getValue();
+  adj.artifacts = myTVArtifacts->getValue();
+  adj.fringing = myTVFringe->getValue();
+  adj.bleed = myTVBleed->getValue();
   instance().frameBuffer().tiaSurface().ntsc().setCustomAdjustables(adj);
 
   // TV phosphor mode
   instance().settings().setValue("tv.phosphor",
                                  myTVPhosphor->getState() ? "always" : "byrom");
   // TV phosphor blend
-  instance().settings().setValue("tv.phosblend", myTVPhosLevel->getValueLabel());
+  instance().settings().setValue("tv.phosblend", myTVPhosLevel->getValueLabel() == "Off"
+                                 ? "0" : myTVPhosLevel->getValueLabel());
 
-  // TV scanline intensity and interpolation
+  // TV scanline intensity
   instance().settings().setValue("tv.scanlines", myTVScanIntense->getValueLabel());
-  instance().settings().setValue("tv.scaninter", myTVScanInterpolate->getState());
 
-  // Finally, issue a complete framebuffer re-initialization
+  if (instance().hasConsole())
+    instance().console().setTIAProperties();
+
+  if (vsizeChanged && instance().hasConsole()) {
+    instance().console().tia().clearFrameBuffer();
+    instance().console().initializeVideo();
+  }
+
+  // Finally, issue a complete framebuffer re-initialization...
   instance().createFrameBuffer();
+
+  // ... and apply potential setting changes to the TIA surface
+  instance().frameBuffer().tiaSurface().updateSurfaceSettings();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -495,16 +501,15 @@ void VideoDialog::setDefaults()
     case 0:  // General
     {
       myRenderer->setSelectedIndex(0);
-      myTIAZoom->setSelected("3", "");
+      myTIAZoom->setValue(300);
       myTIAPalette->setSelected("standard", "");
       myTIAInterpolate->setState(false);
-      myNAspectRatio->setValue(91);
-      myPAspectRatio->setValue(109);
+      myVSizeAdjust->setValue(0);
       mySpeed->setValue(0);
 
       myFullscreen->setState(false);
       //myFullScreenMode->setSelectedIndex(0);
-      myUseStretch->setState(true);
+      myUseStretch->setState(false);
       myUseVSync->setState(true);
       myUIMessages->setState(true);
       myCenter->setState(false);
@@ -525,12 +530,11 @@ void VideoDialog::setDefaults()
 
       // TV scanline intensity and interpolation
       myTVScanIntense->setValue(25);
-      myTVScanInterpolate->setState(true);
 
       // Make sure that mutually-exclusive items are not enabled at the same time
-      handleTVModeChange(NTSCFilter::PRESET_OFF);
+      handleTVModeChange(NTSCFilter::Preset::OFF);
       handlePhosphorChange();
-      loadTVAdjustables(NTSCFilter::PRESET_CUSTOM);
+      loadTVAdjustables(NTSCFilter::Preset::CUSTOM);
       break;
     }
   }
@@ -539,8 +543,7 @@ void VideoDialog::setDefaults()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void VideoDialog::handleTVModeChange(NTSCFilter::Preset preset)
 {
-  bool enable = preset == NTSCFilter::PRESET_CUSTOM;
-  bool scanenable = preset != NTSCFilter::PRESET_OFF;
+  bool enable = preset == NTSCFilter::Preset::CUSTOM;
 
   myTVSharp->setEnabled(enable);
   myTVHue->setEnabled(enable);
@@ -557,10 +560,6 @@ void VideoDialog::handleTVModeChange(NTSCFilter::Preset preset)
   myCloneRGB->setEnabled(enable);
   myCloneBad->setEnabled(enable);
   myCloneCustom->setEnabled(enable);
-
-  myTVScanLabel->setEnabled(scanenable);
-  myTVScanIntense->setEnabled(scanenable);
-  myTVScanInterpolate->setEnabled(scanenable);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -581,6 +580,26 @@ void VideoDialog::loadTVAdjustables(NTSCFilter::Preset preset)
   myTVGamma->setValue(adj.gamma);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void VideoDialog::handleFullScreenChange()
+{
+  bool enable = myFullscreen->getState();
+  myUseStretch->setEnabled(enable);
+  myTVOverscan->setEnabled(enable);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void VideoDialog::handleOverscanChange()
+{
+  if (myTVOverscan->getValue() == 0)
+  {
+    myTVOverscan->setValueLabel("Off");
+    myTVOverscan->setValueUnit("");
+  }
+  else
+    myTVOverscan->setValueUnit("%");
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void VideoDialog::handlePhosphorChange()
 {
@@ -591,7 +610,7 @@ void VideoDialog::handlePhosphorChange()
 void VideoDialog::handleCommand(CommandSender* sender, int cmd,
                                 int data, int id)
 {
-  switch(cmd)
+  switch (cmd)
   {
     case GuiObject::kOKCmd:
       saveConfig();
@@ -602,29 +621,71 @@ void VideoDialog::handleCommand(CommandSender* sender, int cmd,
       setDefaults();
       break;
 
+    case kVSizeChanged:
+    {
+      int adjust = myVSizeAdjust->getValue();
+
+      if (!adjust)
+      {
+        myVSizeAdjust->setValueLabel("Default");
+        myVSizeAdjust->setValueUnit("");
+      }
+      else
+        myVSizeAdjust->setValueUnit("%");
+      break;
+    }
+
     case kSpeedupChanged:
       mySpeed->setValueLabel(formatSpeed(mySpeed->getValue()));
       break;
 
+    case kFullScreenChanged:
+      handleFullScreenChange();
+      break;
+
+    case kOverscanChanged:
+      handleOverscanChange();
+      break;
+
     case kTVModeChanged:
       handleTVModeChange(NTSCFilter::Preset(myTVMode->getSelectedTag().toInt()));
       break;
 
-    case kCloneCompositeCmd: loadTVAdjustables(NTSCFilter::PRESET_COMPOSITE);
+    case kCloneCompositeCmd: loadTVAdjustables(NTSCFilter::Preset::COMPOSITE);
       break;
-    case kCloneSvideoCmd: loadTVAdjustables(NTSCFilter::PRESET_SVIDEO);
+    case kCloneSvideoCmd: loadTVAdjustables(NTSCFilter::Preset::SVIDEO);
       break;
-    case kCloneRGBCmd: loadTVAdjustables(NTSCFilter::PRESET_RGB);
+    case kCloneRGBCmd: loadTVAdjustables(NTSCFilter::Preset::RGB);
       break;
-    case kCloneBadCmd: loadTVAdjustables(NTSCFilter::PRESET_BAD);
+    case kCloneBadCmd: loadTVAdjustables(NTSCFilter::Preset::BAD);
       break;
-    case kCloneCustomCmd: loadTVAdjustables(NTSCFilter::PRESET_CUSTOM);
+    case kCloneCustomCmd: loadTVAdjustables(NTSCFilter::Preset::CUSTOM);
+      break;
+
+    case kScanlinesChanged:
+      if (myTVScanIntense->getValue() == 0)
+      {
+        myTVScanIntense->setValueLabel("Off");
+        myTVScanIntense->setValueUnit("");
+      }
+      else
+        myTVScanIntense->setValueUnit("%");
       break;
 
     case kPhosphorChanged:
       handlePhosphorChange();
       break;
 
+    case kPhosBlendChanged:
+      if (myTVPhosLevel->getValue() == 0)
+      {
+        myTVPhosLevel->setValueLabel("Off");
+        myTVPhosLevel->setValueUnit("");
+      }
+      else
+        myTVPhosLevel->setValueUnit("%");
+      break;
+
     default:
       Dialog::handleCommand(sender, cmd, data, 0);
       break;
