@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -32,17 +32,7 @@ Widget::Widget(GuiObject* boss, const GUI::Font& font,
                int x, int y, int w, int h)
   : GuiObject(boss->instance(), boss->parent(), boss->dialog(), x, y, w, h),
     _boss(boss),
-    _font(font),
-    _id(-1),
-    _flags(0),
-    _hasFocus(false),
-    _bgcolor(kWidColor),
-    _bgcolorhi(kWidColor),
-    _bgcolorlo(kBGColorLo),
-    _textcolor(kTextColor),
-    _textcolorhi(kTextColorHi),
-    _textcolorlo(kBGColorLo),
-    _shadowcolor(kShadowColor)
+    _font(font)
 {
   // Insert into the widget list of the boss
   _next = _boss->_firstWidget;
@@ -50,8 +40,6 @@ Widget::Widget(GuiObject* boss, const GUI::Font& font,
 
   _fontWidth  = _font.getMaxCharWidth();
   _fontHeight = _font.getLineHeight();
-
-  setDirty();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -81,7 +69,7 @@ void Widget::draw()
 
   bool onTop = _boss->dialog().isOnTop();
 
-  bool hasBorder = _flags & WIDGET_BORDER; // currently only used by Dialog widget
+  bool hasBorder = _flags & Widget::FLAG_BORDER; // currently only used by Dialog widget
   int oldX = _x, oldY = _y;
 
   // Account for our relative position in the dialog
@@ -89,20 +77,20 @@ void Widget::draw()
   _y = getAbsY();
 
   // Clear background (unless alpha blending is enabled)
-  if(_flags & WIDGET_CLEARBG)
+  if(_flags & Widget::FLAG_CLEARBG)
   {
     int x = _x, y = _y, w = _w, h = _h;
     if(hasBorder)
     {
       x++; y++; w-=2; h-=2;
     }
-    s.fillRect(x, y, w, h, !onTop ? _bgcolorlo : (_flags & WIDGET_HILITED) && isEnabled() ? _bgcolorhi : _bgcolor);
+    s.fillRect(x, y, w, h, !onTop ? _bgcolorlo : (_flags & Widget::FLAG_HILITED) && isEnabled() ? _bgcolorhi : _bgcolor);
   }
 
   // Draw border
   if(hasBorder)
   {
-    s.frameRect(_x, _y, _w, _h, !onTop ? kColor : (_flags & WIDGET_HILITED) && isEnabled() ? kWidColorHi : kColor);
+    s.frameRect(_x, _y, _w, _h, !onTop ? kColor : (_flags & Widget::FLAG_HILITED) && isEnabled() ? kWidColorHi : kColor);
     _x += 4;
     _y += 4;
     _w -= 8;
@@ -110,7 +98,7 @@ void Widget::draw()
   }
 
   // Now perform the actual widget draw
-  drawWidget((_flags & WIDGET_HILITED) ? true : false);
+  drawWidget((_flags & Widget::FLAG_HILITED) ? true : false);
 
   // Restore x/y
   if (hasBorder)
@@ -140,6 +128,7 @@ void Widget::receivedFocus()
     return;
 
   _hasFocus = true;
+  setFlags(Widget::FLAG_HILITED);
   receivedFocusWidget();
 }
 
@@ -150,14 +139,15 @@ void Widget::lostFocus()
     return;
 
   _hasFocus = false;
+  clearFlags(Widget::FLAG_HILITED);
   lostFocusWidget();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Widget::setEnabled(bool e)
 {
-  if(e) setFlags(WIDGET_ENABLED);
-  else  clearFlags(WIDGET_ENABLED);
+  if(e) setFlags(Widget::FLAG_ENABLED);
+  else  clearFlags(Widget::FLAG_ENABLED);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -282,8 +272,10 @@ Widget* Widget::setFocusForChain(GuiObject* boss, WidgetArray& arr,
 
   if(emitFocusEvents)
     tmp->receivedFocus();
-  else
+  else {
     tmp->_hasFocus = true;
+    tmp->setFlags(Widget::FLAG_HILITED);
+  }
 
   if (onTop)
       s.frameRect(x, y, w, h, kWidFrameColor, FrameStyle::Dashed);
@@ -309,17 +301,15 @@ StaticTextWidget::StaticTextWidget(GuiObject* boss, const GUI::Font& font,
                                    const string& text, TextAlign align,
                                    ColorId shadowColor)
   : Widget(boss, font, x, y, w, h),
+    _label(text),
     _align(align)
 {
-  _flags = WIDGET_ENABLED;
+  _flags = Widget::FLAG_ENABLED;
   _bgcolor = kDlgColor;
   _bgcolorhi = kDlgColor;
   _textcolor = kTextColor;
   _textcolorhi = kTextColor;
   _shadowcolor = shadowColor;
-
-  _label = text;
-  _editable = false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -335,9 +325,7 @@ StaticTextWidget::StaticTextWidget(GuiObject* boss, const GUI::Font& font,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void StaticTextWidget::setValue(int value)
 {
-  char buf[256];
-  std::snprintf(buf, 255, "%d", value);
-  _label = buf;
+  _label = std::to_string(value);
 
   setDirty();
 }
@@ -364,101 +352,107 @@ void StaticTextWidget::drawWidget(bool hilite)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ButtonWidget::ButtonWidget(GuiObject* boss, const GUI::Font& font,
                            int x, int y, int w, int h,
-                           const string& label, int cmd)
+                           const string& label, int cmd, bool repeat)
   : StaticTextWidget(boss, font, x, y, w, h, label, TextAlign::Center),
     CommandSender(boss),
     _cmd(cmd),
-    _useBitmap(false),
-    _bitmap(nullptr),
-    _bmw(0),
-    _bmh(0)
+    _repeat(repeat)
 {
-  _flags = WIDGET_ENABLED | WIDGET_CLEARBG;
+  _flags = Widget::FLAG_ENABLED | Widget::FLAG_CLEARBG;
   _bgcolor = kBtnColor;
   _bgcolorhi = kBtnColorHi;
   _bgcolorlo = kColor;
   _textcolor = kBtnTextColor;
   _textcolorhi = kBtnTextColorHi;
   _textcolorlo = kBGColorLo;
-
-  _editable = false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ButtonWidget::ButtonWidget(GuiObject* boss, const GUI::Font& font,
                            int x, int y, int dw,
-                           const string& label, int cmd)
-  : ButtonWidget(boss, font, x, y, font.getStringWidth(label) + dw, font.getLineHeight() + 4, label, cmd)
+                           const string& label, int cmd, bool repeat)
+  : ButtonWidget(boss, font, x, y, font.getStringWidth(label) + dw,
+                 font.getLineHeight() + 4, label, cmd, repeat)
 {
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ButtonWidget::ButtonWidget(GuiObject* boss, const GUI::Font& font,
                            int x, int y,
-                           const string& label, int cmd)
-  : ButtonWidget(boss, font, x, y, 20, label, cmd)
+                           const string& label, int cmd, bool repeat)
+  : ButtonWidget(boss, font, x, y, 20, label, cmd, repeat)
 {
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ButtonWidget::ButtonWidget(GuiObject* boss, const GUI::Font& font,
                            int x, int y, int w, int h,
-                           uInt32* bitmap, int bmw, int bmh,
-                           int cmd)
-  : ButtonWidget(boss, font, x, y, w, h, "", cmd)
+                           const uInt32* bitmap, int bmw, int bmh,
+                           int cmd, bool repeat)
+  : ButtonWidget(boss, font, x, y, w, h, "", cmd, repeat)
 {
-  _bitmap = bitmap;
-  _bmh = bmh;
-  _bmw = bmw;
   _useBitmap = true;
+  _bitmap = bitmap;
+  _bmw = bmw;
+  _bmh = bmh;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ButtonWidget::handleMouseEntered()
 {
-  setFlags(WIDGET_HILITED);
+  setFlags(Widget::FLAG_HILITED);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ButtonWidget::handleMouseLeft()
 {
-  clearFlags(WIDGET_HILITED);
+  clearFlags(Widget::FLAG_HILITED);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool ButtonWidget::handleEvent(Event::Type e)
 {
-  if(!isEnabled())
+  if(!isEnabled() || e != Event::UISelect)
     return false;
 
-  switch(e)
+  // Simulate mouse event
+  handleMouseUp(0, 0, MouseButton::LEFT, 0);
+  return true;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool ButtonWidget::handleMouseClicks(int x, int y, MouseButton b)
+{
+  return _repeat;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void ButtonWidget::handleMouseDown(int x, int y, MouseButton b, int clickCount)
+{
+  if(_repeat && isEnabled() && x >= 0 && x < _w && y >= 0 && y < _h)
   {
-    case Event::UISelect:
-      // Simulate mouse event
-      handleMouseUp(0, 0, MouseButton::LEFT, 0);
-      return true;
-    default:
-      return false;
+    clearFlags(Widget::FLAG_HILITED);
+    sendCommand(_cmd, 0, _id);
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ButtonWidget::handleMouseUp(int x, int y, MouseButton b, int clickCount)
 {
-  if(isEnabled() && x >= 0 && x < _w && y >= 0 && y < _h)
+  if (!_repeat && isEnabled() && x >= 0 && x < _w && y >= 0 && y < _h)
   {
-    clearFlags(WIDGET_HILITED);
+    clearFlags(Widget::FLAG_HILITED);
     sendCommand(_cmd, 0, _id);
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ButtonWidget::setBitmap(uInt32* bitmap, int bmw, int bmh)
+void ButtonWidget::setBitmap(const uInt32* bitmap, int bmw, int bmh)
 {
+  _useBitmap = true;
   _bitmap = bitmap;
   _bmh = bmh;
   _bmw = bmw;
-  _useBitmap = true;
 
   setDirty();
 }
@@ -484,64 +478,13 @@ void ButtonWidget::drawWidget(bool hilite)
   setDirty();
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-/* 8x8 checkbox bitmap */
-static uInt32 checked_img_active[10] =
-{
-  0b1111111111,
-  0b1111111111,
-  0b1111111111,
-  0b1111111111,
-  0b1111111111,
-  0b1111111111,
-  0b1111111111,
-  0b1111111111,
-  0b1111111111,
-  0b1111111111
-};
-
-static uInt32 checked_img_inactive[10] =
-{
-  0b1111111111,
-  0b1111111111,
-  0b1111001111,
-  0b1110000111,
-  0b1100000011,
-  0b1100000011,
-  0b1110000111,
-  0b1111001111,
-  0b1111111111,
-  0b1111111111
-};
-
-static uInt32 checked_img_circle[10] =
-{
-  0b0001111000,
-  0b0111111110,
-  0b0111111110,
-  0b1111111111,
-  0b1111111111,
-  0b1111111111,
-  0b1111111111,
-  0b0111111110,
-  0b0111111110,
-  0b0001111000
-};
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CheckboxWidget::CheckboxWidget(GuiObject* boss, const GUI::Font& font,
                                int x, int y, const string& label,
                                int cmd)
-  : ButtonWidget(boss, font, x, y, 16, 16, label, cmd),
-    _state(false),
-    _holdFocus(true),
-    _drawBox(true),
-    _changed(false),
-    _fillColor(kColor),
-    _boxY(0),
-    _textY(0)
+  : ButtonWidget(boss, font, x, y, 16, 16, label, cmd)
 {
-  _flags = WIDGET_ENABLED;
+  _flags = Widget::FLAG_ENABLED;
   _bgcolor = _bgcolorhi = kWidColor;
   _bgcolorlo = kDlgColor;
 
@@ -553,7 +496,6 @@ CheckboxWidget::CheckboxWidget(GuiObject* boss, const GUI::Font& font,
     _w = font.getStringWidth(label) + 20;
   _h = font.getFontHeight() < 14 ? 14 : font.getFontHeight();
 
-
   // Depending on font size, either the font or box will need to be
   // centered vertically
   if(_h > 14)  // center box
@@ -561,19 +503,19 @@ CheckboxWidget::CheckboxWidget(GuiObject* boss, const GUI::Font& font,
   else         // center text
     _textY = (14 - _font.getFontHeight()) / 2;
 
-  setFill(CheckboxWidget::Normal);
+  setFill(CheckboxWidget::FillType::Normal);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CheckboxWidget::handleMouseEntered()
 {
-  setFlags(WIDGET_HILITED);
+  setFlags(Widget::FLAG_HILITED);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CheckboxWidget::handleMouseLeft()
 {
-  clearFlags(WIDGET_HILITED);
+  clearFlags(Widget::FLAG_HILITED);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -599,7 +541,7 @@ void CheckboxWidget::setEditable(bool editable)
   else
   {
     _bgcolor = kBGColorHi;
-    setFill(CheckboxWidget::Inactive);
+    setFill(CheckboxWidget::FillType::Inactive);
   }
   setDirty();
 }
@@ -607,18 +549,34 @@ void CheckboxWidget::setEditable(bool editable)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CheckboxWidget::setFill(FillType type)
 {
+  /* 8x8 checkbox bitmap */
+  static constexpr std::array<uInt32, 10> checked_img_active = {
+    0b1111111111,  0b1111111111,  0b1111111111,  0b1111111111,  0b1111111111,
+    0b1111111111,  0b1111111111,  0b1111111111,  0b1111111111,  0b1111111111
+  };
+
+  static constexpr std::array<uInt32, 10> checked_img_inactive = {
+    0b1111111111,  0b1111111111,  0b1111001111,  0b1110000111,  0b1100000011,
+    0b1100000011,  0b1110000111,  0b1111001111,  0b1111111111,  0b1111111111
+  };
+
+  static constexpr std::array<uInt32, 10> checked_img_circle = {
+    0b0001111000,  0b0111111110,  0b0111111110,  0b1111111111,  0b1111111111,
+    0b1111111111,  0b1111111111,  0b0111111110,  0b0111111110,  0b0001111000
+  };
+
   switch(type)
   {
-    case CheckboxWidget::Normal:
-      _img = checked_img_active;
+    case CheckboxWidget::FillType::Normal:
+      _img = checked_img_active.data();
       _drawBox = true;
       break;
-    case CheckboxWidget::Inactive:
-      _img = checked_img_inactive;
+    case CheckboxWidget::FillType::Inactive:
+      _img = checked_img_inactive.data();
       _drawBox = true;
       break;
-    case CheckboxWidget::Circle:
-      _img = checked_img_circle;
+    case CheckboxWidget::FillType::Circle:
+      _img = checked_img_circle.data();
       _drawBox = false;
       break;
   }
@@ -647,7 +605,7 @@ void CheckboxWidget::drawWidget(bool hilite)
   // Do we draw a square or cross?
   s.fillRect(_x + 1, _y + _boxY + 1, 12, 12,
       _changed ? onTop ? kDbgChangedColor : kDlgColor :
-      isEnabled() ? onTop ? _bgcolor : kDlgColor : kColor);
+      isEnabled() && onTop ? _bgcolor : kDlgColor);
   if(_state)
     s.drawBitmap(_img, _x + 2, _y + _boxY + 2, onTop && isEnabled() ? hilite && isEditable() ? kWidColorHi : kCheckColor
                  : kColor, 10);
@@ -663,21 +621,16 @@ void CheckboxWidget::drawWidget(bool hilite)
 SliderWidget::SliderWidget(GuiObject* boss, const GUI::Font& font,
                            int x, int y, int w, int h,
                            const string& label, int labelWidth, int cmd,
-                           int valueLabelWidth, const string& valueUnit, int valueLabelGap)
+                           int valueLabelWidth, const string& valueUnit, int valueLabelGap,
+                           bool forceLabelSign)
   : ButtonWidget(boss, font, x, y, w, h, label, cmd),
-    _value(-1),
-    _stepValue(1),
-    _valueMin(0),
-    _valueMax(100),
-    _isDragging(false),
     _labelWidth(labelWidth),
-    _valueLabel(""),
     _valueUnit(valueUnit),
     _valueLabelGap(valueLabelGap),
     _valueLabelWidth(valueLabelWidth),
-    _numIntervals(0)
+    _forceLabelSign(forceLabelSign)
 {
-  _flags = WIDGET_ENABLED | WIDGET_TRACK_MOUSE;
+  _flags = Widget::FLAG_ENABLED | Widget::FLAG_TRACK_MOUSE;
   _bgcolor = kDlgColor;
   _bgcolorhi = kDlgColor;
 
@@ -686,6 +639,8 @@ SliderWidget::SliderWidget(GuiObject* boss, const GUI::Font& font,
 
   if(_valueLabelWidth == 0)
     _valueLabelGap = 0;
+  if(_valueLabelGap == 0)
+    _valueLabelGap = DEF_LBL_GAP;
 
   _w = w + _labelWidth + _valueLabelGap + _valueLabelWidth;
 }
@@ -694,9 +649,11 @@ SliderWidget::SliderWidget(GuiObject* boss, const GUI::Font& font,
 SliderWidget::SliderWidget(GuiObject* boss, const GUI::Font& font,
                            int x, int y,
                            const string& label, int labelWidth, int cmd,
-                           int valueLabelWidth, const string& valueUnit, int valueLabelGap)
+                           int valueLabelWidth, const string& valueUnit, int valueLabelGap,
+                           bool forceLabelSign)
   : SliderWidget(boss, font, x, y, font.getMaxCharWidth() * 10, font.getLineHeight(),
-                 label, labelWidth, cmd, valueLabelWidth, valueUnit, valueLabelGap)
+                 label, labelWidth, cmd, valueLabelWidth, valueUnit, valueLabelGap,
+                 forceLabelSign)
 {
 }
 
@@ -747,10 +704,7 @@ void SliderWidget::setValueLabel(const string& valueLabel)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void SliderWidget::setValueLabel(int value)
 {
-  char buf[256];
-  std::snprintf(buf, 255, "%d", value);
-  _valueLabel = buf;
-
+  _valueLabel = (_forceLabelSign && value > 0 ? "+" : "") + std::to_string(value);
   setDirty();
 }
 
@@ -762,7 +716,7 @@ void SliderWidget::setValueUnit(const string& valueUnit)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void SliderWidget::setTickmarkInterval(int numIntervals)
+void SliderWidget::setTickmarkIntervals(int numIntervals)
 {
   _numIntervals = numIntervals;
   setDirty();
