@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -29,15 +29,18 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 SerialPortMACOS::SerialPortMACOS()
-  : SerialPort(),
-    myHandle(0)
+  : SerialPort()
 {
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 SerialPortMACOS::~SerialPortMACOS()
 {
-  closePort();
+  if(myHandle)
+  {
+    close(myHandle);
+    myHandle = 0;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -61,22 +64,12 @@ bool SerialPortMACOS::openPort(const string& device)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void SerialPortMACOS::closePort()
+bool SerialPortMACOS::writeByte(uInt8 data)
 {
   if(myHandle)
   {
-    close(myHandle);
-    myHandle = 0;
-  }
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool SerialPortMACOS::writeByte(const uInt8* data)
-{
-  if(myHandle)
-  {
-//    cerr << "SerialPortMACOS::writeByte " << (int)(*data) << endl;
-    return write(myHandle, data, 1) == 1;
+//    cerr << "SerialPortMACOS::writeByte " << int(data) << endl;
+    return write(myHandle, &data, 1) == 1;
   }
   return false;
 }
