@@ -3,50 +3,48 @@ package PropSet;
 # NOTE: If the property types ever change in Stella, the following hashmap
 #       and array must be updated (and stay in sequence)
 my %prop_type = (
-  "Cartridge.MD5"           => 0,
-  "Cartridge.Manufacturer"  => 1,
-  "Cartridge.ModelNo"       => 2,
-  "Cartridge.Name"          => 3,
-  "Cartridge.Note"          => 4,
-  "Cartridge.Rarity"        => 5,
-  "Cartridge.Sound"         => 6,
-  "Cartridge.StartBank"     => 7,
-  "Cartridge.Type"          => 8,
-  "Console.LeftDifficulty"  => 9,
-  "Console.RightDifficulty" => 10,
-  "Console.TelevisionType"  => 11,
+  "Cart.MD5"                => 0,
+  "Cart.Manufacturer"       => 1,
+  "Cart.ModelNo"            => 2,
+  "Cart.Name"               => 3,
+  "Cart.Note"               => 4,
+  "Cart.Rarity"             => 5,
+  "Cart.Sound"              => 6,
+  "Cart.StartBank"          => 7,
+  "Cart.Type"               => 8,
+  "Console.LeftDiff"        => 9,
+  "Console.RightDiff"       => 10,
+  "Console.TVType"          => 11,
   "Console.SwapPorts"       => 12,
   "Controller.Left"         => 13,
   "Controller.Right"        => 14,
   "Controller.SwapPaddles"  => 15,
   "Controller.MouseAxis"    => 16,
   "Display.Format"          => 17,
-  "Display.YStart"          => 18,
-  "Display.Height"          => 19,
-  "Display.Phosphor"        => 20,
-  "Display.PPBlend"         => 21
+  "Display.VCenter"         => 18,
+  "Display.Phosphor"        => 19,
+  "Display.PPBlend"         => 20
 );
 my @prop_type_as_string = (
-  "Cartridge.MD5",
-  "Cartridge.Manufacturer",
-  "Cartridge.ModelNo",
-  "Cartridge.Name",
-  "Cartridge.Note",
-  "Cartridge.Rarity",
-  "Cartridge.Sound",
-  "Cartridge.StartBank",
-  "Cartridge.Type",
-  "Console.LeftDifficulty",
-  "Console.RightDifficulty",
-  "Console.TelevisionType",
+  "Cart.MD5",
+  "Cart.Manufacturer",
+  "Cart.ModelNo",
+  "Cart.Name",
+  "Cart.Note",
+  "Cart.Rarity",
+  "Cart.Sound",
+  "Cart.StartBank",
+  "Cart.Type",
+  "Console.LeftDiff",
+  "Console.RightDiff",
+  "Console.TVType",
   "Console.SwapPorts",
   "Controller.Left",
   "Controller.Right",
   "Controller.SwapPaddles",
   "Controller.MouseAxis",
   "Display.Format",
-  "Display.YStart",
-  "Display.Height",
+  "Display.VCenter",
   "Display.Phosphor",
   "Display.PPBlend"
 );
@@ -55,23 +53,22 @@ my @prop_defaults = (
   "",
   "",
   "",
-  "Untitled",
+  "",
   "",
   "",
   "MONO",
-  "",
+  "AUTO",
   "AUTO",
   "B",
   "B",
   "COLOR",
   "NO",
-  "JOYSTICK",
-  "JOYSTICK",
+  "",
+  "",
   "NO",
   "AUTO",
   "AUTO",
   "0",
-  "0",
   "NO",
   "0"
 );
