@@ -8,15 +8,55 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
+#if defined(RETRON77)
+  #define ROOT_DIR "/mnt/games/"
+#else
+  #define ROOT_DIR "/"
+#endif
+
 #include "FSNodePOSIX.hxx"
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+FilesystemNodePOSIX::FilesystemNodePOSIX()
+  : _path(ROOT_DIR),
+    _displayName(_path)
+{
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+FilesystemNodePOSIX::FilesystemNodePOSIX(const string& path, bool verify)
+{
+  // Default to home directory
+  _path = path.length() > 0 ? path : "~";
+
+  // Expand '~' to the HOME environment variable
+  if(_path[0] == '~')
+  {
+    string home = BSPF::getenv("HOME");
+    if(home != EmptyString)
+      _path.replace(0, 1, home);
+  }
+  // Get absolute path (only used for relative directories)
+  else if(_path[0] == '.')
+  {
+    std::array<char, MAXPATHLEN> buf;
+    if(realpath(_path.c_str(), buf.data()))
+      _path = buf.data();
+  }
+
+  _displayName = lastPathComponent(_path);
+
+  if(verify)
+    setFlags();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FilesystemNodePOSIX::setFlags()
 {
@@ -36,55 +76,15 @@ void FilesystemNodePOSIX::setFlags()
     _isDirectory = _isFile = false;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-FilesystemNodePOSIX::FilesystemNodePOSIX()
-  : _path("/"),          // The root dir.
-    _displayName(_path),
-    _isValid(true),
-    _isFile(false),
-    _isDirectory(true)
-{
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-FilesystemNodePOSIX::FilesystemNodePOSIX(const string& p, bool verify)
-  : _isValid(true),
-    _isFile(false),
-    _isDirectory(true)
-{
-  // Default to home directory
-  _path = p.length() > 0 ? p : "~";
-
-  // Expand '~' to the HOME environment variable
-  if(_path[0] == '~')
-  {
-    const char* home = getenv("HOME");
-    if(home != nullptr)
-      _path.replace(0, 1, home);
-  }
-  // Get absolute path (only used for relative directories)
-  else if(_path[0] == '.')
-  {
-    char buf[MAXPATHLEN];
-    if(realpath(_path.c_str(), buf))
-      _path = buf;
-  }
-
-  _displayName = lastPathComponent(_path);
-
-  if(verify)
-    setFlags();
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string FilesystemNodePOSIX::getShortPath() const
 {
   // If the path starts with the home directory, replace it with '~'
-  const char* home = getenv("HOME");
-  if(home != nullptr && BSPF::startsWithIgnoreCase(_path, home))
+  string home = BSPF::getenv("HOME");
+  if(home != EmptyString && BSPF::startsWithIgnoreCase(_path, home))
   {
     string path = "~";
-    const char* offset = _path.c_str() + strlen(home);
+    const char* offset = _path.c_str() + home.size();
     if(*offset != '/') path += "/";
     path += offset;
     return path;
@@ -93,8 +93,13 @@ string FilesystemNodePOSIX::getShortPath() const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool FilesystemNodePOSIX::getChildren(AbstractFSList& myList, ListMode mode,
-                                      bool hidden) const
+bool FilesystemNodePOSIX::hasParent() const
+{
+  return _path != "" && _path != ROOT_DIR;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool FilesystemNodePOSIX::getChildren(AbstractFSList& myList, ListMode mode) const
 {
   assert(_isDirectory);
 
@@ -106,12 +111,8 @@ bool FilesystemNodePOSIX::getChildren(AbstractFSList& myList, ListMode mode,
   struct dirent* dp;
   while ((dp = readdir(dirp)) != nullptr)
   {
-    // Skip 'invisible' files if necessary
-    if (dp->d_name[0] == '.' && !hidden)
-      continue;
-
-    // Skip '.' and '..' to avoid cycles
-    if ((dp->d_name[0] == '.' && dp->d_name[1] == 0) || (dp->d_name[0] == '.' && dp->d_name[1] == '.'))
+    // Ignore all hidden files
+    if (dp->d_name[0] == '.')
       continue;
 
     string newPath(_path);
@@ -168,11 +169,11 @@ bool FilesystemNodePOSIX::getChildren(AbstractFSList& myList, ListMode mode,
       continue;
 
     // Honor the chosen mode
-    if ((mode == FilesystemNode::kListFilesOnly && !entry._isFile) ||
-        (mode == FilesystemNode::kListDirectoriesOnly && !entry._isDirectory))
+    if ((mode == FilesystemNode::ListMode::FilesOnly && !entry._isFile) ||
+        (mode == FilesystemNode::ListMode::DirectoriesOnly && !entry._isDirectory))
       continue;
 
-    myList.emplace_back(new FilesystemNodePOSIX(entry));
+    myList.emplace_back(make_shared<FilesystemNodePOSIX>(entry));
   }
   closedir(dirp);
 
@@ -185,9 +186,9 @@ bool FilesystemNodePOSIX::makeDir()
   if(mkdir(_path.c_str(), 0777) == 0)
   {
     // Get absolute path
-    char buf[MAXPATHLEN];
-    if(realpath(_path.c_str(), buf))
-      _path = buf;
+    std::array<char, MAXPATHLEN> buf;
+    if(realpath(_path.c_str(), buf.data()))
+      _path = buf.data();
 
     _displayName = lastPathComponent(_path);
     setFlags();
@@ -210,9 +211,9 @@ bool FilesystemNodePOSIX::rename(const string& newfile)
     _path = newfile;
 
     // Get absolute path
-    char buf[MAXPATHLEN];
-    if(realpath(_path.c_str(), buf))
-      _path = buf;
+    std::array<char, MAXPATHLEN> buf;
+    if(realpath(_path.c_str(), buf.data()))
+      _path = buf.data();
 
     _displayName = lastPathComponent(_path);
     setFlags();
@@ -230,7 +231,7 @@ bool FilesystemNodePOSIX::rename(const string& newfile)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 AbstractFSNodePtr FilesystemNodePOSIX::getParent() const
 {
-  if (_path == "/")
+  if (_path == ROOT_DIR)
     return nullptr;
 
   const char* start = _path.c_str();
