@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -113,21 +113,21 @@ inline bool is_operator(char x)
 int const_to_int(char* ch)
 {
   // what base is the input in?
-  Common::Base::Format format = Common::Base::format();
+  Common::Base::Fmt format = Common::Base::format();
 
   switch(*ch) {
     case '\\':
-      format = Common::Base::F_2;
+      format = Common::Base::Fmt::_2;
       ch++;
       break;
 
     case '#':
-      format = Common::Base::F_10;
+      format = Common::Base::Fmt::_10;
       ch++;
       break;
 
     case '$':
-      format = Common::Base::F_16;
+      format = Common::Base::Fmt::_16;
       ch++;
       break;
 
@@ -137,7 +137,7 @@ int const_to_int(char* ch)
 
   int ret = 0;
   switch(format) {
-    case Common::Base::F_2:
+    case Common::Base::Fmt::_2:
       while(*ch) {
         if(*ch != '0' && *ch != '1')
           return -1;
@@ -147,7 +147,7 @@ int const_to_int(char* ch)
       }
       return ret;
 
-    case Common::Base::F_10:
+    case Common::Base::Fmt::_10:
       while(*ch) {
         if(!isdigit(*ch))
           return -1;
@@ -157,7 +157,7 @@ int const_to_int(char* ch)
       }
       return ret;
 
-    case Common::Base::F_16:
+    case Common::Base::Fmt::_16:
       while(*ch) { // FIXME: error check!
         if(!isxdigit(*ch))
           return -1;
@@ -180,7 +180,7 @@ int const_to_int(char* ch)
 CartMethod getCartSpecial(char* ch)
 {
   if(BSPF::equalsIgnoreCase(ch, "_bank"))
-    return &CartDebug::getBank;
+    return &CartDebug::getPCBank;
   else if(BSPF::equalsIgnoreCase(ch, "__lastread"))
     return &CartDebug::lastReadBaseAddress;
   else if(BSPF::equalsIgnoreCase(ch, "__lastwrite"))
@@ -251,7 +251,7 @@ TiaMethod getTiaSpecial(char* ch)
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int yylex() {
-  static char idbuf[255];
+  static char idbuf[255];  // NOLINT  (will be rewritten soon)
   char o, p;
   yylval.val = 0;
   while(*c != '\0') {
